@@ -38,6 +38,7 @@
 
 #include "cpl_conv.h"
 #include "cpl_error.h"
+#include "cpl_mask.h"
 #include "cpl_minixml.h"
 #include "cpl_progress.h"
 #include "cpl_string.h"
@@ -51,7 +52,7 @@
 #include <emmintrin.h>
 #endif
 
-CPL_CVSID("$Id: gdalwarper.cpp 5d5a9396da0e1c66349b4fdbe9d01a6a719eb1e9 2021-10-26 11:22:02 +0200 Even Rouault $")
+CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
@@ -351,7 +352,7 @@ template<class T> static CPLErr GDALWarpNoDataMaskerT( const double *padfNoData,
         if( pData[iOffset] == nNoData )
         {
             bAllValid = FALSE;
-            panValidityMask[iOffset>>5] &= ~(0x01 << (iOffset & 0x1f));
+            CPLMaskClear(panValidityMask, iOffset);
         }
     }
     *pbOutAllValid = bAllValid;
@@ -431,7 +432,7 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                   (!bIsNoDataNan && ARE_REAL_EQUAL(fVal, fNoData)) )
               {
                   bAllValid = FALSE;
-                  panValidityMask[iOffset>>5] &= ~(0x01 << (iOffset & 0x1f));
+                  CPLMaskClear(panValidityMask, iOffset);
               }
           }
           *pbOutAllValid = bAllValid;
@@ -460,7 +461,7 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                   (!bIsNoDataNan && ARE_REAL_EQUAL(dfVal, dfNoData)) )
               {
                   bAllValid = FALSE;
-                  panValidityMask[iOffset>>5] &= ~(0x01 << (iOffset & 0x1f));
+                  CPLMaskClear(panValidityMask, iOffset);
               }
           }
           *pbOutAllValid = bAllValid;
@@ -492,8 +493,7 @@ GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                           iPixel + static_cast<size_t>(iLine) * nXSize;
 
                       bAllValid = FALSE;
-                      panValidityMask[iOffset>>5] &=
-                          ~(0x01 << (iOffset & 0x1f));
+                      CPLMaskClear(panValidityMask, iOffset);
                   }
               }
           }
@@ -790,7 +790,7 @@ GDALWarpSrcMaskMasker( void *pMaskFuncArg,
     for( size_t iPixel = 0; iPixel < nPixels; iPixel++ )
     {
         if( pabySrcMask[iPixel] == 0 )
-            panMask[iPixel>>5] &= ~(0x01 << (iPixel & 0x1f));
+            CPLMaskClear(panMask, iPixel);
     }
 
     CPLFree( pabySrcMask );
@@ -1246,6 +1246,12 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount,
  * window for a given request, and by default it is 1 to take care of rounding
  * error.  Setting this larger will increase the amount of data that needs to
  * be read, but can avoid missing source data.</li>
+ * <li>APPLY_VERTICAL_SHIFT=YES/NO: Force the use of vertical shift.
+ * This option is generally not necessary, except when using an explicit
+ * coordinate transformation (COORDINATE_OPERATION), and not specifying
+ * an explicit source and target SRS.</li>
+ * <li>MULT_FACTOR_VERTICAL_SHIFT: Multiplication factor for the vertical
+ * shift. Default 1.0</li>
  * </ul>
  */
 
@@ -1554,10 +1560,10 @@ GDALWarpResolveWorkingDataType( GDALWarpOptions *psOptions )
         psOptions->papszWarpOptions, "APPLY_VERTICAL_SHIFT", false);
     if( bApplyVerticalShift && GDALDataTypeIsInteger(psOptions->eWorkingDataType) )
     {
-        const double dfMultFactorVerticalShit = CPLAtof(
+        const double dfMultFactorVerticalShift = CPLAtof(
             CSLFetchNameValueDef(psOptions->papszWarpOptions,
                                  "MULT_FACTOR_VERTICAL_SHIFT", "1.0") );
-        if( dfMultFactorVerticalShit != 1 )
+        if( dfMultFactorVerticalShift != 1 )
         {
             psOptions->eWorkingDataType =
                 GDALDataTypeUnion( psOptions->eWorkingDataType, GDT_Float32 );
@@ -1950,11 +1956,17 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 
     if( pszValue != nullptr )
     {
+        CPLXMLNode* psGeoLocNode = CPLSearchXMLNode(psTree, "GeoLocTransformer");
+        if( psGeoLocNode )
+        {
+            CPLCreateXMLElementAndValue(psGeoLocNode, "SourceDataset", pszValue);
+        }
+
         CPLConfigOptionSetter oSetter("CPL_ALLOW_VSISTDIN", "NO", true);
 
         char** papszOpenOptions = GDALDeserializeOpenOptionsFromXML(psTree);
         psWO->hSrcDS = GDALOpenEx(
-            pszValue, GDAL_OF_SHARED | GDAL_OF_RASTER | GDAL_OF_VERBOSE_ERROR,
+            pszValue, GDAL_OF_RASTER | GDAL_OF_VERBOSE_ERROR,
             nullptr,
             papszOpenOptions, nullptr );
         CSLDestroy(papszOpenOptions);
