@@ -28,13 +28,15 @@
 
 #include "cpl_string.h"
 #include "gdal_version.h"
+#include "gdal_priv.h"
 #include "gdal.h"
 #include "commonutils.h"
 #include "ogr_spatialref.h"
 
+#include <cstdlib>
+#include <memory>
 #include <vector>
 
-CPL_CVSID("$Id: gdal_create.cpp 1f859e21fc357aade14433e7b1faef2ac9f0c259 2020-09-13 16:15:14 +0200 Even Rouault $")
 
 static void Usage()
 
@@ -44,11 +46,12 @@ static void Usage()
             "       [-outsize xsize ysize]\n"
             "       [-bands count]\n"
             "       [-burn value]*\n"
-            "       [-ot {Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/\n"
+            "       [-ot {Byte/Int16/UInt16/UInt32/Int32/UInt64/Int64/Float32/Float64/\n"
             "             CInt16/CInt32/CFloat32/CFloat64}] [-strict]\n"
             "       [-a_srs srs_def] [-a_ullr ulx uly lrx lry] [-a_nodata value]\n"
             "       [-mo \"META-TAG=VALUE\"]* [-q]\n"
             "       [-co \"NAME=VALUE\"]*\n"
+            "       [-if input_dataset]\n"
             "       out_dataset\n" );
 
     exit(1);
@@ -104,8 +107,9 @@ MAIN_START(argc, argv)
     CPLStringList aosMetadata;
     std::vector<double> adfBurnValues;
     bool bQuiet = false;
-    bool bSetNoData = false;
-    double dfNoDataValue = 0;
+    int bSetNoData = false;
+    std::string osNoData;
+    const char* pszInputFile = nullptr;
     for( int i = 1; argv != nullptr && argv[i] != nullptr; i++ )
     {
         if( EQUAL(argv[i], "--utility_version") )
@@ -189,7 +193,7 @@ MAIN_START(argc, argv)
             bSetNoData = true;
             ++i;
             // coverity[tainted_data]
-            dfNoDataValue = CPLAtofM(argv[i]);
+            osNoData = argv[i];
         }
 
         else if( i < argc-1 && EQUAL(argv[i],"-burn") )
@@ -214,6 +218,11 @@ MAIN_START(argc, argv)
                 }
             }
         }
+        else if( i < argc-1 && EQUAL(argv[i],"-if") )
+        {
+            ++i;
+            pszInputFile = argv[i];
+        }
         else if( EQUAL(argv[i], "-q") )
         {
             bQuiet = true;
@@ -243,6 +252,74 @@ MAIN_START(argc, argv)
         Usage();
     }
 
+    double adfGeoTransform[6] = {0, 1, 0, 0, 0, 1};
+    if( bGeoTransform && nPixels > 0 && nLines > 0 )
+    {
+        adfGeoTransform[0] = dfULX;
+        adfGeoTransform[1] = (dfLRX - dfULX) / nPixels;
+        adfGeoTransform[2] = 0;
+        adfGeoTransform[3] = dfULY;
+        adfGeoTransform[4] = 0;
+        adfGeoTransform[5] = (dfLRY - dfULY) / nLines;
+    }
+
+    std::unique_ptr<GDALDataset> poInputDS;
+    if( pszInputFile )
+    {
+        poInputDS.reset(GDALDataset::Open(pszInputFile, GDAL_OF_RASTER | GDAL_OF_VERBOSE_ERROR));
+        if( poInputDS == nullptr )
+        {
+            CSLDestroy( argv );
+            GDALDestroyDriverManager();
+            exit( 1 );
+        }
+        if( nPixels == 0 )
+        {
+            nPixels = poInputDS->GetRasterXSize();
+            nLines = poInputDS->GetRasterYSize();
+        }
+        if( nBandCount < 0 )
+        {
+            nBandCount = poInputDS->GetRasterCount();
+        }
+        if( eDT == GDT_Unknown && poInputDS->GetRasterCount() > 0 )
+        {
+            eDT = poInputDS->GetRasterBand(1)->GetRasterDataType();
+        }
+        if( pszOutputSRS == nullptr )
+        {
+            pszOutputSRS = poInputDS->GetProjectionRef();
+        }
+        if( ! (bGeoTransform && nPixels > 0 && nLines > 0) )
+        {
+            if( poInputDS->GetGeoTransform(adfGeoTransform) == CE_None )
+            {
+                bGeoTransform = true;
+            }
+        }
+        if( !bSetNoData && poInputDS->GetRasterCount() > 0 )
+        {
+            if( eDT == GDT_Int64 )
+            {
+                const auto nNoDataValue = poInputDS->GetRasterBand(1)->GetNoDataValueAsInt64(&bSetNoData);
+                if( bSetNoData )
+                    osNoData = CPLSPrintf(CPL_FRMT_GIB, static_cast<GIntBig>(nNoDataValue));
+            }
+            else if( eDT == GDT_UInt64 )
+            {
+                const auto nNoDataValue = poInputDS->GetRasterBand(1)->GetNoDataValueAsUInt64(&bSetNoData);
+                if( bSetNoData )
+                    osNoData = CPLSPrintf(CPL_FRMT_GUIB, static_cast<GUIntBig>(nNoDataValue));
+            }
+            else
+            {
+                const double dfNoDataValue = poInputDS->GetRasterBand(1)->GetNoDataValue(&bSetNoData);
+                if( bSetNoData )
+                    osNoData = CPLSPrintf("%.18g", dfNoDataValue);
+            }
+        }
+    }
+
     GDALDriverH hDriver = GDALGetDriverByName(
         pszFormat ? pszFormat : GetOutputDriverForRaster( pszFilename ).c_str());
     if ( hDriver == nullptr )
@@ -291,7 +368,7 @@ MAIN_START(argc, argv)
         exit( 1 );
     }
 
-    if( pszOutputSRS )
+    if( pszOutputSRS && pszOutputSRS[0] != '\0' && !EQUAL(pszOutputSRS, "NONE") )
     {
         OGRSpatialReference oSRS;
         oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
@@ -327,13 +404,6 @@ MAIN_START(argc, argv)
             GDALDestroyDriverManager();
             exit( 1 );
         }
-        double adfGeoTransform[6];
-        adfGeoTransform[0] = dfULX;
-        adfGeoTransform[1] = (dfLRX - dfULX) / nPixels;
-        adfGeoTransform[2] = 0;
-        adfGeoTransform[3] = dfULY;
-        adfGeoTransform[4] = 0;
-        adfGeoTransform[5] = (dfLRY - dfULY) / nLines;
         if( GDALSetGeoTransform(hDS, adfGeoTransform) != CE_None )
         {
             GDALClose(hDS);
@@ -351,8 +421,22 @@ MAIN_START(argc, argv)
     {
         for( int i = 0; i < nBands; i++ )
         {
-            GDALSetRasterNoDataValue(GDALGetRasterBand(hDS, i+1),
-                                     dfNoDataValue);
+            auto hBand = GDALGetRasterBand(hDS, i+1);
+            if( eDT == GDT_Int64 )
+            {
+                GDALSetRasterNoDataValueAsInt64(hBand,
+                    static_cast<int64_t>(std::strtoll(osNoData.c_str(), nullptr, 10)));
+            }
+            else if( eDT == GDT_UInt64 )
+            {
+                GDALSetRasterNoDataValueAsUInt64(hBand,
+                    static_cast<uint64_t>(std::strtoull(osNoData.c_str(), nullptr, 10)));
+            }
+            else
+            {
+                GDALSetRasterNoDataValue(hBand,
+                                         CPLAtofM(osNoData.c_str()));
+            }
         }
     }
     if( !adfBurnValues.empty() )
