@@ -56,7 +56,6 @@
 #include "ogr_srs_api.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                          GDALGridOptions                             */
@@ -129,14 +128,14 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
             static_cast<GDALGridInverseDistanceToAPowerOptions *>(pOptions);
         CPLprintf("Options are "
                   "\"power=%f:smoothing=%f:radius1=%f:radius2=%f:angle=%f"
-                  ":max_points=%lu:min_points=%lu:nodata=%f\"\n",
+                  ":max_points=%u:min_points=%u:nodata=%f\"\n",
                   pOptions2->dfPower,
                   pOptions2->dfSmoothing,
                   pOptions2->dfRadius1,
                   pOptions2->dfRadius2,
                   pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMaxPoints),
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
+                  pOptions2->nMaxPoints,
+                  pOptions2->nMinPoints,
                   pOptions2->dfNoDataValue);
         break;
     }
@@ -146,15 +145,22 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
         GDALGridInverseDistanceToAPowerNearestNeighborOptions *pOptions2 =
             static_cast<GDALGridInverseDistanceToAPowerNearestNeighborOptions *>(
                 pOptions);
-        CPLprintf("Options are "
-                  "\"power=%f:smoothing=%f:radius=%f"
-                  ":max_points=%lu:min_points=%lu:nodata=%f\"\n",
-                  pOptions2->dfPower,
-                  pOptions2->dfSmoothing,
-                  pOptions2->dfRadius,
-                  static_cast<unsigned long>(pOptions2->nMaxPoints),
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
+        CPLString osStr;
+        osStr.Printf("power=%f:smoothing=%f:radius=%f"
+                     ":max_points=%u:min_points=%u:nodata=%f",
+                     pOptions2->dfPower,
+                     pOptions2->dfSmoothing,
+                     pOptions2->dfRadius,
+                     pOptions2->nMaxPoints,
+                     pOptions2->nMinPoints,
+                     pOptions2->dfNoDataValue);
+        if( pOptions2->nMinPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":min_points_per_quadrant=%u",
+                                pOptions2->nMinPointsPerQuadrant);
+        if( pOptions2->nMaxPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":max_points_per_quadrant=%u",
+                                pOptions2->nMaxPointsPerQuadrant);
+        printf("Options are: \"%s\n", osStr.c_str()); /* ok */
         break;
     }
     case GGA_MovingAverage:
@@ -162,14 +168,24 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
         printf("Algorithm name: \"%s\".\n", szAlgNameAverage);
         GDALGridMovingAverageOptions *pOptions2 =
             static_cast<GDALGridMovingAverageOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
+        CPLString osStr;
+        osStr.Printf("radius1=%f:radius2=%f:angle=%f:min_points=%u"
+                     ":nodata=%f",
+                     pOptions2->dfRadius1,
+                     pOptions2->dfRadius2,
+                     pOptions2->dfAngle,
+                     pOptions2->nMinPoints,
+                     pOptions2->dfNoDataValue);
+        if( pOptions2->nMinPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":min_points_per_quadrant=%u",
+                                pOptions2->nMinPointsPerQuadrant);
+        if( pOptions2->nMaxPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":max_points_per_quadrant=%u",
+                                pOptions2->nMaxPointsPerQuadrant);
+        if( pOptions2->nMaxPoints > 0 )
+            osStr += CPLSPrintf(":max_points=%u",
+                                pOptions2->nMaxPoints);
+        printf("Options are: \"%s\n", osStr.c_str()); /* ok */
         break;
     }
     case GGA_NearestNeighbor:
@@ -186,93 +202,55 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
         break;
     }
     case GGA_MetricMinimum:
-    {
-        printf("Algorithm name: \"%s\".\n", szAlgNameMinimum);
-        GDALGridDataMetricsOptions *pOptions2 =
-            static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
-            break;
-    }
     case GGA_MetricMaximum:
-    {
-        printf("Algorithm name: \"%s\".\n", szAlgNameMaximum);
-        GDALGridDataMetricsOptions *pOptions2 =
-            static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
-        break;
-    }
     case GGA_MetricRange:
-    {
-        printf("Algorithm name: \"%s\".\n", szAlgNameRange);
-        GDALGridDataMetricsOptions *pOptions2 =
-            static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
-        break;
-    }
     case GGA_MetricCount:
-    {
-        printf("Algorithm name: \"%s\".\n", szAlgNameCount);
-        GDALGridDataMetricsOptions *pOptions2 =
-            static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
-        break;
-    }
     case GGA_MetricAverageDistance:
-    {
-        printf("Algorithm name: \"%s\".\n", szAlgNameAverageDistance);
-        GDALGridDataMetricsOptions *pOptions2 =
-            static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
-        break;
-    }
     case GGA_MetricAverageDistancePts:
     {
-        printf("Algorithm name: \"%s\".\n", szAlgNameAverageDistancePts);
+        const char* pszAlgName = "";
+        switch ( eAlgorithm )
+        {
+            case GGA_MetricMinimum:
+                pszAlgName = szAlgNameMinimum;
+                break;
+            case GGA_MetricMaximum:
+                pszAlgName = szAlgNameMaximum;
+                break;
+            case GGA_MetricRange:
+                pszAlgName = szAlgNameRange;
+                break;
+            case GGA_MetricCount:
+                pszAlgName = szAlgNameCount;
+                break;
+            case GGA_MetricAverageDistance:
+                pszAlgName = szAlgNameAverageDistance;
+                break;
+            case GGA_MetricAverageDistancePts:
+                pszAlgName = szAlgNameAverageDistancePts;
+                break;
+            default:
+                CPLAssert(false);
+                break;
+        }
+        printf("Algorithm name: \"%s\".\n", pszAlgName);
         GDALGridDataMetricsOptions *pOptions2 =
             static_cast<GDALGridDataMetricsOptions *>(pOptions);
-        CPLprintf("Options are "
-                  "\"radius1=%f:radius2=%f:angle=%f:min_points=%lu"
-                  ":nodata=%f\"\n",
-                  pOptions2->dfRadius1,
-                  pOptions2->dfRadius2,
-                  pOptions2->dfAngle,
-                  static_cast<unsigned long>(pOptions2->nMinPoints),
-                  pOptions2->dfNoDataValue);
+        CPLString osStr;
+        osStr.Printf("radius1=%f:radius2=%f:angle=%f:min_points=%u"
+                     ":nodata=%f",
+                     pOptions2->dfRadius1,
+                     pOptions2->dfRadius2,
+                     pOptions2->dfAngle,
+                     pOptions2->nMinPoints,
+                     pOptions2->dfNoDataValue);
+        if( pOptions2->nMinPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":min_points_per_quadrant=%u",
+                                pOptions2->nMinPointsPerQuadrant);
+        if( pOptions2->nMaxPointsPerQuadrant > 0 )
+            osStr += CPLSPrintf(":max_points_per_quadrant=%u",
+                                pOptions2->nMaxPointsPerQuadrant);
+        printf("Options are: \"%s\n", osStr.c_str()); /* ok */
         break;
     }
     case GGA_Linear:
@@ -295,107 +273,37 @@ static void PrintAlgorithmAndOptions( GDALGridAlgorithm eAlgorithm,
 }
 
 /************************************************************************/
-/*                          ProcessGeometry()                           */
-/*                                                                      */
 /*  Extract point coordinates from the geometry reference and set the   */
 /*  Z value as requested. Test whether we are in the clipped region     */
 /*  before processing.                                                  */
 /************************************************************************/
 
-static void ProcessGeometry( OGRPoint *poGeom, OGRGeometry *poClipSrc,
-                             int iBurnField, double dfBurnValue,
-                             const double dfIncreaseBurnValue,
-                             const double dfMultiplyBurnValue,
-                             std::vector<double> &adfX,
-                             std::vector<double> &adfY,
-                             std::vector<double> &adfZ )
-
+class GDALGridGeometryVisitor final: public OGRDefaultConstGeometryVisitor
 {
-    if ( poClipSrc && !poGeom->Within(poClipSrc) )
-        return;
+public:
+    const OGRGeometry *poClipSrc = nullptr;
+    int iBurnField = 0;
+    double dfBurnValue = 0;
+    double dfIncreaseBurnValue = 0;
+    double dfMultiplyBurnValue = 1;
+    std::vector<double> adfX{};
+    std::vector<double> adfY{};
+    std::vector<double> adfZ{};
 
-    adfX.push_back( poGeom->getX() );
-    adfY.push_back( poGeom->getY() );
-    if ( iBurnField < 0 )
-        adfZ.push_back(  (poGeom->getZ() + dfIncreaseBurnValue) * dfMultiplyBurnValue  );
-    else
-        adfZ.push_back( (dfBurnValue + dfIncreaseBurnValue) * dfMultiplyBurnValue );
-}
-
-/************************************************************************/
-/*                       ProcessCommonGeometry()                        */
-/*                                                                      */
-/*  Process recursively geometry and extract points.                    */
-/************************************************************************/
-
-static void ProcessCommonGeometry(OGRGeometry* poGeom, OGRGeometry *poClipSrc,
-                                int iBurnField, double dfBurnValue,
-                                const double dfIncreaseBurnValue,
-                                const double dfMultiplyBurnValue,
-                                std::vector<double> &adfX,
-                                std::vector<double> &adfY,
-                                std::vector<double> &adfZ)
-{
-    if (nullptr == poGeom)
-        return;
-
-    OGRwkbGeometryType eType = wkbFlatten(poGeom->getGeometryType());
-    switch (eType)
+    using OGRDefaultConstGeometryVisitor::visit;
+    void visit(const OGRPoint* p) override
     {
-    case wkbPoint:
-        return ProcessGeometry(poGeom->toPoint(), poClipSrc,
-            iBurnField, dfBurnValue, dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
-    case wkbLinearRing:
-    case wkbLineString:
-        {
-            OGRLineString *poLS = poGeom->toLineString();
-            OGRPoint point;
-            for (int pointIndex = 0; pointIndex < poLS->getNumPoints(); pointIndex++)
-            {
-                poLS->getPoint(pointIndex, &point);
-                ProcessCommonGeometry(&point, poClipSrc,
-                    iBurnField, dfBurnValue, dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
-            }
-        }
-        break;
-    case wkbPolygon:
-        {
-            OGRPolygon* poPoly = poGeom->toPolygon();
-            OGRLinearRing* poRing = poPoly->getExteriorRing();
-            ProcessCommonGeometry(poRing, poClipSrc,
-                iBurnField, dfBurnValue, dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
+        if ( poClipSrc && !p->Within(poClipSrc) )
+            return;
 
-            const int nRings = poPoly->getNumInteriorRings();
-            if (nRings > 0)
-            {
-                for (int ir = 0; ir < nRings; ++ir)
-                {
-                    poRing = poPoly->getInteriorRing(ir);
-                    ProcessCommonGeometry(poRing, poClipSrc,
-                        iBurnField, dfBurnValue, dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
-                }
-            }
-        }
-        break;
-    case wkbMultiPoint:
-    case wkbMultiPolygon:
-    case wkbMultiLineString:
-    case wkbGeometryCollection:
-        {
-            OGRGeometryCollection* pOGRGeometryCollection = poGeom->toGeometryCollection();
-            for (int i = 0; i < pOGRGeometryCollection->getNumGeometries(); ++i)
-            {
-                ProcessCommonGeometry(pOGRGeometryCollection->getGeometryRef(i), poClipSrc,
-                    iBurnField, dfBurnValue, dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
-            }
-        }
-        break;
-    case wkbUnknown:
-    case wkbNone:
-    default:
-        break;
+        adfX.push_back( p->getX() );
+        adfY.push_back( p->getY() );
+        if ( iBurnField < 0 )
+            adfZ.push_back( (p->getZ() + dfIncreaseBurnValue) * dfMultiplyBurnValue  );
+        else
+            adfZ.push_back( (dfBurnValue + dfIncreaseBurnValue) * dfMultiplyBurnValue );
     }
-}
+};
 
 /************************************************************************/
 /*                            ProcessLayer()                            */
@@ -405,7 +313,7 @@ static void ProcessCommonGeometry(OGRGeometry* poGeom, OGRGeometry *poClipSrc,
 /************************************************************************/
 
 static CPLErr ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
-                          OGRGeometry *poClipSrc,
+                          const OGRGeometry *poClipSrc,
                           int nXSize, int nYSize, int nBand,
                           bool& bIsXExtentSet, bool& bIsYExtentSet,
                           double& dfXMin, double& dfXMax,
@@ -441,26 +349,28 @@ static CPLErr ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
 /*      Collect the geometries from this layer, and build list of       */
 /*      values to be interpolated.                                      */
 /* -------------------------------------------------------------------- */
-    OGRFeature *poFeat;
-    std::vector<double> adfX, adfY, adfZ;
+    GDALGridGeometryVisitor oVisitor;
+    oVisitor.poClipSrc = poClipSrc;
+    oVisitor.iBurnField = iBurnField;
+    oVisitor.dfIncreaseBurnValue = dfIncreaseBurnValue;
+    oVisitor.dfMultiplyBurnValue = dfMultiplyBurnValue;
 
-    OGR_L_ResetReading( hSrcLayer );
-
-    while( (poFeat = reinterpret_cast<OGRFeature*>(OGR_L_GetNextFeature( hSrcLayer ))) != nullptr )
+    for( auto&& poFeat: OGRLayer::FromHandle(hSrcLayer) )
     {
-        OGRGeometry *poGeom = poFeat->GetGeometryRef();
-        double  dfBurnValue = 0.0;
+        const OGRGeometry *poGeom = poFeat->GetGeometryRef();
+        if( poGeom )
+        {
+            double  dfBurnValue = 0.0;
 
-        if ( iBurnField >= 0 )
-            dfBurnValue = poFeat->GetFieldAsDouble( iBurnField );
+            if ( iBurnField >= 0 )
+                dfBurnValue = poFeat->GetFieldAsDouble( iBurnField );
+            oVisitor.dfBurnValue = dfBurnValue;
 
-        ProcessCommonGeometry(poGeom, poClipSrc, iBurnField, dfBurnValue,
-            dfIncreaseBurnValue, dfMultiplyBurnValue, adfX, adfY, adfZ);
-
-        OGRFeature::DestroyFeature( poFeat );
+            poGeom->accept(&oVisitor);
+        }
     }
 
-    if ( adfX.empty() )
+    if ( oVisitor.adfX.empty() )
     {
         printf( "No point geometry found on layer %s, skipping.\n",
                 OGR_FD_GetName( OGR_L_GetLayerDefn( hSrcLayer ) ) );
@@ -490,6 +400,10 @@ static CPLErr ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
         }
     }
 
+    // Produce north-up images
+    if( dfYMin < dfYMax )
+        std::swap(dfYMin, dfYMax);
+
 /* -------------------------------------------------------------------- */
 /*      Perform gridding.                                               */
 /* -------------------------------------------------------------------- */
@@ -500,15 +414,12 @@ static CPLErr ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
     if ( !bQuiet )
     {
         printf( "Grid data type is \"%s\"\n", GDALGetDataTypeName(eType) );
-        printf("Grid size = (%lu %lu).\n",
-               static_cast<unsigned long>(nXSize),
-               static_cast<unsigned long>(nYSize));
+        printf("Grid size = (%d %d).\n", nXSize, nYSize);
         CPLprintf( "Corner coordinates = (%f %f)-(%f %f).\n",
-                dfXMin - dfDeltaX / 2, dfYMax + dfDeltaY / 2,
-                dfXMax + dfDeltaX / 2, dfYMin - dfDeltaY / 2 );
+                   dfXMin, dfYMin, dfXMax, dfYMax );
         CPLprintf( "Grid cell size = (%f %f).\n", dfDeltaX, dfDeltaY );
         printf("Source point count = %lu.\n",
-               static_cast<unsigned long>(adfX.size()));
+               static_cast<unsigned long>(oVisitor.adfX.size()));
         PrintAlgorithmAndOptions( eAlgorithm, pOptions );
         printf("\n");
     }
@@ -558,8 +469,8 @@ static CPLErr ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
         ((nYSize + nBlockYSize - 1) / nBlockYSize);
 
     GDALGridContext* psContext = GDALGridContextCreate( eAlgorithm, pOptions,
-                                                        static_cast<int>(adfX.size()),
-                                                        &(adfX[0]), &(adfY[0]), &(adfZ[0]),
+                                                        static_cast<int>(oVisitor.adfX.size()),
+                                                        &(oVisitor.adfX[0]), &(oVisitor.adfY[0]), &(oVisitor.adfZ[0]),
                                                         TRUE );
     if( psContext == nullptr )
     {
