@@ -60,7 +60,6 @@
 #include "ogr_spatialref.h"
 #include "vrtdataset.h"
 
-CPL_CVSID("$Id: gdalbuildvrt_lib.cpp f6daca27a7b3cdca6601f6f7757755863dcb5a2f 2022-03-30 21:39:17 +0200 Even Rouault $")
 
 #define GEOTRSFRM_TOPLEFT_X            0
 #define GEOTRSFRM_WE_RES               1
@@ -148,15 +147,15 @@ static int  GetSrcDstWin(DatasetProperty* psDP,
     /* Check that the destination bounding box intersects the source bounding box */
     if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_X] +
          psDP->nRasterXSize *
-         psDP->adfGeoTransform[GEOTRSFRM_WE_RES] < minX )
+         psDP->adfGeoTransform[GEOTRSFRM_WE_RES] <= minX )
          return FALSE;
-    if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_X] > maxX )
+    if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_X] >= maxX )
          return FALSE;
     if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_Y] +
          psDP->nRasterYSize *
-         psDP->adfGeoTransform[GEOTRSFRM_NS_RES] > maxY )
+         psDP->adfGeoTransform[GEOTRSFRM_NS_RES] >= maxY )
          return FALSE;
-    if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_Y] < minY )
+    if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_Y] <= minY )
          return FALSE;
 
     if ( psDP->adfGeoTransform[GEOTRSFRM_TOPLEFT_X] < minX )
@@ -208,7 +207,8 @@ static int  GetSrcDstWin(DatasetProperty* psDP,
         *pdfSrcYSize = *pdfDstYSize / dfSrcToDstYSize;
     }
 
-    return TRUE;
+    return *pdfSrcXSize > 0 && *pdfDstXSize > 0 &&
+           *pdfSrcYSize > 0 && *pdfDstYSize > 0;
 }
 
 /************************************************************************/
@@ -928,6 +928,8 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
         if (psDatasetProperties->isFileOK == FALSE)
             continue;
 
+        const char* dsFileName = ppszInputFilenames[i];
+
         double dfSrcXOff, dfSrcYOff, dfSrcXSize, dfSrcYSize,
                dfDstXOff, dfDstYOff, dfDstXSize, dfDstYSize;
         if (bHasGeoTransform)
@@ -937,7 +939,12 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
                         nRasterXSize, nRasterYSize,
                         &dfSrcXOff, &dfSrcYOff, &dfSrcXSize, &dfSrcYSize,
                         &dfDstXOff, &dfDstYOff, &dfDstXSize, &dfDstYSize) )
+            {
+                CPLDebug("BuildVRT",
+                         "Skipping %s as not intersecting area of interest",
+                         dsFileName);
                 continue;
+            }
         }
         else
         {
@@ -946,23 +953,36 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
             dfSrcYSize = dfDstYSize = nRasterYSize;
         }
 
-        const char* dsFileName = ppszInputFilenames[i];
-
         GDALAddBand(hVRTDS, psDatasetProperties->firstBandType, nullptr);
 
-        GDALProxyPoolDatasetH hProxyDS =
-            GDALProxyPoolDatasetCreate(dsFileName,
-                                        psDatasetProperties->nRasterXSize,
-                                        psDatasetProperties->nRasterYSize,
-                                        GA_ReadOnly, TRUE, pszProjectionRef,
-                                        psDatasetProperties->adfGeoTransform);
-        reinterpret_cast<GDALProxyPoolDataset*>(hProxyDS)->
-                                        SetOpenOptions( papszOpenOptions );
 
-        GDALProxyPoolDatasetAddSrcBandDescription(hProxyDS,
-                                            psDatasetProperties->firstBandType,
-                                            psDatasetProperties->nBlockXSize,
-                                            psDatasetProperties->nBlockYSize);
+        GDALDatasetH hSourceDS;
+        bool bDropRef = false;
+        if( nSrcDSCount == nInputFiles &&
+            GDALGetDatasetDriver(pahSrcDS[i]) != nullptr &&
+            ( dsFileName[0] == '\0' || // could be a unnamed VRT file
+              EQUAL(GDALGetDescription(GDALGetDatasetDriver(pahSrcDS[i])), "MEM")) )
+        {
+            hSourceDS = pahSrcDS[i];
+        }
+        else
+        {
+            bDropRef = true;
+            GDALProxyPoolDatasetH hProxyDS =
+                GDALProxyPoolDatasetCreate(dsFileName,
+                                            psDatasetProperties->nRasterXSize,
+                                            psDatasetProperties->nRasterYSize,
+                                            GA_ReadOnly, TRUE, pszProjectionRef,
+                                            psDatasetProperties->adfGeoTransform);
+            hSourceDS = static_cast<GDALDatasetH>(hProxyDS);
+            reinterpret_cast<GDALProxyPoolDataset*>(hProxyDS)->
+                                            SetOpenOptions( papszOpenOptions );
+
+            GDALProxyPoolDatasetAddSrcBandDescription(hProxyDS,
+                                                psDatasetProperties->firstBandType,
+                                                psDatasetProperties->nBlockXSize,
+                                                psDatasetProperties->nBlockYSize);
+        }
 
         VRTSourcedRasterBandH hVRTBand =
                 static_cast<VRTSourcedRasterBandH>(GDALGetRasterBand(hVRTDS, iBand));
@@ -990,17 +1010,18 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
         VRTSimpleSource* poSimpleSource;
         if (bAllowSrcNoData)
         {
-            poSimpleSource = new VRTComplexSource();
+            auto poComplexSource = new VRTComplexSource();
+            poSimpleSource = poComplexSource;
             if (nSrcNoDataCount > 0)
             {
                 if (iBand-1 < nSrcNoDataCount)
-                    poSimpleSource->SetNoDataValue( padfSrcNoData[iBand-1] );
+                    poComplexSource->SetNoDataValue( padfSrcNoData[iBand-1] );
                 else
-                    poSimpleSource->SetNoDataValue( padfSrcNoData[nSrcNoDataCount - 1] );
+                    poComplexSource->SetNoDataValue( padfSrcNoData[nSrcNoDataCount - 1] );
             }
             else if( psDatasetProperties->abHasNoData[0] )
             {
-                poSimpleSource->SetNoDataValue( psDatasetProperties->adfNoDataValues[0] );
+                poComplexSource->SetNoDataValue( psDatasetProperties->adfNoDataValues[0] );
             }
         }
         else if( bUseSrcMaskBand && psDatasetProperties->abHasMaskBand[0] )
@@ -1015,7 +1036,7 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
         if( pszResampling )
             poSimpleSource->SetResampling(pszResampling);
         poVRTBand->ConfigureSource( poSimpleSource,
-                                    static_cast<GDALRasterBand*>(GDALGetRasterBand(static_cast<GDALDatasetH>(hProxyDS), 1)),
+                                    static_cast<GDALRasterBand*>(GDALGetRasterBand(hSourceDS, 1)),
                                     FALSE,
                                     dfSrcXOff, dfSrcYOff,
                                     dfSrcXSize, dfSrcYSize,
@@ -1030,7 +1051,10 @@ void VRTBuilder::CreateVRTSeparate(VRTDatasetH hVRTDS)
 
         poVRTBand->AddSource( poSimpleSource );
 
-        GDALDereferenceDataset(hProxyDS);
+        if( bDropRef )
+        {
+            GDALDereferenceDataset(hSourceDS);
+        }
 
         iBand ++;
     }
@@ -1088,6 +1112,8 @@ void VRTBuilder::CreateVRTNonSeparate(VRTDatasetH hVRTDS)
         if (psDatasetProperties->isFileOK == FALSE)
             continue;
 
+        const char* dsFileName = ppszInputFilenames[i];
+
         double dfSrcXOff;
         double dfSrcYOff;
         double dfSrcXSize;
@@ -1101,7 +1127,12 @@ void VRTBuilder::CreateVRTNonSeparate(VRTDatasetH hVRTDS)
                         nRasterXSize, nRasterYSize,
                         &dfSrcXOff, &dfSrcYOff, &dfSrcXSize, &dfSrcYSize,
                         &dfDstXOff, &dfDstYOff, &dfDstXSize, &dfDstYSize) )
+        {
+            CPLDebug("BuildVRT",
+                     "Skipping %s as not intersecting area of interest",
+                     dsFileName);
             continue;
+        }
 
         anIdxValidDatasets.push_back(i);
 
@@ -1120,8 +1151,6 @@ void VRTBuilder::CreateVRTNonSeparate(VRTDatasetH hVRTDS)
                 anOverviewFactorsSet.insert(nOvFactor);
         }
 
-        const char* dsFileName = ppszInputFilenames[i];
-
         GDALDatasetH hSourceDS;
         bool bDropRef = false;
 
@@ -1174,8 +1203,9 @@ void VRTBuilder::CreateVRTNonSeparate(VRTDatasetH hVRTDS)
             VRTSimpleSource* poSimpleSource;
             if (bAllowSrcNoData && psDatasetProperties->abHasNoData[nSelBand - 1])
             {
-                poSimpleSource = new VRTComplexSource();
-                poSimpleSource->SetNoDataValue( psDatasetProperties->adfNoDataValues[nSelBand - 1] );
+                auto poComplexSource = new VRTComplexSource();
+                poSimpleSource = poComplexSource;
+                poComplexSource->SetNoDataValue( psDatasetProperties->adfNoDataValues[nSelBand - 1] );
             }
             else if( bUseSrcMaskBand && psDatasetProperties->abHasMaskBand[nSelBand - 1] )
             {
@@ -1271,17 +1301,19 @@ void VRTBuilder::CreateVRTNonSeparate(VRTDatasetH hVRTDS)
             oIter = oIterNext;
         }
     }
-    if( !anOverviewFactorsSet.empty() )
+    if( !anOverviewFactorsSet.empty() &&
+        CPLTestBool(CPLGetConfigOption("VRT_VIRTUAL_OVERVIEWS", "YES")) )
     {
         std::vector<int> anOverviewFactors;
         anOverviewFactors.insert(anOverviewFactors.end(),
                                  anOverviewFactorsSet.begin(),
                                  anOverviewFactorsSet.end());
-        CPLConfigOptionSetter oSetter("VRT_VIRTUAL_OVERVIEWS", "YES", false);
+        const char* const apszOptions [] = { "VRT_VIRTUAL_OVERVIEWS=YES", nullptr };
         poVRTDS->BuildOverviews(pszResampling ? pszResampling : "nearest",
                                 static_cast<int>(anOverviewFactors.size()),
                                 &anOverviewFactors[0],
-                                0, nullptr, nullptr, nullptr);
+                                0, nullptr, nullptr, nullptr,
+                                apszOptions );
     }
 }
 
