@@ -30,12 +30,12 @@
 #include "gdal_utils.h"
 #include "gdal_utils_priv.h"
 
+#include "cpl_json.h"
 #include "cpl_json_streaming_writer.h"
 #include "gdal_priv.h"
 #include <limits>
 #include <set>
 
-CPL_CVSID("$Id: gdalmdiminfo_lib.cpp 2e6733101e6356653e8c999cb7fe808abc22ed11 2020-08-11 14:57:51 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                       GDALMultiDimInfoOptions                        */
@@ -165,6 +165,12 @@ static void DumpValue(CPLJSonStreamingWriter& serializer,
         case GDT_UInt32:
             DumpValue<GUInt32>(serializer, bytes);
             break;
+        case GDT_Int64:
+            DumpValue<std::int64_t>(serializer, bytes);
+            break;
+        case GDT_UInt64:
+            DumpValue<std::uint64_t>(serializer, bytes);
+            break;
         case GDT_Float32:
             DumpValue<float>(serializer, bytes);
             break;
@@ -241,14 +247,89 @@ static void DumpValue(CPLJSonStreamingWriter& serializer,
     }
 }
 
+/************************************************************************/
+/*                          SerializeJSON()                             */
+/************************************************************************/
+
+static void SerializeJSON(const CPLJSONObject& obj,
+                          CPLJSonStreamingWriter& serializer)
+{
+    switch( obj.GetType() )
+    {
+        case CPLJSONObject::Type::Unknown:
+        {
+            CPLAssert(false);
+            break;
+        }
+
+        case CPLJSONObject::Type::Null:
+        {
+            serializer.AddNull();
+            break;
+        }
+
+        case CPLJSONObject::Type::Object:
+        {
+            auto objectContext(serializer.MakeObjectContext());
+            for( const auto& subobj: obj.GetChildren() )
+            {
+                serializer.AddObjKey(subobj.GetName());
+                SerializeJSON(subobj, serializer);
+            }
+            break;
+        }
+
+        case CPLJSONObject::Type::Array:
+        {
+            auto arrayContext(serializer.MakeArrayContext());
+            const CPLJSONArray array = obj.ToArray();
+            for( const auto& subobj: array )
+            {
+                SerializeJSON(subobj, serializer);
+            }
+            break;
+        }
+
+        case CPLJSONObject::Type::Boolean:
+        {
+            serializer.Add( obj.ToBool() );
+            break;
+        }
+
+        case CPLJSONObject::Type::String:
+        {
+            serializer.Add( obj.ToString() );
+            break;
+        }
+
+        case CPLJSONObject::Type::Integer:
+        {
+            serializer.Add( obj.ToInteger() );
+            break;
+        }
+
+        case CPLJSONObject::Type::Long:
+        {
+            serializer.Add( static_cast<int64_t>(obj.ToLong()) );
+            break;
+        }
+
+        case CPLJSONObject::Type::Double:
+        {
+            serializer.Add( obj.ToDouble() );
+            break;
+        }
+    }
+}
+
 /************************************************************************/
 /*                          DumpAttrValue()                             */
 /************************************************************************/
 
-static void DumpAttrValue(std::shared_ptr<GDALAttribute> attr,
+static void DumpAttrValue(const std::shared_ptr<GDALAttribute>& attr,
                           CPLJSonStreamingWriter& serializer)
 {
-    const auto dt(attr->GetDataType());
+    const auto& dt = attr->GetDataType();
     const size_t nEltCount(static_cast<size_t>(attr->GetTotalElementsCount()));
     switch( dt.GetClass() )
     {
@@ -258,9 +339,24 @@ static void DumpAttrValue(std::shared_ptr<GDALAttribute> attr,
             {
                 const char* pszStr = attr->ReadAsString();
                 if( pszStr )
-                    serializer.Add(pszStr);
-                else
-                    serializer.AddNull();
+                {
+                    if( dt.GetSubType() == GEDTST_JSON )
+                    {
+                        CPLJSONDocument oDoc;
+                        if( oDoc.LoadMemory(std::string(pszStr)) )
+                        {
+                            SerializeJSON(oDoc.GetRoot(), serializer);
+                        }
+                        else
+                        {
+                            serializer.Add(pszStr);
+                        }
+                    }
+                    else
+                    {
+                        serializer.Add(pszStr);
+                    }
+                }
             }
             else
             {
@@ -353,7 +449,7 @@ static void DumpAttr(std::shared_ptr<GDALAttribute> attr,
         return;
     }
 
-    const auto dt(attr->GetDataType());
+    const auto& dt = attr->GetDataType();
     auto objectContext(serializer.MakeObjectContext());
     if( bOutputObjType )
     {
@@ -371,6 +467,18 @@ static void DumpAttr(std::shared_ptr<GDALAttribute> attr,
         serializer.AddObjKey("datatype");
         DumpDataType(dt, serializer);
 
+        switch( dt.GetSubType() )
+        {
+            case GEDTST_NONE:
+                break;
+            case GEDTST_JSON:
+            {
+                serializer.AddObjKey("subtype");
+                serializer.Add("JSON");
+                break;
+            }
+        }
+
         serializer.AddObjKey("value");
     }
 
@@ -547,7 +655,7 @@ static void DumpDimensions(const std::vector<std::shared_ptr<GDALDimension>>& di
         serializer.Add(osFullname);
 
         serializer.AddObjKey("size");
-        serializer.Add(dim->GetSize());
+        serializer.Add(static_cast<std::uint64_t>(dim->GetSize()));
 
         const auto& type(dim->GetType());
         if( !type.empty() )
@@ -601,8 +709,7 @@ static void DumpStructuralInfo(CSLConstList papszStructuralInfo,
 /*                             DumpArray()                              */
 /************************************************************************/
 
-static void DumpArray(GDALDataset* poDS,
-                      std::shared_ptr<GDALMDArray> array,
+static void DumpArray(std::shared_ptr<GDALMDArray> array,
                      CPLJSonStreamingWriter& serializer,
                      const GDALMultiDimInfoOptions *psOptions,
                      std::set<std::string>& alreadyDumpedDimensions,
@@ -629,6 +736,34 @@ static void DumpArray(GDALDataset* poDS,
     {
         serializer.AddObjKey("dimensions");
         DumpDimensions(dims, serializer, psOptions, alreadyDumpedDimensions);
+
+        serializer.AddObjKey("dimension_size");
+        auto arrayContext(serializer.MakeArrayContext());
+        for( const auto& poDim: dims )
+        {
+            serializer.Add(static_cast<uint64_t>(poDim->GetSize()));
+        }
+    }
+
+
+    bool hasNonNullBlockSize = false;
+    const auto blockSize = array->GetBlockSize();
+    for( auto v: blockSize )
+    {
+        if( v != 0 )
+        {
+            hasNonNullBlockSize = true;
+            break;
+        }
+    }
+    if( hasNonNullBlockSize )
+    {
+        serializer.AddObjKey("block_size");
+        auto arrayContext(serializer.MakeArrayContext());
+        for( auto v: blockSize )
+        {
+            serializer.Add(static_cast<uint64_t>(v));
+        }
     }
 
     CPLStringList aosOptions;
@@ -727,7 +862,7 @@ static void DumpArray(GDALDataset* poDS,
         double dfMean = 0.0;
         double dfStdDev = 0.0;
         GUInt64 nValidCount = 0;
-        if( array->GetStatistics( poDS, false, true,
+        if( array->GetStatistics( false, true,
                               &dfMin, &dfMax, &dfMean, &dfStdDev,
                               &nValidCount,
                               nullptr, nullptr ) == CE_None )
@@ -750,7 +885,7 @@ static void DumpArray(GDALDataset* poDS,
             }
 
             serializer.AddObjKey("valid_sample_count");
-            serializer.Add(nValidCount);
+            serializer.Add(static_cast<std::uint64_t>(nValidCount));
         }
     }
 }
@@ -759,8 +894,7 @@ static void DumpArray(GDALDataset* poDS,
 /*                            DumpArrays()                              */
 /************************************************************************/
 
-static void DumpArrays(GDALDataset* poDS,
-                       std::shared_ptr<GDALGroup> group,
+static void DumpArrays(std::shared_ptr<GDALGroup> group,
                        const std::vector<std::string>& arrayNames,
                        CPLJSonStreamingWriter& serializer,
                        const GDALMultiDimInfoOptions *psOptions,
@@ -777,7 +911,7 @@ static void DumpArrays(GDALDataset* poDS,
         if( array )
         {
             serializer.AddObjKey(array->GetName());
-            DumpArray( poDS, array, serializer, psOptions,
+            DumpArray( array, serializer, psOptions,
                        alreadyDumpedDimensions, false, false );
         }
     }
@@ -787,8 +921,7 @@ static void DumpArrays(GDALDataset* poDS,
 /*                             DumpGroup()                              */
 /************************************************************************/
 
-static void DumpGroup(GDALDataset* poDS,
-                      std::shared_ptr<GDALGroup> group,
+static void DumpGroup(std::shared_ptr<GDALGroup> group,
                       const char* pszDriverName,
                       CPLJSonStreamingWriter& serializer,
                       const GDALMultiDimInfoOptions *psOptions,
@@ -810,8 +943,15 @@ static void DumpGroup(GDALDataset* poDS,
     {
         serializer.AddObjKey("name");
         serializer.Add(group->GetName());
+
+        // If the root group is not actually the root, print its full path
+        if( pszDriverName != nullptr && group->GetName() != "/" )
+        {
+            serializer.AddObjKey("full_name");
+            serializer.Add(group->GetFullName());
+        }
     }
-    
+
     CPLStringList aosOptionsGetAttr;
     if( psOptions->bDetailed )
         aosOptionsGetAttr.SetNameValue("SHOW_ALL", "YES");
@@ -836,7 +976,7 @@ static void DumpGroup(GDALDataset* poDS,
     if( !arrayNames.empty() )
     {
         serializer.AddObjKey("arrays");
-        DumpArrays(poDS, group, arrayNames, serializer, psOptions,
+        DumpArrays(group, arrayNames, serializer, psOptions,
                    alreadyDumpedDimensions);
     }
 
@@ -860,7 +1000,7 @@ static void DumpGroup(GDALDataset* poDS,
                 if( subgroup )
                 {
                     serializer.AddObjKey(subgroupName);
-                    DumpGroup( poDS, subgroup, nullptr, serializer, psOptions,
+                    DumpGroup(  subgroup, nullptr, serializer, psOptions,
                                alreadyDumpedDimensions, false, false );
                 }
             }
@@ -873,7 +1013,7 @@ static void DumpGroup(GDALDataset* poDS,
                 auto subgroup = group->OpenGroup(subgroupName);
                 if( subgroup )
                 {
-                    DumpGroup( poDS, subgroup, nullptr, serializer, psOptions,
+                    DumpGroup( subgroup, nullptr, serializer, psOptions,
                                alreadyDumpedDimensions, false, true );
                 }
             }
@@ -935,7 +1075,7 @@ char *GDALMultiDimInfo( GDALDatasetH hDataset,
             auto poDriver = poDS->GetDriver();
             if( poDriver )
                 pszDriverName = poDriver->GetDescription();
-            DumpGroup(poDS, group, pszDriverName, serializer, psOptions,
+            DumpGroup(group, pszDriverName, serializer, psOptions,
                       alreadyDumpedDimensions, true, true);
         }
         else
@@ -945,13 +1085,14 @@ char *GDALMultiDimInfo( GDALDatasetH hDataset,
                 psOptions->osArrayName.c_str(), "/", 0));
             for( int i = 0; i < aosTokens.size() - 1; i++ )
             {
-                curGroup = curGroup->OpenGroup(aosTokens[i]);
-                if( !curGroup )
+                auto curGroupNew = curGroup->OpenGroup(aosTokens[i]);
+                if( !curGroupNew )
                 {
                     CPLError(CE_Failure, CPLE_AppDefined,
                              "Cannot find group %s", aosTokens[i]);
                     return nullptr;
                 }
+                curGroup = curGroupNew;
             }
             const char* pszArrayName = aosTokens[aosTokens.size()-1];
             auto array(curGroup->OpenMDArray(pszArrayName));
@@ -961,7 +1102,7 @@ char *GDALMultiDimInfo( GDALDatasetH hDataset,
                          "Cannot find array %s", pszArrayName);
                 return nullptr;
             }
-            DumpArray(poDS, array, serializer, psOptions,
+            DumpArray(array, serializer, psOptions,
                       alreadyDumpedDimensions, true, true);
         }
     }
@@ -991,10 +1132,7 @@ char *GDALMultiDimInfo( GDALDatasetH hDataset,
  *
  * @param papszArgv NULL terminated list of options (potentially including filename and open options too), or NULL.
  *                  The accepted options are the ones of the <a href="/programs/gdalmdiminfo.html">gdalmdiminfo</a> utility.
- * @param psOptionsForBinary (output) may be NULL (and should generally be NULL),
- *                           otherwise (gdalmultidiminfo_bin.cpp use case) must be allocated with
- *                           GDALMultiDimInfoOptionsForBinaryNew() prior to this function. Will be
- *                           filled with potentially present filename, open options, subdataset number...
+ * @param psOptionsForBinary should be nullptr, unless called from gdalmultidiminfo_bin.cpp
  * @return pointer to the allocated GDALMultiDimInfoOptions struct. Must be freed with GDALMultiDimInfoOptionsFree().
  *
  * @since GDAL 3.1
@@ -1017,8 +1155,7 @@ GDALMultiDimInfoOptions *GDALMultiDimInfoOptionsNew(
             i++;
             if( psOptionsForBinary )
             {
-                psOptionsForBinary->papszOpenOptions = CSLAddString(
-                     psOptionsForBinary->papszOpenOptions, papszArgv[i] );
+                psOptionsForBinary->aosOpenOptions.AddString( papszArgv[i] );
             }
         }
         /* Not documented: used by gdalinfo_bin.cpp only */
@@ -1047,6 +1184,21 @@ GDALMultiDimInfoOptions *GDALMultiDimInfoOptionsNew(
         {
             psOptions->bStats = true;
         }
+
+        else if( EQUAL(papszArgv[i], "-if") && papszArgv[i+1] != nullptr )
+        {
+            i++;
+            if( psOptionsForBinary )
+            {
+                if( GDALGetDriverByName(papszArgv[i]) == nullptr )
+                {
+                    CPLError(CE_Warning, CPLE_AppDefined,
+                             "%s is not a recognized driver", papszArgv[i]);
+                }
+                psOptionsForBinary->aosAllowInputDrivers.AddString( papszArgv[i] );
+            }
+        }
+
         else if( papszArgv[i][0] == '-' )
         {
             CPLError(CE_Failure, CPLE_NotSupported,
@@ -1058,7 +1210,7 @@ GDALMultiDimInfoOptions *GDALMultiDimInfoOptionsNew(
         {
             bGotFilename = true;
             if( psOptionsForBinary )
-                psOptionsForBinary->pszFilename = CPLStrdup(papszArgv[i]);
+                psOptionsForBinary->osFilename = papszArgv[i];
         }
         else
         {
