@@ -33,7 +33,6 @@
 #include "gnm_priv.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: gnmanalyse.cpp 2dd182b498f0381aa3bc200928681dfbcf2ecfa6 2020-02-04 10:22:31 +0100 Enrico Weigelt, metux IT consult $")
 
 enum operation
 {
@@ -347,11 +346,9 @@ static void ReportOnLayer( OGRLayer * poLayer, int bVerbose )
 /* -------------------------------------------------------------------- */
 /*      Read, and dump features.                                        */
 /* -------------------------------------------------------------------- */
-    OGRFeature  *poFeature = nullptr;
-    while( (poFeature = poLayer->GetNextFeature()) != nullptr )
+    for( auto& poFeature: poLayer )
     {
         poFeature->DumpReadable( nullptr );
-        OGRFeature::DestroyFeature( poFeature );
     }
 }
 
