@@ -40,7 +40,6 @@
 #include "ogrsf_frmts.h"
 #include "commonutils.h"
 
-CPL_CVSID("$Id: ogrtindex.cpp 355b41831cd2685c85d1aabe5b95665a2c6e99b7 2019-06-19 17:07:04 +0200 Even Rouault $")
 
 typedef enum
 {
@@ -623,7 +622,7 @@ MAIN_START(nArgc, papszArgv)
                     {
                         fprintf(
                             stderr, "Note : you can override this "
-                            "behaviour with -accept_different_schemas option\n"
+                            "behavior with -accept_different_schemas option\n"
                             "but this may result in a tileindex incompatible "
                             "with MapServer\n");
                         bFirstWarningForNonMatchingAttributes = false;
@@ -657,7 +656,7 @@ MAIN_START(nArgc, papszArgv)
                         {
                             fprintf(
                                 stderr, "Note : you can override this "
-                                "behaviour with -accept_different_schemas "
+                                "behavior with -accept_different_schemas "
                                 "option,\nbut this may result in a tileindex "
                                 "incompatible with MapServer\n");
                             bFirstWarningForNonMatchingAttributes = false;
