@@ -33,7 +33,6 @@
 
 #include "georaster_priv.h"
 
-CPL_CVSID("$Id: cpl_vsil_ocilob.cpp $")
 
 // *****************************************************************************
 //                                                             WSIOCILobFSHandle
@@ -322,8 +321,8 @@ size_t VSIOCILobHandle::Read( void* pBuffer, size_t nSize, size_t nCount )
 
     GUIntBig nRead = poStatement->ReadBlob( phLocator,
                                             pBuffer,
-                                            (nCurOff + 1),
-                                            nBytes  );
+                                            static_cast<unsigned long>(nCurOff + 1),
+                                            static_cast<unsigned long>(nBytes)  );
 
     nCurOff += (GUIntBig) nRead;
 
@@ -347,8 +346,8 @@ size_t VSIOCILobHandle::Write( const void * pBuffer,
 
     GUIntBig nWrite = poStatement->WriteBlob( phLocator,
                                               (void*) pBuffer,
-                                              (nCurOff + 1),
-                                              nBytes );
+                                              static_cast<unsigned long>(nCurOff + 1),
+                                              static_cast<unsigned long>(nBytes) );
 
     nCurOff += (GUIntBig) nWrite;
 
