@@ -28,7 +28,6 @@
 
 #include "gifabstractdataset.h"
 
-CPL_CVSID("$Id: gifabstractdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 constexpr int InterlacedOffset[] = { 0, 4, 2, 1 };
 constexpr int InterlacedJumps[] = { 8, 8, 4, 2 };
@@ -46,7 +45,6 @@ constexpr int InterlacedJumps[] = { 8, 8, 4, 2 };
 GIFAbstractDataset::GIFAbstractDataset() :
     fp(nullptr),
     hGifFile(nullptr),
-    pszProjection(nullptr),
     bGeoTransformValid(FALSE),
     nGCPCount(0),
     pasGCPList(nullptr),
@@ -69,9 +67,6 @@ GIFAbstractDataset::~GIFAbstractDataset()
 {
     FlushCache(true);
 
-    if ( pszProjection )
-        CPLFree( pszProjection );
-
     if ( nGCPCount > 0 )
     {
         GDALDeinitGCPs( nGCPCount, pasGCPList );
@@ -230,19 +225,6 @@ char  **GIFAbstractDataset::GetMetadata( const char * pszDomain )
     return GDALPamDataset::GetMetadata(pszDomain);
 }
 
-/************************************************************************/
-/*                        GetProjectionRef()                            */
-/************************************************************************/
-
-const char *GIFAbstractDataset::_GetProjectionRef()
-
-{
-    if ( pszProjection && bGeoTransformValid )
-        return pszProjection;
-
-    return GDALPamDataset::_GetProjectionRef();
-}
-
 /************************************************************************/
 /*                          GetGeoTransform()                           */
 /************************************************************************/
@@ -272,19 +254,6 @@ int GIFAbstractDataset::GetGCPCount()
     return GDALPamDataset::GetGCPCount();
 }
 
-/************************************************************************/
-/*                          GetGCPProjection()                          */
-/************************************************************************/
-
-const char *GIFAbstractDataset::_GetGCPProjection()
-
-{
-    if ( pszProjection && nGCPCount > 0 )
-        return pszProjection;
-
-    return GDALPamDataset::_GetGCPProjection();
-}
-
 /************************************************************************/
 /*                               GetGCPs()                              */
 /************************************************************************/
