@@ -38,7 +38,6 @@
 #include "gdal_frmts.h"
 #include "gdal_pam.h"
 
-CPL_CVSID("$Id: gsbgdataset.cpp 4be4f4da7ab3eaf3e1e1dd662b9f6814e8db5747 2021-10-16 16:47:05 +0200 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -823,7 +822,7 @@ CPLErr GSBGDataset::WriteHeader( VSILFILE *fp, GInt16 nXSize, GInt16 nYSize,
 GDALDataset *GSBGDataset::Create( const char * pszFilename,
                                   int nXSize,
                                   int nYSize,
-                                  CPL_UNUSED int nBands,
+                                  int /* nBands */,
                                   GDALDataType eType,
                                   CPL_UNUSED char **papszParamList )
 {
