@@ -73,15 +73,7 @@ static bool HasZSTDCompression()
 static CPLString GetTmpFilename(const char* pszFilename,
                                 const char* pszExt)
 {
-    const bool bSupportsRandomWrite =
-        !(STARTS_WITH(pszFilename, "/vsitar/") ||
-          STARTS_WITH(pszFilename, "/vsizip/") ||
-          STARTS_WITH(pszFilename, "/vsigzip/") ||
-          STARTS_WITH(pszFilename, "/vsis3") ||
-          STARTS_WITH(pszFilename, "/vsigs") ||
-          STARTS_WITH(pszFilename, "/vsiaz") ||
-          STARTS_WITH(pszFilename, "/vsiadls") ||
-          STARTS_WITH(pszFilename, "/vsioss"));
+    const bool bSupportsRandomWrite = VSISupportsRandomWrite(pszFilename, false);
     CPLString osTmpFilename;
     if( !bSupportsRandomWrite ||
         CPLGetConfigOption( "CPL_TMPDIR", nullptr ) != nullptr )
@@ -635,12 +627,12 @@ static std::unique_ptr<GDALDataset> CreateReprojectedDS(
     const int nBands = poSrcDS->GetRasterCount();
     const char* pszOverviews = CSLFetchNameValueDef(
         papszOptions, "OVERVIEWS", "AUTO");
-    const bool bRecreateOvr = EQUAL(pszOverviews, "FORCE_USE_EXISTING") ||
+    const bool bUseExistingOrNone = EQUAL(pszOverviews, "FORCE_USE_EXISTING") ||
                               EQUAL(pszOverviews, "NONE");
     dfTotalPixelsToProcess =
         double(nXSize) * nYSize * (nBands + (bHasMask ? 1 : 0)) +
-        ((bHasMask && !bRecreateOvr) ? double(nXSize) * nYSize / 3 : 0) +
-        (!bRecreateOvr ? double(nXSize) * nYSize * nBands / 3: 0) +
+        ((bHasMask && !bUseExistingOrNone) ? double(nXSize) * nYSize / 3 : 0) +
+        (!bUseExistingOrNone ? double(nXSize) * nYSize * nBands / 3: 0) +
         double(nXSize) * nYSize * (nBands + (bHasMask ? 1 : 0)) * 4. / 3;
 
     auto psOptions = GDALWarpAppOptionsNew(papszArg, nullptr);
@@ -885,17 +877,21 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
 
     CPLString osOverviews = CSLFetchNameValueDef(
         papszOptions, "OVERVIEWS", "AUTO");
-    const bool bRecreateOvr = EQUAL(osOverviews, "FORCE_USE_EXISTING") ||
+    const bool bUseExistingOrNone = EQUAL(osOverviews, "FORCE_USE_EXISTING") ||
                               EQUAL(osOverviews, "NONE");
+
+    const int nOverviewCount = atoi(
+        CSLFetchNameValueDef(papszOptions, "OVERVIEW_COUNT", "-1"));
+
     const bool bGenerateMskOvr =
-        !bRecreateOvr &&
+        !bUseExistingOrNone &&
         bHasMask &&
-        (nXSize > nOvrThresholdSize || nYSize > nOvrThresholdSize) &&
+        (nXSize > nOvrThresholdSize || nYSize > nOvrThresholdSize || nOverviewCount > 0) &&
         (EQUAL(osOverviews, "IGNORE_EXISTING") ||
          poFirstBand->GetMaskBand()->GetOverviewCount() == 0);
     const bool bGenerateOvr =
-        !bRecreateOvr &&
-        (nXSize > nOvrThresholdSize || nYSize > nOvrThresholdSize) &&
+        !bUseExistingOrNone &&
+        (nXSize > nOvrThresholdSize || nYSize > nOvrThresholdSize || nOverviewCount > 0) &&
         (EQUAL(osOverviews, "IGNORE_EXISTING") ||
          poFirstBand->GetOverviewCount() == 0);
 
@@ -906,12 +902,26 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
     {
         const auto& tmList = poTM->tileMatrixList();
         int nCurLevel = nZoomLevel;
-        while( nTmpXSize > nOvrThresholdSize || nTmpYSize > nOvrThresholdSize )
+        while( true )
         {
+            if( nOverviewCount < 0 )
+            {
+                if( nTmpXSize <= nOvrThresholdSize && nTmpYSize <= nOvrThresholdSize )
+                    break;
+            }
+            else if( static_cast<int>(asOverviewDims.size()) == nOverviewCount ||
+                     (nTmpXSize == 1 && nTmpYSize == 1) )
+            {
+                break;
+            }
             const double dfResRatio = (nCurLevel >= 1) ?
                 tmList[nCurLevel-1].mResX / tmList[nCurLevel].mResX : 2;
             nTmpXSize = static_cast<int>(nTmpXSize / dfResRatio + 0.5);
             nTmpYSize = static_cast<int>(nTmpYSize / dfResRatio + 0.5);
+            if( nTmpXSize == 0 )
+                nTmpXSize = 1;
+            if( nTmpYSize == 0 )
+                nTmpYSize = 1;
             asOverviewDims.push_back(std::pair<int,int>(nTmpXSize, nTmpYSize));
             nCurLevel --;
         }
@@ -922,7 +932,10 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         {
             // If generating only .msk.ovr, use the exact overview size as
             // the overviews of the imagery.
-            for(int i = 0; i < poFirstBand->GetOverviewCount(); i++ )
+            int nIters = poFirstBand->GetOverviewCount();
+            if( nOverviewCount >= 0 && nOverviewCount < nIters )
+                nIters = nOverviewCount;
+            for(int i = 0; i < nIters; i++ )
             {
                 auto poOvrBand = poFirstBand->GetOverview(i);
                 asOverviewDims.push_back(std::pair<int,int>(
@@ -931,11 +944,24 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         }
         else
         {
-            while( nTmpXSize > nOvrThresholdSize ||
-                   nTmpYSize > nOvrThresholdSize )
+            while( true )
             {
+                if( nOverviewCount < 0 )
+                {
+                    if( nTmpXSize <= nOvrThresholdSize && nTmpYSize <= nOvrThresholdSize )
+                        break;
+                }
+                else if( static_cast<int>(asOverviewDims.size()) == nOverviewCount ||
+                         (nTmpXSize == 1 && nTmpYSize == 1) )
+                {
+                    break;
+                }
                 nTmpXSize /= 2;
                 nTmpYSize /= 2;
+                if( nTmpXSize == 0 )
+                    nTmpXSize = 1;
+                if( nTmpYSize == 0 )
+                    nTmpYSize = 1;
                 asOverviewDims.push_back(std::pair<int,int>(nTmpXSize, nTmpYSize));
             }
         }
@@ -1061,7 +1087,8 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
     {
         if( pszQuality && atoi(pszQuality) == 100 )
             aosOptions.SetNameValue("WEBP_LOSSLESS", "YES");
-        aosOptions.SetNameValue("WEBP_LEVEL", pszQuality);
+        else
+            aosOptions.SetNameValue("WEBP_LEVEL", pszQuality);
     }
     else if( EQUAL(osCompress, "DEFLATE") || EQUAL(osCompress, "LERC_DEFLATE") )
     {
@@ -1118,6 +1145,9 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         {
             aosOptions.SetNameValue("@MASK_OVERVIEW_DATASET", m_osTmpMskOverviewFilename);
         }
+        aosOptions.SetNameValue("@OVERVIEW_COUNT",
+                                CSLFetchNameValue(papszOptions, "OVERVIEW_COUNT"));
+
     }
 
     const CPLString osTilingScheme(CSLFetchNameValueDef(papszOptions,
@@ -1137,8 +1167,26 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         papszOptions, "OVERVIEW_COMPRESS", osCompress.c_str());
 
     CPLConfigOptionSetter ovrCompressSetter("COMPRESS_OVERVIEW", pszOverviewCompress, true);
-    CPLConfigOptionSetter ovrQualityJpegSetter("JPEG_QUALITY_OVERVIEW", CSLFetchNameValue(papszOptions, "OVERVIEW_QUALITY"), true);
-    CPLConfigOptionSetter ovrQualityWebpSetter("WEBP_LEVEL_OVERVIEW", CSLFetchNameValue(papszOptions, "OVERVIEW_QUALITY"), true);
+    const char* pszOverviewQuality = CSLFetchNameValue(papszOptions, "OVERVIEW_QUALITY");
+    CPLConfigOptionSetter ovrQualityJpegSetter("JPEG_QUALITY_OVERVIEW", pszOverviewQuality, true);
+
+    std::unique_ptr<CPLConfigOptionSetter> poWebpLosslessSetter;
+    std::unique_ptr<CPLConfigOptionSetter> poWebpLevelSetter;
+    if( EQUAL(pszOverviewCompress, "WEBP") )
+    {
+        if( pszOverviewQuality && CPLAtof(pszOverviewQuality) == 100.0 )
+        {
+            poWebpLosslessSetter.reset(new CPLConfigOptionSetter(
+                "WEBP_LOSSLESS_OVERVIEW", "TRUE", true));
+        }
+        else
+        {
+            poWebpLosslessSetter.reset(new CPLConfigOptionSetter(
+                "WEBP_LOSSLESS_OVERVIEW", "FALSE", true));
+            poWebpLevelSetter.reset(new CPLConfigOptionSetter(
+                "WEBP_LEVEL_OVERVIEW", pszOverviewQuality, true));
+        }
+    }
 
     std::unique_ptr<CPLConfigOptionSetter> poPhotometricSetter;
     if (nBands == 3 && EQUAL(pszOverviewCompress, "JPEG") )
@@ -1278,10 +1326,19 @@ void GDALCOGDriver::InitializeCreationOptionList()
     }
     if( bHasJPEG || bHasWebP )
     {
+        std::string osJPEG_WEBP;
+        if( bHasJPEG )
+            osJPEG_WEBP = "JPEG";
+        if( bHasWebP )
+        {
+            if( !osJPEG_WEBP.empty() )
+                osJPEG_WEBP += '/';
+            osJPEG_WEBP += "WEBP";
+        }
         osOptions += "   <Option name='QUALITY' type='int' "
-                     "description='JPEG/WEBP quality 1-100' default='75'/>"
+                     "description='" + osJPEG_WEBP + " quality 1-100' default='75'/>"
                      "   <Option name='OVERVIEW_QUALITY' type='int' "
-                     "description='Overview JPEG/WEBP quality 1-100' default='75'/>";
+                     "description='Overview " + osJPEG_WEBP + " quality 1-100' default='75'/>";
     }
     if( bHasLERC )
     {
@@ -1320,6 +1377,7 @@ void GDALCOGDriver::InitializeCreationOptionList()
 "     <Value>FORCE_USE_EXISTING</Value>"
 "     <Value>NONE</Value>"
 "   </Option>"
+"  <Option name='OVERVIEW_COUNT' type='int' min='0' description='Number of overviews'/>"
 "  <Option name='TILING_SCHEME' type='string' description='"
         "Which tiling scheme to use pre-defined value or custom inline/outline "
         "JSON definition' default='CUSTOM'>"
@@ -1386,6 +1444,7 @@ void GDALRegister_COG()
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Cloud optimized GeoTIFF generator" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/cog.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "tif tiff" );
 
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
                                "Byte UInt16 Int16 UInt32 Int32 UInt64 Int64 Float32 "
