@@ -29,6 +29,7 @@
  ****************************************************************************/
 
 #include "cpl_port.h"
+
 #include "gt_overview.h"
 
 #include <cstdlib>
@@ -50,7 +51,6 @@
 #include "tifvsi.h"
 #include "xtiffio.h"
 
-CPL_CVSID("$Id: gt_overview.cpp 4d27c2f633a68d4449b15cddb1ddb62aeaf10c98 2022-02-17 00:01:10 +0100 Even Rouault $")
 
 // TODO(schwehr): Explain why 128 and not 127.
 constexpr int knMaxOverviews = 128;
@@ -298,23 +298,24 @@ static int GTIFFGetMaxColorChannels( int photometric )
 
 CPLErr
 GTIFFBuildOverviews( const char * pszFilename,
-                     int nBands, GDALRasterBand **papoBandList,
-                     int nOverviews, int * panOverviewList,
+                     int nBands, GDALRasterBand* const* papoBandList,
+                     int nOverviews, const int * panOverviewList,
                      const char * pszResampling,
-                     GDALProgressFunc pfnProgress, void * pProgressData )
+                     GDALProgressFunc pfnProgress, void * pProgressData,
+                     CSLConstList papszOptions )
 
 {
     return GTIFFBuildOverviewsEx(pszFilename, nBands, papoBandList,
                                  nOverviews, panOverviewList,
                                  nullptr,
                                  pszResampling,
-                                 nullptr,
+                                 papszOptions,
                                  pfnProgress, pProgressData);
 }
 
 CPLErr
 GTIFFBuildOverviewsEx( const char * pszFilename,
-                       int nBands, GDALRasterBand **papoBandList,
+                       int nBands, GDALRasterBand * const* papoBandList,
                        int nOverviews,
                        const int * panOverviewList,
                        const std::pair<int, int>* pasOverviewSize,
@@ -333,9 +334,9 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
     TIFF *hOTIFF = nullptr;
     int nBitsPerPixel = 0;
     int nCompression = COMPRESSION_NONE;
-    int nPhotometric = 0;
+    uint16_t nPhotometric = 0;
     int nSampleFormat = 0;
-    int nPlanarConfig = 0;
+    uint16_t nPlanarConfig = 0;
     int iOverview = 0;
     int nXSize = 0;
     int nYSize = 0;
@@ -377,6 +378,16 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
             nBandFormat = SAMPLEFORMAT_INT;
             break;
 
+          case GDT_UInt64:
+            nBandBits = 64;
+            nBandFormat = SAMPLEFORMAT_UINT;
+            break;
+
+          case GDT_Int64:
+            nBandBits = 64;
+            nBandFormat = SAMPLEFORMAT_INT;
+            break;
+
           case GDT_Float32:
             nBandBits = 32;
             nBandFormat = SAMPLEFORMAT_IEEEFP;
@@ -453,18 +464,33 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         }
     }
 
+    const auto GetOptionValue = [papszOptions](const char* pszOptionKey,
+                                               const char* pszConfigOptionKey,
+                                               const char** ppszKeyUsed = nullptr)
+    {
+        const char* pszVal = CSLFetchNameValue(papszOptions, pszOptionKey);
+        if( pszVal )
+        {
+            if( ppszKeyUsed )
+                *ppszKeyUsed = pszOptionKey;
+            return pszVal;
+        }
+        pszVal = CPLGetConfigOption(pszConfigOptionKey, nullptr);
+        if( pszVal && ppszKeyUsed )
+            *ppszKeyUsed = pszConfigOptionKey;
+        return pszVal;
+    };
+
 /* -------------------------------------------------------------------- */
 /*      Use specified compression method.                               */
 /* -------------------------------------------------------------------- */
-    const char *pszCompress = papszOptions ?
-        CSLFetchNameValue(papszOptions, "COMPRESS") :
-        CPLGetConfigOption( "COMPRESS_OVERVIEW", nullptr );
+    const char *pszCompressKey = "";
+    const char *pszCompress = GetOptionValue("COMPRESS", "COMPRESS_OVERVIEW", &pszCompressKey);
 
     if( pszCompress != nullptr && pszCompress[0] != '\0' )
     {
         nCompression =
-            GTIFFGetCompressionMethod(pszCompress,
-                    papszOptions ? "COMPRESS" : "COMPRESS_OVERVIEW");
+            GTIFFGetCompressionMethod(pszCompress, pszCompressKey);
         if( nCompression < 0 )
             return CE_Failure;
     }
@@ -521,9 +547,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         }
     }
 
-    const char* pszInterleave = papszOptions ?
-        CSLFetchNameValue(papszOptions, "INTERLEAVE") :
-        CPLGetConfigOption( "INTERLEAVE_OVERVIEW", nullptr );
+    const char* pszInterleave = GetOptionValue("INTERLEAVE", "INTERLEAVE_OVERVIEW");
     if( pszInterleave != nullptr && pszInterleave[0] != '\0' )
     {
         if( EQUAL( pszInterleave, "PIXEL" ) )
@@ -563,86 +587,14 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
     else
         nPhotometric = PHOTOMETRIC_MINISBLACK;
 
-    const char* pszPhotometric = papszOptions ?
-        CSLFetchNameValue(papszOptions, "PHOTOMETRIC") :
-        CPLGetConfigOption( "PHOTOMETRIC_OVERVIEW", nullptr );
-    if( pszPhotometric != nullptr && pszPhotometric[0] != '\0' )
+    const char* pszOptionKey = "";
+    const char* pszPhotometric = GetOptionValue("PHOTOMETRIC", "PHOTOMETRIC_OVERVIEW", &pszOptionKey);
+    if( !GTIFFUpdatePhotometric(pszPhotometric, pszOptionKey,
+                                nCompression, pszInterleave,
+                                nBands,
+                                nPhotometric, nPlanarConfig) )
     {
-        if( EQUAL( pszPhotometric, "MINISBLACK" ) )
-            nPhotometric = PHOTOMETRIC_MINISBLACK;
-        else if( EQUAL( pszPhotometric, "MINISWHITE" ) )
-            nPhotometric = PHOTOMETRIC_MINISWHITE;
-        else if( EQUAL( pszPhotometric, "RGB" ))
-        {
-            nPhotometric = PHOTOMETRIC_RGB;
-        }
-        else if( EQUAL( pszPhotometric, "CMYK" ))
-        {
-            nPhotometric = PHOTOMETRIC_SEPARATED;
-        }
-        else if( EQUAL( pszPhotometric, "YCBCR" ))
-        {
-            nPhotometric = PHOTOMETRIC_YCBCR;
-
-            // Because of subsampling, setting YCBCR without JPEG compression
-            // leads to a crash currently. Would need to make
-            // GTiffRasterBand::IWriteBlock() aware of subsampling so that it
-            // doesn't overrun buffer size returned by libtiff.
-            if( nCompression != COMPRESSION_JPEG )
-            {
-                CPLError(
-                    CE_Failure, CPLE_NotSupported,
-                    "Currently, PHOTOMETRIC_OVERVIEW=YCBCR requires "
-                    "COMPRESS_OVERVIEW=JPEG" );
-                return CE_Failure;
-            }
-
-            if( pszInterleave != nullptr &&
-                pszInterleave[0] != '\0' &&
-                nPlanarConfig == PLANARCONFIG_SEPARATE )
-            {
-                CPLError(
-                    CE_Failure, CPLE_NotSupported,
-                    "PHOTOMETRIC_OVERVIEW=YCBCR requires "
-                    "INTERLEAVE_OVERVIEW=PIXEL" );
-                return CE_Failure;
-            }
-            else
-            {
-                nPlanarConfig = PLANARCONFIG_CONTIG;
-            }
-
-            // YCBCR strictly requires 3 bands. Not less, not more
-            // Issue an explicit error message as libtiff one is a bit cryptic:
-            // JPEGLib:Bogus input colorspace.
-            if( nBands != 3 )
-            {
-                CPLError(
-                    CE_Failure, CPLE_NotSupported,
-                    "PHOTOMETRIC_OVERVIEW=YCBCR requires a source raster "
-                    "with only 3 bands (RGB)" );
-                return CE_Failure;
-            }
-        }
-        else if( EQUAL( pszPhotometric, "CIELAB" ))
-        {
-            nPhotometric = PHOTOMETRIC_CIELAB;
-        }
-        else if( EQUAL( pszPhotometric, "ICCLAB" ))
-        {
-            nPhotometric = PHOTOMETRIC_ICCLAB;
-        }
-        else if( EQUAL( pszPhotometric, "ITULAB" ))
-        {
-            nPhotometric = PHOTOMETRIC_ITULAB;
-        }
-        else
-        {
-            CPLError(
-                CE_Warning, CPLE_IllegalArg,
-                "PHOTOMETRIC_OVERVIEW=%s value not recognised, ignoring.",
-                pszPhotometric );
-        }
+        return CE_Failure;
     }
 
 /* -------------------------------------------------------------------- */
@@ -651,9 +603,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
     int nPredictor = PREDICTOR_NONE;
     if( GTIFFSupportsPredictor(nCompression) )
     {
-        const char* pszPredictor = papszOptions ?
-            CSLFetchNameValue(papszOptions, "PREDICTOR") :
-            CPLGetConfigOption( "PREDICTOR_OVERVIEW", nullptr );
+        const char* pszPredictor = GetOptionValue("PREDICTOR", "PREDICTOR_OVERVIEW");
         if( pszPredictor != nullptr )
         {
             nPredictor = atoi( pszPredictor );
@@ -696,9 +646,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
     /* -------------------------------------------------------------------- */
     /*      Should the file be created as a bigtiff file?                   */
     /* -------------------------------------------------------------------- */
-        const char *pszBIGTIFF = papszOptions ?
-            CSLFetchNameValue(papszOptions, "BIGTIFF") :
-            CPLGetConfigOption( "BIGTIFF_OVERVIEW", nullptr );
+        const char *pszBIGTIFF = GetOptionValue("BIGTIFF", "BIGTIFF_OVERVIEW");
 
         if( pszBIGTIFF == nullptr )
             pszBIGTIFF = "IF_SAFER";
@@ -889,9 +837,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         {
             anExtraSamples.push_back(
                 GTiffGetAlphaValue(
-                    papszOptions ?
-                        CSLFetchNameValue(papszOptions, "ALPHA") :
-                        CPLGetConfigOption("GTIFF_ALPHA", nullptr),
+                    GetOptionValue("ALPHA", "GTIFF_ALPHA"),
                     DEFAULT_ALPHA_TYPE));
         }
         else
@@ -949,12 +895,8 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
                              static_cast<int>(anExtraSamples.size()),
                              anExtraSamples.empty() ? nullptr : anExtraSamples.data(),
                              osMetadata,
-                             papszOptions ?
-                                CSLFetchNameValue(papszOptions, "JPEG_QUALITY") :
-                                CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr ),
-                             papszOptions ?
-                                CSLFetchNameValue(papszOptions, "JPEG_TABLESMODE") :
-                                CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr ),
+                             GetOptionValue("JPEG_QUALITY", "JPEG_QUALITY_OVERVIEW"),
+                             GetOptionValue("JPEG_TABLESMODE", "JPEG_TABLESMODE_OVERVIEW"),
                              pszNoData,
                              panLercAddCompressionAndVersion,
                              false
@@ -989,8 +931,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
     aosOpenOptions.SetNameValue("NUM_THREADS",
                                 CSLFetchNameValue(papszOptions, "NUM_THREADS"));
     aosOpenOptions.SetNameValue("SPARSE_OK",
-                                CSLFetchNameValueDef(papszOptions, "SPARSE_OK",
-                                     CPLGetConfigOption("SPARSE_OK_OVERVIEW", nullptr)));
+                                GetOptionValue("SPARSE_OK", "SPARSE_OK_OVERVIEW"));
     aosOpenOptions.SetNameValue("@MASK_OVERVIEW_DATASET",
                                 CSLFetchNameValue(papszOptions, "MASK_OVERVIEW_DATASET"));
     GDALDataset *hODS = GDALDataset::Open( pszFilename,
@@ -1005,10 +946,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     TIFF *hTIFF = static_cast<TIFF *>( hODS->GetInternalHandle(nullptr) );
 
-    const char* pszJPEGQuality =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "JPEG_QUALITY") :
-            CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr );
+    const char* pszJPEGQuality = GetOptionValue("JPEG_QUALITY", "JPEG_QUALITY_OVERVIEW");
     if( nCompression == COMPRESSION_JPEG && pszJPEGQuality != nullptr )
     {
         const int nJpegQuality = atoi(pszJPEGQuality);
@@ -1016,10 +954,8 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
                       nJpegQuality );
         GTIFFSetJpegQuality(GDALDataset::ToHandle(hODS), nJpegQuality);
     }
-    const char* pszWebpLevel =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "WEBP_LEVEL") :
-            CPLGetConfigOption( "WEBP_LEVEL_OVERVIEW", nullptr );
+
+    const char* pszWebpLevel = GetOptionValue("WEBP_LEVEL", "WEBP_LEVEL_OVERVIEW");
     if( nCompression == COMPRESSION_WEBP && pszWebpLevel != nullptr )
     {
         const int nWebpLevel = atoi(pszWebpLevel);
@@ -1030,10 +966,15 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         }
     }
 
-    const char* pszJPEGTablesMode =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "JPEG_TABLESMODE") :
-            CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr );
+    const char* pszWebpLossless = GetOptionValue("WEBP_LOSSLESS", "WEBP_LOSSLESS_OVERVIEW");
+    if( nCompression == COMPRESSION_WEBP && pszWebpLossless != nullptr )
+    {
+        const bool bWebpLossless = CPLTestBool(pszWebpLossless);
+        TIFFSetField( hTIFF, TIFFTAG_WEBP_LOSSLESS, static_cast<int>(bWebpLossless) );
+        GTIFFSetWebPLossless(GDALDataset::ToHandle(hODS), bWebpLossless);
+    }
+
+    const char* pszJPEGTablesMode = GetOptionValue("JPEG_TABLESMODE", "JPEG_TABLESMODE_OVERVIEW");
     if( nCompression == COMPRESSION_JPEG && pszJPEGTablesMode != nullptr )
     {
         const int nJpegTablesMode = atoi(pszJPEGTablesMode);
@@ -1042,10 +983,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         GTIFFSetJpegTablesMode(GDALDataset::ToHandle(hODS), nJpegTablesMode);
     }
 
-    const char* pszZLevel =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "ZLEVEL") :
-            CPLGetConfigOption( "ZLEVEL_OVERVIEW", nullptr );
+    const char* pszZLevel = GetOptionValue("ZLEVEL", "ZLEVEL_OVERVIEW");
     if( (nCompression == COMPRESSION_DEFLATE ||
          anLercAddCompressionAndVersion[1] == LERC_ADD_COMPRESSION_DEFLATE) &&
          pszZLevel != nullptr )
@@ -1058,10 +996,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         }
     }
 
-    const char* pszZSTDLevel =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "ZSTD_LEVEL") :
-            CPLGetConfigOption( "ZSTD_LEVEL_OVERVIEW", nullptr );
+    const char* pszZSTDLevel = GetOptionValue("ZSTD_LEVEL", "ZSTD_LEVEL_OVERVIEW");
     if( (nCompression == COMPRESSION_ZSTD ||
          anLercAddCompressionAndVersion[1] == LERC_ADD_COMPRESSION_ZSTD) &&
          pszZSTDLevel != nullptr )
@@ -1074,10 +1009,7 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
         }
     }
 
-    const char* pszMaxZError =
-        papszOptions ?
-            CSLFetchNameValue(papszOptions, "MAX_Z_ERROR") :
-            CPLGetConfigOption( "MAX_Z_ERROR_OVERVIEW", nullptr );
+    const char* pszMaxZError = GetOptionValue("MAX_Z_ERROR", "MAX_Z_ERROR_OVERVIEW");
     if( nCompression == COMPRESSION_LERC &&
          pszMaxZError != nullptr )
     {
@@ -1130,7 +1062,8 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
           EQUAL(pszResampling, "CUBIC") ||
           EQUAL(pszResampling, "CUBICSPLINE") ||
           EQUAL(pszResampling, "LANCZOS") ||
-          EQUAL(pszResampling, "BILINEAR")) )
+          EQUAL(pszResampling, "BILINEAR") ||
+          EQUAL(pszResampling, "MODE")) )
     {
         // In the case of pixel interleaved compressed overviews, we want to
         // generate the overviews for all the bands block by block, and not
@@ -1226,7 +1159,8 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
                     GDALRegenerateOverviewsMultiBand(
                         nBands, papoBandList,
                         nOverviews, papapoOverviewBands,
-                        pszResampling, pfnProgress, pProgressData );
+                        pszResampling, pfnProgress, pProgressData,
+                        papszOptions);
         }
 
         for( int iBand = 0; iBand < nBands; iBand++ )
@@ -1289,13 +1223,14 @@ GTIFFBuildOverviewsEx( const char * pszFilename,
 
                 if( eErr == CE_None )
                     eErr =
-                        GDALRegenerateOverviews(
+                        GDALRegenerateOverviewsEx(
                             hSrcBand,
                             nDstOverviews,
                             reinterpret_cast<GDALRasterBandH *>( papoOverviews ),
                             pszResampling,
                             GDALScaledProgress,
-                            pScaledProgressData );
+                            pScaledProgressData,
+                            papszOptions);
             }
 
             GDALDestroyScaledProgress( pScaledProgressData );
