@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gt_overview.h bb21d97086093dee5fc9e92d6d3c82f6a51ab108 2021-11-19 18:57:03 +0100 Even Rouault $
+ * $Id$
  *
  * Project:  GeoTIFF Driver
  * Purpose:  Code to build overviews of external databases as a TIFF file.
@@ -64,7 +64,7 @@ void GTIFFBuildOverviewMetadata( const char *pszResampling,
 
 CPLErr
 GTIFFBuildOverviewsEx( const char * pszFilename,
-                       int nBands, GDALRasterBand **papoBandList,
+                       int nBands, GDALRasterBand * const* papoBandList,
                        int nOverviews,
                        const int * panOverviewList,
                        const std::pair<int, int>* pasOverviewSize,
