@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gxf_ogcwkt.c fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $
+ * $Id$
  *
  * Project:  GXF Reader
  * Purpose:  Handle GXF to OGC WKT projection transformation.
@@ -31,7 +31,6 @@
 #include "gxfopen.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: gxf_ogcwkt.c fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 /* -------------------------------------------------------------------- */
 /* the following #defines come from ogr_spatialref.h in the GDAL/OGR	*/
@@ -317,10 +316,6 @@ char *GXFGetMapProjectionAsOGCWKT( GXFHandle hGXF )
                                                 ",", TRUE, TRUE );
     }
 
-#ifdef DBMALLOC
-    malloc_chain_check(1);
-#endif
-
 /* -------------------------------------------------------------------- */
 /*      Create the PROJCS.                                              */
 /* -------------------------------------------------------------------- */
