@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gxf_proj4.c 3b0bbf7a8a012d69a783ee1f9cfeb5c52b370021 2017-06-27 20:57:02Z Even Rouault $
+ * $Id$
  *
  * Project:  GXF Reader
  * Purpose:  Handle GXF to PROJ.4 projection transformation.
@@ -30,7 +30,6 @@
 
 #include "gxfopen.h"
 
-CPL_CVSID("$Id: gxf_proj4.c 3b0bbf7a8a012d69a783ee1f9cfeb5c52b370021 2017-06-27 20:57:02Z Even Rouault $")
 
 
 #define SAFE_strcat(x,y) snprintf(x + strlen(x),sizeof(x) - strlen(x), "%s", y)
@@ -94,10 +93,6 @@ char *GXFGetMapProjectionAsPROJ4( GXFHandle hGXF )
                                                 ",", TRUE, TRUE );
     }
 
-#ifdef DBMALLOC
-    malloc_chain_check(1);
-#endif
-
     if( papszMethods == NULL
         || papszMethods[0] == NULL
         || EQUAL(papszMethods[0],"Geographic") )
