@@ -42,7 +42,6 @@
 #include "hdf4compat.h"
 #include "hdf4dataset.h"
 
-CPL_CVSID("$Id: hdf4dataset.cpp 7efbb574f409465c908e5d83877fa36a501d434d 2020-10-14 11:56:55 +0200 Even Rouault $")
 
 extern const char * const pszGDALSignature;
 
@@ -205,9 +204,9 @@ GDALDataType HDF4Dataset::GetDataType( int32 iNumType )
         case DFNT_UINT32:
             return GDT_UInt32;
         case DFNT_INT64:
-            return GDT_Unknown;
+            return GDT_Int64;
         case DFNT_UINT64:
-            return GDT_Unknown;
+            return GDT_UInt64;
         case DFNT_FLOAT32:
             return GDT_Float32;
         case DFNT_FLOAT64:
@@ -1154,7 +1153,7 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
                             &nAttrs) != 0 )
                 return nullptr;
 
-            if ( iRank == 1 )  // Skip 1D datsets
+            if ( iRank == 1 )  // Skip 1D datasets
                     continue;
 
             // Do sort of known datasets. We will display only image bands
