@@ -38,7 +38,6 @@
 
 #include "gdal_frmts.h"
 
-CPL_CVSID("$Id: ilwisdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 namespace GDAL
 {
@@ -412,7 +411,7 @@ static std::string GDALType2ILWIS(GDALDataType type)
     return sStoreType;
 }
 
-static CPLErr GetStoreType(std::string pszFileName, ilwisStoreType &stStoreType)
+static CPLErr GetStoreType(const std::string& pszFileName, ilwisStoreType &stStoreType)
 {
     std::string st = ReadElement("MapStore", "Type", pszFileName.c_str());
 
@@ -446,10 +445,10 @@ static CPLErr GetStoreType(std::string pszFileName, ilwisStoreType &stStoreType)
 }
 
 ILWISDataset::ILWISDataset() :
-    pszProjection(CPLStrdup("")),
     bGeoDirty(FALSE),
     bNewDataset(FALSE)
 {
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
@@ -466,7 +465,6 @@ ILWISDataset::~ILWISDataset()
 
 {
     ILWISDataset::FlushCache(true);
-    CPLFree( pszProjection );
 }
 
 /************************************************************************/
@@ -600,24 +598,25 @@ CPLErr ILWISDataset::WriteGeoReference()
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
 
-const char *ILWISDataset::_GetProjectionRef()
+const OGRSpatialReference *ILWISDataset::GetSpatialRef() const
 
 {
-   return pszProjection;
+   return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
 }
 
 /************************************************************************/
-/*                           SetProjection()                            */
+/*                           SetSpatialRef()                            */
 /************************************************************************/
 
-CPLErr ILWISDataset::_SetProjection( const char * pszNewProjection )
+CPLErr ILWISDataset::SetSpatialRef( const OGRSpatialReference* poSRS )
 
 {
-    CPLFree( pszProjection );
-    pszProjection = CPLStrdup( pszNewProjection );
+    m_oSRS.Clear();
+    if( poSRS )
+        m_oSRS = *poSRS;
     bGeoDirty = TRUE;
 
     return CE_None;
@@ -864,7 +863,7 @@ void ILWISDataset::FlushCache(bool bAtClosing)
 
 GDALDataset *ILWISDataset::Create(const char* pszFilename,
                                   int nXSize, int nYSize,
-                                  int nBands, GDALDataType eType,
+                                  int nBandsIn, GDALDataType eType,
                                   CPL_UNUSED char** papszParamList)
 {
 /* -------------------------------------------------------------------- */
@@ -908,7 +907,7 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
 
     //Form map/maplist name.
     std::unique_ptr<IniFile> globalFile;
-    if ( nBands == 1 )
+    if ( nBandsIn == 1 )
     {
         pszODFName = std::string(CPLFormFilename(pszPath.c_str(),pszBaseName.c_str(),"mpr"));
         pszDataBaseName = pszBaseName;
@@ -921,13 +920,13 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
         iniFile->SetKeyValue("Ilwis", "Type", "MapList");
         iniFile->SetKeyValue("MapList", "GeoRef", "none.grf");
         iniFile->SetKeyValue("MapList", "Size", std::string(strsize));
-        iniFile->SetKeyValue("MapList", "Maps", CPLSPrintf("%d", nBands));
+        iniFile->SetKeyValue("MapList", "Maps", CPLSPrintf("%d", nBandsIn));
         globalFile.reset(iniFile);
     }
 
-    for( int iBand = 0; iBand < nBands; iBand++ )
+    for( int iBand = 0; iBand < nBandsIn; iBand++ )
     {
-        if ( nBands > 1 )
+        if ( nBandsIn > 1 )
         {
             char szBandName[100];
             snprintf(szBandName, sizeof(szBandName), "%s_band_%d", pszBaseName.c_str(),iBand + 1 );
@@ -987,13 +986,13 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
     ILWISDataset *poDS = new ILWISDataset();
     poDS->nRasterXSize = nXSize;
     poDS->nRasterYSize = nYSize;
-    poDS->nBands = nBands;
+    poDS->nBands = nBandsIn;
     poDS->eAccess = GA_Update;
     poDS->bNewDataset = TRUE;
     poDS->SetDescription(pszFilename);
     poDS->osFileName = pszFileName;
     poDS->pszIlwFileName = std::string(pszFileName);
-    if ( nBands == 1 )
+    if ( nBandsIn == 1 )
         poDS->pszFileType = "Map";
     else
         poDS->pszFileType = "MapList";
@@ -1071,9 +1070,9 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
             georef = pszBaseName + ".grf";
     }
 
-    const char *pszProj = poSrcDS->GetProjectionRef();
-    if( pszProj != nullptr && strlen(pszProj) > 0 )
-        poDS->SetProjection( pszProj );
+    const OGRSpatialReference *poSrcSRS = poSrcDS->GetSpatialRef();
+    if( poSrcSRS )
+        poDS->SetSpatialRef( poSrcSRS );
 
 /* -------------------------------------------------------------------- */
 /*      Create the output raster files for each band                    */
