@@ -135,7 +135,7 @@ class ILWISDataset final: public GDALPamDataset
     friend class ILWISRasterBand;
     CPLString osFileName;
     std::string pszIlwFileName;
-    char         *pszProjection;
+    OGRSpatialReference m_oSRS{};
     double adfGeoTransform[6];
     int    bGeoDirty;
     int    bNewDataset;            /* product of Create() */
@@ -160,15 +160,15 @@ public:
     static GDALDataset *Create(const char* pszFilename,
                                int nXSize, int nYSize,
                                int nBands, GDALDataType eType,
-                               char** papszParmList);
+                               char** papszParamList);
 
     virtual CPLErr  GetGeoTransform( double * padfTransform ) override;
     virtual CPLErr  SetGeoTransform( double * ) override;
 
-    virtual const char *GetProjectionRef() override;
-    virtual CPLErr SetProjection( const char * ) override;
+    const OGRSpatialReference* GetSpatialRef() const override;
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
 
-    virtual void   FlushCache() override;
+    virtual void   FlushCache(bool bAtClosing) override;
 };
 
 // IniFile.h: interface for the IniFile class.
