@@ -48,7 +48,6 @@
 #include "gdal.h"
 #include "gdal_frmts.h"
 
-CPL_CVSID("$Id$")
 
 struct MEMDataset::Private
 {
@@ -65,7 +64,7 @@ GDALRasterBandH MEMCreateRasterBand( GDALDataset *poDS, int nBand,
                                      int bAssumeOwnership )
 
 {
-    return reinterpret_cast<GDALRasterBandH>(
+    return GDALRasterBand::ToHandle(
         new MEMRasterBand( poDS, nBand, pabyData, eType, nPixelOffset,
                            nLineOffset, bAssumeOwnership ) );
 }
@@ -81,7 +80,7 @@ GDALRasterBandH MEMCreateRasterBandEx( GDALDataset *poDS, int nBand,
                                        int bAssumeOwnership )
 
 {
-    return reinterpret_cast<GDALRasterBandH>(
+    return GDALRasterBand::ToHandle(
         new MEMRasterBand( poDS, nBand, pabyData, eType, nPixelOffset,
                            nLineOffset, bAssumeOwnership ) );
 }
@@ -183,7 +182,7 @@ CPLErr MEMRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 
         for( int iPixel = 0; iPixel < nBlockXSize; iPixel++ )
         {
-            memcpy( reinterpret_cast<GByte *>(pImage) + iPixel*nWordSize,
+            memcpy( static_cast<GByte *>(pImage) + iPixel*nWordSize,
                     pabyCur + iPixel*nPixelOffset,
                     nWordSize );
         }
@@ -217,7 +216,7 @@ CPLErr MEMRasterBand::IWriteBlock( CPL_UNUSED int nBlockXOff,
         for( int iPixel = 0; iPixel < nBlockXSize; iPixel++ )
         {
             memcpy( pabyCur + iPixel*nPixelOffset,
-                    reinterpret_cast<GByte *>( pImage ) + iPixel*nWordSize,
+                    static_cast<GByte *>( pImage ) + iPixel*nWordSize,
                     nWordSize );
         }
     }
@@ -259,8 +258,8 @@ CPLErr MEMRasterBand::IRasterIO( GDALRWFlag eRWFlag,
                 nXOff*nPixelOffset,
                 eDataType,
                 static_cast<int>(nPixelOffset),
-                reinterpret_cast<GByte*>( pData ) +
-                nLineSpaceBuf * static_cast<GPtrDiff_t>(iLine),
+                static_cast<GByte*>( pData ) +
+                    nLineSpaceBuf * static_cast<GPtrDiff_t>(iLine),
                 eBufType,
                 static_cast<int>(nPixelSpaceBuf),
                 nXSize );
@@ -271,8 +270,8 @@ CPLErr MEMRasterBand::IRasterIO( GDALRWFlag eRWFlag,
         for( int iLine = 0; iLine < nYSize; iLine++ )
         {
             GDALCopyWords(
-                reinterpret_cast<GByte *>( pData ) +
-                nLineSpaceBuf*static_cast<GPtrDiff_t>(iLine),
+                static_cast<GByte *>( pData ) +
+                    nLineSpaceBuf*static_cast<GPtrDiff_t>(iLine),
                 eBufType,
                 static_cast<int>(nPixelSpaceBuf),
                 pabyData + nLineOffset*static_cast<GPtrDiff_t>(iLine + nYOff) +
@@ -301,47 +300,88 @@ CPLErr MEMDataset::IRasterIO( GDALRWFlag eRWFlag,
 {
     const int eBufTypeSize = GDALGetDataTypeSize(eBufType) / 8;
 
-    // Detect if we have a pixel-interleaved buffer and a pixel-interleaved
-    // dataset.
+    // Detect if we have a pixel-interleaved buffer
     if( nXSize == nBufXSize && nYSize == nBufYSize &&
         nBandCount == nBands && nBands > 1 &&
         nBandSpaceBuf == eBufTypeSize &&
         nPixelSpaceBuf == nBandSpaceBuf * nBands )
     {
-        GDALDataType eDT = GDT_Unknown;
-        GByte* pabyData = nullptr;
-        GSpacing nPixelOffset = 0;
-        GSpacing nLineOffset = 0;
-        int eDTSize = 0;
-        int iBandIndex;
-        for( iBandIndex = 0; iBandIndex < nBandCount; iBandIndex++ )
+        const auto IsPixelInterleaveDataset = [this, nBandCount, panBandMap]()
         {
-            if( panBandMap[iBandIndex] != iBandIndex + 1 )
-                break;
+            GDALDataType eDT = GDT_Unknown;
+            GByte* pabyData = nullptr;
+            GSpacing nPixelOffset = 0;
+            GSpacing nLineOffset = 0;
+            int eDTSize = 0;
+            for( int iBandIndex = 0; iBandIndex < nBandCount; iBandIndex++ )
+            {
+                if( panBandMap[iBandIndex] != iBandIndex + 1 )
+                    return false;
 
-            MEMRasterBand *poBand = reinterpret_cast<MEMRasterBand *>(
-                GetRasterBand(iBandIndex + 1) );
-            if( iBandIndex == 0 )
-            {
-                eDT = poBand->GetRasterDataType();
-                pabyData = poBand->pabyData;
-                nPixelOffset = poBand->nPixelOffset;
-                nLineOffset = poBand->nLineOffset;
-                eDTSize = GDALGetDataTypeSize(eDT) / 8;
-                if( nPixelOffset != static_cast<GSpacing>(nBands) * eDTSize )
-                    break;
+                MEMRasterBand *poBand = cpl::down_cast<MEMRasterBand *>(
+                    GetRasterBand(iBandIndex + 1) );
+                if( iBandIndex == 0 )
+                {
+                    eDT = poBand->GetRasterDataType();
+                    pabyData = poBand->pabyData;
+                    nPixelOffset = poBand->nPixelOffset;
+                    nLineOffset = poBand->nLineOffset;
+                    eDTSize = GDALGetDataTypeSizeBytes(eDT);
+                    if( nPixelOffset != static_cast<GSpacing>(nBands) * eDTSize )
+                        return false;
+                }
+                else if( poBand->GetRasterDataType() != eDT ||
+                         nPixelOffset != poBand->nPixelOffset ||
+                         nLineOffset != poBand->nLineOffset ||
+                         poBand->pabyData != pabyData + iBandIndex * eDTSize )
+                {
+                    return false;
+                }
             }
-            else if( poBand->GetRasterDataType() != eDT ||
-                     nPixelOffset != poBand->nPixelOffset ||
-                     nLineOffset != poBand->nLineOffset ||
-                     poBand->pabyData != pabyData + iBandIndex * eDTSize )
+            return true;
+        };
+
+        const auto IsBandSeparatedDataset = [this, nBandCount, panBandMap]()
+        {
+            GDALDataType eDT = GDT_Unknown;
+            GSpacing nPixelOffset = 0;
+            GSpacing nLineOffset = 0;
+            int eDTSize = 0;
+            for( int iBandIndex = 0; iBandIndex < nBandCount; iBandIndex++ )
             {
-                break;
+                if( panBandMap[iBandIndex] != iBandIndex + 1 )
+                    return false;
+
+                MEMRasterBand *poBand = cpl::down_cast<MEMRasterBand *>(
+                    GetRasterBand(iBandIndex + 1) );
+                if( iBandIndex == 0 )
+                {
+                    eDT = poBand->GetRasterDataType();
+                    nPixelOffset = poBand->nPixelOffset;
+                    nLineOffset = poBand->nLineOffset;
+                    eDTSize = GDALGetDataTypeSizeBytes(eDT);
+                    if( nPixelOffset != eDTSize )
+                        return false;
+                }
+                else if( poBand->GetRasterDataType() != eDT ||
+                         nPixelOffset != poBand->nPixelOffset ||
+                         nLineOffset != poBand->nLineOffset )
+                {
+                    return false;
+                }
             }
-        }
-        if( iBandIndex == nBandCount )
+            return true;
+        };
+
+        if( IsPixelInterleaveDataset() )
         {
             FlushCache(false);
+            const auto poFirstBand = cpl::down_cast<MEMRasterBand *>(papoBands[0]);
+            const GDALDataType eDT = poFirstBand->GetRasterDataType();
+            GByte* pabyData = poFirstBand->pabyData;
+            const GSpacing nPixelOffset = poFirstBand->nPixelOffset;
+            const GSpacing nLineOffset = poFirstBand->nLineOffset;
+            const int eDTSize = GDALGetDataTypeSizeBytes(eDT);
             if( eRWFlag == GF_Read )
             {
                 for(int iLine=0;iLine<nYSize;iLine++)
@@ -352,8 +392,8 @@ CPLErr MEMDataset::IRasterIO( GDALRWFlag eRWFlag,
                         nXOff*nPixelOffset,
                         eDT,
                         eDTSize,
-                        reinterpret_cast<GByte *>( pData ) +
-                        nLineSpaceBuf * static_cast<size_t>(iLine),
+                        static_cast<GByte *>( pData ) +
+                            nLineSpaceBuf * static_cast<size_t>(iLine),
                         eBufType,
                         eBufTypeSize,
                         nXSize * nBands );
@@ -364,8 +404,8 @@ CPLErr MEMDataset::IRasterIO( GDALRWFlag eRWFlag,
                 for(int iLine=0;iLine<nYSize;iLine++)
                 {
                     GDALCopyWords(
-                        reinterpret_cast<GByte *>( pData ) +
-                        nLineSpaceBuf*(size_t)iLine,
+                        static_cast<GByte *>( pData ) +
+                            nLineSpaceBuf*(size_t)iLine,
                         eBufType,
                         eBufTypeSize,
                         pabyData +
@@ -378,6 +418,57 @@ CPLErr MEMDataset::IRasterIO( GDALRWFlag eRWFlag,
             }
             return CE_None;
         }
+        else if( eRWFlag == GF_Write && nBandCount <= 4 &&
+                 IsBandSeparatedDataset() )
+        {
+            // TODO: once we have a GDALInterleave() function, implement the GF_Read
+            // case
+            FlushCache(false);
+            const auto poFirstBand = cpl::down_cast<MEMRasterBand *>(papoBands[0]);
+            const GDALDataType eDT = poFirstBand->GetRasterDataType();
+            void* ppDestBuffer[4] = { nullptr, nullptr, nullptr, nullptr };
+            if( nXOff == 0 &&
+                nXSize == nRasterXSize &&
+                poFirstBand->nLineOffset == poFirstBand->nPixelOffset * nXSize &&
+                nLineSpaceBuf == nPixelSpaceBuf * nXSize )
+            {
+                // Optimization of the general case in the below else() clause:
+                // writing whole strips from a fully packed buffer
+                for( int i = 0; i < nBandCount; ++i )
+                {
+                    const auto poBand = cpl::down_cast<MEMRasterBand *>(papoBands[i]);
+                    ppDestBuffer[i] = poBand->pabyData + poBand->nLineOffset * nYOff;
+                }
+                GDALDeinterleave(
+                        pData,
+                        eBufType,
+                        nBandCount,
+                        ppDestBuffer,
+                        eDT,
+                        static_cast<size_t>(nXSize) * nYSize);
+            }
+            else
+            {
+                for(int iLine=0;iLine<nYSize;iLine++)
+                {
+                    for( int i = 0; i < nBandCount; ++i )
+                    {
+                        const auto poBand = cpl::down_cast<MEMRasterBand *>(papoBands[i]);
+                        ppDestBuffer[i] = poBand->pabyData +
+                            poBand->nPixelOffset * nXOff +
+                            poBand->nLineOffset * (iLine + nYOff);
+                    }
+                    GDALDeinterleave(
+                        static_cast<GByte *>( pData ) + nLineSpaceBuf*static_cast<size_t>(iLine),
+                        eBufType,
+                        nBandCount,
+                        ppDestBuffer,
+                        eDT,
+                        nXSize);
+                }
+            }
+            return CE_None;
+        }
     }
 
     if( nBufXSize != nXSize || nBufYSize != nYSize )
@@ -433,7 +524,7 @@ CPLErr MEMRasterBand::CreateMaskBand( int nFlagsIn )
     if( (nFlagsIn & GMF_PER_DATASET) != 0 && nBand != 1 && poMemDS != nullptr )
     {
         MEMRasterBand* poFirstBand =
-            reinterpret_cast<MEMRasterBand*>(poMemDS->GetRasterBand(1));
+            dynamic_cast<MEMRasterBand*>(poMemDS->GetRasterBand(1));
         if( poFirstBand != nullptr)
             return poFirstBand->CreateMaskBand( nFlagsIn );
     }
@@ -454,7 +545,7 @@ CPLErr MEMRasterBand::CreateMaskBand( int nFlagsIn )
         for( int i = 2; i <= poMemDS->GetRasterCount(); ++i )
         {
             MEMRasterBand* poOtherBand =
-                reinterpret_cast<MEMRasterBand*>(poMemDS->GetRasterBand(i));
+                cpl::down_cast<MEMRasterBand*>(poMemDS->GetRasterBand(i));
             poOtherBand->InvalidateMaskBand();
             poOtherBand->nMaskFlags = nFlagsIn;
             poOtherBand->bOwnMask = false;
@@ -605,7 +696,7 @@ void *MEMDataset::GetInternalHandle( const char * pszRequest )
         if(int BandNumber = static_cast<int>(CPLScanLong(&pszRequest[6], 10)))
         {
             MEMRasterBand *RequestedRasterBand =
-                reinterpret_cast<MEMRasterBand *>( GetRasterBand(BandNumber) );
+                cpl::down_cast<MEMRasterBand *>( GetRasterBand(BandNumber) );
 
             // we're within a MEMDataset so the only thing a RasterBand
             // could be is a MEMRasterBand
@@ -631,13 +722,13 @@ int MEMDataset::GetGCPCount()
 }
 
 /************************************************************************/
-/*                          GetGCPProjection()                          */
+/*                          GetGCPSpatialRef()                          */
 /************************************************************************/
 
-const char *MEMDataset::_GetGCPProjection()
+const OGRSpatialReference *MEMDataset::GetGCPSpatialRef() const
 
 {
-    return osGCPProjection;
+    return m_oGCPSRS.IsEmpty() ? nullptr : &m_oGCPSRS;
 }
 
 /************************************************************************/
@@ -654,17 +745,16 @@ const GDAL_GCP *MEMDataset::GetGCPs()
 /*                              SetGCPs()                               */
 /************************************************************************/
 
-CPLErr MEMDataset::_SetGCPs( int nNewCount, const GDAL_GCP *pasNewGCPList,
-                            const char *pszGCPProjection )
+CPLErr MEMDataset::SetGCPs( int nNewCount, const GDAL_GCP *pasNewGCPList,
+                            const OGRSpatialReference* poSRS )
 
 {
     GDALDeinitGCPs( m_nGCPCount, m_pasGCPs );
     CPLFree( m_pasGCPs );
 
-    if( pszGCPProjection == nullptr )
-        osGCPProjection = "";
-    else
-        osGCPProjection = pszGCPProjection;
+    m_oGCPSRS.Clear();
+    if( poSRS )
+        m_oGCPSRS = *poSRS;
 
     m_nGCPCount = nNewCount;
     m_pasGCPs = GDALDuplicateGCPs( m_nGCPCount, pasNewGCPList );
@@ -697,7 +787,7 @@ CPLErr MEMDataset::AddBand( GDALDataType eType, char **papszOptions )
 #if SIZEOF_VOIDP == 4
             ( nTmp > INT_MAX ) ? nullptr :
 #endif
-            reinterpret_cast<GByte *>(
+            static_cast<GByte *>(
                 VSI_CALLOC_VERBOSE((size_t)nTmp, GetRasterYSize() ) );
 
         if( pData == nullptr )
@@ -716,7 +806,7 @@ CPLErr MEMDataset::AddBand( GDALDataType eType, char **papszOptions )
 /*      Get layout of memory and other flags.                           */
 /* -------------------------------------------------------------------- */
     const char *pszDataPointer = CSLFetchNameValue(papszOptions, "DATAPOINTER");
-    GByte *pData = reinterpret_cast<GByte *>(
+    GByte *pData = static_cast<GByte *>(
         CPLScanPointer( pszDataPointer,
                         static_cast<int>(strlen(pszDataPointer)) ) );
 
@@ -741,15 +831,27 @@ CPLErr MEMDataset::AddBand( GDALDataType eType, char **papszOptions )
     return CE_None;
 }
 
+/************************************************************************/
+/*                           AddMEMBand()                               */
+/************************************************************************/
+
+void MEMDataset::AddMEMBand(GDALRasterBandH hMEMBand)
+{
+    auto poBand = GDALRasterBand::FromHandle(hMEMBand);
+    CPLAssert( dynamic_cast<MEMRasterBand*>(poBand) != nullptr );
+    SetBand( 1 + nBands, poBand);
+}
+
 /************************************************************************/
 /*                          IBuildOverviews()                           */
 /************************************************************************/
 
 CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
-                                     int nOverviews, int *panOverviewList,
-                                     int nListBands, int *panBandList,
-                                     GDALProgressFunc pfnProgress,
-                                     void * pProgressData )
+                                    int nOverviews, const int *panOverviewList,
+                                    int nListBands, const int *panBandList,
+                                    GDALProgressFunc pfnProgress,
+                                    void * pProgressData,
+                                    CSLConstList papszOptions )
 {
     if( nBands == 0 )
     {
@@ -812,7 +914,8 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
                                     1, &panOverviewList[i],
                                     nListBands, panBandList,
                                     GDALScaledProgress,
-                                    pScaledProgress );
+                                    pScaledProgress,
+                                    papszOptions );
             GDALDestroyScaledProgress( pScaledProgress );
             dfAccPixels += dfPixels;
             if( eErr == CE_Failure )
@@ -931,7 +1034,7 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
 
         // If the band has an explicit mask, we need to create overviews
         // for it
-        MEMRasterBand* poMEMBand = reinterpret_cast<MEMRasterBand*>(poBand);
+        MEMRasterBand* poMEMBand = cpl::down_cast<MEMRasterBand*>(poBand);
         const bool bMustGenerateMaskOvr =
             ( (poMEMBand->bOwnMask && poMEMBand->poMask != nullptr) ||
         // Or if it is a per-dataset mask, in which case just do it for the
@@ -944,7 +1047,7 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
             for( int i = 0; i < nNewOverviews; i++ )
             {
                 MEMRasterBand* poMEMOvrBand =
-                    reinterpret_cast<MEMRasterBand*>(papoOverviewBands[i]);
+                    cpl::down_cast<MEMRasterBand*>(papoOverviewBands[i]);
                 if( !(poMEMOvrBand->bOwnMask && poMEMOvrBand->poMask != nullptr) &&
                     (poMEMOvrBand->nMaskFlags & GMF_PER_DATASET) == 0 )
                 {
@@ -958,7 +1061,7 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
                     1.0 * (iBand+0.5) / nBands,
                     pfnProgress, pProgressData );
 
-            MEMRasterBand* poMaskBand = reinterpret_cast<MEMRasterBand*>(
+            MEMRasterBand* poMaskBand = cpl::down_cast<MEMRasterBand*>(
                                                         poBand->GetMaskBand());
             // Make the mask band to be its own mask, similarly to what is
             // done for alpha bands in GDALRegenerateOverviews() (#5640)
@@ -966,12 +1069,13 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
             poMaskBand->bOwnMask = false;
             poMaskBand->poMask = poMaskBand;
             poMaskBand->nMaskFlags = 0;
-            eErr = GDALRegenerateOverviews(
+            eErr = GDALRegenerateOverviewsEx(
                                         (GDALRasterBandH) poMaskBand,
                                         nNewOverviews,
                                         (GDALRasterBandH*)papoMaskOverviewBands,
                                         pszResampling,
-                                        GDALScaledProgress, pScaledProgress );
+                                        GDALScaledProgress, pScaledProgress,
+                                        papszOptions);
             poMaskBand->InvalidateMaskBand();
             GDALDestroyScaledProgress( pScaledProgress );
         }
@@ -983,11 +1087,12 @@ CPLErr MEMDataset::IBuildOverviews( const char *pszResampling,
                     1.0 * (iBand+(bMustGenerateMaskOvr ? 0.5 : 1)) / nBands,
                     1.0 * (iBand+1)/ nBands,
                     pfnProgress, pProgressData );
-            eErr = GDALRegenerateOverviews( (GDALRasterBandH) poBand,
+            eErr = GDALRegenerateOverviewsEx( (GDALRasterBandH) poBand,
                                             nNewOverviews,
                                             (GDALRasterBandH*)papoOverviewBands,
                                             pszResampling,
-                                            GDALScaledProgress, pScaledProgress );
+                                            GDALScaledProgress, pScaledProgress,
+                                            papszOptions);
             GDALDestroyScaledProgress( pScaledProgress );
         }
     }
@@ -1132,7 +1237,7 @@ GDALDataset *MEMDataset::Open( GDALOpenInfo * poOpenInfo )
                                      static_cast<int>(strlen(pszOption)));
 
     const char *pszDataPointer = CSLFetchNameValue(papszOptions,"DATAPOINTER");
-    GByte *pabyData = reinterpret_cast<GByte *>(
+    GByte *pabyData = static_cast<GByte *>(
         CPLScanPointer( pszDataPointer,
                         static_cast<int>(strlen(pszDataPointer)) ) );
 
@@ -1177,7 +1282,7 @@ GDALDataset *MEMDataset::Open( GDALOpenInfo * poOpenInfo )
 /*                               Create()                               */
 /************************************************************************/
 
-GDALDataset *MEMDataset::Create( const char * /* pszFilename */,
+MEMDataset *MEMDataset::Create( const char * /* pszFilename */,
                                  int nXSize,
                                  int nYSize,
                                  int nBandsIn,
@@ -1227,7 +1332,7 @@ GDALDataset *MEMDataset::Create( const char * /* pszFilename */,
     if( bPixelInterleaved )
     {
         apbyBandData.push_back(
-            reinterpret_cast<GByte *>( VSI_CALLOC_VERBOSE( 1, nGlobalSize ) ) );
+            static_cast<GByte *>( VSI_CALLOC_VERBOSE( 1, nGlobalSize ) ) );
 
         if( apbyBandData[0] == nullptr )
             bAllocOK = FALSE;
@@ -1242,7 +1347,7 @@ GDALDataset *MEMDataset::Create( const char * /* pszFilename */,
         for( int iBand = 0; iBand < nBandsIn; iBand++ )
         {
             apbyBandData.push_back(
-                reinterpret_cast<GByte *>(
+                static_cast<GByte *>(
                     VSI_CALLOC_VERBOSE(
                         1,
                         static_cast<size_t>(nWordSize) * nXSize * nYSize ) ) );
@@ -1306,6 +1411,17 @@ GDALDataset *MEMDataset::Create( const char * /* pszFilename */,
     return poDS;
 }
 
+GDALDataset *MEMDataset::CreateBase( const char * pszFilename,
+                                 int nXSize,
+                                 int nYSize,
+                                 int nBandsIn,
+                                 GDALDataType eType,
+                                 char **papszOptions )
+{
+    return Create(pszFilename, nXSize, nYSize, nBandsIn,
+                  eType, papszOptions);
+}
+
 /************************************************************************/
 /*                           GetMDArrayNames()                          */
 /************************************************************************/
@@ -1387,6 +1503,7 @@ std::shared_ptr<GDALGroup> MEMGroup::CreateGroup(const std::string& osName,
 std::shared_ptr<GDALMDArray> MEMGroup::CreateMDArray(const std::string& osName,
                                                      const std::vector<std::shared_ptr<GDALDimension>>& aoDimensions,
                                                      const GDALExtendedDataType& oType,
+                                                     void* pData,
                                                      CSLConstList papszOptions)
 {
     if( osName.empty() )
@@ -1403,15 +1520,11 @@ std::shared_ptr<GDALMDArray> MEMGroup::CreateMDArray(const std::string& osName,
     }
     auto newArray(MEMMDArray::Create(GetFullName(), osName, aoDimensions, oType));
 
-    // Used by NUMPYMultiDimensionalDataset
-    const char *pszDataPointer = CSLFetchNameValue(papszOptions, "DATAPOINTER");
-    GByte* pData = nullptr;
+    GByte* pabyData = nullptr;
     std::vector<GPtrDiff_t> anStrides;
-    if( pszDataPointer )
+    if( pData )
     {
-        pData = reinterpret_cast<GByte *>(
-            CPLScanPointer( pszDataPointer,
-                            static_cast<int>(strlen(pszDataPointer)) ) );
+        pabyData = static_cast<GByte *>(pData);
         const char* pszStrides = CSLFetchNameValue(papszOptions, "STRIDES");
         if( pszStrides )
         {
@@ -1429,12 +1542,45 @@ std::shared_ptr<GDALMDArray> MEMGroup::CreateMDArray(const std::string& osName,
             }
         }
     }
-    if( !newArray->Init(pData, anStrides) )
+    if( !newArray->Init(pabyData, anStrides) )
         return nullptr;
     m_oMapMDArrays[osName] = newArray;
     return newArray;
 }
 
+std::shared_ptr<GDALMDArray> MEMGroup::CreateMDArray(const std::string& osName,
+                                                     const std::vector<std::shared_ptr<GDALDimension>>& aoDimensions,
+                                                     const GDALExtendedDataType& oType,
+                                                     CSLConstList papszOptions)
+{
+    void* pData = nullptr;
+    const char *pszDataPointer = CSLFetchNameValue(papszOptions, "DATAPOINTER");
+    if( pszDataPointer )
+    {
+        // Will not work on architectures with "capability pointers"
+        pData =
+            CPLScanPointer( pszDataPointer,
+                            static_cast<int>(strlen(pszDataPointer)) );
+    }
+    return CreateMDArray(osName, aoDimensions, oType, pData, papszOptions);
+}
+
+/************************************************************************/
+/*                      MEMGroupCreateMDArray()                         */
+/************************************************************************/
+
+// Used by NUMPYMultiDimensionalDataset
+std::shared_ptr<GDALMDArray> MEMGroupCreateMDArray(GDALGroup* poGroup,
+                                                   const std::string& osName,
+                                                   const std::vector<std::shared_ptr<GDALDimension>>& aoDimensions,
+                                                   const GDALExtendedDataType& oDataType,
+                                                   void* pData,
+                                                   CSLConstList papszOptions)
+{
+    return dynamic_cast<MEMGroup*>(poGroup)->CreateMDArray(
+        osName, aoDimensions, oDataType, pData, papszOptions);
+}
+
 /************************************************************************/
 /*                            GetAttribute()                            */
 /************************************************************************/
@@ -1720,7 +1866,7 @@ void MEMAbstractMDArray::ReadWrite(bool bIsWrite,
                 CPLAssert(false);
             }
             else if ( bBothAreNumericDT
-#if SIZEOF_VOIDP == 8
+#if SIZEOF_VOIDP >= 8
                       && src_inc_offset <= std::numeric_limits<int>::max()
                       && dst_inc_offset <= std::numeric_limits<int>::max()
 #endif
@@ -2209,7 +2355,7 @@ void GDALRegister_MEM()
     poDriver->pfnOpen = MEMDataset::Open;
     poDriver->pfnIdentify = MEMDatasetIdentify;
 #endif
-    poDriver->pfnCreate = MEMDataset::Create;
+    poDriver->pfnCreate = MEMDataset::CreateBase;
     poDriver->pfnCreateMultiDimensional = MEMDataset::CreateMultiDimensional;
     poDriver->pfnDelete = MEMDatasetDelete;
 
