@@ -39,7 +39,6 @@
 
 using namespace msg_native_format;
 
-CPL_CVSID("$Id$")
 
 typedef enum {
     MODE_VISIR,     // Visible and Infrared bands (1 through 11) in 10-bit raw mode
@@ -74,7 +73,7 @@ class MSGNDataset final: public GDALDataset
     int                 m_nHRVLowerShiftX = 0;
     int                 m_nHRVUpperShiftX = 0;
     double adfGeoTransform[6];
-    char   *pszProjection;
+    OGRSpatialReference m_oSRS{};
 
   public:
     MSGNDataset();
@@ -83,10 +82,7 @@ class MSGNDataset final: public GDALDataset
     static GDALDataset *Open( GDALOpenInfo * );
 
     CPLErr     GetGeoTransform( double * padfTransform ) override;
-    const char *_GetProjectionRef() override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
+    const OGRSpatialReference* GetSpatialRef() const override;
 };
 
 /************************************************************************/
@@ -241,7 +237,7 @@ CPLErr MSGNRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
                  (p->lineNumberInVisirGrid),
                  poGDS->msg_reader_core->get_line_start(),
                  i_nBlockYOff);
-    
+
         CPLFree( pszRecord );
 
         CPLError( CE_Failure, CPLE_AppDefined, "MSGN Scanline corrupt." );
@@ -329,9 +325,9 @@ double MSGNRasterBand::GetMaximum(int *pbSuccess ) {
 
 MSGNDataset::MSGNDataset() :
     fp(nullptr),
-    msg_reader_core(nullptr),
-    pszProjection(CPLStrdup(""))
+    msg_reader_core(nullptr)
 {
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     std::fill_n(adfGeoTransform, CPL_ARRAYSIZE(adfGeoTransform), 0);
 }
 
@@ -348,8 +344,6 @@ MSGNDataset::~MSGNDataset()
     if (msg_reader_core) {
         delete msg_reader_core;
     }
-
-    CPLFree(pszProjection);
 }
 
 /************************************************************************/
@@ -367,13 +361,13 @@ CPLErr MSGNDataset::GetGeoTransform( double * padfTransform )
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
 
-const char *MSGNDataset::_GetProjectionRef()
+const OGRSpatialReference *MSGNDataset::GetSpatialRef() const
 
 {
-    return pszProjection;
+   return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
 }
 
 /************************************************************************/
@@ -571,7 +565,7 @@ GDALDataset *MSGNDataset::Open( GDALOpenInfo * poOpenInfo )
     unsigned int band_count = 1;
     unsigned int missing_band_count = 0;
     const unsigned char* bands = poDS->msg_reader_core->get_band_map();
-    unsigned char band_map[MSG_NUM_CHANNELS+1];   // map GDAL band numbers to MSG channels
+    unsigned char band_map[MSG_NUM_CHANNELS+1] = {0};   // map GDAL band numbers to MSG channels
     for (i=0; i < MSG_NUM_CHANNELS; i++) {
         if (bands[i]) {
             bool ok_to_add = false;
@@ -606,11 +600,11 @@ GDALDataset *MSGNDataset::Open( GDALOpenInfo * poOpenInfo )
     /* there are a number of 'magic' constants below
        I trimmed them to get registration for MSG4, MSG3, MSG2 with country outlines  from
        http://ec.europa.eu/eurostat/web/gisco/geodata/reference-data/administrative-units-statistical-units
-       
+
        Adjust in two phases P1, P2.
        I describe direction as outline being NSEW of coast shape when number is changed
     */
-  
+
     if (open_mode != MODE_HRV) {
         pixel_gsd_x = 1000.0 * poDS->msg_reader_core->get_col_dir_step();  // convert from km to m
         pixel_gsd_y = 1000.0 * poDS->msg_reader_core->get_line_dir_step(); // convert from km to m
@@ -625,16 +619,16 @@ GDALDataset *MSGNDataset::Open( GDALOpenInfo * poOpenInfo )
             origin_y = -pixel_gsd_y * ((3*Conversions::nlines / 2.0) - idr.plannedCoverage_hrv.lowerSouthLinePlanned+2);    //          N-S -ve S
         }
         else
-        {  
+        {
             origin_x = -pixel_gsd_x * (-(3*Conversions::nlines / 2.0) + 1*poDS->msg_reader_core->get_col_start()-3);  // MSG4, MSG2 HRV E-W -ve E
             origin_y = -pixel_gsd_y * ((3*Conversions::nlines / 2.0) - 1*poDS->msg_reader_core->get_line_start()+4);  //                N-S +ve S
         }
     }
-  
+
     /* the conversion to lat/long is in two parts:
        pixels to m (around imaginary circle r=sta height) in the geo projection (affine transformation)
        geo to lat/long via the GEOS projection (in WKT) and the ellipsoid
-       
+
        CGMS/DOC/12/0017 section 4.4.2
     */
 
@@ -646,11 +640,9 @@ GDALDataset *MSGNDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->adfGeoTransform[4] = 0.0;
     poDS->adfGeoTransform[5] = -pixel_gsd_y;
 
-    OGRSpatialReference oSRS;
+    poDS->m_oSRS.SetProjCS("Geostationary projection (MSG)");
 
-    oSRS.SetProjCS("Geostationary projection (MSG)");
-
-    oSRS.SetGeogCS(
+    poDS->m_oSRS.SetGeogCS(
                "MSG Ellipsoid",
                "MSG_DATUM",
                "MSG_SPHEROID",
@@ -658,17 +650,14 @@ GDALDataset *MSGNDataset::Open( GDALOpenInfo * poOpenInfo )
                1.0/Conversions::oblate // 1 / ( 1 - Conversions::rpol/Conversions::req)
                );
 
-    oSRS.SetGEOS(  idr.longitudeOfSSP, (Conversions::altitude - Conversions::req) * 1000.0,  // we're using metres as length unit
+    poDS->m_oSRS.SetGEOS(  idr.longitudeOfSSP, (Conversions::altitude - Conversions::req) * 1000.0,  // we're using meters as length unit
                0.0 ,
                // false northing to handle the fact RSS is only 1/3 disk
                pixel_gsd_y *((poDS->m_Shape == RSS) ? ((open_mode != MODE_HRV)  ?
-                                   -(idr.plannedCoverage_visir.southernLinePlanned -1) : // MSG-3 vis/NIR N-S P2 
+                                   -(idr.plannedCoverage_visir.southernLinePlanned -1) : // MSG-3 vis/NIR N-S P2
                                    -(idr.plannedCoverage_hrv.lowerSouthLinePlanned +1)) : // MSG-3 HRV N-S P2 -ve N
                      0.0 ));
 
-    CPLFree(poDS->pszProjection);
-    poDS->pszProjection = nullptr;
-    oSRS.exportToWkt( &(poDS->pszProjection) );
     }
 
     const CALIBRATION* cal = poDS->msg_reader_core->get_calibration_parameters();
