@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitfdataset.h 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  NITF Read/Write Translator
  * Purpose:  GDALDataset/GDALRasterBand declarations.
@@ -82,11 +82,11 @@ class NITFDataset final: public GDALPamDataset
     int         bGotGeoTransform;
     double      adfGeoTransform[6];
 
-    char        *pszProjection;
+    OGRSpatialReference m_oSRS{};
 
     int         nGCPCount;
     GDAL_GCP    *pasGCPList;
-    char        *pszGCPProjection;
+    OGRSpatialReference m_oGCPSRS{};
 
     GDALMultiDomainMetadata oSpecialMD;
 
@@ -99,6 +99,7 @@ class NITFDataset final: public GDALPamDataset
     void         InitializeCGMMetadata();
     void         InitializeTextMetadata();
     void         InitializeTREMetadata();
+    void         InitializeImageStructureMetadata();
 
     GIntBig     *panJPEGBlockOffset;
     GByte       *pabyJPEGBlock;
@@ -147,30 +148,16 @@ class NITFDataset final: public GDALPamDataset
                               GSpacing nBandSpace,
                               GDALRasterIOExtraArg* psExtraArg ) override;
 
-    virtual const char *_GetProjectionRef() override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
-    virtual CPLErr _SetProjection( const char * ) override;
-    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
-        return OldSetProjectionFromSetSpatialRef(poSRS);
-    }
+    const OGRSpatialReference* GetSpatialRef() const override;
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
 
     virtual CPLErr GetGeoTransform( double * ) override;
     virtual CPLErr SetGeoTransform( double * ) override;
-    virtual CPLErr _SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
-                            const char *pszGCPProjection ) override;
-    using GDALPamDataset::SetGCPs;
     CPLErr SetGCPs( int nGCPCountIn, const GDAL_GCP *pasGCPListIn,
-                    const OGRSpatialReference* poSRS ) override {
-        return OldSetGCPsFromNew(nGCPCountIn, pasGCPListIn, poSRS);
-    }
+                    const OGRSpatialReference* poSRS ) override;
 
     virtual int    GetGCPCount() override;
-    virtual const char *_GetGCPProjection() override;
-    const OGRSpatialReference* GetGCPSpatialRef() const override {
-        return GetGCPSpatialRefFromOldGetGCPProjection();
-    }
+    const OGRSpatialReference* GetGCPSpatialRef() const override;
     virtual const GDAL_GCP *GetGCPs() override;
     virtual char **GetFileList() override;
 
@@ -179,8 +166,11 @@ class NITFDataset final: public GDALPamDataset
     virtual const char *GetMetadataItem( const char * pszName,
                                          const char * pszDomain = "" ) override;
     virtual void   FlushCache(bool bAtClosing) override;
-    virtual CPLErr IBuildOverviews( const char *, int, int *,
-                                    int, int *, GDALProgressFunc, void * ) override;
+    virtual CPLErr IBuildOverviews( const char *,
+                                    int, const int *,
+                                    int, const int *,
+                                    GDALProgressFunc, void *,
+                                    CSLConstList papszOptions ) override;
 
     static int          Identify( GDALOpenInfo * );
     static NITFDataset *OpenInternal( GDALOpenInfo *,
@@ -307,8 +297,9 @@ class NITFProxyPamRasterBand CPL_NON_FINAL: public GDALPamRasterBand
         virtual int GetOverviewCount() override;
         virtual GDALRasterBand *GetOverview(int) override;
         virtual GDALRasterBand *GetRasterSampleOverview( GUIntBig ) override;
-        virtual CPLErr BuildOverviews( const char *, int, int *,
-                                    GDALProgressFunc, void * ) override;
+        virtual CPLErr BuildOverviews( const char *, int, const int *,
+                                    GDALProgressFunc, void *,
+                                    CSLConstList papszOptions) override;
 
         virtual CPLErr AdviseRead( int nXOff, int nYOff, int nXSize, int nYSize,
                                 int nBufXSize, int nBufYSize,
