@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitflib.h 2c3a0f53a1550c31b62e4ca88bf594b97757ed41 2021-06-09 21:37:57 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  NITF Read/Write Library
  * Purpose:  Main GDAL independent include file for NITF support.
@@ -38,6 +38,12 @@
 
 CPL_C_START
 
+/* 1e-12 - 1 */
+#define NITF_MAX_FILE_SIZE 999999999999ULL
+
+/* 1e-10 - 1 */
+#define NITF_MAX_IMAGE_SIZE 9999999999ULL
+
 typedef struct {
     char szSegmentType[3]; /* one of "IM", ... */
 
@@ -89,9 +95,13 @@ int      CPL_DLL  NITFCreate( const char *pszFilename,
                               char **papszOptions );
 
 int        NITFCreateEx( const char *pszFilename,
-                              int nPixels, int nLines, int nBands,
-                              int nBitsPerSample, const char *pszPVType,
-                              char **papszOptions, int* pnICOffset );
+                         int nPixels, int nLines, int nBands,
+                         int nBitsPerSample, const char *pszPVType,
+                         char **papszOptions,
+                         int* pnIndex,
+                         int* pnImageCount,
+                         vsi_l_offset* pnImageOffset,
+                         vsi_l_offset* pnICOffset );
 
 const char CPL_DLL *NITFFindTRE( const char *pszTREData, int nTREBytes,
                                  const char *pszTag, int *pnFoundTRESize );
@@ -185,6 +195,9 @@ typedef struct {
     int     nTREBytes;
     char    *pachTRE;
 
+    int        nIXSOFL;
+    int        nIXSOFLOffsetInSubfileHeader;
+
     /* Internal information not for application use. */
 
     int        nWordSize;
@@ -268,6 +281,9 @@ void      CPL_DLL  NITFDESFreeTREData( char* pabyTREData );
 
 int       CPL_DLL  NITFDESExtractShapefile(NITFDES* psDES, const char* pszRadixFileName);
 
+CPLXMLNode* NITFCreateXMLDesUserDefinedSubHeader(NITFFile* psFile,
+                                                 const NITFDES* psDES);
+
 CPLXMLNode CPL_DLL *NITFDESGetXml(NITFFile*, int iSegment);
 
 /* -------------------------------------------------------------------- */
