@@ -47,7 +47,6 @@
 #include "nitflib.h"
 #include "vrtdataset.h"
 
-CPL_CVSID("$Id$")
 
 constexpr int GEOTRSFRM_TOPLEFT_X = 0;
 constexpr int GEOTRSFRM_WE_RES = 1;
@@ -74,7 +73,7 @@ constexpr int GEOTRSFRM_NS_RES = 5;
 class RPFTOCDataset final: public GDALPamDataset
 {
   char      **papszSubDatasets;
-  char       *pszProjection;
+  OGRSpatialReference m_oSRS{};
   int         bGotGeoTransform;
   double      adfGeoTransform[6];
 
@@ -83,17 +82,16 @@ class RPFTOCDataset final: public GDALPamDataset
   public:
     RPFTOCDataset() :
         papszSubDatasets(nullptr),
-        pszProjection(nullptr),
         bGotGeoTransform(FALSE),
         papszFileList(nullptr)
     {
+        m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         memset( adfGeoTransform, 0, sizeof(adfGeoTransform) );
     }
 
     virtual ~RPFTOCDataset()
     {
         CSLDestroy( papszSubDatasets );
-        CPLFree( pszProjection );
         CSLDestroy( papszFileList );
     }
 
@@ -126,22 +124,15 @@ class RPFTOCDataset final: public GDALPamDataset
         return CE_None;
     }
 
-    virtual CPLErr _SetProjection( const char * projectionRef ) override
-    {
-        CPLFree(pszProjection);
-        pszProjection = CPLStrdup(projectionRef);
-        return CE_None;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return m_oSRS.IsEmpty() ? nullptr: &m_oSRS;
     }
 
-    virtual const char *_GetProjectionRef(void) override
-    {
-        return (pszProjection) ? pszProjection : "";
-    }
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
     CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
-        return OldSetProjectionFromSetSpatialRef(poSRS);
+        m_oSRS.Clear();
+        if( poSRS )
+            m_oSRS = *poSRS;
+        return CE_None;
     }
 
     static int IsNITFFileTOC(NITFFile *psFile);
