@@ -37,7 +37,6 @@
 #include "../../ogr/ogrsf_frmts/mitab/mitab.h"
 #endif
 
-CPL_CVSID("$Id: grcdataset.cpp f6099e5ed704166bf5cc113a053dd1b2725cb391 2020-03-22 11:20:10 +0100 Kai Pastor $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -55,7 +54,10 @@ class NWT_GRCDataset final : public GDALPamDataset
     GByte abyHeader[1024];
     NWT_GRID *pGrd;
     char **papszCategories;
-    char *pszProjection;
+    mutable OGRSpatialReference m_oSRS{};
+
+    NWT_GRCDataset(const NWT_GRCDataset&) = delete;
+    NWT_GRCDataset& operator= (const NWT_GRCDataset&) = delete;
 
   protected:
     GDALColorTable * poColorTable;
@@ -68,10 +70,7 @@ class NWT_GRCDataset final : public GDALPamDataset
     static int Identify( GDALOpenInfo * poOpenInfo );
 
     CPLErr GetGeoTransform( double *padfTransform ) override;
-    const char *_GetProjectionRef() override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
+    const OGRSpatialReference* GetSpatialRef() const override;
 };
 
 /************************************************************************/
@@ -105,7 +104,7 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDSIn, int nBandIn )
 {
     poDS = poDSIn;
     nBand = nBandIn;
-    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
+    NWT_GRCDataset *poGDS = cpl::down_cast<NWT_GRCDataset *>( poDS );
 
     if( poGDS->pGrd->nBitsPerPixel == 8 )
         eDataType = GDT_Byte;
@@ -130,13 +129,13 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDSIn, int nBandIn )
          i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems );
          i++ )
     {
-        oEntry.c1 = poGDS->pGrd->stClassDict->stClassifedItem[i]->r;
-        oEntry.c2 = poGDS->pGrd->stClassDict->stClassifedItem[i]->g;
-        oEntry.c3 = poGDS->pGrd->stClassDict->stClassifedItem[i]->b;
+        oEntry.c1 = poGDS->pGrd->stClassDict->stClassifiedItem[i]->r;
+        oEntry.c2 = poGDS->pGrd->stClassDict->stClassifiedItem[i]->g;
+        oEntry.c3 = poGDS->pGrd->stClassDict->stClassifiedItem[i]->b;
         oEntry.c4 = 255;            // alpha 255 = solid
 
         poGDS->poColorTable->SetColorEntry( poGDS->pGrd->
-                                          stClassDict->stClassifedItem[i]->
+                                          stClassDict->stClassifiedItem[i]->
                                           usPixVal, &oEntry );
     }
 
@@ -144,8 +143,8 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDSIn, int nBandIn )
     int maxValue = 0;
     for( int i=0; i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems ); i++ )
     {
-        if( poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal > maxValue )
-            maxValue = poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal;
+        if( poGDS->pGrd->stClassDict->stClassifiedItem[i]->usPixVal > maxValue )
+            maxValue = poGDS->pGrd->stClassDict->stClassifiedItem[i]->usPixVal;
     }
 
     // load a value for the null value
@@ -161,13 +160,13 @@ NWT_GRCRasterBand::NWT_GRCRasterBand( NWT_GRCDataset * poDSIn, int nBandIn )
              i < static_cast<int>( poGDS->pGrd->stClassDict->nNumClassifiedItems );
              i++ )
         {
-            if( static_cast<int>( poGDS->pGrd->stClassDict->stClassifedItem[i]->usPixVal ) ==
+            if( static_cast<int>( poGDS->pGrd->stClassDict->stClassifiedItem[i]->usPixVal ) ==
                 val )
             {
                 poGDS->papszCategories =
                     CSLAddString( poGDS->papszCategories,
                                     poGDS->pGrd->stClassDict->
-                                    stClassifedItem[i]->szClassName );
+                                    stClassifiedItem[i]->szClassName );
                 break;
             }
         }
@@ -189,7 +188,7 @@ double NWT_GRCRasterBand::GetNoDataValue( int *pbSuccess )
 // return an array of null terminated strings for the class names
 char **NWT_GRCRasterBand::GetCategoryNames()
 {
-    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
+    NWT_GRCDataset *poGDS = cpl::down_cast<NWT_GRCDataset *>( poDS );
 
     return poGDS->papszCategories;
 }
@@ -197,7 +196,7 @@ char **NWT_GRCRasterBand::GetCategoryNames()
 // return the color table
 GDALColorTable *NWT_GRCRasterBand::GetColorTable()
 {
-    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
+    NWT_GRCDataset *poGDS = cpl::down_cast<NWT_GRCDataset *>( poDS );
 
     return poGDS->poColorTable;
 }
@@ -217,7 +216,7 @@ CPLErr NWT_GRCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
                                       int nBlockYOff,
                                       void *pImage )
 {
-    NWT_GRCDataset *poGDS = reinterpret_cast<NWT_GRCDataset *>( poDS );
+    NWT_GRCDataset *poGDS = cpl::down_cast<NWT_GRCDataset *>( poDS );
     const int nBytesPerPixel = poGDS->pGrd->nBitsPerPixel / 8;
     if( nBytesPerPixel <= 0 || nBlockXSize > INT_MAX / nBytesPerPixel )
         return CE_Failure;
@@ -225,8 +224,8 @@ CPLErr NWT_GRCRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 
     if( nBand == 1 )
     {                            //grc's are just one band of indices
-        VSIFSeekL( poGDS->fp, 1024 + nRecordSize * (vsi_l_offset)nBlockYOff, SEEK_SET );
-        if( (int)VSIFReadL( pImage, 1, nRecordSize, poGDS->fp ) != nRecordSize )
+        VSIFSeekL( poGDS->fp, 1024 + nRecordSize * static_cast<vsi_l_offset>(nBlockYOff), SEEK_SET );
+        if( static_cast<int>(VSIFReadL( pImage, 1, nRecordSize, poGDS->fp )) != nRecordSize )
             return CE_Failure;
     }
     else
@@ -248,9 +247,9 @@ NWT_GRCDataset::NWT_GRCDataset() :
     fp(nullptr),
     pGrd(nullptr),
     papszCategories(nullptr),
-    pszProjection(nullptr),
     poColorTable(nullptr)
 {
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     memset(abyHeader, 0, sizeof(abyHeader) );
 }
 
@@ -262,14 +261,12 @@ NWT_GRCDataset::~NWT_GRCDataset()
     delete poColorTable;
     CSLDestroy( papszCategories );
 
-    NWT_GRCDataset::FlushCache();
+    NWT_GRCDataset::FlushCache(true);
     pGrd->fp = nullptr;       // this prevents nwtCloseGrid from closing the fp
     nwtCloseGrid( pGrd );
 
     if( fp != nullptr )
         VSIFCloseL( fp );
-
-    CPLFree( pszProjection );
 }
 
 /************************************************************************/
@@ -289,21 +286,22 @@ CPLErr NWT_GRCDataset::GetGeoTransform( double *padfTransform )
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
-const char *NWT_GRCDataset::_GetProjectionRef()
+const OGRSpatialReference *NWT_GRCDataset::GetSpatialRef() const
 {
-    if (pszProjection == nullptr)
+    if (m_oSRS.IsEmpty() )
     {
         OGRSpatialReference *poSpatialRef
           = MITABCoordSys2SpatialRef( pGrd->cMICoordSys );
         if (poSpatialRef)
         {
-            poSpatialRef->exportToWkt( &pszProjection );
+            m_oSRS = *poSpatialRef;
+            m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
             poSpatialRef->Release();
         }
     }
-    return (const char *) pszProjection;
+    return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
 }
 
 /************************************************************************/
@@ -350,7 +348,7 @@ GDALDataset *NWT_GRCDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     VSIFSeekL( poDS->fp, 0, SEEK_SET );
     VSIFReadL( poDS->abyHeader, 1, 1024, poDS->fp );
-    poDS->pGrd = reinterpret_cast<NWT_GRID *>( malloc( sizeof (NWT_GRID) ) );
+    poDS->pGrd = static_cast<NWT_GRID *>( malloc( sizeof (NWT_GRID) ) );
 
     poDS->pGrd->fp = poDS->fp;
 
