@@ -33,7 +33,6 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: pcidskdataset2.cpp a5d5ed208537a05de4437e97b6a09b7ba44f76c9 2020-03-24 08:27:48 +0100 Kai Pastor $")
 
 const PCIDSK::PCIDSKInterfaces *PCIDSK2GetInterfaces(void);
 
@@ -626,6 +625,7 @@ CPLErr PCIDSK2Band::SetMetadata( char **papszMD,
 /* -------------------------------------------------------------------- */
     CSLDestroy( papszLastMDListValue );
     papszLastMDListValue = nullptr;
+    m_oCacheMetadataItem.clear();
 
     if( GetAccess() == GA_ReadOnly )
     {
@@ -679,6 +679,7 @@ CPLErr PCIDSK2Band::SetMetadataItem( const char *pszName,
 /* -------------------------------------------------------------------- */
     CSLDestroy( papszLastMDListValue );
     papszLastMDListValue = nullptr;
+    m_oCacheMetadataItem.clear();
 
     if( GetAccess() == GA_ReadOnly )
     {
@@ -729,23 +730,29 @@ const char *PCIDSK2Band::GetMetadataItem( const char *pszName,
         return GDALPamRasterBand::GetMetadataItem( pszName, pszDomain );
 
 /* -------------------------------------------------------------------- */
-/*      Try and fetch.                                                  */
+/*      Try and fetch (use cached value if available)                   */
 /* -------------------------------------------------------------------- */
+    auto oIter = m_oCacheMetadataItem.find(pszName);
+    if( oIter != m_oCacheMetadataItem.end() )
+    {
+        return oIter->second.empty() ? nullptr : oIter->second.c_str();
+    }
+
+    CPLString osValue;
     try
     {
-        osLastMDValue = poChannel->GetMetadataValue( pszName );
+        osValue = poChannel->GetMetadataValue( pszName );
     }
     catch( const PCIDSKException& ex )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "%s", ex.what() );
+                "%s", ex.what() );
         return nullptr;
     }
 
-    if( osLastMDValue == "" )
-        return nullptr;
-
-    return osLastMDValue.c_str();
+    oIter = m_oCacheMetadataItem.insert(
+        std::pair<std::string, std::string>(pszName, osValue)).first;
+    return oIter->second.empty() ? nullptr : oIter->second.c_str();
 }
 
 /************************************************************************/
@@ -821,7 +828,7 @@ PCIDSK2Dataset::PCIDSK2Dataset() :
 #endif
 PCIDSK2Dataset::~PCIDSK2Dataset()
 {
-    PCIDSK2Dataset::FlushCache();
+    PCIDSK2Dataset::FlushCache(true);
 
     while( !apoLayers.empty() )
     {
@@ -1029,10 +1036,10 @@ void PCIDSK2Dataset::ProcessRPC()
 /*                             FlushCache()                             */
 /************************************************************************/
 
-void PCIDSK2Dataset::FlushCache()
+void PCIDSK2Dataset::FlushCache(bool bAtClosing)
 
 {
-    GDALPamDataset::FlushCache();
+    GDALPamDataset::FlushCache(bAtClosing);
 
     if( poFile )
     {
@@ -1066,6 +1073,7 @@ CPLErr PCIDSK2Dataset::SetMetadata( char **papszMD,
 /* -------------------------------------------------------------------- */
     CSLDestroy( papszLastMDListValue );
     papszLastMDListValue = nullptr;
+    m_oCacheMetadataItem.clear();
 
     if( GetAccess() == GA_ReadOnly )
     {
@@ -1118,6 +1126,7 @@ CPLErr PCIDSK2Dataset::SetMetadataItem( const char *pszName,
 /* -------------------------------------------------------------------- */
     CSLDestroy( papszLastMDListValue );
     papszLastMDListValue = nullptr;
+    m_oCacheMetadataItem.clear();
 
     if( GetAccess() == GA_ReadOnly )
     {
@@ -1166,23 +1175,29 @@ const char *PCIDSK2Dataset::GetMetadataItem( const char *pszName,
         return GDALPamDataset::GetMetadataItem( pszName, pszDomain );
 
 /* -------------------------------------------------------------------- */
-/*      Try and fetch.                                                  */
+/*      Try and fetch (use cached value if available)                   */
 /* -------------------------------------------------------------------- */
+    auto oIter = m_oCacheMetadataItem.find(pszName);
+    if( oIter != m_oCacheMetadataItem.end() )
+    {
+        return oIter->second.empty() ? nullptr : oIter->second.c_str();
+    }
+
+    CPLString osValue;
     try
     {
-        osLastMDValue = poFile->GetMetadataValue( pszName );
+        osValue = poFile->GetMetadataValue( pszName );
     }
     catch( const PCIDSKException& ex )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
-                  "%s", ex.what() );
+                "%s", ex.what() );
         return nullptr;
     }
 
-    if( osLastMDValue == "" )
-        return nullptr;
-
-    return osLastMDValue.c_str();
+    oIter = m_oCacheMetadataItem.insert(
+        std::pair<std::string, std::string>(pszName, osValue)).first;
+    return oIter->second.empty() ? nullptr : oIter->second.c_str();
 }
 
 /************************************************************************/
@@ -1499,10 +1514,11 @@ const OGRSpatialReference *PCIDSK2Dataset::GetSpatialRef() const
 /************************************************************************/
 
 CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
-                                        int nOverviews, int *panOverviewList,
-                                        int nListBands, int *panBandList,
+                                        int nOverviews, const int *panOverviewList,
+                                        int nListBands, const int *panBandList,
                                         GDALProgressFunc pfnProgress,
-                                        void *pProgressData )
+                                        void *pProgressData,
+                                        CSLConstList papszOptions )
 
 {
     PCIDSK2Band *poBand = reinterpret_cast<PCIDSK2Band*>(
@@ -1544,7 +1560,8 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
 
         return GDALDataset::IBuildOverviews(
             pszResampling, nOverviews, panOverviewList,
-            nListBands, panBandList, pfnProgress, pProgressData );
+            nListBands, panBandList, pfnProgress, pProgressData,
+            papszOptions);
     }
 
     if( nListBands == 0 )
@@ -1569,6 +1586,7 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
     int nNewOverviews = 0;
     int *panNewOverviewList = reinterpret_cast<int *>(
         CPLCalloc( sizeof( int ), nOverviews ) );
+    std::vector<bool> abFoundOverviewFactor(nOverviews);
     for( int i = 0; i < nOverviews && poBand != nullptr; i++ )
     {
         for( int j = 0; j < poBand->GetOverviewCount(); j++ )
@@ -1584,13 +1602,11 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
                 || nOvFactor == GDALOvLevelAdjust2( panOverviewList[i],
                                                     poBand->GetXSize(),
                                                     poBand->GetYSize() ) )
-                panOverviewList[i] *= -1;
+                abFoundOverviewFactor[i] = true;
         }
 
-        if( panOverviewList[i] > 0 )
+        if( !abFoundOverviewFactor[i] )
             panNewOverviewList[nNewOverviews++] = panOverviewList[i];
-        else
-            panOverviewList[i] *= -1;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1664,11 +1680,12 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
 
         if( nNewOverviews > 0 )
         {
-            eErr = GDALRegenerateOverviews( (GDALRasterBandH) poBand,
+            eErr = GDALRegenerateOverviewsEx( (GDALRasterBandH) poBand,
                                             nNewOverviews,
                                             reinterpret_cast<GDALRasterBandH*>( papoOverviewBands ),
                                             pszResampling,
-                                            pfnProgress, pProgressData );
+                                            pfnProgress, pProgressData,
+                                            papszOptions );
 
             // Mark the regenerated overviews as valid.
             for( int i = 0; i < static_cast<int>( anRegenLevels.size() ); i++ )
@@ -1745,11 +1762,13 @@ GDALDataset *PCIDSK2Dataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Try opening the file.                                           */
 /* -------------------------------------------------------------------- */
     PCIDSKFile *poFile = nullptr;
+    const int nMaxBandCount = atoi(CPLGetConfigOption("GDAL_MAX_BAND_COUNT", "65536"));
     try {
         poFile =
             PCIDSK::Open( poOpenInfo->pszFilename,
                           poOpenInfo->eAccess == GA_ReadOnly ? "r" : "r+",
-                          PCIDSK2GetInterfaces() );
+                          PCIDSK2GetInterfaces(),
+                          nMaxBandCount );
         if( poFile == nullptr )
         {
             CPLError( CE_Failure, CPLE_OpenFailed,
@@ -1825,7 +1844,7 @@ GDALDataset *PCIDSK2Dataset::Open( GDALOpenInfo * poOpenInfo )
 
 GDALDataset *PCIDSK2Dataset::LLOpen( const char *pszFilename,
                                      PCIDSK::PCIDSKFile *poFile,
-                                     GDALAccess eAccess,
+                                     GDALAccess eAccessIn,
                                      char** papszSiblingFiles )
 
 {
@@ -1834,7 +1853,7 @@ GDALDataset *PCIDSK2Dataset::LLOpen( const char *pszFilename,
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
     poDS->poFile = poFile;
-    poDS->eAccess = eAccess;
+    poDS->eAccess = eAccessIn;
     poDS->nRasterXSize = poFile->GetWidth();
     poDS->nRasterYSize = poFile->GetHeight();
 
@@ -1929,7 +1948,7 @@ GDALDataset *PCIDSK2Dataset::LLOpen( const char *pszFilename,
         {
             PCIDSK::PCIDSKVectorSegment* poVecSeg = dynamic_cast<PCIDSK::PCIDSKVectorSegment*>( segobj );
             if( poVecSeg )
-                poDS->apoLayers.push_back( new OGRPCIDSKLayer( segobj, poVecSeg, eAccess == GA_Update ) );
+                poDS->apoLayers.push_back( new OGRPCIDSKLayer( segobj, poVecSeg, eAccessIn == GA_Update ) );
         }
 
 /* -------------------------------------------------------------------- */
@@ -1978,9 +1997,9 @@ GDALDataset *PCIDSK2Dataset::LLOpen( const char *pszFilename,
 /************************************************************************/
 
 GDALDataset *PCIDSK2Dataset::Create( const char * pszFilename,
-                                     int nXSize, int nYSize, int nBands,
+                                     int nXSize, int nYSize, int nBandsIn,
                                      GDALDataType eType,
-                                     char **papszParmList )
+                                     char **papszParamList )
 
 {
 /* -------------------------------------------------------------------- */
@@ -1989,24 +2008,24 @@ GDALDataset *PCIDSK2Dataset::Create( const char * pszFilename,
     std::vector<eChanType> aeChanTypes;
 
     if( eType == GDT_Float32 )
-      aeChanTypes.resize( std::max(1, nBands), CHN_32R );
+      aeChanTypes.resize( std::max(1, nBandsIn), CHN_32R );
     else if( eType == GDT_Int16 )
-        aeChanTypes.resize( std::max(1, nBands), CHN_16S );
+        aeChanTypes.resize( std::max(1, nBandsIn), CHN_16S );
     else if( eType == GDT_UInt16 )
-        aeChanTypes.resize( std::max(1, nBands), CHN_16U );
+        aeChanTypes.resize( std::max(1, nBandsIn), CHN_16U );
     else if( eType == GDT_CInt16 )
-        aeChanTypes.resize( std::max(1, nBands), CHN_C16S );
+        aeChanTypes.resize( std::max(1, nBandsIn), CHN_C16S );
     else if( eType == GDT_CFloat32 )
-        aeChanTypes.resize( std::max(1, nBands), CHN_C32R );
+        aeChanTypes.resize( std::max(1, nBandsIn), CHN_C32R );
     else
-        aeChanTypes.resize( std::max(1, nBands), CHN_8U );
+        aeChanTypes.resize( std::max(1, nBandsIn), CHN_8U );
 
 /* -------------------------------------------------------------------- */
 /*      Reformat options.  Currently no support for jpeg compression    */
 /*      quality.                                                        */
 /* -------------------------------------------------------------------- */
     CPLString osOptions;
-    const char *pszValue = CSLFetchNameValue( papszParmList, "INTERLEAVING" );
+    const char *pszValue = CSLFetchNameValue( papszParamList, "INTERLEAVING" );
     if( pszValue == nullptr )
         pszValue = "BAND";
 
@@ -2014,16 +2033,23 @@ GDALDataset *PCIDSK2Dataset::Create( const char * pszFilename,
 
     if( osOptions == "TILED" )
     {
-        pszValue = CSLFetchNameValue( papszParmList, "TILESIZE" );
+        pszValue = CSLFetchNameValue( papszParamList, "TILESIZE" );
         if( pszValue != nullptr )
             osOptions += pszValue;
 
-        pszValue = CSLFetchNameValue( papszParmList, "COMPRESSION" );
+        pszValue = CSLFetchNameValue( papszParamList, "COMPRESSION" );
         if( pszValue != nullptr )
         {
             osOptions += " ";
             osOptions += pszValue;
         }
+
+        pszValue = CSLFetchNameValue( papszParamList, "TILEVERSION" );
+        if( pszValue != nullptr )
+        {
+            osOptions += " TILEV";
+            osOptions += pszValue;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -2031,12 +2057,12 @@ GDALDataset *PCIDSK2Dataset::Create( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 
     try {
-        if( nBands == 0 )
+        if( nBandsIn == 0 )
         {
             nXSize = 512;
             nYSize = 512;
         }
-        PCIDSKFile *poFile = PCIDSK::Create( pszFilename, nXSize, nYSize, nBands,
+        PCIDSKFile *poFile = PCIDSK::Create( pszFilename, nXSize, nYSize, nBandsIn,
                                              &(aeChanTypes[0]), osOptions,
                                              PCIDSK2GetInterfaces() );
 
@@ -2044,14 +2070,14 @@ GDALDataset *PCIDSK2Dataset::Create( const char * pszFilename,
 /*      Apply band descriptions, if provided as creation options.       */
 /* -------------------------------------------------------------------- */
         for( size_t i = 0;
-             papszParmList != nullptr && papszParmList[i] != nullptr;
+             papszParamList != nullptr && papszParamList[i] != nullptr;
              i++ )
         {
-            if( STARTS_WITH_CI(papszParmList[i], "BANDDESC") )
+            if( STARTS_WITH_CI(papszParamList[i], "BANDDESC") )
             {
-                int nBand = atoi(papszParmList[i] + 8 );
-                const char *pszDescription = strstr(papszParmList[i],"=");
-                if( pszDescription && nBand > 0 && nBand <= nBands )
+                int nBand = atoi(papszParamList[i] + 8 );
+                const char *pszDescription = strstr(papszParamList[i],"=");
+                if( pszDescription && nBand > 0 && nBand <= nBandsIn )
                 {
                     poFile->GetChannel(nBand)->SetDescription( pszDescription+1 );
                 }
@@ -2250,6 +2276,8 @@ void GDALRegister_PCIDSK()
     poDriver->SetDescription( "PCIDSK" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_LAYER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_FIELD, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "PCIDSK Database File" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/pcidsk.html" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
@@ -2270,9 +2298,11 @@ void GDALRegister_PCIDSK()
 "       <Value>JPEG</Value>"
 "   </Option>"
 "   <Option name='TILESIZE' type='int' default='127' description='Tile Size (INTERLEAVING=TILED only)'/>"
+"   <Option name='TILEVERSION' type='int' default='2' description='Tile Version (INTERLEAVING=TILED only)'/>"
 "</CreationOptionList>" );
     poDriver->SetMetadataItem( GDAL_DS_LAYER_CREATIONOPTIONLIST,
                                "<LayerCreationOptionList/>" );
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "OGRSQL SQLITE" );
 
     poDriver->pfnIdentify = PCIDSK2Dataset::Identify;
     poDriver->pfnOpen = PCIDSK2Dataset::Open;
