@@ -46,7 +46,6 @@ constexpr int RECORD_SIZE = 512;
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: isis2dataset.cpp a5d5ed208537a05de4437e97b6a09b7ba44f76c9 2020-03-24 08:27:48 +0100 Kai Pastor $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -64,7 +63,7 @@ class ISIS2Dataset final: public RawDataset
     int         bGotTransform;
     double      adfGeoTransform[6];
 
-    CPLString   osProjection;
+    OGRSpatialReference m_oSRS{};
 
     int parse_label(const char *file, char *keyword, char *value);
     int strstrip(char instr[], char outstr[], int position);
@@ -84,26 +83,23 @@ public:
     virtual ~ISIS2Dataset();
 
     virtual CPLErr GetGeoTransform( double * padfTransform ) override;
-    virtual const char *_GetProjectionRef(void) override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
+    const OGRSpatialReference* GetSpatialRef() const override;
 
     virtual char **GetFileList() override;
 
     static int          Identify( GDALOpenInfo * );
     static GDALDataset *Open( GDALOpenInfo * );
     static GDALDataset *Create( const char * pszFilename,
-                                int nXSize, int nYSize, int nBands,
-                                GDALDataType eType, char ** papszParmList );
+                                int nXSize, int nYSize, int nBandsIn,
+                                GDALDataType eType, char ** papszParamList );
 
     // Write related.
     static int WriteRaster(CPLString osFilename, bool includeLabel, GUIntBig iRecord, GUIntBig iLabelRecords, GDALDataType eType, const char * pszInterleaving);
 
-    static int WriteLabel(CPLString osFilename, CPLString osRasterFile, CPLString sObjectTag, unsigned int nXSize, unsigned int nYSize, unsigned int nBands, GDALDataType eType,
+    static int WriteLabel(CPLString osFilename, CPLString osRasterFile, CPLString sObjectTag, unsigned int nXSize, unsigned int nYSize, unsigned int nBandsIn, GDALDataType eType,
                           GUIntBig iRecords, const char * pszInterleaving, GUIntBig & iLabelRecords, bool bRelaunch=false);
     static int WriteQUBE_Information(VSILFILE *fpLabel, unsigned int iLevel, unsigned int & nWritingBytes,
-                                     unsigned int nXSize, unsigned int nYSize, unsigned int nBands, GDALDataType eType, const char * pszInterleaving);
+                                     unsigned int nXSize, unsigned int nYSize, unsigned int nBandsIn, GDALDataType eType, const char * pszInterleaving);
 
     static unsigned int WriteKeyword(VSILFILE *fpLabel, unsigned int iLevel, CPLString key, CPLString value);
     static unsigned int WriteFormatting(VSILFILE *fpLabel, CPLString data);
@@ -118,6 +114,7 @@ ISIS2Dataset::ISIS2Dataset() :
     fpImage(nullptr),
     bGotTransform(FALSE)
 {
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
@@ -133,7 +130,7 @@ ISIS2Dataset::ISIS2Dataset() :
 ISIS2Dataset::~ISIS2Dataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if( fpImage != nullptr )
         VSIFCloseL( fpImage );
 }
@@ -154,16 +151,14 @@ char **ISIS2Dataset::GetFileList()
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                         GetSpatialRef()                              */
 /************************************************************************/
 
-const char *ISIS2Dataset::_GetProjectionRef()
-
+const OGRSpatialReference* ISIS2Dataset::GetSpatialRef() const
 {
-    if( !osProjection.empty() )
-        return osProjection;
-
-    return GDALPamDataset::_GetProjectionRef();
+    if( !m_oSRS.IsEmpty() )
+        return &m_oSRS;
+    return GDALPamDataset::GetSpatialRef();
 }
 
 /************************************************************************/
@@ -212,12 +207,11 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     VSILFILE *fpQube = poOpenInfo->fpL;
     poOpenInfo->fpL = nullptr;
 
-    ISIS2Dataset *poDS = new ISIS2Dataset();
+    auto poDS = cpl::make_unique<ISIS2Dataset>();
 
     if( ! poDS->oKeywords.Ingest( fpQube, 0 ) )
     {
         VSIFCloseL( fpQube );
-        delete poDS;
         return nullptr;
     }
 
@@ -232,7 +226,7 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     // ^QUBE = "ui31s015.img" - which implies no label or skip value
 
     const char *pszQube = poDS->GetKeyword( "^QUBE" );
-    GUIntBig nQube = 0;
+    int nQube = 0;
     int bByteLocation = FALSE;
     CPLString osTargetFile = poOpenInfo->pszFilename;
 
@@ -282,7 +276,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
                   "*** ISIS 2 cube file has invalid SUFFIX_ITEMS parameters:\n"
                   "*** gdal isis2 driver requires (0, 0, 0), thus no sideplanes or backplanes\n"
                   "found: (%i, %i, %i)\n\n", s_ix, s_iy, s_iz );
-        delete poDS;
         return nullptr;
     }
 
@@ -303,7 +296,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     else {
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "%s layout not supported. Abort\n\n", value);
-        delete poDS;
         return nullptr;
     }
 
@@ -316,7 +308,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     const int record_bytes = atoi(poDS->GetKeyword("RECORD_BYTES"));
     if( record_bytes < 0 )
     {
-        delete poDS;
         return nullptr;
     }
 
@@ -376,7 +367,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Itype of %d is not supported in ISIS 2.",
                   itype);
-        delete poDS;
         return nullptr;
     }
 
@@ -454,6 +444,7 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     CPLDebug("ISIS2","using projection %s", map_proj_name.c_str() );
 
     OGRSpatialReference oSRS;
+    oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     bool bProjectionSet = true;
 
     //Set oSRS projection and parameters
@@ -572,10 +563,7 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         }
 
         // translate back into a projection string.
-        char *pszResult = nullptr;
-        oSRS.exportToWkt( &pszResult );
-        poDS->osProjection = pszResult;
-        CPLFree( pszResult );
+        poDS->m_oSRS = oSRS;
     }
 
 /* END ISIS2 Label Read */
@@ -589,7 +577,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     if( !GDALCheckDatasetDimensions(nCols, nRows) ||
         !GDALCheckBandCount(nBands, false) )
     {
-        delete poDS;
         return nullptr;
     }
 
@@ -613,7 +600,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         CPLError( CE_Failure, CPLE_OpenFailed,
                   "Failed to open %s with write permission.\n%s",
                   osTargetFile.c_str(), VSIStrerror( errno ) );
-        delete poDS;
         return nullptr;
     }
 
@@ -631,7 +617,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         nPixelOffset = nItemSize * nBands;
         if( nPixelOffset > INT_MAX / nBands )
         {
-            delete poDS;
             return nullptr;
         }
         nLineOffset = nPixelOffset * nCols;
@@ -642,7 +627,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         nPixelOffset = nItemSize;
         if( nPixelOffset > INT_MAX / nCols )
         {
-            delete poDS;
             return nullptr;
         }
         nLineOffset = nPixelOffset * nCols;
@@ -654,7 +638,6 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
         if( nPixelOffset > INT_MAX / nBands ||
             nPixelOffset * nBands > INT_MAX / nCols )
         {
-            delete poDS;
             return nullptr;
         }
         nLineOffset = nItemSize * nBands * nCols;
@@ -668,7 +651,7 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
     for( int i = 0; i < poDS->nBands; i++ )
     {
         RawRasterBand *poBand =
-            new RawRasterBand( poDS, i+1, poDS->fpImage,
+            new RawRasterBand( poDS.get(), i+1, poDS->fpImage,
                                nSkipBytes + nBandOffset * i,
                                nPixelOffset, nLineOffset, eDataType,
 #ifdef CPL_LSB
@@ -704,14 +687,7 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
 
         char **papszLines = CSLLoad( pszPrjFile );
 
-        OGRSpatialReference oSRS2;
-        if( oSRS2.importFromESRI( papszLines ) == OGRERR_NONE )
-        {
-            char *pszResult = nullptr;
-            oSRS2.exportToWkt( &pszResult );
-            poDS->osProjection = pszResult;
-            CPLFree( pszResult );
-        }
+        poDS->m_oSRS.importFromESRI( papszLines );
 
         CSLDestroy( papszLines );
     }
@@ -746,9 +722,9 @@ GDALDataset *ISIS2Dataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
+    poDS->oOvManager.Initialize( poDS.get(), poOpenInfo->pszFilename );
 
-    return poDS;
+    return poDS.release();
 }
 
 /************************************************************************/
@@ -835,8 +811,8 @@ void ISIS2Dataset::CleanString( CPLString &osInput )
  */
 
 GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
-                                  int nXSize, int nYSize, int nBands,
-                                  GDALDataType eType, char** papszParmList) {
+                                  int nXSize, int nYSize, int nBandsIn,
+                                  GDALDataType eType, char** papszParamList) {
 
     /* Verify settings. In Isis 2 core pixel values can be represented in
      * three different ways : 1, 2 4, or 8 Bytes */
@@ -852,7 +828,7 @@ GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
         (SAMPLE, BAND, LINE) - Band Interleaved by Line (BIL)
         (BAND, SAMPLE, LINE) - Band Interleaved by Pixel (BIP) */
     const char *pszInterleaving = "(SAMPLE,LINE,BAND)";
-    const char *pszInterleavingParam = CSLFetchNameValue( papszParmList, "INTERLEAVE" );
+    const char *pszInterleavingParam = CSLFetchNameValue( papszParamList, "INTERLEAVE" );
     if ( pszInterleavingParam ) {
         if ( STARTS_WITH_CI(pszInterleavingParam, "bip") )
             pszInterleaving = "(BAND,SAMPLE,LINE)";
@@ -865,7 +841,7 @@ GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
     /* default labeling method is attached */
     bool bAttachedLabelingMethod = true;
     /* check if labeling method is set : check the all three first chars */
-    const char *pszLabelingMethod = CSLFetchNameValue( papszParmList, "LABELING_METHOD" );
+    const char *pszLabelingMethod = CSLFetchNameValue( papszParamList, "LABELING_METHOD" );
     if ( pszLabelingMethod ){
         if ( STARTS_WITH_CI( pszLabelingMethod, "det" /* "detached" */ ) ){
             bAttachedLabelingMethod = false;
@@ -885,7 +861,7 @@ GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
     else
     {
         CPLString sExtension = "cub";
-        const char* pszExtension = CSLFetchNameValue( papszParmList, "IMAGE_EXTENSION" );
+        const char* pszExtension = CSLFetchNameValue( papszParamList, "IMAGE_EXTENSION" );
         if( pszExtension ){
             sExtension = pszExtension;
         }
@@ -903,7 +879,7 @@ GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
         osOutFile = osLabelFile;
     }
 
-    const char *pszObject = CSLFetchNameValue( papszParmList, "OBJECT" );
+    const char *pszObject = CSLFetchNameValue( papszParamList, "OBJECT" );
     CPLString sObject = "QUBE"; // default choice
     if (pszObject) {
         if ( EQUAL( pszObject, "IMAGE") ){
@@ -914,18 +890,18 @@ GDALDataset *ISIS2Dataset::Create(const char* pszFilename,
         }
     }
 
-    GUIntBig iRecords = ISIS2Dataset::RecordSizeCalculation(nXSize, nYSize, nBands, eType);
+    GUIntBig iRecords = ISIS2Dataset::RecordSizeCalculation(nXSize, nYSize, nBandsIn, eType);
     GUIntBig iLabelRecords(2);
 
     CPLDebug("ISIS2","irecord = %i",static_cast<int>(iRecords));
 
     if( bAttachedLabelingMethod )
     {
-        ISIS2Dataset::WriteLabel(osRasterFile, "", sObject, nXSize, nYSize, nBands, eType, iRecords, pszInterleaving, iLabelRecords, true);
+        ISIS2Dataset::WriteLabel(osRasterFile, "", sObject, nXSize, nYSize, nBandsIn, eType, iRecords, pszInterleaving, iLabelRecords, true);
     }
     else
     {
-        ISIS2Dataset::WriteLabel(osLabelFile, osRasterFile, sObject, nXSize, nYSize, nBands, eType, iRecords, pszInterleaving, iLabelRecords);
+        ISIS2Dataset::WriteLabel(osLabelFile, osRasterFile, sObject, nXSize, nYSize, nBandsIn, eType, iRecords, pszInterleaving, iLabelRecords);
     }
 
     if( !ISIS2Dataset::WriteRaster(osRasterFile, bAttachedLabelingMethod,
@@ -983,16 +959,16 @@ int ISIS2Dataset::WriteRaster(CPLString osFilename,
 /************************************************************************/
 /*                       RecordSizeCalculation()                        */
 /************************************************************************/
-GUIntBig ISIS2Dataset::RecordSizeCalculation(unsigned int nXSize, unsigned int nYSize, unsigned int nBands, GDALDataType eType )
+GUIntBig ISIS2Dataset::RecordSizeCalculation(unsigned int nXSize, unsigned int nYSize, unsigned int nBandsIn, GDALDataType eType )
 
 {
-    const GUIntBig n = static_cast<GUIntBig>(nXSize) * nYSize * nBands * (  GDALGetDataTypeSize(eType) / 8);
+    const GUIntBig n = static_cast<GUIntBig>(nXSize) * nYSize * nBandsIn * (  GDALGetDataTypeSize(eType) / 8);
     // size of pds file is a multiple of RECORD_SIZE Bytes.
     CPLDebug("ISIS2","n = %i", static_cast<int>(n));
     CPLDebug("ISIS2","RECORD SIZE = %i", RECORD_SIZE);
     CPLDebug("ISIS2","nXSize = %i", nXSize);
     CPLDebug("ISIS2","nYSize = %i", nYSize);
-    CPLDebug("ISIS2","nBands = %i", nBands);
+    CPLDebug("ISIS2","nBands = %i", nBandsIn);
     CPLDebug("ISIS2","DataTypeSize = %i", GDALGetDataTypeSize(eType));
     return static_cast<GUIntBig>( ceil(static_cast<float>( n ) / RECORD_SIZE) );
 }
@@ -1003,7 +979,7 @@ GUIntBig ISIS2Dataset::RecordSizeCalculation(unsigned int nXSize, unsigned int n
 
 int ISIS2Dataset::WriteQUBE_Information(
     VSILFILE *fpLabel, unsigned int iLevel, unsigned int & nWritingBytes,
-    unsigned int nXSize,  unsigned int nYSize, unsigned int nBands,
+    unsigned int nXSize,  unsigned int nYSize, unsigned int nBandsIn,
     GDALDataType eType, const char * pszInterleaving)
 
 {
@@ -1015,9 +991,9 @@ int ISIS2Dataset::WriteQUBE_Information(
     nWritingBytes += ISIS2Dataset::WriteKeyword( fpLabel, iLevel, "AXIS_NAME", pszInterleaving);
     nWritingBytes += ISIS2Dataset::WriteFormatting( fpLabel, "/* Core description */");
 
-    CPLDebug("ISIS2","%d,%d,%d",nXSize,nYSize,nBands);
+    CPLDebug("ISIS2","%d,%d,%d",nXSize,nYSize,nBandsIn);
 
-    nWritingBytes += ISIS2Dataset::WriteKeyword( fpLabel, iLevel, "CORE_ITEMS",CPLString().Printf("(%d,%d,%d)",nXSize,nYSize,nBands));
+    nWritingBytes += ISIS2Dataset::WriteKeyword( fpLabel, iLevel, "CORE_ITEMS",CPLString().Printf("(%d,%d,%d)",nXSize,nYSize,nBandsIn));
     nWritingBytes += ISIS2Dataset::WriteKeyword( fpLabel, iLevel, "CORE_NAME", "\"RAW DATA NUMBER\"");
     nWritingBytes += ISIS2Dataset::WriteKeyword( fpLabel, iLevel, "CORE_UNIT", "\"N/A\"");
     // TODO change for eType
@@ -1072,7 +1048,7 @@ int ISIS2Dataset::WriteQUBE_Information(
 
 int ISIS2Dataset::WriteLabel(
     CPLString osFilename, CPLString osRasterFile, CPLString sObjectTag,
-    unsigned int nXSize, unsigned int nYSize, unsigned int nBands,
+    unsigned int nXSize, unsigned int nYSize, unsigned int nBandsIn,
     GDALDataType eType,
     GUIntBig iRecords, const char * pszInterleaving,
     GUIntBig &iLabelRecords,
@@ -1115,7 +1091,7 @@ int ISIS2Dataset::WriteLabel(
     }
 
     if(EQUAL(sObjectTag, "QUBE")){
-        ISIS2Dataset::WriteQUBE_Information(fpLabel, iLevel, nWritingBytes, nXSize, nYSize, nBands, eType, pszInterleaving);
+        ISIS2Dataset::WriteQUBE_Information(fpLabel, iLevel, nWritingBytes, nXSize, nYSize, nBandsIn, eType, pszInterleaving);
     }
 
     nWritingBytes += ISIS2Dataset::WriteFormatting( fpLabel, "END");
@@ -1129,7 +1105,7 @@ int ISIS2Dataset::WriteLabel(
         VSIFPrintfL(fpLabel,"%*c", nSpaceBytesToWrite, ' ');
     }else{
         iLabelRecords = q+1;
-        ISIS2Dataset::WriteLabel(osFilename, osRasterFile, sObjectTag, nXSize, nYSize, nBands, eType, iRecords, pszInterleaving, iLabelRecords);
+        ISIS2Dataset::WriteLabel(osFilename, osRasterFile, sObjectTag, nXSize, nYSize, nBandsIn, eType, iRecords, pszInterleaving, iLabelRecords);
     }
     VSIFCloseL( fpLabel );
 
