@@ -222,7 +222,7 @@ private:
     char * pszPrimaryKeyName;
     GBool  bIsFastPK;
     int bHasTriedFetchingPrimaryKeyName;
-    char* pszProjection;
+    mutable OGRSpatialReference m_oSRS{};
     ResolutionStrategy resolutionStrategy;
     WorkingMode nMode;
     OutDBResolution eOutDBResolution{OutDBResolution::SERVER_SIDE};
@@ -324,14 +324,9 @@ public:
     static CPLErr Delete(const char*);
     virtual char      **GetMetadataDomainList() override;
     char ** GetMetadata(const char *) override;
-    const char* _GetProjectionRef() override;
-    CPLErr _SetProjection(const char*) override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
-    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
-        return OldSetProjectionFromSetSpatialRef(poSRS);
-    }
+
+    const OGRSpatialReference* GetSpatialRef() const override;
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
 
     CPLErr SetGeoTransform(double *) override;
     CPLErr GetGeoTransform(double *) override;
