@@ -37,8 +37,7 @@
 #include "ogr_srs_api.h"
 
 #include <cstdlib>
-
-CPL_CVSID("$Id: byndataset.cpp 3e807d4e7b08d81a5e11be242abdb7b9bee9306f 2022-01-21 22:06:02 +0100 Even Rouault $")
+#include <limits>
 
 // Specification at
 // https://www.nrcan.gc.ca/sites/www.nrcan.gc.ca/files/earthsciences/pdf/gpshgrid_e.pdf
@@ -125,11 +124,11 @@ CPLErr BYNRasterBand::SetScale( double dfNewValue )
 /* ==================================================================== */
 /************************************************************************/
 
-BYNDataset::BYNDataset() : 
+BYNDataset::BYNDataset() :
         fpImage(nullptr),
-        pszProjection(nullptr),
         hHeader{0,0,0,0,0,0,0,0,0.0,0,0,0,0,0,0,0,0,0.0,0.0,0,0,0.0,0}
 {
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
@@ -157,8 +156,6 @@ BYNDataset::~BYNDataset()
             CPLError( CE_Failure, CPLE_FileIO, "I/O error" );
         }
     }
-
-    CPLFree( pszProjection );
 }
 
 /************************************************************************/
@@ -224,25 +221,25 @@ int BYNDataset::Identify( GDALOpenInfo *poOpenInfo )
 
     if( hHeader.nScale == 0 )
     {
-        if( ( std::abs( static_cast<GIntBig>( hHeader.nSouth ) - 
+        if( ( std::abs( static_cast<GIntBig>( hHeader.nSouth ) -
                         ( hHeader.nDLat / 2 ) ) > BYN_MAX_LAT ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nNorth ) + 
+            ( std::abs( static_cast<GIntBig>( hHeader.nNorth ) +
                         ( hHeader.nDLat / 2 ) ) > BYN_MAX_LAT ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nWest ) - 
+            ( std::abs( static_cast<GIntBig>( hHeader.nWest ) -
                         ( hHeader.nDLon / 2 ) ) > BYN_MAX_LON ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nEast ) + 
+            ( std::abs( static_cast<GIntBig>( hHeader.nEast ) +
                         ( hHeader.nDLon / 2 ) ) > BYN_MAX_LON ) )
             return FALSE;
     }
     else
     {
-        if( ( std::abs( static_cast<GIntBig>( hHeader.nSouth ) - 
+        if( ( std::abs( static_cast<GIntBig>( hHeader.nSouth ) -
                         ( hHeader.nDLat / 2 ) ) > BYN_MAX_LAT_SCL ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nNorth ) + 
+            ( std::abs( static_cast<GIntBig>( hHeader.nNorth ) +
                         ( hHeader.nDLat / 2 ) ) > BYN_MAX_LAT_SCL ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nWest ) - 
+            ( std::abs( static_cast<GIntBig>( hHeader.nWest ) -
                         ( hHeader.nDLon / 2 ) ) > BYN_MAX_LON_SCL ) ||
-            ( std::abs( static_cast<GIntBig>( hHeader.nEast ) + 
+            ( std::abs( static_cast<GIntBig>( hHeader.nEast ) +
                         ( hHeader.nDLon / 2 ) ) > BYN_MAX_LON_SCL ) )
             return FALSE;
     }
@@ -287,7 +284,7 @@ GDALDataset *BYNDataset::Open( GDALOpenInfo * poOpenInfo )
     double dfDLat  = poDS->hHeader.nDLat;
     double dfDLon  = poDS->hHeader.nDLon;
 
-    if( poDS->hHeader.nScale == 1 ) 
+    if( poDS->hHeader.nScale == 1 )
     {
         dfSouth *= BYN_SCALE;
         dfNorth *= BYN_SCALE;
@@ -340,7 +337,7 @@ GDALDataset *BYNDataset::Open( GDALOpenInfo * poOpenInfo )
     /*********************/
     /* Set data type     */
     /*********************/
- 
+
     GDALDataType eDT = GDT_Unknown;
 
     if ( poDS->hHeader.nSizeOf == 2 )
@@ -363,7 +360,7 @@ GDALDataset *BYNDataset::Open( GDALOpenInfo * poOpenInfo )
 
     BYNRasterBand *poBand = new BYNRasterBand(
         poDS, 1, poDS->fpImage, BYN_HDR_SZ,
-        nDTSize, poDS->nRasterXSize * nDTSize, eDT, 
+        nDTSize, poDS->nRasterXSize * nDTSize, eDT,
         CPL_IS_LSB == bIsLSB );
 
     poDS->SetBand( 1, poBand );
@@ -413,24 +410,21 @@ CPLErr BYNDataset::SetGeoTransform( double * padfTransform )
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
 
-const char *BYNDataset::_GetProjectionRef()
+const OGRSpatialReference* BYNDataset::GetSpatialRef() const
 
 {
-    if( pszProjection )
-        return pszProjection;
-
-    OGRSpatialReference oSRS;
+    if( !m_oSRS.IsEmpty() )
+        return &m_oSRS;
 
     /* Try to use a prefefined EPSG compound CS */
 
-    if( hHeader.nDatum == 1 && hHeader.nVDatum == 2 ) 
+    if( hHeader.nDatum == 1 && hHeader.nVDatum == 2 )
     {
-        oSRS.importFromEPSG( BYN_DATUM_1_VDATUM_2 );
-        oSRS.exportToWkt( &pszProjection );
-        return pszProjection;
+        m_oSRS.importFromEPSG( BYN_DATUM_1_VDATUM_2 );
+        return &m_oSRS;
     }
 
     /* Build the GEOGCS based on Datum ( or Ellipsoid )*/
@@ -438,17 +432,17 @@ const char *BYNDataset::_GetProjectionRef()
     bool bNoGeogCS = false;
 
     if( hHeader.nDatum == 0 )
-        oSRS.importFromEPSG( BYN_DATUM_0 );
+        m_oSRS.importFromEPSG( BYN_DATUM_0 );
     else if( hHeader.nDatum == 1 )
-        oSRS.importFromEPSG( BYN_DATUM_1 );
+        m_oSRS.importFromEPSG( BYN_DATUM_1 );
     else
     {
         /* Build GEOGCS based on Ellipsoid (Table 3) */
 
-        if( hHeader.nEllipsoid > -1 && 
-            hHeader.nEllipsoid < static_cast<GInt16> 
+        if( hHeader.nEllipsoid > -1 &&
+            hHeader.nEllipsoid < static_cast<GInt16>
                                  (CPL_ARRAYSIZE(EllipsoidTable)))
-            oSRS.SetGeogCS( 
+            m_oSRS.SetGeogCS(
                 CPLSPrintf("BYN Ellipsoid(%d)", hHeader.nEllipsoid),
                 "Unspecified",
                 EllipsoidTable[ hHeader.nEllipsoid ].pszName,
@@ -478,51 +472,47 @@ const char *BYNDataset::_GetProjectionRef()
         if( bNoGeogCS )
             return nullptr;
 
-        oSRS.exportToWkt( &pszProjection );
-        return pszProjection;
+        return &m_oSRS;
     }
 
     oSRSVert.importFromEPSG( nVertCS );
 
     /* Create CPMPD_CS with GEOGCS and VERT_CS */
 
-    if( oSRSComp.SetCompoundCS( 
-            CPLSPrintf("BYN Datum(%d) & VDatum(%d)", 
-            hHeader.nDatum, hHeader.nDatum), 
-            &oSRS, 
+    if( oSRSComp.SetCompoundCS(
+            CPLSPrintf("BYN Datum(%d) & VDatum(%d)",
+            hHeader.nDatum, hHeader.nDatum),
+            &m_oSRS,
             &oSRSVert ) == CE_None )
     {
         /* Return COMPD_CS with GEOGCS and VERT_CS */
 
-        oSRSComp.exportToWkt( &pszProjection );
-        return pszProjection;
+        m_oSRS = oSRSComp;
+        m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+        return &m_oSRS;
     }
 
-    return "";
+    return nullptr;
 }
 
 /************************************************************************/
-/*                          SetProjectionRef()                          */
+/*                          SetSpatialRef()                             */
 /************************************************************************/
 
-CPLErr BYNDataset::_SetProjection( const char* pszProjString )
+CPLErr BYNDataset::SetSpatialRef( const OGRSpatialReference* poSRS )
 
 {
-    OGRSpatialReference oSRS;
-
-    OGRErr eOGRErr = oSRS.importFromWkt( pszProjString );
-
-    if( eOGRErr != OGRERR_NONE )
-    {
+    if( poSRS == nullptr )
         return CE_Failure;
-    }
+
+    m_oSRS = *poSRS;
 
     /* Try to recognize prefefined EPSG compound CS */
 
-    if( oSRS.IsCompound() )
+    if( poSRS->IsCompound() )
     {
-        const char* pszAuthName = oSRS.GetAuthorityName( "COMPD_CS" );
-        const char* pszAuthCode = oSRS.GetAuthorityCode( "COMPD_CS" );
+        const char* pszAuthName = poSRS->GetAuthorityName( "COMPD_CS" );
+        const char* pszAuthCode = poSRS->GetAuthorityCode( "COMPD_CS" );
 
         if( pszAuthName != nullptr &&
             pszAuthCode != nullptr &&
@@ -534,40 +524,40 @@ CPLErr BYNDataset::_SetProjection( const char* pszProjString )
             return CE_None;
         }
     }
-    
+
     OGRSpatialReference oSRSTemp;
 
     /* Try to match GEOGCS */
 
-    if( oSRS.IsGeographic() )
+    if( poSRS->IsGeographic() )
     {
         oSRSTemp.importFromEPSG( BYN_DATUM_0 );
-        if( oSRS.IsSameGeogCS( &oSRSTemp ) )
+        if( poSRS->IsSameGeogCS( &oSRSTemp ) )
             hHeader.nDatum = 0;
         else
         {
             oSRSTemp.importFromEPSG( BYN_DATUM_1 );
-            if( oSRS.IsSameGeogCS( &oSRSTemp ) )
+            if( poSRS->IsSameGeogCS( &oSRSTemp ) )
                 hHeader.nDatum = 1;
         }
     }
 
     /* Try to match VERT_CS */
 
-    if( oSRS.IsVertical() )
+    if( poSRS->IsVertical() )
     {
         oSRSTemp.importFromEPSG( BYN_VDATUM_1 );
-        if( oSRS.IsSameVertCS( &oSRSTemp ) )
+        if( poSRS->IsSameVertCS( &oSRSTemp ) )
             hHeader.nVDatum = 1;
         else
         {
             oSRSTemp.importFromEPSG( BYN_VDATUM_2 );
-            if( oSRS.IsSameVertCS( &oSRSTemp ) )
+            if( poSRS->IsSameVertCS( &oSRSTemp ) )
                 hHeader.nVDatum = 2;
             else
             {
                 oSRSTemp.importFromEPSG( BYN_VDATUM_3 );
-                if( oSRS.IsSameVertCS( &oSRSTemp ) )
+                if( poSRS->IsSameVertCS( &oSRSTemp ) )
                     hHeader.nVDatum = 3;
             }
         }
@@ -729,7 +719,7 @@ GDALDataset *BYNDataset::Create( const char * pszFilename,
                                  GDALDataType eType,
                                  char ** /* papszOptions */ )
 {
-    if( eType != GDT_Int16 && 
+    if( eType != GDT_Int16 &&
         eType != GDT_Int32 )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
@@ -814,7 +804,7 @@ void BYNDataset::UpdateHeader()
     double dfSouth = dfNorth - ( ( nRasterYSize - 1 ) * dfDLat );
     double dfEast  = dfWest  + ( ( nRasterXSize - 1 ) * dfDLon );
 
-    if( hHeader.nScale == 1 ) 
+    if( hHeader.nScale == 1 )
     {
         dfSouth /= BYN_SCALE;
         dfNorth /= BYN_SCALE;
@@ -836,43 +826,43 @@ void BYNDataset::UpdateHeader()
     header2buffer( &hHeader, abyBuf );
 
     const char* pszValue = GetMetadataItem("GLOBAL");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nGlobal  = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("TYPE");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nType    = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("DESCRIPTION");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nDescrip = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("SUBTYPE");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nSubType = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("WO");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.dfWo     = CPLAtof( pszValue );
 
     pszValue = GetMetadataItem("GM");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.dfGM     = CPLAtof( pszValue );
 
     pszValue = GetMetadataItem("TIDESYSTEM");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nTideSys = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("REALIZATION");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nRealiz  = static_cast<GInt16>( atoi( pszValue ) );
 
     pszValue = GetMetadataItem("EPOCH");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.dEpoch   = static_cast<float>( CPLAtof( pszValue ) );
 
     pszValue = GetMetadataItem("PTTYPE");
-    if(pszValue != nullptr) 
+    if(pszValue != nullptr)
         hHeader.nPtType  = static_cast<GInt16>( atoi( pszValue ) );
 
     CPL_IGNORE_RET_VAL(VSIFSeekL( fpImage, 0, SEEK_SET ));
