@@ -32,7 +32,6 @@
 #include "ogr_srs_api.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: ctable2dataset.cpp 6e221b5fd9bb196492f09e52fd257158573f93b3 2019-12-28 02:02:25 +0100 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -45,6 +44,7 @@ class CTable2Dataset final: public RawDataset
     VSILFILE    *fpImage;  // image data file.
 
     double      adfGeoTransform[6];
+    OGRSpatialReference m_oSRS{};
 
     CPL_DISALLOW_COPY_ASSIGN(CTable2Dataset)
 
@@ -54,11 +54,8 @@ class CTable2Dataset final: public RawDataset
 
     CPLErr SetGeoTransform( double * padfTransform ) override;
     CPLErr GetGeoTransform( double * padfTransform ) override;
-    const char *_GetProjectionRef() override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
-    void   FlushCache(void) override;
+
+    const OGRSpatialReference* GetSpatialRef() const override { return &m_oSRS; }
 
     static GDALDataset *Open( GDALOpenInfo * );
     static int          Identify( GDALOpenInfo * );
@@ -80,6 +77,9 @@ class CTable2Dataset final: public RawDataset
 CTable2Dataset::CTable2Dataset() :
     fpImage(nullptr)
 {
+    m_oSRS.SetFromUserInput(SRS_WKT_WGS84_LAT_LONG);
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+
     memset( adfGeoTransform, 0, sizeof(adfGeoTransform) );
 }
 
@@ -90,7 +90,7 @@ CTable2Dataset::CTable2Dataset() :
 CTable2Dataset::~CTable2Dataset()
 
 {
-    CTable2Dataset::FlushCache();
+    CTable2Dataset::FlushCache(true);
 
     if( fpImage != nullptr )
     {
@@ -101,16 +101,6 @@ CTable2Dataset::~CTable2Dataset()
     }
 }
 
-/************************************************************************/
-/*                             FlushCache()                             */
-/************************************************************************/
-
-void CTable2Dataset::FlushCache()
-
-{
-    RawDataset::FlushCache();
-}
-
 /************************************************************************/
 /*                              Identify()                              */
 /************************************************************************/
@@ -330,16 +320,6 @@ CPLErr CTable2Dataset::SetGeoTransform( double * padfTransform )
     return CE_None;
 }
 
-/************************************************************************/
-/*                          GetProjectionRef()                          */
-/************************************************************************/
-
-const char *CTable2Dataset::_GetProjectionRef()
-
-{
-    return SRS_WKT_WGS84_LAT_LONG;
-}
-
 /************************************************************************/
 /*                               Create()                               */
 /************************************************************************/
@@ -347,7 +327,7 @@ const char *CTable2Dataset::_GetProjectionRef()
 GDALDataset *CTable2Dataset::Create( const char * pszFilename,
                                      int nXSize,
                                      int nYSize,
-                                     CPL_UNUSED int nBands,
+                                     int /* nBandsIn */,
                                      GDALDataType eType,
                                      char ** papszOptions )
 {
