@@ -31,7 +31,6 @@
 #include "gdal_frmts.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: krodataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 // http://www.autopano.net/wiki-en/Format_KRO
 
@@ -52,8 +51,8 @@ class KRODataset final: public RawDataset
                    ~KRODataset();
 
     static GDALDataset *Open( GDALOpenInfo * );
-    static GDALDataset *Create( const char * pszFilename,
-                                int nXSize, int nYSize, int nBands,
+    static GDALDataset *Create(const char * pszFilename,
+                                int nXSize, int nYSize, int nBandsIn,
                                 GDALDataType eType, char ** papszOptions );
     static int          Identify( GDALOpenInfo * );
 };
@@ -240,7 +239,7 @@ GDALDataset *KRODataset::Open( GDALOpenInfo * poOpenInfo )
 GDALDataset *KRODataset::Create( const char * pszFilename,
                                  int nXSize,
                                  int nYSize,
-                                 int nBands,
+                                 int nBandsIn,
                                  GDALDataType eType,
                                  char ** /* papszOptions */ )
 {
@@ -251,7 +250,7 @@ GDALDataset *KRODataset::Create( const char * pszFilename,
                   GDALGetDataTypeName( eType ) );
         return nullptr;
     }
-    if( nXSize == 0 || nYSize == 0 || nBands == 0 )
+    if( nXSize == 0 || nYSize == 0 || nBandsIn == 0 )
     {
         return nullptr;
     }
@@ -285,7 +284,7 @@ GDALDataset *KRODataset::Create( const char * pszFilename,
     CPL_MSBPTR32(&nTmp);
     nRet += VSIFWriteL(&nTmp, 4, 1, fp);
 
-    nTmp = nBands;
+    nTmp = nBandsIn;
     CPL_MSBPTR32(&nTmp);
     nRet += VSIFWriteL(&nTmp, 4, 1, fp);
 
@@ -296,7 +295,7 @@ GDALDataset *KRODataset::Create( const char * pszFilename,
     CPL_IGNORE_RET_VAL(
         VSIFSeekL( fp,
                    static_cast<vsi_l_offset>(nXSize) * nYSize *
-                   GDALGetDataTypeSizeBytes(eType) * nBands - 1,
+                   GDALGetDataTypeSizeBytes(eType) * nBandsIn - 1,
               SEEK_CUR));
     GByte byNul = 0;
     nRet += VSIFWriteL(&byNul, 1, 1, fp);
