@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: rdbdataset.cpp 95727499e1e118c5eeae7ca5bd5c4774ce32cb8a 2020-03-24 08:33:14 +0100 Kai Pastor $
+ * $Id$
  *
  * Project:  RIEGL RDB 2 driver
  * Purpose:  Add support for reading *.mpx RDB 2 files.
@@ -31,6 +31,8 @@
 
 #include "rdbdataset.hpp"
 
+#include <algorithm>
+#include <limits>
 #include <cmath>
 #include <sstream>
 
@@ -99,14 +101,14 @@ template <typename T> class RDBRasterBandInternal final: public RDBRasterBand
     RDBRasterBandInternal(
         RDBDataset *poDSIn, const std::string &osAttributeNameIn,
         const riegl::rdb::pointcloud::PointAttribute &oPointAttributeIn,
-        int nBandIn, GDALDataType eDataTypeIn, int nLevelIn, int nNumerOfLayers)
+        int nBandIn, GDALDataType eDataTypeIn, int nLevelIn, int nNumberOfLayers)
         : RDBRasterBandInternal(poDSIn, osAttributeNameIn, oPointAttributeIn,
                                 nBandIn, eDataTypeIn, nLevelIn)
     {
-        aoOverviewBands.resize(nNumerOfLayers);
-        poDSIn->apoVRTDataset.resize(nNumerOfLayers);
+        aoOverviewBands.resize(nNumberOfLayers);
+        poDSIn->apoVRTDataset.resize(nNumberOfLayers);
 
-        for(int i = nNumerOfLayers - 2; i >= 0; i--)
+        for(int i = nNumberOfLayers - 2; i >= 0; i--)
         {
             aoOverviewBands[i].reset(new RDBRasterBandInternal<T>(
                 poDSIn, osAttributeNameIn, oPointAttributeIn, nBandIn,
@@ -194,7 +196,7 @@ template <typename T> class RDBRasterBandInternal final: public RDBRasterBand
             // this could maybe be a problem when combining multiple files.
 
             // Using always the maximum or minimum value in such cases might be
-            // at least consistend across multiple files.
+            // at least consistent across multiple files.
             if(pbSuccess != nullptr)
             {
                 *pbSuccess = FALSE;
@@ -336,7 +338,7 @@ void RDBDataset::SetBandInternal(
     RDBDataset *poDs, const std::string &osAttributeName,
     const riegl::rdb::pointcloud::PointAttribute &oPointAttribute,
     riegl::rdb::pointcloud::DataType eRDBDataType, int nLevel,
-    int nNumerOfLayers, int &nBandIndex)
+    int nNumberOfLayers, int &nBandIndex)
 {
     RDBRasterBand *poBand = nullptr;
     // map riegl rdb datatype to gdal data type
@@ -347,37 +349,37 @@ void RDBDataset::SetBandInternal(
     case riegl::rdb::pointcloud::DataType::INT8:
         poBand = new RDBRasterBandInternal<std::uint8_t>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Byte,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::UINT16:
         poBand = new RDBRasterBandInternal<std::uint16_t>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_UInt16,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::INT16:
         poBand = new RDBRasterBandInternal<std::int16_t>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Int16,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::UINT32:
         poBand = new RDBRasterBandInternal<std::uint32_t>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_UInt32,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::INT32:
         poBand = new RDBRasterBandInternal<std::int32_t>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Int32,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::FLOAT32:
         poBand = new RDBRasterBandInternal<float>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Float32,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     case riegl::rdb::pointcloud::DataType::FLOAT64:
         poBand = new RDBRasterBandInternal<double>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Float64,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     default:
         // for all reamining use double. e.g. u/int64_t
@@ -386,7 +388,7 @@ void RDBDataset::SetBandInternal(
         // multiple files.
         poBand = new RDBRasterBandInternal<double>(
             poDs, osAttributeName, oPointAttribute, nBandIndex, GDT_Float64,
-            nLevel, nNumerOfLayers);
+            nLevel, nNumberOfLayers);
         break;
     }
 
