@@ -41,7 +41,6 @@
 
 /*! @cond Doxygen_Suppress */
 
-CPL_CVSID("$Id: vrtderivedrasterband.cpp 33f3d049c42d69081955817eedf05c72339ae2ca 2021-02-25 03:00:58 -0500 Elliott Sales de Andrade $")
 
 using namespace GDALPy;
 
@@ -52,7 +51,7 @@ using namespace GDALPy;
 #define GDAL_VRT_ENABLE_PYTHON_DEFAULT "TRUSTED_MODULES"
 #endif
 
-static std::map<CPLString, GDALDerivedPixelFunc> osMapPixelFunction;
+static std::map<CPLString, std::pair<VRTDerivedRasterBand::PixelFunc, CPLString>> osMapPixelFunction;
 
 /* Flags for getting buffers */
 #define PyBUF_WRITABLE 0x0001
@@ -228,7 +227,7 @@ void VRTDerivedRasterBand::Cleanup()
  * that it will apply, and if a pixel function matching the name is not
  * found the IRasterIO() call will do nothing.
  *
- * @param pszFuncName Name used to access pixel function
+ * @param pszName Name used to access pixel function
  * @param pfnNewFunction Pixel function associated with name.  An
  *  existing pixel function registered with the same name will be
  *  replaced with the new one.
@@ -236,16 +235,63 @@ void VRTDerivedRasterBand::Cleanup()
  * @return CE_None, invalid (NULL) parameters are currently ignored.
  */
 CPLErr CPL_STDCALL
-GDALAddDerivedBandPixelFunc( const char *pszFuncName,
+GDALAddDerivedBandPixelFunc( const char *pszName,
                              GDALDerivedPixelFunc pfnNewFunction )
 {
-    if( pszFuncName == nullptr || pszFuncName[0] == '\0' ||
+    if( pszName == nullptr || pszName[0] == '\0' ||
         pfnNewFunction == nullptr )
     {
       return CE_None;
     }
 
-    osMapPixelFunction[pszFuncName] = pfnNewFunction;
+    osMapPixelFunction[pszName] = {
+        [pfnNewFunction](void **papoSources, int nSources, void *pData,
+                                         int nBufXSize, int nBufYSize,
+                                         GDALDataType eSrcType, GDALDataType eBufType,
+                                         int nPixelSpace, int nLineSpace, CSLConstList papszFunctionArgs) {
+            (void) papszFunctionArgs;
+            return pfnNewFunction(papoSources, nSources, pData, nBufXSize, nBufYSize,
+                                eSrcType, eBufType, nPixelSpace, nLineSpace);
+        },
+        ""
+    };
+
+    return CE_None;
+}
+
+/**
+ * This adds a pixel function to the global list of available pixel
+ * functions for derived bands.  Pixel functions must be registered
+ * in this way before a derived band tries to access data.
+ *
+ * Derived bands are stored with only the name of the pixel function
+ * that it will apply, and if a pixel function matching the name is not
+ * found the IRasterIO() call will do nothing.
+ *
+ * @param pszName Name used to access pixel function
+ * @param pfnNewFunction Pixel function associated with name.  An
+ *  existing pixel function registered with the same name will be
+ *  replaced with the new one.
+ * @param pszMetadata Pixel function metadata (not currently implemented)
+ *
+ * @return CE_None, invalid (NULL) parameters are currently ignored.
+ * @since GDAL 3.4
+ */
+CPLErr CPL_STDCALL
+GDALAddDerivedBandPixelFuncWithArgs(const char *pszName,
+                                    GDALDerivedPixelFuncWithArgs pfnNewFunction,
+                                    const char *pszMetadata)
+{
+    if( pszName == nullptr || pszName[0] == '\0' ||
+        pfnNewFunction == nullptr )
+    {
+        return CE_None;
+    }
+
+    osMapPixelFunction[pszName] = {
+        pfnNewFunction,
+        pszMetadata != nullptr ? pszMetadata : ""
+    };
 
     return CE_None;
 }
@@ -256,9 +302,9 @@ GDALAddDerivedBandPixelFunc( const char *pszFuncName,
  * This adds a pixel function to the global list of available pixel
  * functions for derived bands.
  *
- * This is the same as the c function GDALAddDerivedBandPixelFunc()
+ * This is the same as the C function GDALAddDerivedBandPixelFunc()
  *
- * @param pszFuncName Name used to access pixel function
+ * @param pszFuncNameIn Name used to access pixel function
  * @param pfnNewFunction Pixel function associated with name.  An
  *  existing pixel function registered with the same name will be
  *  replaced with the new one.
@@ -267,9 +313,18 @@ GDALAddDerivedBandPixelFunc( const char *pszFuncName,
  */
 CPLErr
 VRTDerivedRasterBand::AddPixelFunction(
-    const char *pszFuncName, GDALDerivedPixelFunc pfnNewFunction )
+    const char *pszFuncNameIn, GDALDerivedPixelFunc pfnNewFunction )
 {
-    return GDALAddDerivedBandPixelFunc(pszFuncName, pfnNewFunction);
+    return GDALAddDerivedBandPixelFunc(pszFuncNameIn, pfnNewFunction);
+}
+
+CPLErr
+VRTDerivedRasterBand::AddPixelFunction(
+        const char *pszFuncNameIn,
+        GDALDerivedPixelFuncWithArgs pfnNewFunction,
+        const char *pszMetadata)
+{
+    return GDALAddDerivedBandPixelFuncWithArgs(pszFuncNameIn, pfnNewFunction, pszMetadata);
 }
 
 /************************************************************************/
@@ -280,26 +335,25 @@ VRTDerivedRasterBand::AddPixelFunction(
  * Get a pixel function previously registered using the global
  * AddPixelFunction.
  *
- * @param pszFuncName The name associated with the pixel function.
+ * @param pszFuncNameIn The name associated with the pixel function.
  *
  * @return A derived band pixel function, or NULL if none have been
  * registered for pszFuncName.
  */
-GDALDerivedPixelFunc
-VRTDerivedRasterBand::GetPixelFunction( const char *pszFuncName )
+std::pair<VRTDerivedRasterBand::PixelFunc, CPLString>*
+VRTDerivedRasterBand::GetPixelFunction( const char *pszFuncNameIn )
 {
-    if( pszFuncName == nullptr || pszFuncName[0] == '\0' )
+    if( pszFuncNameIn == nullptr || pszFuncNameIn[0] == '\0' )
     {
         return nullptr;
     }
 
-    std::map<CPLString, GDALDerivedPixelFunc>::iterator oIter =
-        osMapPixelFunction.find(pszFuncName);
+    auto oIter = osMapPixelFunction.find(pszFuncNameIn);
 
     if( oIter == osMapPixelFunction.end())
         return nullptr;
 
-    return oIter->second;
+    return &(oIter->second);
 }
 
 /************************************************************************/
@@ -315,6 +369,7 @@ VRTDerivedRasterBand::GetPixelFunction( const char *pszFuncName )
  */
 void VRTDerivedRasterBand::SetPixelFunctionName( const char *pszFuncNameIn )
 {
+    CPLFree(pszFuncName);
     pszFuncName = CPLStrdup( pszFuncNameIn );
 }
 
@@ -692,6 +747,76 @@ bool VRTDerivedRasterBand::InitializePython()
     return true;
 }
 
+CPLErr
+VRTDerivedRasterBand::GetPixelFunctionArguments(
+  const CPLString& osMetadata,
+  std::vector<std::pair<CPLString, CPLString>>& oAdditionalArgs)
+{
+
+    auto poArgs = CPLXMLTreeCloser(CPLParseXMLString(osMetadata));
+    if (poArgs != nullptr && poArgs->eType == CXT_Element &&
+        !strcmp(poArgs->pszValue,
+                 "PixelFunctionArgumentsList"))
+    {
+        for (CPLXMLNode* psIter = poArgs->psChild; psIter != nullptr;
+             psIter = psIter->psNext)
+        {
+            if (psIter->eType == CXT_Element &&
+                !strcmp(psIter->pszValue, "Argument"))
+            {
+                CPLString osName, osType, osValue;
+                auto pszName = CPLGetXMLValue(psIter, "name", nullptr);
+                if (pszName != nullptr)
+                    osName = pszName;
+                auto pszType = CPLGetXMLValue(psIter, "type", nullptr);
+                if (pszType != nullptr)
+                    osType = pszType;
+                auto pszValue = CPLGetXMLValue(psIter, "value", nullptr);
+                if (pszValue != nullptr)
+                    osValue = pszValue;
+                if (osType == "constant" && osValue != "" && osName != "")
+                    oAdditionalArgs.push_back(
+                      std::pair<CPLString, CPLString>(osName, osValue));
+                if (osType == "builtin")
+                {
+                    double dfVal;
+                    int success;
+                    if (osValue == "NoData")
+                        dfVal = this->GetNoDataValue(&success);
+                    else if (osValue == "scale")
+                        dfVal = this->GetScale(&success);
+                    else if (osValue == "offset")
+                        dfVal = this->GetOffset(&success);
+                    else
+                    {
+                        CPLError(CE_Failure,
+                                 CPLE_NotSupported,
+                                 "PixelFunction builtin %s not supported",
+                                 osValue.c_str());
+                        return CE_Failure;
+                    }
+                    if (!success)
+                    {
+                        CPLError(CE_Failure,
+                                 CPLE_AppDefined,
+                                 "Raster has no %s",
+                                 osValue.c_str());
+                        return CE_Failure;
+                    }
+
+                    oAdditionalArgs.push_back(std::pair<CPLString, CPLString>(
+                      osValue, CPLSPrintf("%.18g", dfVal)));
+                    CPLDebug("VRT", "Added builtin pixel function argument %s = %s",
+                           osValue.c_str(),
+                           CPLSPrintf("%.18g", dfVal));
+                }
+            }
+        }
+    }
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                             IRasterIO()                              */
 /************************************************************************/
@@ -775,7 +900,7 @@ CPLErr VRTDerivedRasterBand::IRasterIO( GDALRWFlag eRWFlag,
 /*      Initialize the buffer to some background value. Use the         */
 /*      nodata value if available.                                      */
 /* -------------------------------------------------------------------- */
-    if( bSkipBufferInitialization )
+    if( SkipBufferInitialization() )
     {
         // Do nothing
     }
@@ -812,12 +937,13 @@ CPLErr VRTDerivedRasterBand::IRasterIO( GDALRWFlag eRWFlag,
     }
 
     /* ---- Get pixel function for band ---- */
-    GDALDerivedPixelFunc pfnPixelFunc = nullptr;
+    std::pair<PixelFunc, CPLString> *poPixelFunc = nullptr;
+    std::vector<std::pair<CPLString, CPLString>> oAdditionalArgs;
 
     if( EQUAL(m_poPrivate->m_osLanguage, "C") )
     {
-        pfnPixelFunc = VRTDerivedRasterBand::GetPixelFunction(pszFuncName);
-        if( pfnPixelFunc == nullptr )
+        poPixelFunc = VRTDerivedRasterBand::GetPixelFunction(pszFuncName);
+        if( poPixelFunc == nullptr )
         {
             CPLError( CE_Failure, CPLE_IllegalArg,
                     "VRTDerivedRasterBand::IRasterIO:"
@@ -825,6 +951,15 @@ CPLErr VRTDerivedRasterBand::IRasterIO( GDALRWFlag eRWFlag,
                     this->pszFuncName) ;
             return CE_Failure;
         }
+
+        if (poPixelFunc->second != "")
+        {
+            if (GetPixelFunctionArguments(poPixelFunc->second,
+                                        oAdditionalArgs) != CE_None)
+            {
+                return CE_Failure;
+            }
+        }
     }
 
     /* TODO: It would be nice to use a MallocBlock function for each
@@ -1153,11 +1288,26 @@ CPLErr VRTDerivedRasterBand::IRasterIO( GDALRWFlag eRWFlag,
             VSIFree(pabyTmpBuffer);
         }
     }
-    else if( eErr == CE_None && pfnPixelFunc != nullptr ) {
-        eErr = pfnPixelFunc( static_cast<void **>( pBuffers ), nSources,
-                             pData, nBufXSize, nBufYSize,
-                             eSrcType, eBufType, static_cast<int>(nPixelSpace),
-                             static_cast<int>(nLineSpace) );
+    else if( eErr == CE_None && poPixelFunc != nullptr )
+    {
+        char **papszArgs = nullptr;
+
+        oAdditionalArgs.insert(oAdditionalArgs.end(),
+            m_poPrivate->m_oFunctionArgs.begin(), m_poPrivate->m_oFunctionArgs.end());
+        for (const auto& oArg : oAdditionalArgs)
+        {
+            const char *pszKey = oArg.first.c_str();
+            const char *pszValue = oArg.second.c_str();
+            papszArgs = CSLSetNameValue(papszArgs, pszKey, pszValue);
+        }
+
+        eErr = (poPixelFunc->first)(static_cast<void **>( pBuffers ), nSources,
+                              pData, nBufXSize, nBufYSize,
+                              eSrcType, eBufType, static_cast<int>(nPixelSpace),
+                              static_cast<int>(nLineSpace),
+                              papszArgs);
+
+        CSLDestroy(papszArgs);
     }
 end:
     // Release buffers.
@@ -1191,12 +1341,10 @@ int  VRTDerivedRasterBand::IGetDataCoverageStatus( int /* nXOff */,
 
 CPLErr VRTDerivedRasterBand::XMLInit( CPLXMLNode *psTree,
                                       const char *pszVRTPath,
-                                      void* pUniqueHandle,
                                       std::map<CPLString, GDALDataset*>& oMapSharedSources )
 
 {
     const CPLErr eErr = VRTSourcedRasterBand::XMLInit( psTree, pszVRTPath,
-                                                       pUniqueHandle,
                                                        oMapSharedSources );
     if( eErr != CE_None )
         return eErr;
@@ -1249,12 +1397,6 @@ CPLErr VRTDerivedRasterBand::XMLInit( CPLXMLNode *psTree,
     CPLXMLNode* psArgs = CPLGetXMLNode( psTree, "PixelFunctionArguments" );
     if( psArgs != nullptr )
     {
-        if( !EQUAL(m_poPrivate->m_osLanguage, "Python") )
-        {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "PixelFunctionArguments can only be used with Python");
-            return CE_Failure;
-        }
         for( CPLXMLNode* psIter = psArgs->psChild;
                          psIter != nullptr;
                          psIter = psIter->psNext )
