@@ -37,6 +37,7 @@
 #include <cstring>
 
 #include <algorithm>
+#include <limits>
 #include <map>
 #include <set>
 #include <string>
@@ -54,7 +55,6 @@
 #include "ogr_core.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: vrtpansharpened.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                    GDALCreatePansharpenedVRT()                       */
@@ -1578,8 +1578,7 @@ CPLErr VRTPansharpenedRasterBand::IRasterIO( GDALRWFlag eRWFlag,
             = static_cast<GUIntBig>( nXSize ) * nYSizeToCache * nDataTypeSize
             * psOptions->nOutPansharpenedBands;
         // Check the we don't overflow (for 32 bit platforms)
-        if( static_cast<GUIntBig>( static_cast<size_t>( nBufferSize ) )
-            != nBufferSize )
+        if( nBufferSize > std::numeric_limits<size_t>::max() / 2 )
         {
             CPLError(CE_Failure, CPLE_OutOfMemory,
                  "Out of memory error while allocating working buffers");
