@@ -660,6 +660,7 @@ CPLString GetKeywords(CPLXMLNode *root,
             std::sort(epsg_codes.begin(), epsg_codes.end());
             unsigned int pajazzo = 0, i = 0, a = 0, b = 0;
             while (1) {
+                // cppcheck-suppress containerOutOfBounds
                 unsigned int c = i < epsg_codes.size() ? epsg_codes[i] : 0;
                 if (pajazzo == 1) {
                     if (c > a + 1) {
@@ -779,7 +780,7 @@ bool CRS2Projection(const CPLString &crs, OGRSpatialReference *sr, char **projec
     }
     OGRSpatialReference local_sr;
     OGRSpatialReference *sr_pointer = sr != nullptr ? sr : &local_sr;
-    if (sr_pointer->SetFromUserInput(crs2) == OGRERR_NONE) {
+    if (sr_pointer->SetFromUserInput(crs2, OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS_get()) == OGRERR_NONE) {
         sr_pointer->exportToWkt(projection);
         return true;
     }
