@@ -38,7 +38,6 @@
 #include "gdal.h"
 #include "gdal_priv.h"
 
-CPL_CVSID("$Id: gdalproxydataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /*! @cond Doxygen_Suppress */
 /* ******************************************************************** */
@@ -137,12 +136,13 @@ CPLErr GDALProxyDataset::IRasterIO( GDALRWFlag eRWFlag,
 
 D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, IBuildOverviews,
                         ( const char *pszResampling,
-                          int nOverviews, int *panOverviewList,
-                          int nListBands, int *panBandList,
+                          int nOverviews, const int *panOverviewList,
+                          int nListBands, const int *panBandList,
                           GDALProgressFunc pfnProgress,
-                          void * pProgressData ),
+                          void * pProgressData, CSLConstList papszOptions ),
                         ( pszResampling, nOverviews, panOverviewList,
-                          nListBands, panBandList, pfnProgress, pProgressData ))
+                          nListBands, panBandList, pfnProgress, pProgressData,
+                          papszOptions ))
 
 void  GDALProxyDataset::FlushCache(bool bAtClosing)
 {
@@ -166,9 +166,7 @@ D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, SetMetadataItem,
                         (const char * pszName, const char * pszValue, const char * pszDomain),
                         (pszName, pszValue, pszDomain))
 
-D_PROXY_METHOD_WITH_RET(const char *, nullptr, _GetProjectionRef, (), ())
 D_PROXY_METHOD_WITH_RET(const OGRSpatialReference *, nullptr, GetSpatialRef, () const, ())
-D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, _SetProjection, (const char* pszProjection), (pszProjection))
 D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, SetSpatialRef, (const OGRSpatialReference* poSRS), (poSRS))
 D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, GetGeoTransform, (double* padfGeoTransform), (padfGeoTransform))
 D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, SetGeoTransform, (double* padfGeoTransform), (padfGeoTransform))
@@ -177,13 +175,8 @@ D_PROXY_METHOD_WITH_RET(void *, nullptr, GetInternalHandle, ( const char * arg1)
 D_PROXY_METHOD_WITH_RET(GDALDriver *, nullptr, GetDriver, (), ())
 D_PROXY_METHOD_WITH_RET(char **, nullptr, GetFileList, (), ())
 D_PROXY_METHOD_WITH_RET(int, 0, GetGCPCount, (), ())
-D_PROXY_METHOD_WITH_RET(const char *, nullptr, _GetGCPProjection, (), ())
 D_PROXY_METHOD_WITH_RET(const OGRSpatialReference *, nullptr, GetGCPSpatialRef, () const, ())
 D_PROXY_METHOD_WITH_RET(const GDAL_GCP *, nullptr, GetGCPs, (), ())
-D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, _SetGCPs,
-                        (int nGCPCount, const GDAL_GCP *pasGCPList,
-                         const char *pszGCPProjection),
-                        (nGCPCount, pasGCPList, pszGCPProjection))
 D_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, SetGCPs,
                         (int nGCPCount, const GDAL_GCP *pasGCPList,
                          const OGRSpatialReference *poGCP_SRS),
@@ -390,9 +383,9 @@ RB_PROXY_METHOD_WITH_RET(GDALRasterBand*, nullptr,  GetRasterSampleOverview,
                         (GUIntBig arg1), (arg1))
 
 RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, BuildOverviews,
-                        (const char * arg1, int arg2, int *arg3,
-                        GDALProgressFunc arg4, void * arg5),
-                        (arg1, arg2, arg3, arg4, arg5))
+                        (const char * arg1, int arg2, const int *arg3,
+                        GDALProgressFunc arg4, void * arg5, CSLConstList papszOptions),
+                        (arg1, arg2, arg3, arg4, arg5, papszOptions))
 
 RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, AdviseRead,
                         ( int nXOff, int nYOff, int nXSize, int nYSize,
@@ -429,6 +422,8 @@ RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, SetDefaultRAT,
 RB_PROXY_METHOD_WITH_RET(GDALRasterBand*, nullptr, GetMaskBand, (), ())
 RB_PROXY_METHOD_WITH_RET(int, 0, GetMaskFlags, (), ())
 RB_PROXY_METHOD_WITH_RET(CPLErr, CE_Failure, CreateMaskBand, ( int nFlagsIn ), (nFlagsIn))
+RB_PROXY_METHOD_WITH_RET(bool, false, IsMaskBand, () const, ())
+RB_PROXY_METHOD_WITH_RET(GDALMaskValueRange, GMVR_UNKNOWN, GetMaskValueRange, () const, ())
 
 RB_PROXY_METHOD_WITH_RET(CPLVirtualMem*, nullptr, GetVirtualMemAuto,
                          ( GDALRWFlag eRWFlag, int *pnPixelSpace, GIntBig *pnLineSpace, char **papszOptions ),
@@ -439,7 +434,7 @@ RB_PROXY_METHOD_WITH_RET(CPLVirtualMem*, nullptr, GetVirtualMemAuto,
 /************************************************************************/
 
 void GDALProxyRasterBand::UnrefUnderlyingRasterBand(
-    GDALRasterBand* /* poUnderlyingRasterBand */ )
+    GDALRasterBand* /* poUnderlyingRasterBand */ ) const
 {}
 
 /*! @endcond */
