@@ -44,7 +44,6 @@
 
 //! @cond Doxygen_Suppress
 
-CPL_CVSID("$Id$")
 
 /* We *must* share the same mutex as the gdaldataset.cpp file, as we are */
 /* doing GDALOpen() calls that can indirectly call GDALOpenShared() on */
@@ -650,8 +649,7 @@ GDALProxyPoolDataset::GDALProxyPoolDataset(const char* pszSourceDatasetDescripti
                                    double * padfGeoTransform,
                                    const char* pszOwner):
     responsiblePID(GDALGetResponsiblePIDForCurrentThread()),
-    pszProjectionRef(pszProjectionRefIn ? CPLStrdup(pszProjectionRefIn): nullptr),
-    bHasSrcProjection(pszProjectionRefIn != nullptr)
+    pszProjectionRef(pszProjectionRefIn ? CPLStrdup(pszProjectionRefIn): nullptr)
 {
     GDALDatasetPool::Ref();
 
@@ -914,32 +912,6 @@ const OGRSpatialReference* GDALProxyPoolDataset::GetSpatialRef() const
     }
 }
 
-/************************************************************************/
-/*                        SetProjection()                               */
-/************************************************************************/
-
-CPLErr GDALProxyPoolDataset::_SetProjection(const char* pszProjectionRefIn)
-{
-    bHasSrcProjection = false;
-    return GDALProxyDataset::_SetProjection(pszProjectionRefIn);
-}
-
-/************************************************************************/
-/*                        GetProjectionRef()                            */
-/************************************************************************/
-
-const char *GDALProxyPoolDataset::_GetProjectionRef()
-{
-    if (bHasSrcProjection)
-        return pszProjectionRef;
-    else
-    {
-        CPLFree(pszProjectionRef);
-        pszProjectionRef = CPLStrdup( GDALProxyDataset::_GetProjectionRef() );
-        return pszProjectionRef;
-    }
-}
-
 /************************************************************************/
 /*                        SetGeoTransform()                             */
 /************************************************************************/
@@ -1058,28 +1030,6 @@ const OGRSpatialReference *GDALProxyPoolDataset::GetGCPSpatialRef() const
     return m_poGCPSRS;
 }
 
-/************************************************************************/
-/*                     GetGCPProjection()                               */
-/************************************************************************/
-
-const char *GDALProxyPoolDataset::_GetGCPProjection()
-{
-    GDALDataset* poUnderlyingDataset = RefUnderlyingDataset();
-    if (poUnderlyingDataset == nullptr)
-        return nullptr;
-
-    CPLFree(pszGCPProjection);
-    pszGCPProjection = nullptr;
-
-    const char* pszUnderlyingGCPProjection = poUnderlyingDataset->_GetGCPProjection();
-    if (pszUnderlyingGCPProjection)
-        pszGCPProjection = CPLStrdup(pszUnderlyingGCPProjection);
-
-    UnrefUnderlyingDataset(poUnderlyingDataset);
-
-    return pszGCPProjection;
-}
-
 /************************************************************************/
 /*                            GetGCPs()                                 */
 /************************************************************************/
