@@ -44,6 +44,12 @@ void OSRCleanupTLSContext();
 
 class OSRProjTLSCache
 {
+        struct OSRPJDeleter
+        {
+            void operator()(PJ* pj) const { proj_destroy(pj); }
+        };
+        typedef std::unique_ptr<PJ, OSRPJDeleter> UniquePtrPJ;
+
         struct EPSGCacheKey
         {
             int nCode_;
@@ -70,17 +76,23 @@ class OSRProjTLSCache
             }
         };
 
-        lru11::Cache<EPSGCacheKey, std::shared_ptr<PJ>,
+        PJ_CONTEXT* m_tlsContext = nullptr; // never use it directly. use GetPJContext()
+        lru11::Cache<EPSGCacheKey, UniquePtrPJ,
                      lru11::NullLock,
                       std::unordered_map<
                         EPSGCacheKey,
                         typename std::list<lru11::KeyValuePair<EPSGCacheKey,
-                            std::shared_ptr<PJ>>>::iterator,
+                            UniquePtrPJ>>::iterator,
                             EPSGCacheKeyHasher>> m_oCacheEPSG{};
-        lru11::Cache<std::string, std::shared_ptr<PJ>> m_oCacheWKT{};
+        lru11::Cache<std::string, UniquePtrPJ> m_oCacheWKT{};
+
+        PJ_CONTEXT* GetPJContext();
+
+        OSRProjTLSCache(const OSRProjTLSCache&) = delete;
+        OSRProjTLSCache& operator=(const OSRProjTLSCache&) = delete;
 
     public:
-        OSRProjTLSCache() = default;
+        explicit OSRProjTLSCache(PJ_CONTEXT* tlsContext): m_tlsContext(tlsContext) {}
 
         void clear();
 
