@@ -42,7 +42,6 @@
 #include "ogr_p.h"
 #include "ograpispy.h"
 
-CPL_CVSID("$Id: ogrfeaturedefn.cpp 1e4510d0d88bbf73885b7f18b79f50d5a6696131 2021-08-21 19:26:01 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           OGRFeatureDefn()                           */
@@ -952,7 +951,7 @@ OGRwkbGeometryType OGRFeatureDefn::GetGeomType() const
     if( poGFldDefn == nullptr )
         return wkbNone;
     OGRwkbGeometryType eType = poGFldDefn->GetType();
-    if( eType == (wkbUnknown | wkb25DBitInternalUse) &&
+    if( eType == (/*wkbUnknown |*/ wkb25DBitInternalUse) &&
         CPLTestBool(CPLGetConfigOption("QGIS_HACK", "NO")) )
         eType = wkbUnknown;
     return eType;
@@ -1544,7 +1543,7 @@ std::vector<int> OGRFeatureDefn::ComputeMapForSetFrom( const OGRFeatureDefn* poS
     for( int i = 0; i < nFieldCount; i++ )
     {
         const OGRFieldDefn* poFldDefn = GetFieldDefn(i);
-        assert(poFldDefn); /* Make GCC-8 -Wnull-dereference happy */
+        if( poFldDefn == nullptr ) continue;
         const char* pszName = poFldDefn->GetNameRef();
 
         // In the insane case where there are several matches, arbitrarily
@@ -1561,7 +1560,7 @@ std::vector<int> OGRFeatureDefn::ComputeMapForSetFrom( const OGRFeatureDefn* poS
     for( int i = 0; i < nSrcFieldCount; i++ )
     {
         const OGRFieldDefn* poSrcFldDefn = poSrcFDefn->GetFieldDefn(i);
-        assert(poSrcFldDefn); /* Make GCC-8 -Wnull-dereference happy */
+        if( poSrcFldDefn == nullptr ) continue;
         const char* pszSrcName = poSrcFldDefn->GetNameRef();
 
         auto oIter = oMapNameToTargetFieldIndex.find(pszSrcName);
@@ -1573,7 +1572,7 @@ std::vector<int> OGRFeatureDefn::ComputeMapForSetFrom( const OGRFeatureDefn* poS
                 for( int j = 0; j < nFieldCount; j++ )
                 {
                     const OGRFieldDefn* poFldDefn = GetFieldDefn(j);
-                    assert(poFldDefn); /* Make GCC-8 -Wnull-dereference happy */
+                    if( poFldDefn == nullptr ) continue;
                     oMapNameToTargetFieldIndexUC[
                         CPLString(poFldDefn->GetNameRef()).toupper()] = j;
                 }
