@@ -43,7 +43,6 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ogrgeometrycollection.cpp 3798cbe48457b7127606931896549f26507469db 2021-04-09 15:04:16 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                       OGRGeometryCollection()                        */
@@ -796,8 +795,10 @@ std::string OGRGeometryCollection::exportToWkt(const OGRWktOptions& opts,
 
 
 //! @cond Doxygen_Suppress
-std::string OGRGeometryCollection::exportToWktInternal(const OGRWktOptions& opts,
-    OGRErr *err, std::string exclude) const
+std::string OGRGeometryCollection::exportToWktInternal(
+    const OGRWktOptions& opts,
+    OGRErr *err,
+    const std::string& exclude) const
 {
     bool first = true;
     const size_t excludeSize = exclude.size();
