@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgeogeometry.h b1c9c12ad373e40b955162b45d704070d4ebf7b0 2019-06-19 16:50:15 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements decoder of shapebin geometry for PGeo
@@ -75,7 +75,7 @@
 #define ESRI_LAYERGEOMTYPE_POLYGON       4
 #define ESRI_LAYERGEOMTYPE_MULTIPATCH    9
 
-OGRGeometry* OGRCreateFromMultiPatch( int nParts,
+OGRGeometry CPL_DLL* OGRCreateFromMultiPatch( int nParts,
                                       const GInt32* panPartStart,
                                       const GInt32* panPartType,
                                       int nPoints,
@@ -91,14 +91,14 @@ OGRErr CPL_DLL OGRWriteToShapeBin( const OGRGeometry *poGeom,
                            GByte **ppabyShape,
                            int *pnBytes );
 
-OGRErr OGRCreateMultiPatch( const OGRGeometry *poGeom,
-                            int bAllowSHPTTriangle,
-                            int& nParts,
-                            int*& panPartStart,
-                            int*& panPartType,
-                            int& nPoints,
-                            OGRRawPoint*& poPoints,
-                            double*& padfZ );
+OGRErr CPL_DLL OGRCreateMultiPatch( const OGRGeometry *poGeom,
+                                    int bAllowSHPTTriangle,
+                                    int& nParts,
+                                    int*& panPartStart,
+                                    int*& panPartType,
+                                    int& nPoints,
+                                    OGRRawPoint*& poPoints,
+                                    double*& padfZ );
 
 OGRErr CPL_DLL OGRWriteMultiPatchToShapeBin( const OGRGeometry *poGeom,
                            GByte **ppabyShape,
