@@ -31,7 +31,6 @@
 #include "ogrgeojsonreader.h"
 #include <sstream>
 
-CPL_CVSID("$Id: ogramigoclouddatasource.cpp 18767e3250cfdb60bb8ce468a1819c0686cf0bf1 2021-03-11 11:23:57 +0100 Even Rouault $")
 
 CPLString OGRAMIGOCLOUDGetOptionValue(const char* pszFilename, const char* pszOptionName);
 
@@ -94,6 +93,8 @@ int OGRAmigoCloudDataSource::TestCapability( const char * pszCap )
         return TRUE;
     else if( EQUAL(pszCap,ODsCRandomLayerWrite) )
         return bReadWrite;
+    else if( EQUAL(pszCap,ODsCZGeometries) )
+        return TRUE;
     else
         return FALSE;
 }
@@ -803,6 +804,14 @@ OGRLayer * OGRAmigoCloudDataSource::ExecuteSQL( const char *pszSQLCommand,
                                         const char *pszDialect )
 
 {
+/* -------------------------------------------------------------------- */
+/*      Use generic implementation for recognized dialects              */
+/* -------------------------------------------------------------------- */
+    if( IsGenericSQLDialect(pszDialect) )
+        return GDALDataset::ExecuteSQL( pszSQLCommand,
+                                          poSpatialFilter,
+                                          pszDialect );
+
     return ExecuteSQLInternal(pszSQLCommand, poSpatialFilter, pszDialect, true);
 }
 
