@@ -28,7 +28,6 @@
 
 #include "ogr_amigocloud.h"
 
-CPL_CVSID("$Id: ogramigoclouddriver.cpp 1761acd90777d5bcc49eddbc13c193098f0ed40b 2020-10-01 12:12:00 +0200 Even Rouault $")
 
 extern "C" void RegisterOGRAmigoCloud();
 
@@ -100,6 +99,8 @@ void RegisterOGRAmigoCloud()
     poDriver->SetDescription( "AmigoCloud" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,  "AmigoCloud" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_LAYER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_DELETE_LAYER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/amigocloud.html" );
     poDriver->SetMetadataItem( GDAL_DMD_CONNECTION_PREFIX, "AMIGOCLOUD:" );
 
@@ -122,6 +123,8 @@ void RegisterOGRAmigoCloud()
     poDriver->SetMetadataItem( GDAL_DCAP_NOTNULL_FIELDS, "YES" );
     poDriver->SetMetadataItem( GDAL_DCAP_DEFAULT_FIELDS, "YES" );
     poDriver->SetMetadataItem( GDAL_DCAP_NOTNULL_GEOMFIELDS, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_Z_GEOMETRIES, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "NATIVE OGRSQL SQLITE" );
 
     poDriver->pfnOpen = OGRAmigoCloudDriverOpen;
     poDriver->pfnIdentify = OGRAmigoCloudDriverIdentify;
