@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc_e00gen.cpp 8257eed38341fc286c5ecd94b7a142d5850d4b12 2018-03-01 20:46:13Z Even Rouault $
+ * $Id$
  *
  * Name:     avc_e00gen.c
  * Project:  Arc/Info vector coverage (AVC)  BIN->E00 conversion library
@@ -1063,8 +1063,8 @@ const char *AVCE00GenTx6(AVCE00GenInfo *psInfo, AVCTxt *psTxt, GBool bCont)
 
         psInfo->iCurItem++;
     }
-    else if (psInfo->iCurItem < psInfo->numItems &&
-             psInfo->iCurItem >= psInfo->numItems-((psTxt->numChars-1)/80 + 1))
+    else if (psInfo->iCurItem < psInfo->numItems /* &&
+             psInfo->iCurItem >= psInfo->numItems-((psTxt->numChars-1)/80 + 1) */ )
     {
         /*-------------------------------------------------------------
          * Last line, contains the text string
