@@ -39,7 +39,7 @@ class CADWrapperRasterBand : public GDALProxyRasterBand
   GDALRasterBand* poBaseBand;
 
   protected:
-    virtual GDALRasterBand* RefUnderlyingRasterBand() override { return poBaseBand; }
+    virtual GDALRasterBand* RefUnderlyingRasterBand() const override { return poBaseBand; }
 
   public:
     explicit CADWrapperRasterBand( GDALRasterBand* poBaseBandIn ) :
@@ -161,7 +161,7 @@ int GDALCADDataset::Open( GDALOpenInfo* poOpenInfo, CADFileIO* pFileIO,
         return FALSE;
     }
 
-    OGRSpatialReference *poSpatialRef = GetSpatialReference();
+    const OGRSpatialReference *poSpatialRef = GetSpatialRef();
     int nRasters = 1;
 
     if( nSubRasterLayer != -1 && nSubRasterFID != -1 )
@@ -195,8 +195,11 @@ int GDALCADDataset::Open( GDALOpenInfo* poOpenInfo, CADFileIO* pFileIO,
             if( poOpenInfo->nOpenFlags & GDAL_OF_VECTOR &&
                 oLayer.getGeometryCount() > 0 )
             {
-                papoLayers[nLayers++] = new OGRCADLayer( oLayer, poSpatialRef,
+                OGRSpatialReference* poSRS = poSpatialRef ? poSpatialRef->Clone() : nullptr;
+                papoLayers[nLayers++] = new OGRCADLayer( oLayer, poSRS,
                                                          nEncoding);
+                if( poSRS )
+                    poSRS->Release();
             }
 
             if( poOpenInfo->nOpenFlags & GDAL_OF_RASTER )
@@ -313,6 +316,8 @@ int GDALCADDataset::TestCapability( const char * pszCap )
         return TRUE;
     else if( EQUAL(pszCap,ODsCMeasuredGeometries) )
         return TRUE;
+    else if( EQUAL(pszCap,ODsCZGeometries) )
+        return TRUE;
     return FALSE;
 }
 
@@ -357,7 +362,7 @@ int GDALCADDataset::GetCadEncoding() const
                                         CADHeader::DWGCODEPAGE, 0 ).getDecimal() );
 }
 
-OGRSpatialReference *GDALCADDataset::GetSpatialReference()
+const OGRSpatialReference *GDALCADDataset::GetSpatialRef() const
 {
     if( poSpatialReference )
         return poSpatialReference;
@@ -412,17 +417,10 @@ OGRSpatialReference *GDALCADDataset::GetSpatialReference()
         }
     }
 
-    if( poSpatialReference )
-    {
-        char *pszProjection = nullptr;
-        poSpatialReference->exportToWkt( &pszProjection );
-        soWKT = pszProjection;
-        CPLFree( pszProjection );
-    }
     return poSpatialReference;
 }
 
-const char* GDALCADDataset::GetPrjFilePath()
+const char* GDALCADDataset::GetPrjFilePath() const
 {
     const char * pszPRJFilename = CPLResetExtension( osCADFilename, "prj" );
     if ( CPLCheckForFile( (char*)pszPRJFilename, nullptr ) == TRUE )
@@ -435,11 +433,6 @@ const char* GDALCADDataset::GetPrjFilePath()
     return "";
 }
 
-const char *GDALCADDataset::_GetProjectionRef(void)
-{
-    return soWKT;
-}
-
 CPLErr GDALCADDataset::GetGeoTransform( double* padfGeoTransform )
 {
     memcpy( padfGeoTransform, adfGeoTransform, sizeof(double) * 6 );
