@@ -47,7 +47,6 @@
 #include "ogreditablelayer.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                     OGRCSVEditableLayerSynchronizer                  */
@@ -102,7 +101,7 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
     }
     const char chDelimiter = m_poCSVLayer->GetDelimiter();
     OGRCSVLayer *poCSVTmpLayer = new OGRCSVLayer(
-        osLayerName, nullptr, osTmpFilename, true, true, chDelimiter);
+        osLayerName, nullptr, -1, osTmpFilename, true, true, chDelimiter);
     poCSVTmpLayer->BuildFeatureDefn(nullptr, nullptr, m_papszOpenOptions);
     poCSVTmpLayer->SetCRLF(m_poCSVLayer->GetCRLF());
     poCSVTmpLayer->SetCreateCSVT(bCreateCSVT || bHasCSVT);
@@ -285,7 +284,7 @@ OGRErr OGRCSVEditableLayerSynchronizer::EditableSyncToDisk(
         return OGRERR_FAILURE;
     }
 
-    m_poCSVLayer = new OGRCSVLayer(osLayerName, fp,
+    m_poCSVLayer = new OGRCSVLayer(osLayerName, fp, -1,
                                    osFilename,
                                    false, /* new */
                                    true, /* update */
@@ -443,6 +442,8 @@ int OGRCSVDataSource::TestCapability( const char * pszCap )
         return TRUE;
     else if( EQUAL(pszCap, ODsCMeasuredGeometries) )
         return TRUE;
+    else if( EQUAL(pszCap, ODsCZGeometries) )
+        return TRUE;
     else if( EQUAL(pszCap, ODsCRandomLayerWrite) )
         return bUpdate;
     else
@@ -752,16 +753,31 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
         }
     }
 
+    int nMaxLineSize = atoi(CPLGetConfigOption(
+        "OGR_CSV_MAX_LINE_SIZE",
+        CSLFetchNameValueDef(papszOpenOptionsIn,
+             "MAX_LINE_SIZE", CPLSPrintf("%d", OGR_CSV_DEFAULT_MAX_LINE_SIZE))));
+    size_t nMaxLineSizeAsSize_t = static_cast<size_t>(nMaxLineSize);
+    if( nMaxLineSize == 0 )
+    {
+        nMaxLineSize = -1;
+        nMaxLineSizeAsSize_t = static_cast<size_t>(-1);
+    }
+
     // Read and parse a line.  Did we get multiple fields?
 
-    const char *pszLine = CPLReadLineL(fp);
-    if( pszLine == nullptr )
+    std::string osLine;
     {
-        VSIFCloseL(fp);
-        return false;
+        const char *pszLine = CPLReadLine2L(fp, nMaxLineSize, nullptr);
+        if( pszLine == nullptr )
+        {
+            VSIFCloseL(fp);
+            return false;
+        }
+        osLine = pszLine;
     }
-    char chDelimiter = CSVDetectSeperator(pszLine);
-    if( chDelimiter != '\t' && strchr(pszLine, '\t') != nullptr )
+    char chDelimiter = CSVDetectSeperator(osLine.c_str());
+    if( chDelimiter != '\t' && osLine.find('\t') != std::string::npos )
     {
         // Force the delimiter to be TAB for a .tsv file that has a tabulation
         // in its first line */
@@ -780,7 +796,7 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
                 // of fields, if using tabulation.
                 VSIRewindL(fp);
                 char **papszTokens =
-                    CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE, "\t",
+                    CSVReadParseLine3L(fp, nMaxLineSizeAsSize_t, "\t",
                                        bHonourStrings,
                                        false, // bKeepLeadingAndClosingQuotes
                                        false, // bMergeDelimiter
@@ -789,7 +805,7 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
                 const int nTokens1 = CSLCount(papszTokens);
                 CSLDestroy(papszTokens);
                 papszTokens =
-                    CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE, "\t",
+                    CSVReadParseLine3L(fp, nMaxLineSizeAsSize_t, "\t",
                                        bHonourStrings,
                                        false, // bKeepLeadingAndClosingQuotes
                                        false, // bMergeDelimiter
@@ -832,13 +848,13 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
 #endif
 
     // GNIS specific.
-    if( pszGeonamesGeomFieldPrefix != nullptr && strchr(pszLine, '|') != nullptr )
+    if( pszGeonamesGeomFieldPrefix != nullptr && osLine.find('|') != std::string::npos )
         chDelimiter = '|';
 
     char szDelimiter[2];
     szDelimiter[0] = chDelimiter;
     szDelimiter[1] = 0;
-    char **papszFields = CSVReadParseLine3L(fp, OGR_CSV_MAX_LINE_SIZE,
+    char **papszFields = CSVReadParseLine3L(fp, nMaxLineSizeAsSize_t,
                                             szDelimiter,
                                             true, // bHonourStrings,
                                             false, // bKeepLeadingAndClosingQuotes
@@ -875,7 +891,8 @@ bool OGRCSVDataSource::OpenTable( const char *pszFilename,
     if (EQUAL(pszFilename, "/vsistdin/"))
         osLayerName = "layer";
 
-    OGRCSVLayer *poCSVLayer = new OGRCSVLayer(osLayerName, fp, pszFilename,
+    OGRCSVLayer *poCSVLayer = new OGRCSVLayer(osLayerName, fp, nMaxLineSize,
+                                              pszFilename,
                                               FALSE, bUpdate, chDelimiter);
     poCSVLayer->BuildFeatureDefn(pszNfdcRunwaysGeomField,
                                  pszGeonamesGeomFieldPrefix,
@@ -975,7 +992,8 @@ OGRCSVDataSource::ICreateLayer( const char *pszLayerName,
 
     // Create a layer.
 
-    OGRCSVLayer *poCSVLayer = new OGRCSVLayer(pszLayerName, nullptr, osFilename,
+    OGRCSVLayer *poCSVLayer = new OGRCSVLayer(pszLayerName, nullptr, -1,
+                                              osFilename,
                                               true, true, chDelimiter);
 
     poCSVLayer->BuildFeatureDefn();
