@@ -32,7 +32,6 @@
 #include "cpl_string.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogrdxfreader.cpp 9e5efa394ced27cb93794156bc869c414772527b 2021-08-10 11:51:28 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                            OGRDXFReader()                            */
@@ -71,7 +70,7 @@ void OGRDXFReader::Initialize( VSILFILE *fpIn )
 /*                          ResetReadPointer()                          */
 /************************************************************************/
 
-void OGRDXFReader::ResetReadPointer( int iNewOffset,
+void OGRDXFReader::ResetReadPointer( unsigned int iNewOffset,
     int nNewLineNumber /* = 0 */ )
 
 {
@@ -94,8 +93,6 @@ void OGRDXFReader::ResetReadPointer( int iNewOffset,
 void OGRDXFReader::LoadDiskChunk()
 
 {
-    CPLAssert( iSrcBufferOffset >= 0 );
-
     if( nSrcBufferBytes - iSrcBufferOffset > 511 )
         return;
 
@@ -137,7 +134,7 @@ int OGRDXFReader::ReadValueRaw( char *pszValueBuf, int nValueBufSize )
 /* -------------------------------------------------------------------- */
 /*      Capture the value code, and skip past it.                       */
 /* -------------------------------------------------------------------- */
-    int iStartSrcBufferOffset = iSrcBufferOffset;
+    unsigned int iStartSrcBufferOffset = iSrcBufferOffset;
     int nValueCode = atoi(achSrcBuffer + iSrcBufferOffset);
 
     nLineNumber ++;
@@ -166,7 +163,7 @@ int OGRDXFReader::ReadValueRaw( char *pszValueBuf, int nValueBufSize )
 /* -------------------------------------------------------------------- */
 /*      Capture the value string.                                       */
 /* -------------------------------------------------------------------- */
-    int iEOL = iSrcBufferOffset;
+    unsigned int iEOL = iSrcBufferOffset;
     CPLString osValue;
 
     nLineNumber ++;
@@ -178,10 +175,10 @@ int OGRDXFReader::ReadValueRaw( char *pszValueBuf, int nValueBufSize )
         iEOL++;
 
     bool bLongLine = false;
-    while( achSrcBuffer[iEOL] == '\0' )
+    while( achSrcBuffer[iEOL] == '\0' || (achSrcBuffer[iEOL] == '\r' && achSrcBuffer[iEOL+1] == '\0') )
     {
-        // The line is longer than the buffer. Let's copy what we have so
-        // far into our string, and read more
+        // The line is longer than the buffer (or the line ending is split at end of buffer).
+        // Let's copy what we have so far into our string, and read more
         const auto nValueLength = osValue.length();
 
         if( nValueLength + iEOL - iSrcBufferOffset > 1048576 )
@@ -201,15 +198,15 @@ int OGRDXFReader::ReadValueRaw( char *pszValueBuf, int nValueBufSize )
         iEOL = iSrcBufferOffset;
         bLongLine = true;
 
+        // Have we prematurely reached the end of the file?
+        if( achSrcBuffer[iEOL] == '\0' )
+            return -1;
+
         // Proceed to newline again
         while( achSrcBuffer[iEOL] != '\n'
                && achSrcBuffer[iEOL] != '\r'
                && achSrcBuffer[iEOL] != '\0' )
             iEOL++;
-
-        // If nothing was read, we have prematurely reached the end of the file
-        if( iEOL == iSrcBufferOffset )
-            return -1;
     }
 
     size_t nValueBufLen = 0;
@@ -231,7 +228,7 @@ int OGRDXFReader::ReadValueRaw( char *pszValueBuf, int nValueBufSize )
     }
 
     // Copy the last (normally, the only) section of this line into the buffer
-    if( (iEOL - iSrcBufferOffset) >
+    if( static_cast<int>(iEOL - iSrcBufferOffset) >
         nValueBufSize - static_cast<int>(nValueBufLen) - 1 )
     {
         strncpy( pszValueBuf + nValueBufLen,
