@@ -29,7 +29,6 @@
 #include "ogr_elastic.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrelasticdriver.cpp 6fd775839f9ee5a69fd0c13c9a6e5b25dd9d4bc2 2020-11-17 09:04:12 +0100 Even Rouault $")
 
 /************************************************************************/
 /*                   OGRElasticsearchDriverIdentify()                   */
@@ -95,11 +94,15 @@ void RegisterOGRElastic() {
 
     poDriver->SetDescription( "Elasticsearch" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_LAYER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_DELETE_LAYER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_FIELD, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Elastic Search" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/elasticsearch.html" );
     poDriver->SetMetadataItem( GDAL_DMD_CONNECTION_PREFIX, "ES:" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
                                "<CreationOptionList/>");
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "OGRSQL SQLITE ES" );
 
     poDriver->SetMetadataItem( GDAL_DS_LAYER_CREATIONOPTIONLIST,
     "<LayerCreationOptionList>"
@@ -140,6 +143,7 @@ void RegisterOGRElastic() {
 "  <Option name='USERPWD' type='string' "
         "description='Basic authentication as username:password'/>"
 "  <Option name='LAYER' type='string' description='Index name or index_mapping to use for restricting layer listing'/>"
+"  <Option name='AGGREGATION' type='string' description='JSon serialized description of an aggregation request'/>"
 "  <Option name='BATCH_SIZE' type='integer' description='Number of features to retrieve per batch' default='100'/>"
 "  <Option name='FEATURE_COUNT_TO_ESTABLISH_FEATURE_DEFN' type='integer' description='Number of features to retrieve to establish feature definition. -1 = unlimited' default='100'/>"
 "  <Option name='SINGLE_QUERY_TIMEOUT' type='float' description='Timeout in second for request such as GetFeatureCount() or GetExtent()'/>"
@@ -152,6 +156,7 @@ void RegisterOGRElastic() {
 "  <Option name='BULK_SIZE' type='integer' description='Size in bytes of the buffer for bulk upload' default='1000000'/>"
 "  <Option name='FID' type='string' description='Field name, with integer values, to use as FID' default='ogc_fid'/>"
 "  <Option name='FORWARD_HTTP_HEADERS_FROM_ENV' type='string' description='Comma separated list of http_header_name=env_variable_name'/>"
+"  <Option name='ADD_SOURCE_INDEX_NAME' type='boolean' description='Whether to add the index name as a field for wildcard layers' default='NO'/>"
 "</OpenOptionList>");
 
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONFIELDDATATYPES,
