@@ -1,5 +1,5 @@
 /******************************************************************************
-* $Id: ogr_fgdb.h 5b3261184527657a6ffb2a6ec76055ae84001a09 2022-02-16 23:45:11 +0100 Even Rouault $
+* $Id$
 *
 * Project:  OpenGIS Simple Features Reference Implementation
 * Purpose:  Standard includes and class definitions ArcObjects OGR driver.
@@ -31,10 +31,6 @@
 #ifndef OGR_FGDB_H_INCLUDED
 #define OGR_FGDB_H_INCLUDED
 
-#ifdef DEBUG_BOOL
-#define DO_NOT_USE_DEBUG_BOOL
-#endif
-
 #include <vector>
 #include <set>
 #include "ogrsf_frmts.h"
@@ -61,6 +57,7 @@
 #define FGDB_FEATURE_DATASET "";
 #define FGDB_GEOMETRY_NAME "SHAPE"
 #define FGDB_OID_NAME "OBJECTID"
+constexpr const char* pszRelationshipTypeUUID = "{B606A7E1-FA5B-439C-849C-6E9C2481537B}";
 
 /* The ESRI FGDB API namespace */
 using namespace FileGDBAPI;
@@ -222,6 +219,8 @@ public:
 
   virtual const char* GetMetadataItem(const char* pszName, const char* pszDomain) override;
 
+  virtual OGRErr      Rename(const char* pszNewName) override;
+
 protected:
 
   bool GDBToOGRFields(CPLXMLNode* psFields);
@@ -286,6 +285,7 @@ class FGdbDataSource final: public OGRDataSource
   CPLString             m_osPublicName;
   std::set<OGRLayer*>   m_oSetSelectLayers;
   std::shared_ptr<GDALGroup>     m_poRootGroup{};
+  std::map<std::string,std::unique_ptr<GDALRelationship>> m_osMapRelationships{};
 
   int        FixIndexes();
   int        bPerLayerCopyingForTransaction;
@@ -316,6 +316,20 @@ public:
   int TestCapability( const char * ) override;
 
   const OGRFieldDomain* GetFieldDomain(const std::string& name) const override;
+  std::vector<std::string> GetFieldDomainNames(CSLConstList papszOptions = nullptr) const override;
+
+  bool        AddFieldDomain(std::unique_ptr<OGRFieldDomain>&& domain,
+                             std::string& failureReason) override;
+
+  bool        DeleteFieldDomain(const std::string& name,
+                                std::string& failureReason) override;
+
+  bool        UpdateFieldDomain(std::unique_ptr<OGRFieldDomain>&& domain,
+                                std::string& failureReason) override;
+
+  std::vector<std::string> GetRelationshipNames(CSLConstList papszOptions = nullptr) const override;
+
+  const GDALRelationship* GetRelationship(const std::string& name) const override;
 
   std::shared_ptr<GDALGroup> GetRootGroup() const override { return m_poRootGroup; }
 
