@@ -53,14 +53,9 @@ void GeometryWriter::writePoint(const OGRPoint *p)
 
 void GeometryWriter::writeMultiPoint(const OGRMultiPoint *mp)
 {
-    for (int i = 0; i < mp->getNumGeometries(); i++)
-    {
-        const auto part = mp->getGeometryRef(i);
-        if( !part->IsEmpty() )
-        {
-            writePoint(part->toPoint());
-        }
-    }
+    for (const auto part: *mp)
+        if (!part->IsEmpty())
+            writePoint(part);
 }
 
 uint32_t GeometryWriter::writeSimpleCurve(const OGRSimpleCurve *sc)
@@ -90,15 +85,9 @@ uint32_t GeometryWriter::writeSimpleCurve(const OGRSimpleCurve *sc)
 void GeometryWriter::writeMultiLineString(const OGRMultiLineString *mls)
 {
     uint32_t e = 0;
-    const auto numGeometries = mls->getNumGeometries();
-    for (int i = 0; i < numGeometries; i++)
-    {
-        const auto part = mls->getGeometryRef(i);
-        if( !part->IsEmpty() )
-        {
-            m_ends.push_back(e += writeSimpleCurve(part->toLineString()));
-        }
-    }
+    for (const auto part: *mls)
+        if (!part->IsEmpty())
+            m_ends.push_back(e += writeSimpleCurve(part));
 }
 
 void GeometryWriter::writePolygon(const OGRPolygon *p)
@@ -106,7 +95,7 @@ void GeometryWriter::writePolygon(const OGRPolygon *p)
     const auto exteriorRing = p->getExteriorRing();
     const auto numInteriorRings = p->getNumInteriorRings();
     uint32_t e = writeSimpleCurve(exteriorRing);
-    // NOTE: do not have to write ends if only exterior ring
+    // NOTE: should not write ends if only exterior ring
     if (numInteriorRings > 0) {
         m_ends.push_back(e);
         for (int i = 0; i < numInteriorRings; i++)
@@ -117,80 +106,53 @@ void GeometryWriter::writePolygon(const OGRPolygon *p)
 const Offset<Geometry> GeometryWriter::writeMultiPolygon(const OGRMultiPolygon *mp, int depth)
 {
     std::vector<Offset<Geometry>> parts;
-    for (int i = 0; i < mp->getNumGeometries(); i++) {
-        const auto part = mp->getGeometryRef(i)->toPolygon();
-        if( !part->IsEmpty() )
-        {
-            GeometryWriter writer { m_fbb, part, GeometryType::Polygon, m_hasZ, m_hasM };
-            parts.push_back(writer.write(depth + 1));
-        }
-    }
+    for (const auto part: *mp)
+        if (!part->IsEmpty())
+            parts.push_back(writePart(part, GeometryType::Polygon, depth + 1));
     return CreateGeometryDirect(m_fbb, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, m_geometryType, &parts);
 }
 
 const Offset<Geometry> GeometryWriter::writeGeometryCollection(const OGRGeometryCollection *gc, int depth)
 {
     std::vector<Offset<Geometry>> parts;
-    for (int i = 0; i < gc->getNumGeometries(); i++) {
-        auto part = gc->getGeometryRef(i);
-        if( !part->IsEmpty() )
-        {
-            GeometryWriter writer { m_fbb, part, m_hasZ, m_hasM };
-            parts.push_back(writer.write(depth + 1));
-        }
-    }
+    for (const auto part: *gc)
+        if (!part->IsEmpty())
+            parts.push_back(writePart(part, depth + 1));
     return CreateGeometryDirect(m_fbb, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, m_geometryType, &parts);
 }
 
 const Offset<Geometry> GeometryWriter::writeCompoundCurve(const OGRCompoundCurve *cc, int depth)
 {
     std::vector<Offset<Geometry>> parts;
-    for (int i = 0; i < cc->getNumCurves(); i++) {
-        const auto part = cc->getCurve(i);
-        GeometryWriter writer { m_fbb, part, m_hasZ, m_hasM };
-        parts.push_back(writer.write(depth + 1));
-    }
+    for (const auto curve : *cc)
+        parts.push_back(writePart(curve, depth + 1));
     return CreateGeometryDirect(m_fbb, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, m_geometryType, &parts);
 }
 
 const Offset<Geometry> GeometryWriter::writeCurvePolygon(const OGRCurvePolygon *cp, int depth)
 {
-    std::vector<const OGRCurve *> curves;
     std::vector<Offset<Geometry>> parts;
-    curves.push_back(cp->getExteriorRingCurve());
-    for (int i = 0; i < cp->getNumInteriorRings(); i++)
-        curves.push_back(cp->getInteriorRingCurve(i));
-    for (auto curve : curves) {
-        GeometryWriter writer { m_fbb, curve, m_hasZ, m_hasM };
-        parts.push_back(writer.write(depth + 1));
-    }
+    for (const auto curve : *cp)
+        parts.push_back(writePart(curve, depth + 1));
     return CreateGeometryDirect(m_fbb, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, m_geometryType, &parts);
 }
 
 const Offset<Geometry> GeometryWriter::writePolyhedralSurface(const OGRPolyhedralSurface *p, int depth)
 {
     std::vector<Offset<Geometry>> parts;
-    for (int i = 0; i < p->getNumGeometries(); i++) {
-        auto part = p->getGeometryRef(i);
-        GeometryWriter writer { m_fbb, part, m_hasZ, m_hasM };
-        parts.push_back(writer.write(depth + 1));
-    }
+    for (const auto part: *p)
+        parts.push_back(writePart(part, depth + 1));
     return CreateGeometryDirect(m_fbb, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, m_geometryType, &parts);
 }
 
 void GeometryWriter::writeTIN(const OGRTriangulatedSurface *ts)
 {
-    auto numGeometries = ts->getNumGeometries();
-    if (numGeometries == 1) {
-        auto lr = ts->getGeometryRef(0)->toTriangle()->getExteriorRing();
-        writeSimpleCurve(lr);
-        return;
-    }
+    const auto numGeometries = ts->getNumGeometries();
+    if (numGeometries == 1)
+        return (void) writeSimpleCurve(ts->getGeometryRef(0)->getExteriorRing());
     uint32_t e = 0;
-    for (int i = 0; i < numGeometries; i++) {
-        auto lr = ts->getGeometryRef(i)->toTriangle()->getExteriorRing();
-        m_ends.push_back(e += writeSimpleCurve(lr));
-    }
+    for (const auto part: *ts)
+        m_ends.push_back(e += writeSimpleCurve(part->getExteriorRing()));
 }
 
 const Offset<Geometry> GeometryWriter::write(int depth)
