@@ -38,7 +38,6 @@
 
 //! @cond Doxygen_Suppress
 
-CPL_CVSID("$Id$")
 
 class OGRGenSQLGeomFieldDefn final: public OGRGeomFieldDefn
 {
@@ -293,6 +292,9 @@ OGRGenSQLResultsLayer::OGRGenSQLResultsLayer( GDALDataset *poSrcDSIn,
                 oFDefn.SetWidth( poSrcFDefn->GetWidth() );
                 oFDefn.SetPrecision( poSrcFDefn->GetPrecision() );
             }
+
+            if( psColDef->col_func == SWQCF_NONE )
+                oFDefn.SetDomainName(poSrcFDefn->GetDomainName());
         }
         else if( poSrcGFDefn != nullptr )
         {
@@ -1985,6 +1987,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
         {
             GUIntBig nNewFeaturesAlloc = static_cast<GUIntBig>(nFeaturesAlloc)
                                                         + nFeaturesAlloc / 3;
+#if SIZEOF_SIZE_T == 4
             if( static_cast<size_t>(nNewFeaturesAlloc) != nNewFeaturesAlloc ||
                 static_cast<size_t>(sizeof(OGRField) * nOrderItems *
                                     nNewFeaturesAlloc) !=
@@ -1999,6 +2002,7 @@ void OGRGenSQLResultsLayer::CreateOrderByIndex()
                 delete poSrcFeat;
                 return;
             }
+#endif
             OGRField* pasNewIndexFields = static_cast<OGRField *>(
                 VSI_REALLOC_VERBOSE(pasIndexFields,
                            sizeof(OGRField) * nOrderItems *
