@@ -31,7 +31,6 @@
 #include "ogr_api.h"
 #include "ograpispy.h"
 
-CPL_CVSID("$Id: ogrdatasource.cpp ba3554dcc5a5f63eb7bc2a92a6cbb3f995b729ee 2020-10-21 15:43:49 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           ~OGRDataSource()                           */
@@ -312,7 +311,7 @@ OGRErr OGR_DS_SyncToDisk( OGRDataSourceH hDS )
 {
     VALIDATE_POINTER1( hDS, "OGR_DS_SyncToDisk", OGRERR_INVALID_HANDLE );
 
-    reinterpret_cast<GDALDataset *>(hDS)->FlushCache();
+    reinterpret_cast<GDALDataset *>(hDS)->FlushCache(false);
     if( CPLGetLastErrorType() != 0 )
         return OGRERR_FAILURE;
     else
