@@ -31,7 +31,6 @@
 
 #include <map>
 
-CPL_CVSID("$Id$")
 
 //! @cond Doxygen_Suppress
 
@@ -428,6 +427,22 @@ OGRErr      OGREditableLayer::ICreateFeature( OGRFeature *poFeature )
     return eErr;
 }
 
+/************************************************************************/
+/*                          IUpsertFeature()                            */
+/************************************************************************/
+
+OGRErr      OGREditableLayer::IUpsertFeature( OGRFeature* poFeature )
+{
+   if( GetFeature(poFeature->GetFID()) )
+   {
+      return ISetFeature(poFeature);
+   }
+   else
+   {
+      return ICreateFeature(poFeature);
+   }
+}
+
 /************************************************************************/
 /*                          DeleteFeature()                             */
 /************************************************************************/
@@ -648,6 +663,7 @@ int         OGREditableLayer::TestCapability( const char * pszCap )
         EQUAL(pszCap, OLCDeleteField) ||
         EQUAL(pszCap, OLCReorderFields) ||
         EQUAL(pszCap, OLCAlterFieldDefn) ||
+        EQUAL(pszCap, OLCAlterGeomFieldDefn) ||
         EQUAL(pszCap, OLCDeleteFeature) )
     {
         return m_poDecoratedLayer->TestCapability(OLCCreateField) == TRUE ||
@@ -774,6 +790,29 @@ OGRErr      OGREditableLayer::AlterFieldDefn( int iField,
     return eErr;
 }
 
+/************************************************************************/
+/*                         AlterGeomFieldDefn()                         */
+/************************************************************************/
+
+OGRErr      OGREditableLayer::AlterGeomFieldDefn( int iGeomField,
+                                                  const OGRGeomFieldDefn* poNewGeomFieldDefn,
+                                                  int nFlagsIn )
+{
+    if( !m_poDecoratedLayer ) return OGRERR_FAILURE;
+
+    OGRErr eErr = m_poMemLayer->AlterGeomFieldDefn(iGeomField, poNewGeomFieldDefn, nFlagsIn);
+    if( eErr == OGRERR_NONE )
+    {
+        OGRGeomFieldDefn* poFieldDefn = m_poEditableFeatureDefn->GetGeomFieldDefn(iGeomField);
+        OGRGeomFieldDefn* poMemFieldDefn = m_poMemLayer->GetLayerDefn()->GetGeomFieldDefn(iGeomField);
+        poFieldDefn->SetName(poMemFieldDefn->GetNameRef());
+        poFieldDefn->SetType(poMemFieldDefn->GetType());
+        poFieldDefn->SetNullable(poMemFieldDefn->IsNullable());
+        poFieldDefn->SetSpatialRef(poMemFieldDefn->GetSpatialRef());
+        m_bStructureModified = true;
+    }
+    return eErr;
+}
 /************************************************************************/
 /*                          CreateGeomField()                          */
 /************************************************************************/
