@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrlayerdecorator.h 355b41831cd2685c85d1aabe5b95665a2c6e99b7 2019-06-19 17:07:04 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Defines OGRLayerDecorator class
@@ -64,8 +64,13 @@ class CPL_DLL OGRLayerDecorator : public OGRLayer
     virtual OGRFeature *GetFeature( GIntBig nFID ) override;
     virtual OGRErr      ISetFeature( OGRFeature *poFeature ) override;
     virtual OGRErr      ICreateFeature( OGRFeature *poFeature ) override;
+    virtual OGRErr      IUpsertFeature( OGRFeature* poFeature ) override;
     virtual OGRErr      DeleteFeature( GIntBig nFID ) override;
 
+    virtual GDALDataset* GetDataset() override;
+    virtual bool         GetArrowStream(struct ArrowArrayStream* out_stream,
+                                        CSLConstList papszOptions = nullptr) override;
+
     virtual const char *GetName() override;
     virtual OGRwkbGeometryType GetGeomType() override;
     virtual OGRFeatureDefn *GetLayerDefn() override;
@@ -83,6 +88,7 @@ class CPL_DLL OGRLayerDecorator : public OGRLayer
     virtual OGRErr      DeleteField( int iField ) override;
     virtual OGRErr      ReorderFields( int* panMap ) override;
     virtual OGRErr      AlterFieldDefn( int iField, OGRFieldDefn* poNewFieldDefn, int nFlags ) override;
+    virtual OGRErr      AlterGeomFieldDefn( int iGeomField, const OGRGeomFieldDefn* poNewGeomFieldDefn, int nFlags ) override;
 
     virtual OGRErr      CreateGeomField( OGRGeomFieldDefn *poField,
                                          int bApproxOK = TRUE ) override;
@@ -111,6 +117,7 @@ class CPL_DLL OGRLayerDecorator : public OGRLayer
     virtual CPLErr      SetMetadataItem( const char * pszName,
                                          const char * pszValue,
                                          const char * pszDomain = "" ) override;
+    virtual OGRErr      Rename(const char* pszNewName) override;
 
     OGRLayer* GetBaseLayer() const { return m_poDecoratedLayer; }
 };
