@@ -30,7 +30,6 @@
 
 #include "ogrwarpedlayer.h"
 
-CPL_CVSID("$Id: ogrwarpedlayer.cpp 355b41831cd2685c85d1aabe5b95665a2c6e99b7 2019-06-19 17:07:04 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                          OGRWarpedLayer()                            */
@@ -290,6 +289,25 @@ OGRErr      OGRWarpedLayer::ICreateFeature( OGRFeature *poFeature )
     return eErr;
 }
 
+/************************************************************************/
+/*                            IUpsertFeature()                          */
+/************************************************************************/
+
+OGRErr      OGRWarpedLayer::IUpsertFeature( OGRFeature* poFeature )
+{
+    OGRErr eErr;
+
+    OGRFeature* poFeatureNew = WarpedFeatureToSrcFeature(poFeature);
+    if( poFeatureNew == nullptr )
+        return OGRERR_FAILURE;
+
+    eErr = m_poDecoratedLayer->UpsertFeature(poFeatureNew);
+
+    delete poFeatureNew;
+
+    return eErr;
+}
+
 /************************************************************************/
 /*                            GetLayerDefn()                           */
 /************************************************************************/
