@@ -28,17 +28,16 @@
 
 #include "ogr_geopackage.h"
 
-CPL_CVSID("$Id: ogrgeopackageselectlayer.cpp b55a33407a80673ec314b165c82f47dd02e9dc9c 2020-04-27 20:37:55 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                        OGRGeoPackageSelectLayer()                    */
 /************************************************************************/
 
 OGRGeoPackageSelectLayer::OGRGeoPackageSelectLayer( GDALGeoPackageDataset *poDS,
-                                            CPLString osSQLIn,
+                                            const CPLString& osSQLIn,
                                             sqlite3_stmt *hStmtIn,
-                                            int bUseStatementForGetNextFeature,
-                                            int bEmptyLayer ) :
+                                            bool bUseStatementForGetNextFeature,
+                                            bool bEmptyLayer ) :
     OGRGeoPackageLayer(poDS)
 {
     // Cannot be moved to initializer list because of use of this, which MSVC 2008 doesn't like
@@ -106,12 +105,12 @@ OGRErr OGRGeoPackageSelectLayer::ResetStatement()
     bDoStep = true;
 
 #ifdef DEBUG
-    CPLDebug( "OGR_GPKG", "prepare_v2(%s)", poBehavior->osSQLCurrent.c_str() );
+    CPLDebug( "OGR_GPKG", "prepare_v2(%s)", poBehavior->m_osSQLCurrent.c_str() );
 #endif
 
     const int rc =
-        sqlite3_prepare_v2( m_poDS->GetDB(), poBehavior->osSQLCurrent,
-                         static_cast<int>(poBehavior->osSQLCurrent.size()),
+        sqlite3_prepare_v2( m_poDS->GetDB(), poBehavior->m_osSQLCurrent,
+                         static_cast<int>(poBehavior->m_osSQLCurrent.size()),
                          &m_poQueryStatement, nullptr );
 
     if( rc == SQLITE_OK )
@@ -122,7 +121,7 @@ OGRErr OGRGeoPackageSelectLayer::ResetStatement()
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "In ResetStatement(): sqlite3_prepare_v2(%s):\n  %s",
-                  poBehavior->osSQLCurrent.c_str(), sqlite3_errmsg(m_poDS->GetDB()) );
+                  poBehavior->m_osSQLCurrent.c_str(), sqlite3_errmsg(m_poDS->GetDB()) );
         m_poQueryStatement = nullptr;
         return OGRERR_FAILURE;
     }
