@@ -29,7 +29,6 @@
 #include "ogrgeopackageutility.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrgeopackageutility.cpp 38970 2017-06-08 08:32:05Z rouault $");
 
 /* Requirement 20: A GeoPackage SHALL store feature table geometries */
 /* with the basic simple feature geometry types (Geometry, Point, */
@@ -77,7 +76,15 @@ OGRFieldType GPkgFieldToOGR(const char *pszGpkgType, OGRFieldSubType& eSubType,
 
     /* Integer types */
     if ( STRNCASECMP("INT", pszGpkgType, 3) == 0 )
+    {
+        if( !EQUAL("INT", pszGpkgType) && !EQUAL("INTEGER", pszGpkgType) )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Field format '%s' not supported. "
+                     "Interpreted as INT", pszGpkgType);
+        }
         return OFTInteger64;
+    }
     else if ( EQUAL("MEDIUMINT", pszGpkgType) )
         return OFTInteger;
     else if ( EQUAL("SMALLINT", pszGpkgType) )
@@ -104,16 +111,35 @@ OGRFieldType GPkgFieldToOGR(const char *pszGpkgType, OGRFieldSubType& eSubType,
     else if ( EQUAL("REAL", pszGpkgType) )
         return OFTReal;
 
+    // Only used normally in gpkg_data_column_constraints table, and we
+    // need this only is reading it through ExecuteSQL()
+    else if ( EQUAL("NUMERIC", pszGpkgType) )
+        return OFTReal;
+
     /* String/binary types */
     else if ( STRNCASECMP("TEXT", pszGpkgType, 4) == 0 )
     {
         if( pszGpkgType[4] == '(' )
             nMaxWidth = atoi(pszGpkgType+5);
+        else if( pszGpkgType[4] != '\0' )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Field format '%s' not supported. "
+                     "Interpreted as TEXT", pszGpkgType);
+        }
         return OFTString;
     }
 
     else if ( STRNCASECMP("BLOB", pszGpkgType, 4) == 0 )
+    {
+        if( pszGpkgType[4] != '(' && pszGpkgType[4] != '\0' )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Field format '%s' not supported. "
+                     "Interpreted as BLOB", pszGpkgType);
+        }
         return OFTBinary;
+    }
 
     /* Date types */
     else if ( EQUAL("DATE", pszGpkgType) )
@@ -123,7 +149,14 @@ OGRFieldType GPkgFieldToOGR(const char *pszGpkgType, OGRFieldSubType& eSubType,
 
     /* Illegal! */
     else
-        return (OGRFieldType)(OFTMaxType + 1);
+    {
+        if( GPkgGeometryTypeToWKB(pszGpkgType, false, false) == wkbNone )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined,
+                    "Field format '%s' not supported", pszGpkgType);
+        }
+        return static_cast<OGRFieldType>(OFTMaxType + 1);
+    }
 }
 
 /* Requirement 5: The columns of tables in a GeoPackage SHALL only be */
@@ -204,11 +237,11 @@ const char* GPkgFieldFromOGR(OGRFieldType eType, OGRFieldSubType eSubType,
 GByte* GPkgGeometryFromOGR(const OGRGeometry *poGeometry, int iSrsId,
                            size_t *pnWkbLen)
 {
-    CPLAssert( poGeometry != NULL );
+    CPLAssert( poGeometry != nullptr );
 
     GByte byFlags = 0;
     GByte byEnv = 1;
-    OGRwkbByteOrder eByteOrder = (OGRwkbByteOrder)CPL_IS_LSB;
+    OGRwkbByteOrder eByteOrder = static_cast<OGRwkbByteOrder>(CPL_IS_LSB);
     OGRErr err;
     OGRBoolean bPoint = (wkbFlatten(poGeometry->getGeometryType()) == wkbPoint);
     OGRBoolean bEmpty = poGeometry->IsEmpty();
@@ -225,7 +258,7 @@ GByte* GPkgGeometryFromOGR(const OGRGeometry *poGeometry, int iSrsId,
 
     /* Total BLOB size is header + WKB size */
     size_t nWkbLen = nHeaderLen + poGeometry->WkbSize();
-    GByte *pabyWkb = (GByte *)CPLMalloc(nWkbLen);
+    GByte *pabyWkb = static_cast<GByte *>(CPLMalloc(nWkbLen));
     if (pnWkbLen)
         *pnWkbLen = nWkbLen;
 
@@ -275,7 +308,7 @@ GByte* GPkgGeometryFromOGR(const OGRGeometry *poGeometry, int iSrsId,
     /* Write envelope */
     if ( ! bEmpty && ! bPoint )
     {
-        double *padPtr = (double*)(pabyWkb+8);
+        double *padPtr = reinterpret_cast<double*>(pabyWkb+8);
         if ( iDims == 3 )
         {
             OGREnvelope3D oEnv3d;
@@ -305,7 +338,7 @@ GByte* GPkgGeometryFromOGR(const OGRGeometry *poGeometry, int iSrsId,
     if ( err != OGRERR_NONE )
     {
         CPLFree(pabyWkb);
-        return NULL;
+        return nullptr;
     }
 
     return pabyWkb;
@@ -313,8 +346,8 @@ GByte* GPkgGeometryFromOGR(const OGRGeometry *poGeometry, int iSrsId,
 
 OGRErr GPkgHeaderFromWKB(const GByte *pabyGpkg, size_t nGpkgLen, GPkgHeader *poHeader)
 {
-    CPLAssert( pabyGpkg != NULL );
-    CPLAssert( poHeader != NULL );
+    CPLAssert( pabyGpkg != nullptr );
+    CPLAssert( poHeader != nullptr );
 
     /* Magic (match required) */
     if ( nGpkgLen < 8 ||
@@ -329,7 +362,7 @@ OGRErr GPkgHeaderFromWKB(const GByte *pabyGpkg, size_t nGpkgLen, GPkgHeader *poH
     GByte byFlags = pabyGpkg[3];
     poHeader->bEmpty = (byFlags & (0x01 << 4)) >> 4;
     poHeader->bExtended = (byFlags & (0x01 << 5)) >> 5;
-    poHeader->eByteOrder = (OGRwkbByteOrder)(byFlags & 0x01);
+    poHeader->eByteOrder = static_cast<OGRwkbByteOrder>(byFlags & 0x01);
     poHeader->bExtentHasXY = false;
     poHeader->bExtentHasZ = false;
 #ifdef notdef
@@ -389,7 +422,7 @@ OGRErr GPkgHeaderFromWKB(const GByte *pabyGpkg, size_t nGpkgLen, GPkgHeader *poH
     }
 
     /* Envelope */
-    double *padPtr = (double*)(pabyGpkg+8);
+    const double *padPtr = reinterpret_cast<const double*>(pabyGpkg+8);
     if ( poHeader->bExtentHasXY )
     {
         poHeader->MinX = padPtr[0];
@@ -435,25 +468,25 @@ OGRErr GPkgHeaderFromWKB(const GByte *pabyGpkg, size_t nGpkgLen, GPkgHeader *poH
 
 OGRGeometry* GPkgGeometryToOGR(const GByte *pabyGpkg, size_t nGpkgLen, OGRSpatialReference *poSrs)
 {
-    CPLAssert( pabyGpkg != NULL );
+    CPLAssert( pabyGpkg != nullptr );
 
     GPkgHeader oHeader;
 
     /* Read header */
     OGRErr err = GPkgHeaderFromWKB(pabyGpkg, nGpkgLen, &oHeader);
     if ( err != OGRERR_NONE )
-        return NULL;
+        return nullptr;
 
     /* WKB pointer */
     const GByte *pabyWkb = pabyGpkg + oHeader.nHeaderLen;
     size_t nWkbLen = nGpkgLen - oHeader.nHeaderLen;
 
     /* Parse WKB */
-    OGRGeometry *poGeom = NULL;
-    err = OGRGeometryFactory::createFromWkb((GByte*)pabyWkb, poSrs, &poGeom,
+    OGRGeometry *poGeom = nullptr;
+    err = OGRGeometryFactory::createFromWkb(pabyWkb, poSrs, &poGeom,
                                             static_cast<int>(nWkbLen));
     if ( err != OGRERR_NONE )
-        return NULL;
+        return nullptr;
 
     return poGeom;
 }
