@@ -29,7 +29,6 @@
 #include "ogr_jml.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id$")
 
 #ifdef HAVE_EXPAT
 
@@ -843,7 +842,12 @@ void OGRJMLLayer::endElementLoadSchemaCbk( const char * /* pszName */ )
 int OGRJMLLayer::TestCapability( const char * pszCap )
 
 {
-    return EQUAL(pszCap,OLCStringsAsUTF8);
+    if ( EQUAL(pszCap, OLCStringsAsUTF8) )
+        return true;
+    else if ( EQUAL(pszCap, OLCZGeometries) )
+        return true;
+
+    return false;
 }
 
 #endif /* HAVE_EXPAT */
