@@ -39,7 +39,6 @@
 #include "cpl_error.h"
 #include "ogr_geometry.h"
 
-CPL_CVSID("$Id: kmlnode.cpp b1c9c12ad373e40b955162b45d704070d4ebf7b0 2019-06-19 16:50:15 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           Help functions                             */
@@ -279,7 +278,10 @@ int KMLNode::classify(KML* poKML, int nRecLevel)
         // Compare and return if it is mixed
         if(curr != all && all != Empty && curr != Empty)
         {
-            if (sName_.compare("MultiGeometry") == 0)
+            if (sName_.compare("MultiGeometry") == 0 ||
+                sName_.compare("MultiPolygon") == 0 ||
+                sName_.compare("MultiLineString") == 0 ||
+                sName_.compare("MultiPoint") == 0 )
                 eType_ = MultiGeometry;
             else
                 eType_ = Mixed;
@@ -292,7 +294,10 @@ int KMLNode::classify(KML* poKML, int nRecLevel)
 
     if(eType_ == Unknown)
     {
-        if (sName_.compare("MultiGeometry") == 0)
+        if (sName_.compare("MultiGeometry") == 0 ||
+            sName_.compare("MultiPolygon") == 0 ||
+            sName_.compare("MultiLineString") == 0 ||
+            sName_.compare("MultiPoint") == 0 )
         {
             if (all == Point)
                 eType_ = MultiPoint;
@@ -425,8 +430,7 @@ void KMLNode::addContent(std::string const& text)
 
 void KMLNode::appendContent(std::string const& text)
 {
-    std::string& tmp = pvsContent_->back();
-    tmp += text;
+    pvsContent_->back() += text;
 }
 
 std::string KMLNode::getContent(std::size_t index) const
@@ -459,14 +463,13 @@ int KMLNode::getLayerNumber() const
 
 std::string KMLNode::getNameElement() const
 {
-    kml_nodes_t::size_type subsize = 0;
     const kml_nodes_t::size_type size = pvpoChildren_->size();
 
     for( kml_nodes_t::size_type i = 0; i < size; ++i )
     {
         if( (*pvpoChildren_)[i]->sName_.compare("name") == 0 )
         {
-            subsize = (*pvpoChildren_)[i]->pvsContent_->size();
+            const auto subsize = (*pvpoChildren_)[i]->pvsContent_->size();
             if( subsize > 0 )
             {
                 return (*(*pvpoChildren_)[i]->pvsContent_)[0];
@@ -479,13 +482,12 @@ std::string KMLNode::getNameElement() const
 
 std::string KMLNode::getDescriptionElement() const
 {
-    kml_nodes_t::size_type subsize = 0;
     const kml_nodes_t::size_type size = pvpoChildren_->size();
     for( kml_nodes_t::size_type i = 0; i < size; ++i )
     {
         if( (*pvpoChildren_)[i]->sName_.compare("description") == 0 )
         {
-            subsize = (*pvpoChildren_)[i]->pvsContent_->size();
+            const auto subsize = (*pvpoChildren_)[i]->pvsContent_->size();
             if ( subsize > 0 )
             {
                 return (*(*pvpoChildren_)[i]->pvsContent_)[0];
@@ -690,7 +692,10 @@ OGRGeometry* KMLNode::getGeometry(Nodetype eType)
         if( poLinearRing )
             poGeom->toPolygon()->addRingDirectly(poLinearRing);
     }
-    else if (sName_.compare("MultiGeometry") == 0)
+    else if (sName_.compare("MultiGeometry") == 0  ||
+             sName_.compare("MultiPolygon") == 0 ||
+             sName_.compare("MultiLineString") == 0 ||
+             sName_.compare("MultiPoint") == 0)
     {
         if (eType == MultiPoint)
             poGeom = new OGRMultiPoint();
@@ -778,7 +783,11 @@ Feature* KMLNode::getFeature(std::size_t nNum, int& nLastAsked, int &nLastCount)
 
     for(nCount = 0; nCount < poFeat->pvpoChildren_->size(); nCount++)
     {
-        if((*poFeat->pvpoChildren_)[nCount]->sName_.compare(sElementName) == 0)
+        const auto& sName = (*poFeat->pvpoChildren_)[nCount]->sName_;
+        if(sName.compare(sElementName) == 0 ||
+            (sElementName == "MultiGeometry" &&
+             (sName == "MultiPolygon" || sName == "MultiLineString" ||
+              sName == "MultiPoint")))
         {
             poTemp = (*poFeat->pvpoChildren_)[nCount];
             psReturn->poGeom = poTemp->getGeometry(poFeat->eType_);
