@@ -48,7 +48,6 @@
 #include "ogr_spatialref.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrmemlayer.cpp ee96623c61ea9a3a712e3bd6876be0ca6d7e4c73 2019-08-15 21:24:45 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                      IOGRMemLayerFeatureIterator                     */
@@ -202,16 +201,16 @@ OGRErr OGRMemLayer::SetNextByIndex( GIntBig nIndex )
 }
 
 /************************************************************************/
-/*                             GetFeature()                             */
+/*                         GetFeatureRef()                              */
 /************************************************************************/
 
-OGRFeature *OGRMemLayer::GetFeature( GIntBig nFeatureId )
+const OGRFeature *OGRMemLayer::GetFeatureRef( GIntBig nFeatureId )
 
 {
     if( nFeatureId < 0 )
         return nullptr;
 
-    OGRFeature *poFeature = nullptr;
+    const OGRFeature *poFeature = nullptr;
     if( m_papoFeatures != nullptr )
     {
         if( nFeatureId >= m_nMaxFeatureCount )
@@ -224,10 +223,19 @@ OGRFeature *OGRMemLayer::GetFeature( GIntBig nFeatureId )
         if( oIter != m_oMapFeatures.end() )
             poFeature = oIter->second;
     }
-    if( poFeature == nullptr )
-        return nullptr;
 
-    return poFeature->Clone();
+    return poFeature;
+}
+
+/************************************************************************/
+/*                             GetFeature()                             */
+/************************************************************************/
+
+OGRFeature *OGRMemLayer::GetFeature( GIntBig nFeatureId )
+
+{
+    const OGRFeature* poFeature = GetFeatureRef(nFeatureId);
+    return poFeature ? poFeature->Clone() : nullptr;
 }
 
 /************************************************************************/
@@ -455,6 +463,26 @@ OGRErr OGRMemLayer::ICreateFeature( OGRFeature *poFeature )
     return SetFeature(poFeature);
 }
 
+/************************************************************************/
+/*                           UpsertFeature()                            */
+/************************************************************************/
+
+OGRErr OGRMemLayer::IUpsertFeature(OGRFeature* poFeature)
+
+{
+   if( !TestCapability(OLCUpsertFeature) )
+      return OGRERR_FAILURE;
+
+   if( GetFeatureRef(poFeature->GetFID()) )
+   {
+      return SetFeature(poFeature);
+   }
+   else
+   {
+      return CreateFeature(poFeature);
+   }
+}
+
 /************************************************************************/
 /*                           DeleteFeature()                            */
 /************************************************************************/
@@ -536,14 +564,15 @@ int OGRMemLayer::TestCapability( const char *pszCap )
     else if( EQUAL(pszCap, OLCFastSpatialFilter) )
         return FALSE;
 
-    else if( EQUAL(pszCap, OLCDeleteFeature) )
+    else if( EQUAL(pszCap, OLCDeleteFeature) || EQUAL(pszCap, OLCUpsertFeature) )
         return m_bUpdatable;
 
     else if( EQUAL(pszCap, OLCCreateField) ||
              EQUAL(pszCap, OLCCreateGeomField) ||
              EQUAL(pszCap, OLCDeleteField) ||
              EQUAL(pszCap, OLCReorderFields) ||
-             EQUAL(pszCap, OLCAlterFieldDefn) )
+             EQUAL(pszCap, OLCAlterFieldDefn) ||
+             EQUAL(pszCap, OLCAlterGeomFieldDefn) )
         return m_bUpdatable;
 
     else if( EQUAL(pszCap, OLCFastSetNextByIndex) )
@@ -560,6 +589,9 @@ int OGRMemLayer::TestCapability( const char *pszCap )
     else if( EQUAL(pszCap, OLCMeasuredGeometries) )
         return TRUE;
 
+    else if( EQUAL(pszCap, OLCZGeometries) )
+        return TRUE;
+
     return FALSE;
 }
 
@@ -819,6 +851,74 @@ OGRErr OGRMemLayer::AlterFieldDefn( int iField, OGRFieldDefn *poNewFieldDefn,
     return OGRERR_NONE;
 }
 
+/************************************************************************/
+/*                         AlterGeomFieldDefn()                         */
+/************************************************************************/
+
+OGRErr OGRMemLayer::AlterGeomFieldDefn( int iGeomField,
+                                        const OGRGeomFieldDefn* poNewGeomFieldDefn,
+                                        int nFlagsIn )
+{
+    if( !m_bUpdatable )
+        return OGRERR_FAILURE;
+
+    if( iGeomField < 0 || iGeomField >= m_poFeatureDefn->GetGeomFieldCount() )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported, "Invalid field index");
+        return OGRERR_FAILURE;
+    }
+
+    auto poFieldDefn = m_poFeatureDefn->GetGeomFieldDefn(iGeomField);
+
+    if( nFlagsIn & ALTER_GEOM_FIELD_DEFN_NAME_FLAG )
+        poFieldDefn->SetName(poNewGeomFieldDefn->GetNameRef());
+    if( nFlagsIn & ALTER_GEOM_FIELD_DEFN_TYPE_FLAG )
+    {
+        if( poNewGeomFieldDefn->GetType() == wkbNone )
+            return OGRERR_FAILURE;
+        poFieldDefn->SetType(poNewGeomFieldDefn->GetType());
+    }
+    if( nFlagsIn & ALTER_GEOM_FIELD_DEFN_NULLABLE_FLAG )
+        poFieldDefn->SetNullable(poNewGeomFieldDefn->IsNullable());
+
+    if( nFlagsIn & ALTER_GEOM_FIELD_DEFN_SRS_FLAG )
+    {
+        OGRSpatialReference* poSRSNew = nullptr;
+        const auto poSRSNewRef = poNewGeomFieldDefn->GetSpatialRef();
+        if( poSRSNewRef )
+        {
+            poSRSNew = poSRSNewRef->Clone();
+            if( (nFlagsIn & ALTER_GEOM_FIELD_DEFN_SRS_COORD_EPOCH_FLAG) == 0 )
+            {
+                const auto poSRSOld = poFieldDefn->GetSpatialRef();
+                if( poSRSOld )
+                    poSRSNew->SetCoordinateEpoch(poSRSOld->GetCoordinateEpoch());
+                else
+                    poSRSNew->SetCoordinateEpoch(0);
+            }
+        }
+        poFieldDefn->SetSpatialRef(poSRSNew);
+        if( poSRSNew )
+            poSRSNew->Release();
+    }
+    else if( nFlagsIn & ALTER_GEOM_FIELD_DEFN_SRS_COORD_EPOCH_FLAG )
+    {
+        const auto poSRSOld = poFieldDefn->GetSpatialRef();
+        const auto poSRSNewRef = poNewGeomFieldDefn->GetSpatialRef();
+        if( poSRSOld && poSRSNewRef )
+        {
+            auto poSRSNew = poSRSOld->Clone();
+            poSRSNew->SetCoordinateEpoch(poSRSNewRef->GetCoordinateEpoch());
+            poFieldDefn->SetSpatialRef(poSRSNew);
+            poSRSNew->Release();
+        }
+    }
+
+    m_bUpdated = true;
+
+    return OGRERR_NONE;
+}
+
 /************************************************************************/
 /*                          CreateGeomField()                           */
 /************************************************************************/
