@@ -36,12 +36,12 @@
 #include <cstddef>
 
 #include "cpl_conv.h"
+#include "cpl_csv.h"
 #include "cpl_error.h"
 #include "cpl_string.h"
 #include "cpl_vsi.h"
 #include "mitab_priv.h"
 
-CPL_CVSID("$Id: mitab_middatafile.cpp b4f8e2941b9293c66cb4646cea332d5fb132405a 2021-11-02 19:25:58 +0100 Even Rouault $")
 
 /*=====================================================================
  *                      class MIDDATAFile
@@ -209,6 +209,32 @@ const char *MIDDATAFile::GetLastLine()
     return nullptr;
 }
 
+char** MIDDATAFile::GetTokenizedNextLine()
+{
+    static const int nMaxLineLength = atoi(
+        CPLGetConfigOption("MITAB_MAX_LINE_LENGTH", "1000000"));
+    char** papszTokens = CSVReadParseLine3L( m_fp,
+                                             nMaxLineLength,
+                                             m_pszDelimiter,
+                                             true, // bHonourStrings
+                                             false, // bKeepLeadingAndClosingQuotes
+                                             false, // bMergeDelimiter
+                                             false // bSkipBOM
+                                            );
+    if( papszTokens == nullptr )
+    {
+        if( strstr(CPLGetLastErrorMsg(), "Maximum number of characters allowed reached") )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Maximum number of characters allowed reached. "
+                     "You can set the MITAB_MAX_LINE_LENGTH configuration option "
+                     "to the desired number of bytes (or -1 for unlimited)");
+        }
+        SetEof(TRUE);
+    }
+    return papszTokens;
+}
+
 void MIDDATAFile::WriteLine(const char *pszFormat, ...)
 {
     va_list args;
