@@ -47,7 +47,6 @@
 #include "ogr_spatialref.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: mitab_tabseamless.cpp 7506db93e683abe039fb754a3977ab029235a1ed 2019-08-15 21:29:31 +0200 Even Rouault $")
 
 /*=====================================================================
  *                      class TABSeamless
@@ -203,7 +202,7 @@ int TABSeamless::OpenForRead(const char *pszFname,
      * but we do not support it in this class.
      *------------------------------------------------------------*/
     GBool bSeamlessFound = FALSE;
-    for (int i=0; !bSeamlessFound && papszTABFile && papszTABFile[i]; i++)
+    for (int i=0; !bSeamlessFound && papszTABFile[i]; i++)
     {
         const char *pszStr = papszTABFile[i];
         while(*pszStr != '\0' && isspace(static_cast<unsigned char>(*pszStr)))
