@@ -50,7 +50,6 @@
 #include "ogr_geometry.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: mitab_tabview.cpp 8ca42e1b9c2e54b75d35e49885df9789a2643aa4 2020-05-17 21:43:40 +0200 Even Rouault $")
 
 /*=====================================================================
  *                      class TABView
@@ -209,7 +208,7 @@ int TABView::OpenForRead(const char *pszFname,
      *------------------------------------------------------------*/
     GBool bCreateViewFound = FALSE;
     for (int i=0;
-         !bCreateViewFound && m_papszTABFile && m_papszTABFile[i];
+         !bCreateViewFound && m_papszTABFile[i];
          i++)
     {
         const char *pszStr = m_papszTABFile[i];
