@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_mssqlspatial.h 3f43e75e42f950a2a864d3d46ae1544cf5adb738 2021-12-11 12:39:35 +0100 Even Rouault $
+ * $Id$
  *
  * Project:  MSSQL Spatial driver
  * Purpose:  Definition of classes for OGR MSSQL Spatial driver.
@@ -38,7 +38,7 @@
 #include <sqlncli.h>
 #endif
 #ifdef MSODBCSQL_VERSION
-#include <msodbcsql.h>
+#include "include_msodbcsql.h"
 #endif
 
 class OGRMSSQLSpatialDataSource;
@@ -329,7 +329,7 @@ class OGRMSSQLSpatialLayer CPL_NON_FINAL: public OGRLayer
     virtual const char *GetGeometryColumn() override;
 
     virtual int         TestCapability( const char * ) override;
-    char*               GByteArrayToHexString( const GByte* pabyData, int nLen);
+    static char*        GByteArrayToHexString( const GByte* pabyData, int nLen);
 
     void               SetLayerStatus( int nStatus ) { nLayerStatus = nStatus; }
     int                GetLayerStatus() { return nLayerStatus; }
@@ -589,7 +589,7 @@ public:
                                     const char *pszDialect ) override;
     virtual void        ReleaseResultSet( OGRLayer * poLayer ) override;
 
-    char                *LaunderName( const char *pszSrcName );
+    static char        *LaunderName( const char *pszSrcName );
     OGRErr              InitializeMetadataTables();
 
     OGRSpatialReference* FetchSRS( int nId );
