@@ -33,7 +33,6 @@
 
 #include <memory>
 
-CPL_CVSID("$Id: ogrmssqlspatialtablelayer.cpp 9be722d4162edd5087a9c2d57ccf3ab310e1c282 2021-12-14 12:23:32 +0100 Even Rouault $")
 
 #define UNSUPPORTED_OP_READ_ONLY "%s : unsupported operation on a read-only datasource."
 
@@ -828,6 +827,8 @@ int OGRMSSQLSpatialTableLayer::TestCapability( const char * pszCap )
         return TRUE;
     else if (EQUAL(pszCap, OLCMeasuredGeometries))
         return TRUE;
+    else if (EQUAL(pszCap, OLCZGeometries))
+        return TRUE;
     else
         return OGRMSSQLSpatialLayer::TestCapability( pszCap );
 }
@@ -1388,7 +1389,7 @@ int OGRMSSQLSpatialTableLayer::InitBCP(const char* pszDSN)
 
 {
     /* Create a different connection for BCP upload */
-    if( Failed( SQLAllocHandle( SQL_HANDLE_ENV, NULL, &hEnvBCP ) ) )
+    if( Failed( SQLAllocHandle( SQL_HANDLE_ENV, nullptr, &hEnvBCP ) ) )
     return FALSE;
 
     /* Notify ODBC that this is an ODBC 3.0 app. */
@@ -1411,14 +1412,13 @@ int OGRMSSQLSpatialTableLayer::InitBCP(const char* pszDSN)
         return FALSE;
     }
 
-    SQLUINTEGER timeout = 30;
-    Failed(SQLSetConnectAttr(hDBCBCP, SQL_ATTR_LOGIN_TIMEOUT, (void*)timeout, SQL_IS_INTEGER));
+    Failed(SQLSetConnectAttr(hDBCBCP, SQL_ATTR_LOGIN_TIMEOUT, (void*)30, SQL_IS_INTEGER));
 
     SQLCHAR szOutConnString[1024];
     SQLSMALLINT nOutConnStringLen = 0;
 
     if ( Failed(
-        SQLDriverConnect( hDBCBCP, NULL,
+        SQLDriverConnect( hDBCBCP, nullptr,
                             (SQLCHAR *) pszDSN, (SQLSMALLINT)strlen(pszDSN),
                             szOutConnString, sizeof(szOutConnString),
                             &nOutConnStringLen, SQL_DRIVER_NOPROMPT ) ))
@@ -1448,7 +1448,7 @@ void OGRMSSQLSpatialTableLayer::CloseBCP()
         for( iCol = 0; iCol < nRawColumns; iCol++ )
             CPLFree(papstBindBuffer[iCol]);
         CPLFree(papstBindBuffer);
-        papstBindBuffer = NULL;
+        papstBindBuffer = nullptr;
 
         if( bIdentityInsert )
         {
@@ -1456,18 +1456,18 @@ void OGRMSSQLSpatialTableLayer::CloseBCP()
         }
     }
 
-    if( hDBCBCP!=NULL )
+    if( hDBCBCP!=nullptr )
     {
         CPLDebug( "ODBC", "SQLDisconnect()" );
         SQLDisconnect( hDBCBCP );
         SQLFreeHandle( SQL_HANDLE_DBC, hDBCBCP );
-        hDBCBCP = NULL;
+        hDBCBCP = nullptr;
     }
 
-    if( hEnvBCP!=NULL )
+    if( hEnvBCP!=nullptr )
     {
         SQLFreeHandle( SQL_HANDLE_ENV, hEnvBCP );
-        hEnvBCP = NULL;
+        hEnvBCP = nullptr;
     }
 }
 
@@ -1481,7 +1481,7 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
     int iCol;
     int iField = 0;
 
-    if ( hDBCBCP == NULL )
+    if ( hDBCBCP == nullptr )
     {
         nBCPCount = 0;
 
@@ -1498,13 +1498,13 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
         {
             /* need to re-create layer defn */
             poFeatureDefn->Release();
-            poFeatureDefn = NULL;
+            poFeatureDefn = nullptr;
         }
         GetLayerDefn();
         if (!poFeatureDefn)
             return OGRERR_FAILURE;
 
-        if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != NULL && bIsIdentityFid )
+        if( poFeature->GetFID() != OGRNullFID && pszFIDColumn != nullptr && bIsIdentityFid )
         {
             bIdentityInsert = TRUE;
         }
@@ -1513,7 +1513,7 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
             return OGRERR_FAILURE;
 
         /* Initialize the bulk copy */
-        if (Failed2( bcp_init(hDBCBCP, CPLSPrintf("[%s].[%s]", pszSchemaName, pszTableName), NULL, NULL, DB_IN) ))
+        if (Failed2( bcp_init(hDBCBCP, CPLSPrintf("[%s].[%s]", pszSchemaName, pszTableName), nullptr, nullptr, DB_IN) ))
         {
             CloseBCP();
             return OGRERR_FAILURE;
@@ -1534,13 +1534,13 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
 
         for( iCol = 0; iCol < nRawColumns; iCol++ )
         {
-            papstBindBuffer[iCol] = NULL;
+            papstBindBuffer[iCol] = nullptr;
 
             if (iCol == nGeomColumnIndex)
             {
                 papstBindBuffer[iCol] = (BCPData*)CPLMalloc(sizeof(BCPData));
-                if (Failed2( bcp_bind(hDBCBCP, NULL /* data is provided later */,
-                    0, 0/*or any value < 8000*/, NULL, 0, SQLUDT, iCol + 1) ))
+                if (Failed2( bcp_bind(hDBCBCP, nullptr /* data is provided later */,
+                    0, 0/*or any value < 8000*/, nullptr, 0, SQLUDT, iCol + 1) ))
                     return OGRERR_FAILURE;
             }
             else if (iCol == nFIDColumnIndex)
@@ -1582,7 +1582,7 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
 
                     if (Failed2( bcp_bind(hDBCBCP, (LPCBYTE)papstBindBuffer[iCol],
                         sizeof(papstBindBuffer[iCol]->Integer.iIndicator), sizeof(papstBindBuffer[iCol]->Integer.Value),
-                            NULL, 0, SQLINT4, iCol + 1) ))
+                            nullptr, 0, SQLINT4, iCol + 1) ))
                         return OGRERR_FAILURE;
                 }
                 else if( poFDefn->GetType() == OFTInteger64 )
@@ -1615,15 +1615,15 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
                     papstBindBuffer[iCol]->VarChar.nSize = poFDefn->GetWidth();
                     if (poFDefn->GetWidth() == 0)
                     {
-                        if (Failed2( bcp_bind(hDBCBCP, NULL /* data is provided later */,
-                            0, 0/*or any value < 8000*/, NULL, 0, 0, iCol + 1) ))
+                        if (Failed2( bcp_bind(hDBCBCP, nullptr /* data is provided later */,
+                            0, 0/*or any value < 8000*/, nullptr, 0, 0, iCol + 1) ))
                             return OGRERR_FAILURE;
                     }
                     else
                     {
                         if (Failed2( bcp_bind(hDBCBCP, (LPCBYTE)papstBindBuffer[iCol],
                             sizeof(papstBindBuffer[iCol]->VarChar.nSize), poFDefn->GetWidth(),
-                                NULL, 0, SQLNVARCHAR, iCol + 1) ))
+                                nullptr, 0, SQLNVARCHAR, iCol + 1) ))
                             return OGRERR_FAILURE;
                     }
                 }
@@ -1664,8 +1664,8 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
                 {
                     /* image */
                     papstBindBuffer[iCol] = (BCPData*)CPLMalloc(sizeof(BCPData));
-                    if (Failed2( bcp_bind(hDBCBCP, NULL /* data is provided later */,
-                        0, 0/*or any value < 8000*/, NULL, 0, 0, iCol + 1) ))
+                    if (Failed2( bcp_bind(hDBCBCP, nullptr /* data is provided later */,
+                        0, 0/*or any value < 8000*/, nullptr, 0, 0, iCol + 1) ))
                         return OGRERR_FAILURE;
                 }
                 else
@@ -1761,7 +1761,7 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
         {
             OGRFieldDefn* poFDefn = poFeatureDefn->GetFieldDefn(iField);
 
-            if (papstBindBuffer[iCol] == NULL)
+            if (papstBindBuffer[iCol] == nullptr)
             {
                 ++iField;
                 continue; /* column requires no data */
@@ -1990,14 +1990,14 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
 
                 }
                 CPLFree(papstBindBuffer[iCol]->RawData.pData);
-                if (Failed2( bcp_moretext( hDBCBCP, 0, NULL) ))
+                if (Failed2( bcp_moretext( hDBCBCP, 0, nullptr) ))
                 {
 
                 }
             }
             else
             {
-                if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, NULL) ))
+                if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, nullptr) ))
                 {
 
                 }
@@ -2041,14 +2041,14 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
                             CPLFree(buffer);
                         }
 
-                        if (Failed2( bcp_moretext( hDBCBCP, 0, NULL) ))
+                        if (Failed2( bcp_moretext( hDBCBCP, 0, nullptr) ))
                         {
 
                         }
                     }
                     else
                     {
-                        if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, NULL) ))
+                        if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, nullptr) ))
                         {
 
                         }
@@ -2068,14 +2068,14 @@ OGRErr OGRMSSQLSpatialTableLayer::CreateFeatureBCP( OGRFeature *poFeature )
 
                         }
                     }
-                    if (Failed2( bcp_moretext( hDBCBCP, 0, NULL) ))
+                    if (Failed2( bcp_moretext( hDBCBCP, 0, nullptr) ))
                     {
 
                     }
                 }
                 else
                 {
-                    if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, NULL) ))
+                    if (Failed2( bcp_moretext( hDBCBCP, SQL_NULL_DATA, nullptr) ))
                     {
 
                     }
