@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_mysql.h f890db52002f641b0c01872720a7473aa7c6c0cf 2020-05-27 23:21:17 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Declarations for MySQL OGR Driver Classes.
@@ -269,7 +269,7 @@ class OGRMySQLDataSource final: public OGRDataSource
 
     void                ReportError( const char * = nullptr );
 
-    char               *LaunderName( const char * );
+    static char        *LaunderName( const char * );
 
     void                RequestLongResult( OGRMySQLLayer * );
     void                InterruptLongResult();
