@@ -32,7 +32,6 @@
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp 980581525ac7edce204f844b7ea48e7cb2ecb0e4 2021-08-12 22:57:39 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
@@ -649,6 +648,12 @@ int OGRMySQLTableLayer::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCSequentialWrite) )
         return bUpdateAccess;
 
+    else if( EQUAL(pszCap,OLCMeasuredGeometries) )
+        return TRUE;
+
+    else if( EQUAL(pszCap,OLCZGeometries) )
+        return TRUE;
+
     else
         return FALSE;
 }
