@@ -32,12 +32,14 @@
 #include "cpl_http.h"
 #include "gdal_proxy.h"
 
+#include <limits>
+
 class NGWWrapperRasterBand : public GDALProxyRasterBand
 {
     GDALRasterBand *poBaseBand;
 
 protected:
-    virtual GDALRasterBand *RefUnderlyingRasterBand() override { return poBaseBand; }
+    virtual GDALRasterBand *RefUnderlyingRasterBand() const override { return poBaseBand; }
 
 public:
     explicit NGWWrapperRasterBand( GDALRasterBand* poBaseBandIn ) :
@@ -74,7 +76,7 @@ OGRNGWDataset::OGRNGWDataset() :
 OGRNGWDataset::~OGRNGWDataset()
 {
     // Last sync with server.
-    OGRNGWDataset::FlushCache();
+    OGRNGWDataset::FlushCache(true);
 
     if( poRasterDS != nullptr )
     {
@@ -143,6 +145,10 @@ int OGRNGWDataset::TestCapability( const char *pszCap )
     {
         return stPermissions.bDataCanRead;
     }
+    else if( EQUAL(pszCap, ODsCZGeometries) )
+    {
+        return TRUE;
+    }
     else
     {
         return FALSE;
@@ -580,7 +586,7 @@ OGRLayer *OGRNGWDataset::ICreateLayer( const char *pszNameIn,
     }
 
     // Check input parameters.
-    if( (eGType < wkbPoint || eGType > wkbMultiPolygon) && 
+    if( (eGType < wkbPoint || eGType > wkbMultiPolygon) &&
         (eGType < wkbPoint25D || eGType > wkbMultiPolygon25D) )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -796,9 +802,9 @@ CPLErr OGRNGWDataset::SetMetadataItem( const char *pszName,
 /*
  * FlushCache()
  */
-void OGRNGWDataset::FlushCache()
+void OGRNGWDataset::FlushCache(bool bAtClosing)
 {
-    GDALDataset::FlushCache();
+    GDALDataset::FlushCache(bAtClosing);
     FlushMetadata( GetMetadata("NGW") );
 }
 
@@ -1121,7 +1127,7 @@ OGRLayer *OGRNGWDataset::ExecuteSQL( const char *pszStatement,
                 {
                     osNgwSelect += psKeyDef->field_name;
                 }
-                else 
+                else
                 {
                     osNgwSelect += "-" + std::string(psKeyDef->field_name);
                 }
