@@ -29,7 +29,6 @@
 #include "ntf.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrntflayer.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
 
 /************************************************************************/
 /*                            OGRNTFLayer()                             */
@@ -179,9 +178,12 @@ OGRFeature *OGRNTFLayer::GetNextFeature()
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRNTFLayer::TestCapability( const char * /* pszCap */ )
+int OGRNTFLayer::TestCapability( const char * pszCap )
 
 {
+    if (EQUAL(pszCap, OLCZGeometries))
+        return TRUE;
+
     return FALSE;
 }
 
