@@ -29,7 +29,6 @@
 #include "ogr_odbc.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrodbcdriver.cpp f60db6b9481fc35b2fac0c1f7808f0240c15b7d6 2021-08-31 10:17:36 +1000 Nyall Dawson $")
 
 /************************************************************************/
 /*                     OGRODBCDriverIdentify()                          */
@@ -38,9 +37,7 @@ CPL_CVSID("$Id: ogrodbcdriver.cpp f60db6b9481fc35b2fac0c1f7808f0240c15b7d6 2021-
 static int OGRODBCDriverIdentify( GDALOpenInfo* poOpenInfo )
 
 {
-    if (STARTS_WITH_CI(poOpenInfo->pszFilename, "WALK:")
-        || STARTS_WITH_CI(poOpenInfo->pszFilename, "GEOMEDIA:")
-        || STARTS_WITH_CI(poOpenInfo->pszFilename, "PGEO:"))
+    if (STARTS_WITH_CI(poOpenInfo->pszFilename, "PGEO:"))
     {
         return FALSE;
     }
@@ -50,7 +47,7 @@ static int OGRODBCDriverIdentify( GDALOpenInfo* poOpenInfo )
 
     const char* psExtension(CPLGetExtension(poOpenInfo->pszFilename));
     if( EQUAL(psExtension,"mdb") )
-        return -1; // Could potentially be a PGeo, Walk or Geomedia MDB database
+        return -1; // Could potentially be a PGeo MDB database
 
     if ( OGRODBCDataSource::IsSupportedMsAccessFileExtension( psExtension ) )
         return TRUE; // An Access database which isn't a .MDB file (we checked that above), so this is the only candidate driver
@@ -100,6 +97,7 @@ void RegisterOGRODBC()
     poDriver->SetMetadataItem(GDAL_DMD_EXTENSIONS, "mdb accdb" );
     poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/vector/odbc.html");
     poDriver->SetMetadataItem( GDAL_DCAP_MULTIPLE_VECTOR_LAYERS, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "NATIVE OGRSQL SQLITE" );
 
     poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST, "<OpenOptionList>"
 "  <Option name='LIST_ALL_TABLES' type='string-select' scope='vector' description='Whether all tables, including system and internal tables (such as MSys* tables) should be listed' default='NO'>"
