@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_pgeo.h c62ee35e511f27eb73a8668006d6f2dc9c4edbc4 2021-08-29 09:01:23 +1000 Nyall Dawson $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for Personal Geodatabase driver.
@@ -43,11 +43,15 @@
 
 class OGRPGeoDataSource;
 
+constexpr const char* pszRelationshipTypeUUID = "{B606A7E1-FA5B-439C-849C6E9C2481537B}";
+
 class OGRPGeoLayer CPL_NON_FINAL: public OGRLayer
 {
   protected:
     OGRFeatureDefn     *poFeatureDefn;
 
+    int                 m_nStatementFlags = 0;
+
     CPLODBCStatement   *poStmt;
 
     // Layer spatial reference system, and srid.
@@ -108,7 +112,7 @@ class OGRPGeoTableLayer final: public OGRPGeoLayer
     std::string         m_osDocumentation;
 
   public:
-    explicit            OGRPGeoTableLayer( OGRPGeoDataSource * );
+    explicit            OGRPGeoTableLayer( OGRPGeoDataSource *,int );
                         virtual ~OGRPGeoTableLayer();
 
     CPLErr              Initialize( const char *pszTableName,
@@ -181,10 +185,15 @@ class OGRPGeoDataSource final: public OGRDataSource
 
     std::vector<std::unique_ptr<OGRLayer>> m_apoInvisibleLayers;
 
+    std::map<std::string,std::unique_ptr<GDALRelationship>> m_osMapRelationships{};
+
 #ifndef _WIN32
     mutable bool        m_COUNT_STAR_state_known = false;
     mutable bool        m_COUNT_STAR_working = false;
 #endif
+
+    int                 m_nStatementFlags = 0;
+
     static bool         IsPrivateLayerName( const CPLString& osName );
   public:
                         OGRPGeoDataSource();
@@ -208,6 +217,10 @@ class OGRPGeoDataSource final: public OGRDataSource
                                     const char *pszDialect ) override;
     virtual void        ReleaseResultSet( OGRLayer * poLayer ) override;
 
+    std::vector<std::string> GetRelationshipNames(CSLConstList papszOptions = nullptr) const override;
+
+    const GDALRelationship* GetRelationship(const std::string& name) const override;
+
     // Internal use
     CPLODBCSession     *GetSession() { return &oSession; }
 
