@@ -34,7 +34,6 @@
 #include <algorithm>
 #include <set>
 
-CPL_CVSID("$Id$")
 
 /************************************************************************/
 /*                          OGRS57DataSource()                          */
@@ -144,10 +143,13 @@ const char *OGRS57DataSource::GetOption( const char * pszOption )
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRS57DataSource::TestCapability( const char * )
+int OGRS57DataSource::TestCapability( const char * pszCap )
 
 {
-    return FALSE;
+    if( EQUAL(pszCap,ODsCZGeometries) )
+        return true;
+
+    return false;
 }
 
 /************************************************************************/
