@@ -32,7 +32,6 @@
 #include "ogr_api.h"
 #include "s57.h"
 
-CPL_CVSID("$Id: s57featuredefns.cpp 38140 2017-04-26 08:41:23Z rouault $");
 
 /************************************************************************/
 /*                     S57GenerateGeomFeatureDefn()                     */
@@ -178,7 +177,7 @@ OGRFeatureDefn *S57GenerateGeomFeatureDefn( OGRwkbGeometryType eGType,
                                             int nOptionFlags )
 
 {
-    OGRFeatureDefn      *poFDefn = NULL;
+    OGRFeatureDefn      *poFDefn = nullptr;
 
     if( eGType == wkbPoint )
     {
@@ -206,7 +205,7 @@ OGRFeatureDefn *S57GenerateGeomFeatureDefn( OGRwkbGeometryType eGType,
         poFDefn->SetGeomType( eGType );
     }
     else
-        return NULL;
+        return nullptr;
 
     poFDefn->Reference();
     S57GenerateStandardAttributes( poFDefn, nOptionFlags );
@@ -222,7 +221,7 @@ OGRFeatureDefn *
 S57GenerateVectorPrimitiveFeatureDefn( int nRCNM,
                                        int /* nOptionFlags */ )
 {
-    OGRFeatureDefn      *poFDefn = NULL;
+    OGRFeatureDefn      *poFDefn = nullptr;
 
     if( nRCNM == RCNM_VI )
     {
@@ -245,7 +244,7 @@ S57GenerateVectorPrimitiveFeatureDefn( int nRCNM,
         poFDefn->SetGeomType( wkbPolygon );
     }
     else
-        return NULL;
+        return nullptr;
 
     poFDefn->Reference();
 
@@ -332,7 +331,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn(
 
 {
     if( !poClassContentExplorer->SelectClass( nOBJL ) )
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Create the feature definition based on the object class         */
@@ -388,7 +387,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn(
     char **papszAttrList = poClassContentExplorer->GetAttributeList();
 
     for( int iAttr = 0;
-         papszAttrList != NULL && papszAttrList[iAttr] != NULL;
+         papszAttrList != nullptr && papszAttrList[iAttr] != nullptr;
          iAttr++ )
     {
         const int iAttrIndex = poCR->FindAttrByAcronym( papszAttrList[iAttr] );
@@ -421,7 +420,15 @@ OGRFeatureDefn *S57GenerateObjectClassDefn(
             break;
 
           case SAT_LIST:
-            oField.SetType( OFTString );
+            if( (nOptionFlags & S57M_LIST_AS_STRING) )
+            {
+                // Legacy behavior
+                oField.SetType( OFTString );
+            }
+            else
+            {
+                oField.SetType( OFTStringList );
+            }
             break;
         }
 
@@ -432,7 +439,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn(
 /*      Do we need to add DEPTH attributes to soundings?                */
 /* -------------------------------------------------------------------- */
     const char* pszClassAcronym = poClassContentExplorer->GetAcronym();
-    if( pszClassAcronym != NULL && EQUAL(pszClassAcronym, "SOUNDG")
+    if( pszClassAcronym != nullptr && EQUAL(pszClassAcronym, "SOUNDG")
         && (nOptionFlags & S57M_ADD_SOUNDG_DEPTH) )
     {
         OGRFieldDefn oField( "DEPTH", OFTReal );
