@@ -49,7 +49,6 @@
 #include "ogrshape.h"
 #include "shapefil.h"
 
-CPL_CVSID("$Id: shape2ogr.cpp 30fcaee8bb4fca62567d1a5b41c301d5f43469e5 2022-02-18 00:16:01 +0100 Even Rouault $")
 
 /************************************************************************/
 /*                        RingStartEnd                                  */
@@ -864,7 +863,16 @@ OGRErr SHPWriteOGRObject( SHPHandle hSHP, int iShape,
                 }
 
                 const int nNumInteriorRings = poPoly->getNumInteriorRings();
-                apoRings.reserve(apoRings.size() + nNumInteriorRings + 1);
+                // to avoid coverity scan warning: "To avoid a quadratic time
+                // penalty when using reserve(), always increase the capacity
+                /// by a multiple of its current value"
+                if( apoRings.size() + nNumInteriorRings + 1 > apoRings.capacity() &&
+                    apoRings.size() < std::numeric_limits<size_t>::max() / 2 )
+                {
+                    apoRings.reserve(std::max(
+                        2 * apoRings.size(),
+                        apoRings.size() + apoRings.size() + nNumInteriorRings + 1));
+                }
                 for(const auto poRing: poPoly)
                 {
                     const int nNumPoints = poRing->getNumPoints();
