@@ -32,7 +32,6 @@
 
 #include "ogr_sosi.h"
 
-CPL_CVSID("$Id$")
 
 static int bFYBAInit = FALSE;
 static std::mutex oMutex;
@@ -132,11 +131,13 @@ void RegisterOGRSOSI() {
 
     poDriver->SetDescription( "SOSI" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_FIELD, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Norwegian SOSI Standard" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/sosi.html" );
     poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,  "<OpenOptionList>"
     "<Option name='appendFieldsMap' type='string' description='Default is that all rows for equal field names will be appended in a feature, but with this parameter you select what field this should be valid for. With appendFieldsMap=f1&amp;f2, Append will be done for field f1 and f2 using a comma as delimiter.'/>"
     "</OpenOptionList>");
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "OGRSQL SQLITE" );
 
 
     poDriver->pfnOpen = OGRSOSIDriverOpen;
