@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_sqlite.h 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/SQLite driver.
@@ -280,6 +280,7 @@ class OGRSQLiteTableLayer final: public OGRSQLiteLayer
 
     bool                m_bLaunderColumnNames = true;
     bool                m_bSpatialite2D = false;
+    bool                m_bStrict = false;
 
     CPLString           m_osWHERE{};
     CPLString           m_osQuery{};
@@ -328,7 +329,8 @@ class OGRSQLiteTableLayer final: public OGRSQLiteLayer
 
     bool                CheckSpatialIndexTable(int iGeomCol);
 
-    CPLErr              EstablishFeatureDefn(const char* pszGeomCol);
+    CPLErr              EstablishFeatureDefn(const char* pszGeomCol,
+                                             bool bMayEmitError);
 
     void                LoadStatistics();
     void                LoadStatisticsSpatialite4DB();
@@ -345,7 +347,8 @@ class OGRSQLiteTableLayer final: public OGRSQLiteLayer
     CPLErr              Initialize( const char *pszTableName,
                                     bool bIsTable,
                                     bool bIsVirtualShapeIn,
-                                    bool bDeferredCreation);
+                                    bool bDeferredCreation,
+                                    bool bMayEmitError );
     void                SetCreationParameters( const char *pszFIDColumnName,
                                                OGRwkbGeometryType eGeomType,
                                                const char *pszGeomFormat,
@@ -393,6 +396,7 @@ class OGRSQLiteTableLayer final: public OGRSQLiteLayer
     void                SetDeferredSpatialIndexCreation( bool bFlag )
                                 { m_bDeferredSpatialIndexCreation = bFlag; }
     void                SetCompressedColumns( const char* pszCompressedColumns );
+    void                SetStrictFlag( bool bFlag ) { m_bStrict = bFlag; }
 
     int                 CreateSpatialIndex(int iGeomCol);
 
@@ -587,7 +591,7 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     CPLStringList       m_aosSubDatasets{};
     bool                m_bGeoTransformValid = false;
     double              m_adfGeoTransform[6];
-    CPLString           m_osProjection{};
+    OGRSpatialReference m_oSRS{};
     bool                m_bPromote1BitAs8Bit = false;
     bool                OpenRaster();
     bool                OpenRasterSubDataset(const char* pszConnectionId);
@@ -609,7 +613,8 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
 
     bool                OpenTable( const char *pszTableName,
                                    bool IsTable,
-                                   bool bIsVirtualShape );
+                                   bool bIsVirtualShape,
+                                   bool bMayEmitError );
     bool                OpenView( const char *pszViewName,
                                    const char *pszViewGeometry,
                                    const char *pszViewRowid,
@@ -644,12 +649,9 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     virtual char**      GetMetadata(const char* pszDomain = "") override;
 
     virtual CPLErr      GetGeoTransform( double* padfGeoTransform ) override;
-    virtual const char* _GetProjectionRef() override;
-    const OGRSpatialReference* GetSpatialRef() const override {
-        return GetSpatialRefFromOldGetProjectionRef();
-    }
+    const OGRSpatialReference* GetSpatialRef() const override;
 
-    char               *LaunderName( const char * );
+    static char        *LaunderName( const char * );
     int                 FetchSRSId( const OGRSpatialReference * poSRS );
     OGRSpatialReference*FetchSRS( int nSRID );
 
@@ -671,6 +673,10 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     int                 GetUndefinedSRID() const { return m_nUndefinedSRID; }
     bool                HasGeometryColumns() const { return m_bHaveGeometryColumns; }
 
+    std::vector<std::string> GetRelationshipNames(CSLConstList papszOptions = nullptr) const override;
+
+    const GDALRelationship* GetRelationship(const std::string& name) const override;
+
     void                ReloadLayers();
 
 #ifdef HAVE_RASTERLITE2
@@ -680,8 +686,11 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     const double*       GetGeoTransform() const { return m_adfGeoTransform; }
     bool                IsRL2MixedResolutions() const { return m_bRL2MixedResolutions; }
 
-    virtual CPLErr IBuildOverviews( const char *, int, int *,
-                                    int, int *, GDALProgressFunc, void * ) override;
+    virtual CPLErr IBuildOverviews( const char *,
+                                    int, const int *,
+                                    int, const int *,
+                                    GDALProgressFunc, void *,
+                                    CSLConstList papszOptions ) override;
 
 #endif
     OGRSQLiteDataSource* GetParentDS() const { return m_poParentDS; }
@@ -729,7 +738,8 @@ class RL2RasterBand final: public GDALPamRasterBand
 #endif // HAVE_RASTERLITE2
 
 CPLString OGRSQLiteFieldDefnToSQliteFieldDefn( OGRFieldDefn* poFieldDefn,
-                                               bool bSQLiteDialectInternalUse );
+                                               bool bSQLiteDialectInternalUse,
+                                               bool bStrict );
 
 void OGRSQLiteRegisterInflateDeflate(sqlite3* hDB);
 
