@@ -32,14 +32,12 @@
 
 #include <cstdlib>
 #include <string>
-#include <regex>
 #include <iostream>
 #include <sstream>
 
 #include "cpl_error.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrsqliteutility.cpp 498a80077f2504ce756950b02fccc7144c28e163 2021-08-13 10:52:53 +0200 Even Rouault $")
 
 SQLResult::SQLResult(char** result, int nRow, int nCol)
     : papszResult(result), nRowCount(nRow), nColCount(nCol) {
@@ -348,98 +346,152 @@ std::set<std::string> SQLGetUniqueFieldUCConstraints(sqlite3* poDb,
     // set names (in upper case) of fields with unique constraint
     std::set<std::string> uniqueFieldsUC;
 
-    // std::regex in gcc < 4.9 is broken
-#if !defined(__GNUC__) || defined(__clang__) || __GNUC__ >= 5
+    // Unique fields detection
+    const std::string upperTableName { CPLString( pszTableName ).toupper() };
+    char* pszTableDefinitionSQL = sqlite3_mprintf(
+        "SELECT sql, type FROM sqlite_master "
+        "WHERE type IN ('table', 'view') AND UPPER(name)='%q'", upperTableName.c_str() );
+    auto oResultTable = SQLQuery(poDb, pszTableDefinitionSQL);
+    sqlite3_free(pszTableDefinitionSQL);
 
-    try
+    if ( !oResultTable || oResultTable->RowCount() == 0 )
     {
-        static int hasWorkingRegex = std::regex_match("c", std::regex("a|b|c"));
-        if( !hasWorkingRegex )
+        if( oResultTable )
+            CPLError( CE_Failure, CPLE_AppDefined, "Cannot find table %s", pszTableName );
+
+        return uniqueFieldsUC;
+    }
+    if( std::string(oResultTable->GetValue(1, 0)) == "view" )
+    {
+        return uniqueFieldsUC;
+    }
+
+    // Parses strings like "colum_name1" KEYWORD1 KEYWORD2 'some string', `column_name2`,"column_name3"
+    const auto GetNextToken = [](const std::string& osStr, size_t& pos, bool keepQuotes)
+    {
+        if( pos >= osStr.size() )
+            return std::string();
+        pos = osStr.find_first_not_of(" \t\n\r", pos);
+        if( pos == std::string::npos )
+            return std::string();
+
+        std::string osToken;
+        if( osStr[pos] == '"' || osStr[pos] == '\'' || osStr[pos] == '`' )
         {
-            // Can happen if we build with clang, but run against libstdc++ of gcc < 4.9.
-            return uniqueFieldsUC;
-        }
-
-        // Unique fields detection
-        const std::string upperTableName { CPLString( pszTableName ).toupper() };
-        char* pszTableDefinitionSQL = sqlite3_mprintf(
-            "SELECT sql, type FROM sqlite_master "
-            "WHERE type IN ('table', 'view') AND UPPER(name)='%q'", upperTableName.c_str() );
-        auto oResultTable = SQLQuery(poDb, pszTableDefinitionSQL);
-        sqlite3_free(pszTableDefinitionSQL);
-
-        if ( !oResultTable || oResultTable->RowCount() == 0 )
-        {
-            if( oResultTable )
-                CPLError( CE_Failure, CPLE_AppDefined, "Cannot find table %s", pszTableName );
-
-            return uniqueFieldsUC;
-        }
-        if( std::string(oResultTable->GetValue(1, 0)) == "view" )
-        {
-            return uniqueFieldsUC;
-        }
-
-        // Match identifiers with ", ', or ` or no delimiter (and no spaces).
-        std::string tableDefinition { oResultTable->GetValue(0, 0) };
-        tableDefinition = tableDefinition.substr(tableDefinition.find('('), tableDefinition.rfind(')') );
-        std::stringstream tableDefinitionStream { tableDefinition };
-        std::smatch uniqueFieldMatch;
-        while (tableDefinitionStream.good()) {
-            std::string fieldStr;
-            std::getline( tableDefinitionStream, fieldStr, ',' );
-            if( CPLString( fieldStr ).toupper().find( "UNIQUE" ) != std::string::npos )
+            const char chQuoteChar = osStr[pos];
+            if( keepQuotes )
+                osToken += chQuoteChar;
+            ++pos;
+            while( pos < osStr.size() )
             {
-                static const std::regex sFieldIdentifierRe {
-                    R"raw(^\s*((["'`]([^"'`]+)["'`])|(([^"'`\s]+)\s)).*UNIQUE.*)raw",
-                    std::regex_constants::icase};
-                if( std::regex_search(fieldStr, uniqueFieldMatch, sFieldIdentifierRe) )
+                if( osStr[pos] == chQuoteChar )
                 {
-                    const std::string quoted { uniqueFieldMatch.str( 3 ) };
-                    uniqueFieldsUC.insert( CPLString(
-                        !quoted.empty() ? quoted: uniqueFieldMatch.str( 5 )).toupper() );
+                    if( pos + 1 < osStr.size() &&
+                        osStr[pos+1] == chQuoteChar )
+                    {
+                        osToken += chQuoteChar;
+                        pos += 2;
+                    }
+                    else
+                    {
+                        if( keepQuotes )
+                            osToken += chQuoteChar;
+                        pos ++;
+                        break;
+                    }
+                }
+                else
+                {
+                    osToken += osStr[pos];
+                    pos ++;
                 }
             }
         }
-
-        // Search indexes:
-        pszTableDefinitionSQL = sqlite3_mprintf(
-            "SELECT sql FROM sqlite_master WHERE type='index' AND"
-            " UPPER(tbl_name)=UPPER('%q') AND UPPER(sql) "
-            "LIKE 'CREATE UNIQUE INDEX%%'", upperTableName.c_str() );
-        oResultTable = SQLQuery(poDb, pszTableDefinitionSQL);
-        sqlite3_free(pszTableDefinitionSQL);
-
-        if ( !oResultTable  )
+        else if( osStr[pos] == ',' )
         {
-            CPLError( CE_Failure, CPLE_AppDefined,
-                      "Error searching indexes for table %s", pszTableName );
+            osToken = ',';
+            pos ++;
         }
-        else if (oResultTable->RowCount() >= 0 )
+        else
         {
-            for( int rowCnt = 0; rowCnt < oResultTable->RowCount(); ++rowCnt )
+            size_t pos2 = osStr.find_first_of(" \t\n\r,", pos);
+            if( pos2 == std::string::npos )
+                osToken = osStr.substr(pos);
+            else
+                osToken = osStr.substr(pos, pos2 - pos);
+            pos = pos2;
+        }
+        return osToken;
+    };
+
+    // Parses CREATE TABLE definition for column UNIQUE keywords
+    {
+        std::string tableDefinition { oResultTable->GetValue(0, 0) };
+        const auto nPosStart = tableDefinition.find('(');
+        const auto nPosEnd = tableDefinition.rfind(')');
+        if( nPosStart != std::string::npos &&
+            nPosEnd != std::string::npos &&
+            nPosEnd > nPosStart &&
+            CPLString( tableDefinition ).toupper().find( "UNIQUE" ) != std::string::npos )
+        {
+            tableDefinition = tableDefinition.substr(nPosStart+1, nPosEnd-nPosStart-1);
+            size_t pos = 0;
+            while( true )
             {
-                std::string indexDefinition { oResultTable->GetValue(0, rowCnt) };
-                if ( CPLString (indexDefinition ).toupper().find( "UNIQUE" ) != std::string::npos )
+                const std::string osColName = GetNextToken(tableDefinition, pos, false);
+                if( osColName.empty() )
                 {
-                    indexDefinition = indexDefinition.substr(
-                        indexDefinition.find('('), indexDefinition.rfind(')') );
-                    static const std::regex sFieldIndexIdentifierRe {
-                        R"raw(\(\s*[`"]?([^",`\)]+)["`]?\s*\))raw" };
-                    if( std::regex_search(indexDefinition, uniqueFieldMatch,
-                        sFieldIndexIdentifierRe) )
+                    break;
+                }
+                while( true )
+                {
+                    const std::string osToken = GetNextToken(tableDefinition, pos, true);
+                    if( osToken.empty() || osToken == "," )
+                        break;
+                    if( EQUAL(osToken.c_str(), "UNIQUE") )
                     {
-                        uniqueFieldsUC.insert( CPLString(uniqueFieldMatch.str( 1 )).toupper() );
+                        uniqueFieldsUC.insert( CPLString(osColName).toupper() );
                     }
                 }
             }
         }
     }
-    catch( const std::regex_error& e )
+
+    // Search indexes:
+    pszTableDefinitionSQL = sqlite3_mprintf(
+        "SELECT sql FROM sqlite_master WHERE type='index' AND"
+        " UPPER(tbl_name)=UPPER('%q') AND UPPER(sql) "
+        "LIKE 'CREATE UNIQUE INDEX%%'", upperTableName.c_str() );
+    oResultTable = SQLQuery(poDb, pszTableDefinitionSQL);
+    sqlite3_free(pszTableDefinitionSQL);
+
+    if ( !oResultTable  )
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "regex_error: %s", e.what());
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Error searching indexes for table %s", pszTableName );
+    }
+    else if (oResultTable->RowCount() >= 0 )
+    {
+        for( int rowCnt = 0; rowCnt < oResultTable->RowCount(); ++rowCnt )
+        {
+            std::string indexDefinition { oResultTable->GetValue(0, rowCnt) };
+            const auto nPosStart = indexDefinition.find('(');
+            const auto nPosEnd = indexDefinition.rfind(')');
+            if( nPosStart != std::string::npos &&
+                nPosEnd != std::string::npos &&
+                nPosEnd > nPosStart )
+            {
+                indexDefinition = indexDefinition.substr(nPosStart+1, nPosEnd - nPosStart-1);
+                size_t pos = 0;
+                const std::string osColName = GetNextToken(indexDefinition, pos, false);
+                // Only matches index on single columns
+                if( GetNextToken(indexDefinition, pos, false).empty() )
+                {
+                    uniqueFieldsUC.insert( CPLString(osColName).toupper() );
+                }
+            }
+        }
     }
 
-#endif  // <-- Unique detection
     return uniqueFieldsUC;
 }
