@@ -100,7 +100,7 @@ bool OGRSQLiteDataSource::OpenRaster()
         const char* pszAbstract = papszRow[2];
         if( pszCoverageName != nullptr )
         {
-            rl2CoveragePtr cvg = rl2_create_coverage_from_dbms( 
+            rl2CoveragePtr cvg = rl2_create_coverage_from_dbms(
                                                             hDB,
                                                             nullptr,
                                                             pszCoverageName );
@@ -356,18 +356,8 @@ bool OGRSQLiteDataSource::OpenRasterSubDataset(CPL_UNUSED
         OGRSpatialReference* poSRS = FetchSRS( nSRID );
         if( poSRS != nullptr )
         {
-            OGRSpatialReference oSRS(*poSRS);
-            char* pszWKT = nullptr;
-            if( oSRS.EPSGTreatsAsLatLong() ||
-                oSRS.EPSGTreatsAsNorthingEasting() )
-            {
-                oSRS.GetRoot()->StripNodes( "AXIS" );
-            }
-            if( oSRS.exportToWkt( &pszWKT ) == OGRERR_NONE )
-            {
-                m_osProjection = pszWKT;
-            }
-            CPLFree(pszWKT);
+            m_oSRS = *poSRS;
+            m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         }
     }
 
@@ -2319,9 +2309,10 @@ GDALDataset *OGRSQLiteDriverCreateCopy( const char* pszName,
 
 CPLErr OGRSQLiteDataSource::IBuildOverviews(
     const char * pszResampling,
-    int nOverviews, int * panOverviewList,
-    int nBandsIn, int * /*panBandList */,
-    GDALProgressFunc /*pfnProgress*/, void * /*pProgressData*/ )
+    int nOverviews, const int * panOverviewList,
+    int nBandsIn, const int * /*panBandList */,
+    GDALProgressFunc /*pfnProgress*/, void * /*pProgressData*/,
+    CSLConstList /* papszOptions */)
 
 {
     if( nBandsIn != nBands )
@@ -2442,12 +2433,12 @@ CPLErr OGRSQLiteDataSource::GetGeoTransform( double* padfGeoTransform )
 }
 
 /************************************************************************/
-/*                           GetProjectionRef()                         */
+/*                            GetSpatialRef()                           */
 /************************************************************************/
 
-const char* OGRSQLiteDataSource::_GetProjectionRef()
+const OGRSpatialReference* OGRSQLiteDataSource::GetSpatialRef() const
 {
-    if( !m_osProjection.empty() )
-        return m_osProjection.c_str();
-    return GDALPamDataset::_GetProjectionRef();
+    if( !m_oSRS.IsEmpty() )
+        return &m_oSRS;
+    return GDALPamDataset::GetSpatialRef();
 }
