@@ -37,7 +37,6 @@
 #include "ogr_srs_api.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: ogrsxflayer.cpp df398e80769422a4bbd5d4a295f4ede443c9fec6 2021-04-04 00:17:15 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                        OGRSXFLayer()                                 */
@@ -54,7 +53,7 @@ OGRSXFLayer::OGRSXFLayer(
     m_nSXFFormatVer(nVer),
     sFIDColumn_("ogc_fid"),
     m_hIOMutex(hIOMutex),
-    m_dfCoeff(sxfMapDesc.dfScale / sxfMapDesc.nResolution)
+    m_dfCoeff(sxfMapDesc.nResolution == 0 ? 0.0 : sxfMapDesc.dfScale / sxfMapDesc.nResolution)
 {
     stSXFMapDescription.pSpatRef->Reference();
     oNextIt = mnRecordDesc.begin();
@@ -413,6 +412,8 @@ int OGRSXFLayer::TestCapability( const char * pszCap )
         return TRUE;
     else if (EQUAL(pszCap, OLCFastSetNextByIndex))
         return TRUE;
+    else if (EQUAL(pszCap, OLCZGeometries))
+        return TRUE;
 
     return FALSE;
 }
