@@ -29,19 +29,18 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerpip.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
 
 static const char FILE_CODE[] = "P";
 
 static const TigerFieldInfo rtP_2002_fields[] = {
-  // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet bWrite
-  { "MODULE",     ' ', ' ', OFTString,     0,   0,   8,       1,   0,     0 },
-  { "FILE",       'L', 'N', OFTInteger,    6,  10,   5,       1,   1,     1 },
-  { "CENID",      'L', 'A', OFTString,    11,  15,   5,       1,   1,     1 },
-  { "POLYID",     'R', 'N', OFTInteger,   16,  25,  10,       1,   1,     1 },
-  { "POLYLONG",   'R', 'N', OFTInteger,   26,  35,  10,       1,   1,     1 },
-  { "POLYLAT",    'R', 'N', OFTInteger,   36,  44,   9,       1,   1,     1 },
-  { "WATER",      'L', 'N', OFTInteger,   45,  45,   1,       1,   1,     1 },
+  // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet
+  { "MODULE",     ' ', ' ', OFTString,     0,   0,   8,       1,   0 },
+  { "FILE",       'L', 'N', OFTInteger,    6,  10,   5,       1,   1 },
+  { "CENID",      'L', 'A', OFTString,    11,  15,   5,       1,   1 },
+  { "POLYID",     'R', 'N', OFTInteger,   16,  25,  10,       1,   1 },
+  { "POLYLONG",   'R', 'N', OFTInteger,   26,  35,  10,       1,   1 },
+  { "POLYLAT",    'R', 'N', OFTInteger,   36,  44,   9,       1,   1 },
+  { "WATER",      'L', 'N', OFTInteger,   45,  45,   1,       1,   1 },
 };
 static const TigerRecordInfo rtP_2002_info =
   {
@@ -51,13 +50,13 @@ static const TigerRecordInfo rtP_2002_info =
   };
 
 static const TigerFieldInfo rtP_fields[] = {
-  // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet bWrite
-  { "MODULE",     ' ', ' ', OFTString,     0,   0,   8,       1,   0,     0 },
-  { "FILE",       'L', 'N', OFTString,     6,  10,   5,       1,   1,     1 },
-  { "STATE",      'L', 'N', OFTInteger,    6,   7,   2,       1,   1,     1 },
-  { "COUNTY",     'L', 'N', OFTInteger,    8,  10,   3,       1,   1,     1 },
-  { "CENID",      'L', 'A', OFTString,    11,  15,   5,       1,   1,     1 },
-  { "POLYID",     'R', 'N', OFTInteger,   16,  25,  10,       1,   1,     1 }
+  // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet
+  { "MODULE",     ' ', ' ', OFTString,     0,   0,   8,       1,   0 },
+  { "FILE",       'L', 'N', OFTString,     6,  10,   5,       1,   1 },
+  { "STATE",      'L', 'N', OFTInteger,    6,   7,   2,       1,   1 },
+  { "COUNTY",     'L', 'N', OFTInteger,    8,  10,   3,       1,   1 },
+  { "CENID",      'L', 'A', OFTString,    11,  15,   5,       1,   1 },
+  { "POLYID",     'R', 'N', OFTInteger,   16,  25,  10,       1,   1 }
 };
 static const TigerRecordInfo rtP_info =
   {
@@ -72,7 +71,7 @@ static const TigerRecordInfo rtP_info =
 
 TigerPIP::TigerPIP( OGRTigerDataSource * poDSIn,
                     CPL_UNUSED const char * pszPrototypeModule )
-  : TigerPoint(TRUE, nullptr, FILE_CODE)
+  : TigerPoint(nullptr, FILE_CODE)
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "PIP" );
@@ -93,9 +92,3 @@ OGRFeature *TigerPIP::GetFeature( int nRecordId )
                                  26, 35,
                                  36, 44 );
 }
-
-OGRErr TigerPIP::CreateFeature( OGRFeature *poFeature )
-{
-  return TigerPoint::CreateFeature( poFeature,
-                                    26 );
-}
