@@ -31,7 +31,6 @@
 #include "cpl_time.h"
 #include <map>
 
-CPL_CVSID("$Id: ogrvdvdatasource.cpp 1761acd90777d5bcc49eddbc13c193098f0ed40b 2020-10-01 12:12:00 +0200 Even Rouault $")
 
 #ifndef STARTS_WITH_CI
 #define STARTS_WITH(a,b)               (strncmp(a,b,strlen(b)) == 0)
@@ -241,7 +240,7 @@ void OGRIDFDataSource::Parse()
         double x;
         double y;
         double z;
-        Point(double xIn = 0, double yIn = 0, double zIn = 0):
+        explicit Point(double xIn = 0, double yIn = 0, double zIn = 0):
             x(xIn), y(yIn), z(zIn) {}
     };
     std::map<GIntBig, Point > oMapNode; // map from NODE_ID to Point
@@ -594,6 +593,22 @@ OGRLayer* OGRIDFDataSource::GetLayer( int iLayer )
     return m_poTmpDS->GetLayer(iLayer);
 }
 
+/************************************************************************/
+/*                              TestCapability()                        */
+/************************************************************************/
+
+int OGRIDFDataSource::TestCapability( const char* pszCap )
+{
+    if( EQUAL(pszCap,ODsCMeasuredGeometries) )
+        return true;
+    else if( EQUAL(pszCap,ODsCCurveGeometries) )
+        return true;
+    else if( EQUAL(pszCap,ODsCZGeometries) )
+        return true;
+
+    return false;
+}
+
 /************************************************************************/
 /*                           OGRVDVDataSource()                         */
 /************************************************************************/
@@ -1091,10 +1106,14 @@ int OGRVDVLayer::TestCapability(const char* pszCap)
     {
         return TRUE;
     }
-    if( EQUAL(pszCap, OLCStringsAsUTF8) )
+    else if( EQUAL(pszCap, OLCStringsAsUTF8) )
     {
         return m_bRecodeFromLatin1;
     }
+    else if( EQUAL(pszCap, OLCZGeometries) )
+    {
+        return TRUE;
+    }
     return FALSE;
 }
 
@@ -1930,7 +1949,10 @@ int OGRVDVDataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
         return m_bUpdate;
-    return FALSE;
+    else if( EQUAL(pszCap,ODsCZGeometries) )
+        return true;
+
+    return false;
 }
 
 /************************************************************************/
@@ -1998,6 +2020,16 @@ void RegisterOGRVDV()
 
     poDriver->SetDescription( "VDV" );
     poDriver->SetMetadataItem( GDAL_DCAP_VECTOR, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_LAYER, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CREATE_FIELD, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_DELETE_FIELD, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_REORDER_FIELDS, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_MEASURED_GEOMETRIES, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_CURVE_GEOMETRIES, "YES" );
+    poDriver->SetMetadataItem( GDAL_DCAP_Z_GEOMETRIES, "YES" );
+    poDriver->SetMetadataItem( GDAL_DMD_ALTER_FIELD_DEFN_FLAGS, "Name Type WidthPrecision" );
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "OGRSQL SQLITE" );
+
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                "VDV-451/VDV-452/INTREST Data Format" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/vdv.html" );
