@@ -1305,6 +1305,7 @@ void OGROAPIFLayer::EstablishFeatureDefn()
     for( const auto& osItemAsset: m_aosItemAssetNames )
     {
         OGRFieldDefn oFieldDefn( ("asset_" + osItemAsset + "_href").c_str(), OFTString );
+        // cppcheck-suppress danglingTemporaryLifetime
         m_poFeatureDefn->AddFieldDefn( &oFieldDefn );
     }
 
@@ -2483,6 +2484,7 @@ void RegisterOGROAPIF()
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/oapif.html" );
 
     poDriver->SetMetadataItem( GDAL_DMD_CONNECTION_PREFIX, "OAPIF:" );
+    poDriver->SetMetadataItem( GDAL_DMD_SUPPORTED_SQL_DIALECTS, "OGRSQL SQLITE" );
 
     poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,
 "<OpenOptionList>"
