@@ -40,7 +40,6 @@
 #include "cpl_time.h"
 #include "swq_parser.hpp"
 
-CPL_CVSID("$Id: swq.cpp 1ea2397244cca10e862b7182a4785812c4b764fc 2021-06-01 15:35:55 +0200 Even Rouault $")
 
 #define YYSTYPE swq_expr_node *
 
@@ -292,8 +291,7 @@ int swqlex( YYSTYPE *ppNode, swq_parse_context *context )
             nReturn = SWQT_OFFSET;
 
         // Unhandled by OGR SQL.
-        else if( EQUAL(osToken, "LIMIT") ||
-                 EQUAL(osToken, "OUTER") ||
+        else if( EQUAL(osToken, "OUTER") ||
                  EQUAL(osToken, "INNER") )
             nReturn = SWQT_RESERVED_KEYWORD;
 
@@ -868,6 +866,7 @@ swq_expr_node* swq_create_and_or_or(swq_op op, swq_expr_node* left, swq_expr_nod
                    right->nSubExprCount * sizeof(swq_expr_node*));
 
             right->nSubExprCount = 0;
+            CPLFree(right->papoSubExpr);
             right->papoSubExpr = nullptr;
             delete right;
         }
