@@ -38,7 +38,6 @@
 #include "cpl_http.h"
 #include <algorithm>
 
-CPL_CVSID("$Id$")
 
 // #define DEBUG_VERBOSE 1
 
@@ -68,6 +67,10 @@ static std::string gosSourceProfileAccessKeyId;
 static std::string gosSourceProfileSecretAccessKey;
 static std::string gosSourceProfileSessionToken;
 
+// The below variables are used for web identity settings in aws/config
+static std::string gosRoleArnWebIdentity;
+static std::string gosWebIdentityTokenFile;
+
 /************************************************************************/
 /*                         CPLGetLowerCaseHex()                         */
 /************************************************************************/
@@ -318,6 +321,7 @@ CPLGetAWS_SIGN4_Authorization( const CPLString& osSecretAccessKey,
                                const CPLString& osCanonicalURI,
                                const CPLString& osCanonicalQueryString,
                                const CPLString& osXAMZContentSHA256,
+                               bool bAddHeaderAMZContentSHA256,
                                const CPLString& osTimestamp )
 {
     CPLString osSignedHeaders;
@@ -332,7 +336,7 @@ CPLGetAWS_SIGN4_Authorization( const CPLString& osSecretAccessKey,
                                                     osCanonicalURI,
                                                     osCanonicalQueryString,
                                                     osXAMZContentSHA256,
-                                                    true, // bAddHeaderAMZContentSHA256
+                                                    bAddHeaderAMZContentSHA256,
                                                     osTimestamp,
                                                     osSignedHeaders));
 
@@ -371,10 +375,10 @@ CPLGetAWS_SIGN4_Authorization( const CPLString& osSecretAccessKey,
 /*                        CPLGetAWS_SIGN4_Timestamp()                   */
 /************************************************************************/
 
-CPLString CPLGetAWS_SIGN4_Timestamp()
+CPLString CPLGetAWS_SIGN4_Timestamp(GIntBig timestamp)
 {
     struct tm brokenDown;
-    CPLUnixTimeToYMDHMS(time(nullptr), &brokenDown);
+    CPLUnixTimeToYMDHMS(timestamp, &brokenDown);
 
     char szTimeStamp[80] = {};
     snprintf(szTimeStamp, sizeof(szTimeStamp), "%04d%02d%02dT%02d%02d%02dZ",
@@ -730,36 +734,173 @@ static bool IsMachinePotentiallyEC2Instance()
 #endif
 }
 
+
+/************************************************************************/
+/*                   ReadAWSWebIdentityTokenFile()                      */
+/************************************************************************/
+
+static bool ReadAWSWebIdentityTokenFile(const std::string& osWebIdentityTokenFile,
+                                        CPLString& webIdentityToken)
+{
+    GByte *pabyOut = nullptr;
+    if( !VSIIngestFile( nullptr, osWebIdentityTokenFile.c_str(), &pabyOut, nullptr, -1 ) )
+        return false;
+
+    webIdentityToken = reinterpret_cast<char *>(pabyOut);
+    VSIFree(pabyOut);
+    // Remove trailing end-of-line character
+    if( !webIdentityToken.empty() && webIdentityToken.back() == '\n' )
+        webIdentityToken.resize(webIdentityToken.size() - 1);
+    return !webIdentityToken.empty();
+}
+
+/************************************************************************/
+/*          GetConfigurationFromAssumeRoleWithWebIdentity()             */
+/************************************************************************/
+
+bool VSIS3HandleHelper::GetConfigurationFromAssumeRoleWithWebIdentity(bool bForceRefresh,
+                                                                      const std::string& osPathForOption,
+                                                                      const std::string& osRoleArnIn,
+                                                                      const std::string& osWebIdentityTokenFileIn,
+                                                                      CPLString& osSecretAccessKey,
+                                                                      CPLString& osAccessKeyId,
+                                                                      CPLString& osSessionToken)
+{
+    CPLMutexHolder oHolder( &ghMutex );
+    if( !bForceRefresh )
+    {
+        time_t nCurTime;
+        time(&nCurTime);
+        // Try to reuse credentials if they are still valid, but
+        // keep one minute of margin...
+        if( !gosGlobalAccessKeyId.empty() && nCurTime < gnGlobalExpiration - 60 )
+        {
+            osAccessKeyId = gosGlobalAccessKeyId;
+            osSecretAccessKey = gosGlobalSecretAccessKey;
+            osSessionToken = gosGlobalSessionToken;
+            return true;
+        }
+    }
+
+    const CPLString roleArn = !osRoleArnIn.empty() ? osRoleArnIn :
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_ROLE_ARN", "");
+    if( roleArn.empty() )
+    {
+        CPLDebug("AWS", "AWS_ROLE_ARN configuration option not defined");
+        return false;
+    }
+
+    const CPLString webIdentityTokenFile = !osWebIdentityTokenFileIn.empty() ? osWebIdentityTokenFileIn :
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_WEB_IDENTITY_TOKEN_FILE", "");
+    if( webIdentityTokenFile.empty() )
+    {
+        CPLDebug("AWS", "AWS_WEB_IDENTITY_TOKEN_FILE configuration option not defined");
+        return false;
+    }
+
+    const CPLString stsRegionalEndpoints = VSIGetPathSpecificOption(osPathForOption.c_str(),
+                                                            "AWS_STS_REGIONAL_ENDPOINTS", "regional");
+
+    std::string osStsDefaultUrl;
+    if (stsRegionalEndpoints == "regional") {
+        const CPLString osRegion = VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_REGION", "us-east-1");
+        osStsDefaultUrl = "https://sts." + osRegion + ".amazonaws.com";
+    } else {
+        osStsDefaultUrl = "https://sts.amazonaws.com";
+    }
+    const CPLString osStsRootUrl(
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "CPL_AWS_STS_ROOT_URL", osStsDefaultUrl.c_str()));
+
+    // Get token from web identity token file
+    CPLString webIdentityToken;
+    if(!ReadAWSWebIdentityTokenFile(webIdentityTokenFile, webIdentityToken) )
+    {
+        CPLDebug("AWS", "%s is empty", webIdentityTokenFile.c_str());
+        return false;
+    }
+
+    // Get credentials from sts AssumeRoleWithWebIdentity
+    std::string osExpiration;
+    {
+        const CPLString osSTS_asuume_role_with_web_identity_URL =
+            osStsRootUrl + "/?Action=AssumeRoleWithWebIdentity&RoleSessionName=gdal"
+            "&Version=2011-06-15&RoleArn=" + CPLAWSURLEncode(roleArn) + "&WebIdentityToken=" + CPLAWSURLEncode(webIdentityToken);
+
+        CPLPushErrorHandler(CPLQuietErrorHandler);
+
+        CPLHTTPResult* psResult = CPLHTTPFetch( osSTS_asuume_role_with_web_identity_URL.c_str(), nullptr );
+        CPLPopErrorHandler();
+        if( psResult )
+        {
+            if( psResult->nStatus == 0 && psResult->pabyData != nullptr )
+            {
+                CPLXMLTreeCloser oTree(CPLParseXMLString(reinterpret_cast<char*>(psResult->pabyData)));
+                if( oTree )
+                {
+                    const auto psCredentials = CPLGetXMLNode(oTree.get(),
+                        "=AssumeRoleWithWebIdentityResponse.AssumeRoleWithWebIdentityResult.Credentials");
+                    if( psCredentials )
+                    {
+                        osAccessKeyId = CPLGetXMLValue(psCredentials, "AccessKeyId", "");
+                        osSecretAccessKey = CPLGetXMLValue(psCredentials, "SecretAccessKey", "");
+                        osSessionToken = CPLGetXMLValue(psCredentials, "SessionToken", "");
+                        osExpiration = CPLGetXMLValue(psCredentials, "Expiration", "");
+                    }
+                }
+            }
+            CPLHTTPDestroyResult(psResult);
+        }
+    }
+
+    GIntBig nExpirationUnix = 0;
+    if( !osAccessKeyId.empty() &&
+        !osSecretAccessKey.empty() &&
+        !osSessionToken.empty() &&
+        Iso8601ToUnixTime(osExpiration.c_str(), &nExpirationUnix) )
+    {
+        gosGlobalAccessKeyId = osAccessKeyId;
+        gosGlobalSecretAccessKey = osSecretAccessKey;
+        gosGlobalSessionToken = osSessionToken;
+        gnGlobalExpiration = nExpirationUnix;
+        CPLDebug("AWS", "Storing AIM credentials until %s", osExpiration.c_str());
+    }
+    return !osAccessKeyId.empty() && !osSecretAccessKey.empty() && !osSessionToken.empty();
+}
+
 /************************************************************************/
 /*                      GetConfigurationFromEC2()                       */
 /************************************************************************/
 
-bool VSIS3HandleHelper::GetConfigurationFromEC2(const std::string& osPathForOption,
+bool VSIS3HandleHelper::GetConfigurationFromEC2(bool bForceRefresh,
+                                                const std::string& osPathForOption,
                                                 CPLString& osSecretAccessKey,
                                                 CPLString& osAccessKeyId,
                                                 CPLString& osSessionToken)
 {
     CPLMutexHolder oHolder( &ghMutex );
-    time_t nCurTime;
-    time(&nCurTime);
-    // Try to reuse credentials if they are still valid, but
-    // keep one minute of margin...
-    if( !gosGlobalAccessKeyId.empty() && nCurTime < gnGlobalExpiration - 60 )
+    if( !bForceRefresh )
     {
-        osAccessKeyId = gosGlobalAccessKeyId;
-        osSecretAccessKey = gosGlobalSecretAccessKey;
-        osSessionToken = gosGlobalSessionToken;
-        return true;
+        time_t nCurTime;
+        time(&nCurTime);
+        // Try to reuse credentials if they are still valid, but
+        // keep one minute of margin...
+        if( !gosGlobalAccessKeyId.empty() && nCurTime < gnGlobalExpiration - 60 )
+        {
+            osAccessKeyId = gosGlobalAccessKeyId;
+            osSecretAccessKey = gosGlobalSecretAccessKey;
+            osSessionToken = gosGlobalSessionToken;
+            return true;
+        }
     }
 
     CPLString osURLRefreshCredentials;
     const CPLString osEC2DefaultURL("http://169.254.169.254");
     // coverity[tainted_data]
     const CPLString osEC2RootURL(
-        VSIGetCredential(osPathForOption.c_str(), "CPL_AWS_EC2_API_ROOT_URL", osEC2DefaultURL));
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "CPL_AWS_EC2_API_ROOT_URL", osEC2DefaultURL));
     // coverity[tainted_data]
     const CPLString osECSRelativeURI(
-        VSIGetCredential(osPathForOption.c_str(), "AWS_CONTAINER_CREDENTIALS_RELATIVE_URI", ""));
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_CONTAINER_CREDENTIALS_RELATIVE_URI", ""));
     CPLString osToken;
     if( osEC2RootURL == osEC2DefaultURL && !osECSRelativeURI.empty() )
     {
@@ -965,6 +1106,7 @@ static bool ReadAWSCredentials(const std::string& osProfile,
 
 bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
                                                 const std::string& osPathForOption,
+                                                const char* pszProfile,
                                                 CPLString& osSecretAccessKey,
                                                 CPLString& osAccessKeyId,
                                                 CPLString& osSessionToken,
@@ -974,15 +1116,20 @@ bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
                                                 CPLString& osSourceProfile,
                                                 CPLString& osExternalId,
                                                 CPLString& osMFASerial,
-                                                CPLString& osRoleSessionName)
+                                                CPLString& osRoleSessionName,
+                                                CPLString& osWebIdentityTokenFile)
 {
     // See http://docs.aws.amazon.com/cli/latest/userguide/cli-config-files.html
     // If AWS_DEFAULT_PROFILE is set (obsolete, no longer documented), use it in priority
     // Otherwise use AWS_PROFILE
     // Otherwise fallback to "default"
-    const char* pszProfile = VSIGetCredential(osPathForOption.c_str(), "AWS_DEFAULT_PROFILE", "");
-    if( pszProfile[0] == '\0' )
-        pszProfile = VSIGetCredential(osPathForOption.c_str(), "AWS_PROFILE", "");
+    const char* pszProfileOri = pszProfile;
+    if( pszProfile == nullptr )
+    {
+        pszProfile = VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_DEFAULT_PROFILE", "");
+        if( pszProfile[0] == '\0' )
+            pszProfile = VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_PROFILE", "");
+    }
     const CPLString osProfile(pszProfile[0] != '\0' ? pszProfile : "default");
 
 #ifdef WIN32
@@ -1002,7 +1149,7 @@ bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
     // GDAL specific config option (mostly for testing purpose, but also
     // used in production in some cases)
     const char* pszCredentials =
-        VSIGetCredential(osPathForOption.c_str(), "CPL_AWS_CREDENTIALS_FILE", nullptr );
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "CPL_AWS_CREDENTIALS_FILE", nullptr );
     if( pszCredentials )
     {
         osCredentials = pszCredentials;
@@ -1019,7 +1166,7 @@ bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
 
     // And then ~/.aws/config file (unless AWS_CONFIG_FILE is defined)
     const char* pszAWSConfigFileEnv =
-        VSIGetCredential(osPathForOption.c_str(), "AWS_CONFIG_FILE", nullptr );
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_CONFIG_FILE", nullptr );
     CPLString osConfig;
     if( pszAWSConfigFileEnv )
     {
@@ -1106,6 +1253,10 @@ bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
                     {
                         osRoleSessionName = pszValue;
                     }
+                    else if( strcmp(pszKey, "web_identity_token_file") == 0 )
+                    {
+                        osWebIdentityTokenFile = pszValue;
+                    }
                 }
                 CPLFree(pszKey);
             }
@@ -1123,7 +1274,8 @@ bool VSIS3HandleHelper::GetConfigurationFromAWSConfigFiles(
     }
 
     return (!osAccessKeyId.empty() && !osSecretAccessKey.empty()) ||
-           (!osRoleArn.empty() && !osSourceProfile.empty());
+           (!osRoleArn.empty() && !osSourceProfile.empty()) ||
+           (pszProfileOri != nullptr && !osRoleArn.empty() && !osWebIdentityTokenFile.empty());
 }
 
 
@@ -1147,7 +1299,7 @@ static bool GetTemporaryCredentialsForRole(const std::string& osRoleArn,
 {
     std::string osXAMZDate = CPLGetConfigOption("AWS_TIMESTAMP", "");
     if( osXAMZDate.empty() )
-        osXAMZDate = CPLGetAWS_SIGN4_Timestamp();
+        osXAMZDate = CPLGetAWS_SIGN4_Timestamp(time(nullptr));
     std::string osDate(osXAMZDate);
     osDate.resize(8);
 
@@ -1157,11 +1309,6 @@ static bool GetTemporaryCredentialsForRole(const std::string& osRoleArn,
     const std::string osHost(CPLGetConfigOption("AWS_STS_ENDPOINT", "sts.amazonaws.com"));
 
     std::map<std::string, std::string> oMap;
-    oMap["X-Amz-Algorithm"] = "AWS4-HMAC-SHA256";
-    oMap["X-Amz-Credential"] =
-        osAccessKeyId + "/" + osDate + "/" + osRegion + "/" + osService + "/aws4_request";
-    oMap["X-Amz-Date"] = osXAMZDate;
-    oMap["X-Amz-SignedHeaders"] = "host";
     oMap["Version"] = "2011-06-15";
     oMap["Action"] = "AssumeRole";
     oMap["RoleArn"] = osRoleArn;
@@ -1186,28 +1333,36 @@ static bool GetTemporaryCredentialsForRole(const std::string& osRoleArn,
     }
     CPLString osCanonicalQueryString(osQueryString.substr(1));
 
-    CPLString osSignedHeaders;
-    const CPLString osSignature =
-      CPLGetAWS_SIGN4_Signature(
+    const CPLString osAuthorization =
+      CPLGetAWS_SIGN4_Authorization(
         osSecretAccessKey,
+        osAccessKeyId,
         osSessionToken,
         osRegion,
         std::string(), //m_osRequestPayer,
         osService,
         osVerb,
-        nullptr, /* existing headers */
+        nullptr, // psExistingHeaders,
         osHost,
         "/",
         osCanonicalQueryString,
         CPLGetLowerCaseHexSHA256(std::string()),
         false, // bAddHeaderAMZContentSHA256
-        osXAMZDate,
-        osSignedHeaders);
+        osXAMZDate);
 
     bool bRet = false;
     const bool bUseHTTPS = CPLTestBool(CPLGetConfigOption("AWS_HTTPS", "YES"));
-    const std::string osURL = (bUseHTTPS ? "https://" : "http://") + osHost + "/" + osQueryString + "&X-Amz-Signature=" + osSignature;
-    CPLHTTPResult* psResult = CPLHTTPFetch( osURL.c_str(), nullptr );
+
+    CPLStringList aosOptions;
+    std::string headers;
+    if( !osSessionToken.empty() )
+        headers += "X-Amz-Security-Token: " + osSessionToken + "\r\n";
+    headers += "X-Amz-Date: " + osXAMZDate + "\r\n";
+    headers += "Authorization: " + osAuthorization;
+    aosOptions.AddNameValue("HEADERS", headers.c_str());
+
+    const std::string osURL = (bUseHTTPS ? "https://" : "http://") + osHost + "/" + osQueryString;
+    CPLHTTPResult* psResult = CPLHTTPFetch( osURL.c_str(), aosOptions.List() );
     if( psResult )
     {
         if( psResult->nStatus == 0 && psResult->pabyData != nullptr )
@@ -1239,23 +1394,47 @@ static bool GetTemporaryCredentialsForRole(const std::string& osRoleArn,
 /*               GetOrRefreshTemporaryCredentialsForRole()              */
 /************************************************************************/
 
-static bool GetOrRefreshTemporaryCredentialsForRole(CPLString& osSecretAccessKey,
+bool VSIS3HandleHelper::GetOrRefreshTemporaryCredentialsForRole(bool bForceRefresh,
+                                                    CPLString& osSecretAccessKey,
                                                     CPLString& osAccessKeyId,
                                                     CPLString& osSessionToken,
                                                     CPLString& osRegion)
 {
     CPLMutexHolder oHolder( &ghMutex );
-    time_t nCurTime;
-    time(&nCurTime);
-    // Try to reuse credentials if they are still valid, but
-    // keep one minute of margin...
-    if( !gosGlobalAccessKeyId.empty() && nCurTime < gnGlobalExpiration - 60 )
+    if( !bForceRefresh )
     {
-        osAccessKeyId = gosGlobalAccessKeyId;
-        osSecretAccessKey = gosGlobalSecretAccessKey;
-        osSessionToken = gosGlobalSessionToken;
-        osRegion = gosRegion;
-        return true;
+        time_t nCurTime;
+        time(&nCurTime);
+        // Try to reuse credentials if they are still valid, but
+        // keep one minute of margin...
+        if( !gosGlobalAccessKeyId.empty() && nCurTime < gnGlobalExpiration - 60 )
+        {
+            osAccessKeyId = gosGlobalAccessKeyId;
+            osSecretAccessKey = gosGlobalSecretAccessKey;
+            osSessionToken = gosGlobalSessionToken;
+            osRegion = gosRegion;
+            return true;
+        }
+    }
+
+    if( !gosRoleArnWebIdentity.empty() )
+    {
+        if( GetConfigurationFromAssumeRoleWithWebIdentity(bForceRefresh,
+                                                          std::string(),
+                                                          gosRoleArnWebIdentity,
+                                                          gosWebIdentityTokenFile,
+                                                          osSecretAccessKey,
+                                                          osAccessKeyId,
+                                                          osSessionToken) )
+        {
+            gosSourceProfileSecretAccessKey = osSecretAccessKey;
+            gosSourceProfileAccessKeyId = osAccessKeyId;
+            gosSourceProfileSessionToken = osSessionToken;
+        }
+        else
+        {
+            return false;
+        }
     }
 
     std::string osExpiration;
@@ -1301,9 +1480,9 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
     // AWS_REGION is GDAL specific. Later overloaded by standard
     // AWS_DEFAULT_REGION
     osRegion = CSLFetchNameValueDef(papszOptions, "AWS_REGION",
-            VSIGetCredential(osPathForOption.c_str(), "AWS_REGION", "us-east-1"));
+            VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_REGION", "us-east-1"));
 
-    if( CPLTestBool(VSIGetCredential(osPathForOption.c_str(), "AWS_NO_SIGN_REQUEST", "NO")) )
+    if( CPLTestBool(VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_NO_SIGN_REQUEST", "NO")) )
     {
         osSecretAccessKey.clear();
         osAccessKeyId.clear();
@@ -1313,12 +1492,12 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
 
     osSecretAccessKey = CSLFetchNameValueDef(papszOptions,
         "AWS_SECRET_ACCESS_KEY",
-        VSIGetCredential(osPathForOption.c_str(), "AWS_SECRET_ACCESS_KEY", ""));
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_SECRET_ACCESS_KEY", ""));
     if( !osSecretAccessKey.empty() )
     {
         osAccessKeyId = CSLFetchNameValueDef(papszOptions,
             "AWS_ACCESS_KEY_ID",
-            VSIGetCredential(osPathForOption.c_str(), "AWS_ACCESS_KEY_ID", ""));
+            VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_ACCESS_KEY_ID", ""));
         if( osAccessKeyId.empty() )
         {
             VSIError(VSIE_AWSInvalidCredentials,
@@ -1328,7 +1507,7 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
 
         osSessionToken = CSLFetchNameValueDef(papszOptions,
             "AWS_SESSION_TOKEN",
-            VSIGetCredential(osPathForOption.c_str(), "AWS_SESSION_TOKEN", ""));
+            VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_SESSION_TOKEN", ""));
         return true;
     }
 
@@ -1339,7 +1518,8 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
         bAssumedRole = !gosRoleArn.empty();
     }
     if( bAssumedRole &&
-        GetOrRefreshTemporaryCredentialsForRole(osSecretAccessKey,
+        GetOrRefreshTemporaryCredentialsForRole(/* bForceRefresh = */ false,
+                                                osSecretAccessKey,
                                                 osAccessKeyId,
                                                 osSessionToken,
                                                 osRegion) )
@@ -1355,8 +1535,10 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
     CPLString osExternalId;
     CPLString osMFASerial;
     CPLString osRoleSessionName;
+    CPLString osWebIdentityTokenFile;
     // coverity[tainted_data]
     if( GetConfigurationFromAWSConfigFiles(osPathForOption,
+                                           /* pszProfile = */ nullptr,
                                            osSecretAccessKey, osAccessKeyId,
                                            osSessionToken, osRegion,
                                            osCredentials,
@@ -1364,21 +1546,67 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
                                            osSourceProfile,
                                            osExternalId,
                                            osMFASerial,
-                                           osRoleSessionName) )
+                                           osRoleSessionName,
+                                           osWebIdentityTokenFile) )
     {
         if( osSecretAccessKey.empty() && !osRoleArn.empty() )
         {
-            // Get the credentials for the source profile, that will be
-            // used to sign the STS AssumedRole request.
-            if( !ReadAWSCredentials(osSourceProfile, osCredentials,
-                                    osSecretAccessKey,
-                                    osAccessKeyId,
-                                    osSessionToken) )
+            // Check if the default profile is pointing to another profile
+            // that has a role_arn and web_identity_token_file settings.
+            if( !osSourceProfile.empty() )
             {
-                VSIError(VSIE_AWSInvalidCredentials,
-                         "Cannot retrieve credentials for source profile %s",
-                         osSourceProfile.c_str());
-                return false;
+                CPLString osSecretAccessKeySP;
+                CPLString osAccessKeyIdSP;
+                CPLString osSessionTokenSP;
+                CPLString osRegionSP;
+                CPLString osCredentialsSP;
+                CPLString osRoleArnSP;
+                CPLString osSourceProfileSP;
+                CPLString osExternalIdSP;
+                CPLString osMFASerialSP;
+                CPLString osRoleSessionNameSP;
+                if( GetConfigurationFromAWSConfigFiles(osPathForOption,
+                                                       osSourceProfile.c_str(),
+                                                       osSecretAccessKeySP,
+                                                       osAccessKeyIdSP,
+                                                       osSessionTokenSP,
+                                                       osRegionSP,
+                                                       osCredentialsSP,
+                                                       osRoleArnSP,
+                                                       osSourceProfileSP,
+                                                       osExternalIdSP,
+                                                       osMFASerialSP,
+                                                       osRoleSessionNameSP,
+                                                       osWebIdentityTokenFile) )
+                {
+                    if( GetConfigurationFromAssumeRoleWithWebIdentity(/* bForceRefresh = */ false,
+                                                                      osPathForOption,
+                                                                      osRoleArnSP,
+                                                                      osWebIdentityTokenFile,
+                                                                      osSecretAccessKey, osAccessKeyId,
+                                                                      osSessionToken) )
+                    {
+                        CPLMutexHolder oHolder( &ghMutex );
+                        gosRoleArnWebIdentity = osRoleArnSP;
+                        gosWebIdentityTokenFile = osWebIdentityTokenFile;
+                    }
+                }
+            }
+
+            if( gosRoleArnWebIdentity.empty() )
+            {
+                // Get the credentials for the source profile, that will be
+                // used to sign the STS AssumedRole request.
+                if( !ReadAWSCredentials(osSourceProfile, osCredentials,
+                                        osSecretAccessKey,
+                                        osAccessKeyId,
+                                        osSessionToken) )
+                {
+                    VSIError(VSIE_AWSInvalidCredentials,
+                             "Cannot retrieve credentials for source profile %s",
+                             osSourceProfile.c_str());
+                    return false;
+                }
             }
 
             std::string osTempSecretAccessKey;
@@ -1427,8 +1655,24 @@ bool VSIS3HandleHelper::GetConfiguration(const std::string& osPathForOption,
         return true;
     }
 
+    if( CPLTestBool(CPLGetConfigOption("CPL_AWS_WEB_IDENTITY_ENABLE", "YES")) )
+    {
+        // WebIdentity method: use Web Identity Token
+        if( GetConfigurationFromAssumeRoleWithWebIdentity(/* bForceRefresh = */ false,
+                                                          osPathForOption,
+                                                          /* osRoleArnIn = */ std::string(),
+                                                          /* osWebIdentityTokenFileIn = */ std::string(),
+                                                          osSecretAccessKey, osAccessKeyId,
+                                                          osSessionToken) )
+        {
+            eCredentialsSource = AWSCredentialsSource::WEB_IDENTITY;
+            return true;
+        }
+    }
+
     // Last method: use IAM role security credentials on EC2 instances
-    if( GetConfigurationFromEC2(osPathForOption,
+    if( GetConfigurationFromEC2(/* bForceRefresh = */ false,
+                                osPathForOption,
                                 osSecretAccessKey, osAccessKeyId,
                                 osSessionToken) )
     {
@@ -1475,6 +1719,8 @@ void VSIS3HandleHelper::ClearCache()
     gosSourceProfileSecretAccessKey.clear();
     gosSourceProfileSessionToken.clear();
     gosRegion.clear();
+    gosRoleArnWebIdentity.clear();
+    gosWebIdentityTokenFile.clear();
 }
 
 /************************************************************************/
@@ -1507,16 +1753,16 @@ VSIS3HandleHelper* VSIS3HandleHelper::BuildFromURI( const char* pszURI,
     // " This variable overrides the default region of the in-use profile, if set."
     const CPLString osDefaultRegion = CSLFetchNameValueDef(
         papszOptions, "AWS_DEFAULT_REGION",
-        VSIGetCredential(osPathForOption.c_str(), "AWS_DEFAULT_REGION", ""));
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_DEFAULT_REGION", ""));
     if( !osDefaultRegion.empty() )
     {
         osRegion = osDefaultRegion;
     }
 
     const CPLString osEndpoint =
-        VSIGetCredential(osPathForOption.c_str(), "AWS_S3_ENDPOINT", "s3.amazonaws.com");
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_S3_ENDPOINT", "s3.amazonaws.com");
     const CPLString osRequestPayer =
-        VSIGetCredential(osPathForOption.c_str(), "AWS_REQUEST_PAYER", "");
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_REQUEST_PAYER", "");
     CPLString osBucket;
     CPLString osObjectKey;
     if( pszURI != nullptr && pszURI[0] != '\0' &&
@@ -1525,12 +1771,12 @@ VSIS3HandleHelper* VSIS3HandleHelper::BuildFromURI( const char* pszURI,
     {
         return nullptr;
     }
-    const bool bUseHTTPS = CPLTestBool(VSIGetCredential(osPathForOption.c_str(), "AWS_HTTPS", "YES"));
+    const bool bUseHTTPS = CPLTestBool(VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_HTTPS", "YES"));
     const bool bIsValidNameForVirtualHosting =
         osBucket.find('.') == std::string::npos;
     const bool bUseVirtualHosting = CPLTestBool(
         CSLFetchNameValueDef(papszOptions, "AWS_VIRTUAL_HOSTING",
-                VSIGetCredential(osPathForOption.c_str(), "AWS_VIRTUAL_HOSTING",
+                VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_VIRTUAL_HOSTING",
                            bIsValidNameForVirtualHosting ? "TRUE" : "FALSE")));
     return new VSIS3HandleHelper(osSecretAccessKey, osAccessKeyId,
                                  osSessionToken,
@@ -1599,6 +1845,61 @@ CPLString IVSIS3LikeHandleHelper::GetURLNoKVP() const
     return osURL;
 }
 
+/************************************************************************/
+/*                          RefreshCredentials()                        */
+/************************************************************************/
+
+void VSIS3HandleHelper::RefreshCredentials(const std::string& osPathForOption,
+                                           bool bForceRefresh) const
+{
+    if( m_eCredentialsSource == AWSCredentialsSource::EC2 )
+    {
+        CPLString osSecretAccessKey, osAccessKeyId, osSessionToken;
+        if( GetConfigurationFromEC2(bForceRefresh,
+                                    osPathForOption.c_str(),
+                                    osSecretAccessKey,
+                                    osAccessKeyId,
+                                    osSessionToken) )
+        {
+            m_osSecretAccessKey = osSecretAccessKey;
+            m_osAccessKeyId = osAccessKeyId;
+            m_osSessionToken = osSessionToken;
+        }
+    }
+    else if( m_eCredentialsSource == AWSCredentialsSource::ASSUMED_ROLE )
+    {
+        CPLString osSecretAccessKey, osAccessKeyId, osSessionToken;
+        CPLString osRegion;
+        if( GetOrRefreshTemporaryCredentialsForRole(bForceRefresh,
+                                                    osSecretAccessKey,
+                                                    osAccessKeyId,
+                                                    osSessionToken,
+                                                    osRegion) )
+        {
+            m_osSecretAccessKey = osSecretAccessKey;
+            m_osAccessKeyId = osAccessKeyId;
+            m_osSessionToken = osSessionToken;
+        }
+    }
+    else if( m_eCredentialsSource == AWSCredentialsSource::WEB_IDENTITY )
+    {
+        CPLString osSecretAccessKey, osAccessKeyId, osSessionToken;
+        CPLString osRegion;
+        if( GetConfigurationFromAssumeRoleWithWebIdentity(bForceRefresh,
+                                                          osPathForOption.c_str(),
+                                                          std::string(),
+                                                          std::string(),
+                                                          osSecretAccessKey,
+                                                          osAccessKeyId,
+                                                          osSessionToken) )
+        {
+            m_osSecretAccessKey = osSecretAccessKey;
+            m_osAccessKeyId = osAccessKeyId;
+            m_osSessionToken = osSessionToken;
+        }
+    }
+}
+
 /************************************************************************/
 /*                           GetCurlHeaders()                           */
 /************************************************************************/
@@ -1614,37 +1915,11 @@ VSIS3HandleHelper::GetCurlHeaders( const CPLString& osVerb,
     osPathForOption += '/';
     osPathForOption += m_osObjectKey;
 
-    if( m_eCredentialsSource == AWSCredentialsSource::EC2 )
-    {
-        CPLString osSecretAccessKey, osAccessKeyId, osSessionToken;
-        if( GetConfigurationFromEC2(osPathForOption.c_str(),
-                                    osSecretAccessKey,
-                                    osAccessKeyId,
-                                    osSessionToken) )
-        {
-            m_osSecretAccessKey = osSecretAccessKey;
-            m_osAccessKeyId = osAccessKeyId;
-            m_osSessionToken = osSessionToken;
-        }
-    }
-    else if( m_eCredentialsSource == AWSCredentialsSource::ASSUMED_ROLE )
-    {
-        CPLString osSecretAccessKey, osAccessKeyId, osSessionToken;
-        CPLString osRegion;
-        if( GetOrRefreshTemporaryCredentialsForRole(osSecretAccessKey,
-                                                    osAccessKeyId,
-                                                    osSessionToken,
-                                                    osRegion) )
-        {
-            m_osSecretAccessKey = osSecretAccessKey;
-            m_osAccessKeyId = osAccessKeyId;
-            m_osSessionToken = osSessionToken;
-        }
-    }
+    RefreshCredentials(osPathForOption, /* bForceRefresh = */ false);
 
-    CPLString osXAMZDate = VSIGetCredential(osPathForOption.c_str(), "AWS_TIMESTAMP", "");
+    CPLString osXAMZDate = VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_TIMESTAMP", "");
     if( osXAMZDate.empty() )
-        osXAMZDate = CPLGetAWS_SIGN4_Timestamp();
+        osXAMZDate = CPLGetAWS_SIGN4_Timestamp(time(nullptr));
 
     const CPLString osXAMZContentSHA256 =
         CPLGetLowerCaseHexSHA256(pabyDataContent, nBytesContent);
@@ -1671,6 +1946,7 @@ VSIS3HandleHelper::GetCurlHeaders( const CPLString& osVerb,
         CPLAWSURLEncode("/" + m_osBucket + "/" + m_osObjectKey, false).c_str(),
         osCanonicalQueryString,
         osXAMZContentSHA256,
+        true, // bAddHeaderAMZContentSHA256
         osXAMZDate);
 
     struct curl_slist *headers=nullptr;
@@ -1902,15 +2178,51 @@ CPLString VSIS3HandleHelper::GetSignedURL(CSLConstList papszOptions)
     osPathForOption += m_osObjectKey;
 
     CPLString osXAMZDate = CSLFetchNameValueDef(papszOptions, "START_DATE",
-        VSIGetCredential(osPathForOption.c_str(), "AWS_TIMESTAMP", ""));
+        VSIGetPathSpecificOption(osPathForOption.c_str(), "AWS_TIMESTAMP", ""));
     if( osXAMZDate.empty() )
-        osXAMZDate = CPLGetAWS_SIGN4_Timestamp();
+        osXAMZDate = CPLGetAWS_SIGN4_Timestamp(time(nullptr));
     CPLString osDate(osXAMZDate);
     osDate.resize(8);
 
     CPLString osXAMZExpires =
         CSLFetchNameValueDef(papszOptions, "EXPIRATION_DELAY", "3600");
 
+    if( m_eCredentialsSource != AWSCredentialsSource::REGULAR )
+    {
+        // For credentials that have an expiration, we must check their
+        // expiration compared to the expiration of the signed URL, since
+        // if the effective expiration is min(desired_expiration, credential_expiration)
+        // Cf https://aws.amazon.com/premiumsupport/knowledge-center/presigned-url-s3-bucket-expiration
+        int nYear, nMonth, nDay, nHour = 0, nMin = 0, nSec = 0;
+        if( sscanf(osXAMZDate, "%04d%02d%02dT%02d%02d%02dZ",
+                    &nYear, &nMonth, &nDay, &nHour, &nMin, &nSec) < 3 )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Bad format for START_DATE");
+            return CPLString();
+        }
+        struct tm brokendowntime;
+        brokendowntime.tm_year = nYear - 1900;
+        brokendowntime.tm_mon = nMonth - 1;
+        brokendowntime.tm_mday = nDay;
+        brokendowntime.tm_hour = nHour;
+        brokendowntime.tm_min = nMin;
+        brokendowntime.tm_sec = nSec;
+        const GIntBig nStartDate = CPLYMDHMSToUnixTime(&brokendowntime);
+
+        {
+            CPLMutexHolder oHolder( &ghMutex );
+
+            // Try to reuse credentials if they will still be valid after the
+            // desired end of the validity of the signed URL,
+            // with one minute of margin
+            if( nStartDate + CPLAtoGIntBig(osXAMZExpires.c_str()) >= gnGlobalExpiration - 60 )
+            {
+                RefreshCredentials(osPathForOption, /* bForceRefresh = */ true);
+            }
+        }
+    }
+
     CPLString osVerb(CSLFetchNameValueDef(papszOptions, "VERB", "GET"));
 
     ResetQueryParameters();
@@ -1919,6 +2231,8 @@ CPLString VSIS3HandleHelper::GetSignedURL(CSLConstList papszOptions)
         m_osAccessKeyId + "/" + osDate + "/" + m_osRegion + "/s3/aws4_request");
     AddQueryParameter("X-Amz-Date", osXAMZDate);
     AddQueryParameter("X-Amz-Expires", osXAMZExpires);
+    if( !m_osSessionToken.empty() )
+        AddQueryParameter("X-Amz-Security-Token", m_osSessionToken);
     AddQueryParameter("X-Amz-SignedHeaders", "host");
 
     CPLString osCanonicalQueryString(GetQueryString(true).substr(1));
@@ -1929,7 +2243,7 @@ CPLString VSIS3HandleHelper::GetSignedURL(CSLConstList papszOptions)
     const CPLString osSignature =
       CPLGetAWS_SIGN4_Signature(
         m_osSecretAccessKey,
-        m_osSessionToken,
+        std::string(), // sessionToken set to empty as we include it in query parameters
         m_osRegion,
         m_osRequestPayer,
         "s3",
