@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_odbc.h 658d4f6254f467214d228ee9513838e9b1f878f9 2021-08-27 15:00:59 +1000 Nyall Dawson $
+ * $Id$
  *
  * Project:  OGR ODBC Driver
  * Purpose:  Declarations for ODBC Access Cover API.
@@ -174,11 +174,14 @@ class CPL_DLL CPLODBCSession {
 
     CPL_DISALLOW_COPY_ASSIGN(CPLODBCSession)
 
+/*! @cond Doxygen_Suppress */
+  protected:
     CPLString m_osLastError{};
     HENV      m_hEnv = nullptr;
     HDBC      m_hDBC = nullptr;
     int       m_bInTransaction = false;
     int       m_bAutoCommit = true;
+/*! @endcond */
 
   public:
     CPLODBCSession();
@@ -225,6 +228,10 @@ class CPL_DLL CPLODBCStatement {
 
     CPL_DISALLOW_COPY_ASSIGN(CPLODBCStatement)
 
+/*! @cond Doxygen_Suppress */
+  protected:
+    int m_nFlags = 0;
+
     CPLODBCSession     *m_poSession = nullptr;
     HSTMT               m_hStmt = nullptr;
 
@@ -239,20 +246,46 @@ class CPL_DLL CPLODBCStatement {
 
     char         **m_papszColValues = nullptr;
     CPL_SQLLEN       *m_panColValueLengths = nullptr;
+    double        *m_padColValuesAsDouble = nullptr;
 
     int            Failed( int );
 
     char          *m_pszStatement = nullptr;
     size_t         m_nStatementMax = 0;
     size_t         m_nStatementLen = 0;
+/*! @endcond */
 
   public:
-    explicit CPLODBCStatement( CPLODBCSession * );
+
+    /**
+     * Flags which control ODBC statement behavior.
+     */
+    enum Flag
+    {
+       /**
+        * Numeric column values should be retrieved as doubles, using either the SQL_C_DOUBLE or SQL_C_FLOAT types.
+        *
+        * By default numeric column values are retrieved as characters. Retrieving as character is the safest behavior, but can risk loss
+        * of precision.
+        *
+        * If set, GetColDataAsDouble should be used for numeric columns instead of GetColData.
+        *
+        * Warning: this flag can expose issues in particular ODBC drivers on different platforms. Use with caution.
+        */
+       RetrieveNumericColumnsAsDouble = 1 << 0,
+    };
+
+    explicit CPLODBCStatement( CPLODBCSession *, int flags = 0 );
     ~CPLODBCStatement();
 
     /** Return statement handle */
     HSTMT          GetStatement() { return m_hStmt; }
 
+    /**
+     * Returns statement flags.
+     */
+    int            Flags() const { return m_nFlags; }
+
     // Command buffer related.
     void           Clear();
     void           AppendEscaped( const char * );
@@ -279,10 +312,14 @@ class CPL_DLL CPLODBCStatement {
     short          GetColNullable( int );
     const char    *GetColColumnDef( int );
 
-    int            GetColId( const char * );
+    int            GetColId( const char * ) const;
     const char    *GetColData( int, const char * = nullptr );
     const char    *GetColData( const char *, const char * = nullptr );
     int            GetColDataLength( int );
+
+    double         GetColDataAsDouble( int ) const;
+    double         GetColDataAsDouble( const char * ) const;
+
     int            GetRowCountAffected();
 
     // Fetch special metadata.
