@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_port.h 9b04a6ce329ade6272643ab59d0ffaabd313955c 2021-10-11 23:32:03 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  CPL - Common Portability Library
  * Author:   Frank Warmerdam, warmerdam@pobox.com
@@ -78,7 +78,7 @@
 #error "Unexpected value for SIZEOF_UNSIGNED_LONG"
 #endif
 
-#if !defined(SIZEOF_VOIDP) || (SIZEOF_VOIDP != 4 && SIZEOF_VOIDP != 8)
+#if !defined(SIZEOF_VOIDP)
 #error "Unexpected value for SIZEOF_VOIDP"
 #endif
 
@@ -144,9 +144,7 @@
 
 #include <time.h>
 
-#if defined(HAVE_ERRNO_H)
-#  include <errno.h>
-#endif
+#include <errno.h>
 
 #ifdef HAVE_LOCALE_H
 #  include <locale.h>
@@ -160,16 +158,6 @@
 #  include <strings.h>
 #endif
 
-#if defined(HAVE_LIBDBMALLOC) && defined(HAVE_DBMALLOC_H) && defined(DEBUG)
-#  define DBMALLOC
-#  include <dbmalloc.h>
-#endif
-
-#if !defined(DBMALLOC) && defined(HAVE_DMALLOC_H)
-#  define USE_DMALLOC
-#  include <dmalloc.h>
-#endif
-
 /* ==================================================================== */
 /*      Base portability stuff ... this stuff may need to be            */
 /*      modified for new platforms.                                     */
@@ -237,8 +225,6 @@ typedef int             GBool;
 /*      64bit support                                                   */
 /* -------------------------------------------------------------------- */
 
-#if HAVE_LONG_LONG
-
 /** Large signed integer type (generally 64-bit integer type).
  *  Use GInt64 when exactly 64 bit is needed */
 typedef long long        GIntBig;
@@ -271,13 +257,11 @@ typedef GUIntBig         GUInt64;
 /** Minimum GUInt64 value */
 #define GUINT64_MAX     GUINTBIG_MAX
 
-#else
-
-#error "64bit integer support required"
-
-#endif
-
-#if SIZEOF_VOIDP == 8
+#if SIZEOF_VOIDP > 8
+#include <stddef.h> // ptrdiff_t
+/** Integer type large enough to hold the difference between 2 addresses */
+typedef ptrdiff_t        GPtrDiff_t;
+#elif SIZEOF_VOIDP == 8
 /** Integer type large enough to hold the difference between 2 addresses */
 typedef GIntBig          GPtrDiff_t;
 #else
@@ -286,26 +270,17 @@ typedef int              GPtrDiff_t;
 #endif
 
 #ifdef GDAL_COMPILATION
-#if HAVE_UINTPTR_T
 #include <stdint.h>
 typedef uintptr_t GUIntptr_t;
-#elif SIZEOF_VOIDP == 8
-typedef GUIntBig GUIntptr_t;
-#else
-typedef unsigned int  GUIntptr_t;
-#endif
-
 #define CPL_IS_ALIGNED(ptr, quant) ((CPL_REINTERPRET_CAST(GUIntptr_t, CPL_STATIC_CAST(const void*, ptr)) % (quant)) == 0)
 
 #endif
 
-#if defined(__MSVCRT__) || (defined(WIN32) && defined(_MSC_VER))
+#if (defined(__MSVCRT__) && !(defined(__MINGW64__) && __GNUC__ >= 10)) || (defined(WIN32) && defined(_MSC_VER))
   #define CPL_FRMT_GB_WITHOUT_PREFIX     "I64"
-#elif HAVE_LONG_LONG
+#else
 /** Printf formatting suffix for GIntBig */
   #define CPL_FRMT_GB_WITHOUT_PREFIX     "ll"
-#else
-  #define CPL_FRMT_GB_WITHOUT_PREFIX     "l"
 #endif
 
 /** Printf formatting for GIntBig */
@@ -1033,86 +1008,6 @@ inline static bool CPL_TO_BOOL(int x) { return x != 0; }
 #endif
 
 /*! @cond Doxygen_Suppress */
-// Define DEBUG_BOOL to compile in "MSVC mode", ie error out when
-// a integer is assigned to a bool
-// WARNING: use only at compilation time, since it is know to not work
-//  at runtime for unknown reasons (crash in MongoDB driver for example)
-#if defined(__cplusplus) && defined(DEBUG_BOOL) && !defined(DO_NOT_USE_DEBUG_BOOL) && !defined(CPL_SUPRESS_CPLUSPLUS)
-extern "C++" {
-class MSVCPedanticBool
-{
-
-        friend bool operator== (const bool& one, const MSVCPedanticBool& other);
-        friend bool operator!= (const bool& one, const MSVCPedanticBool& other);
-
-        bool b;
-        MSVCPedanticBool(int bIn);
-
-    public:
-        /* b not initialized on purpose in default ctor to flag use. */
-        /* cppcheck-suppress uninitMemberVar */
-        MSVCPedanticBool() {}
-        MSVCPedanticBool(bool bIn) : b(bIn) {}
-        MSVCPedanticBool(const MSVCPedanticBool& other) : b(other.b) {}
-
-        MSVCPedanticBool& operator= (const MSVCPedanticBool& other) { b = other.b; return *this; }
-        MSVCPedanticBool& operator&= (const MSVCPedanticBool& other) { b &= other.b; return *this; }
-        MSVCPedanticBool& operator|= (const MSVCPedanticBool& other) { b |= other.b; return *this; }
-
-        bool operator== (const bool& other) const { return b == other; }
-        bool operator!= (const bool& other) const { return b != other; }
-        bool operator< (const bool& other) const { return b < other; }
-        bool operator== (const MSVCPedanticBool& other) const { return b == other.b; }
-        bool operator!= (const MSVCPedanticBool& other) const { return b != other.b; }
-        bool operator< (const MSVCPedanticBool& other) const { return b < other.b; }
-
-        bool operator! () const { return !b; }
-        operator bool() const { return b; }
-        operator int() const { return b; }
-        operator GIntBig() const { return b; }
-};
-
-inline bool operator== (const bool& one, const MSVCPedanticBool& other) { return one == other.b; }
-inline bool operator!= (const bool& one, const MSVCPedanticBool& other) { return one != other.b; }
-
-/* We must include all C++ stuff before to avoid issues with templates that use bool */
-#include <vector>
-#include <map>
-#include <set>
-#include <string>
-#include <cstddef>
-#include <limits>
-#include <sstream>
-#include <fstream>
-#include <algorithm>
-#include <functional>
-#include <memory>
-#include <queue>
-#include <mutex>
-#include <unordered_map>
-#include <thread>
-#include <unordered_set>
-#include <complex>
-#include <iomanip>
-
-} /* extern C++ */
-
-#undef FALSE
-#define FALSE false
-#undef TRUE
-#define TRUE true
-
-/* In the very few cases we really need a "simple" type, fallback to bool */
-#define EMULATED_BOOL int
-
-/* Use our class instead of bool */
-#define bool MSVCPedanticBool
-
-/* "volatile bool" with the below substitution doesn't really work. */
-/* Just for the sake of the debug, we don't really need volatile */
-#define VOLATILE_BOOL bool
-
-#else /* defined(__cplusplus) && defined(DEBUG_BOOL) */
 
 #ifndef FALSE
 #  define FALSE 0
@@ -1122,11 +1017,6 @@ inline bool operator!= (const bool& one, const MSVCPedanticBool& other) { return
 #  define TRUE 1
 #endif
 
-#define EMULATED_BOOL bool
-#define VOLATILE_BOOL volatile bool
-
-#endif /* defined(__cplusplus) && defined(DEBUG_BOOL) */
-
 #if __clang_major__ >= 4 || (__clang_major__ == 3 && __clang_minor__ >= 8)
 #define CPL_NOSANITIZE_UNSIGNED_INT_OVERFLOW __attribute__((no_sanitize("unsigned-integer-overflow")))
 #else
@@ -1144,9 +1034,6 @@ inline C CPLUnsanitizedAdd(A a, B b)
 }
 #endif
 
-/*! @endcond */
-
-/*! @cond Doxygen_Suppress */
 #if defined(__cplusplus) && !defined(CPL_SUPRESS_CPLUSPLUS)
 #define CPL_NULLPTR nullptr
 #else
