@@ -27,7 +27,6 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id$")
 
 #ifdef CPL_RECODE_ICONV
 
@@ -61,6 +60,26 @@ void CPLClearRecodeIconvWarningFlags()
     bHaveWarned2 = false;
 }
 
+/************************************************************************/
+/*                      CPLFixInputEncoding()                           */
+/************************************************************************/
+
+static const char* CPLFixInputEncoding(const char *pszSrcEncoding,
+                                       int nFirstByteVal)
+{
+    // iconv on Alpine Linux seems to assume BE order, when it is not explicit
+    if( EQUAL(pszSrcEncoding, CPL_ENC_UCS2) )
+        pszSrcEncoding = "UCS-2LE";
+    else if( EQUAL(pszSrcEncoding, CPL_ENC_UTF16) &&
+             nFirstByteVal != 0xFF &&
+             nFirstByteVal != 0xFE )
+    {
+        // Only force UTF-16LE if there's no starting endianness marker
+        pszSrcEncoding = "UTF-16LE";
+    }
+    return pszSrcEncoding;
+}
+
 /************************************************************************/
 /*                          CPLRecodeIconv()                            */
 /************************************************************************/
@@ -83,6 +102,9 @@ char *CPLRecodeIconv( const char *pszSource,
                       const char *pszDstEncoding )
 
 {
+    pszSrcEncoding = CPLFixInputEncoding(pszSrcEncoding,
+                                         static_cast<unsigned char>(pszSource[0]));
+
     iconv_t sConv;
 
     sConv = iconv_open( pszDstEncoding, pszSrcEncoding );
@@ -197,6 +219,8 @@ char *CPLRecodeFromWCharIconv( const wchar_t *pwszSource,
                                const char *pszDstEncoding )
 
 {
+    pszSrcEncoding = CPLFixInputEncoding(pszSrcEncoding, pwszSource[0]);
+
 /* -------------------------------------------------------------------- */
 /*      What is the source length.                                      */
 /* -------------------------------------------------------------------- */
