@@ -37,6 +37,7 @@
 #endif
 
 #include <algorithm>
+#include <limits>
 #include <map>
 #include <memory>
 #include <utility>
@@ -45,44 +46,12 @@
 #include "cpl_error.h"
 #include "cpl_vsi.h"
 #include "cpl_vsi_virtual.h"
+#include "cpl_mem_cache.h"
+#include "cpl_noncopyablevector.h"
 
 //! @cond Doxygen_Suppress
 
-CPL_CVSID("$Id: cpl_vsil_cache.cpp d4c2d0da2a653242903804163f22189561a0b7e4 2021-09-28 19:24:15 +0200 Even Rouault $")
 
-/************************************************************************/
-/* ==================================================================== */
-/*                             VSICacheChunk                            */
-/* ==================================================================== */
-/************************************************************************/
-
-class VSICacheChunk
-{
-    CPL_DISALLOW_COPY_ASSIGN(VSICacheChunk)
-
-public:
-    VSICacheChunk() = default;
-
-    virtual ~VSICacheChunk()
-    {
-        VSIFree( pabyData );
-    }
-
-    bool Allocate( size_t nChunkSize )
-    {
-        CPLAssert( pabyData == nullptr );
-        pabyData = static_cast<GByte *>(VSIMalloc( nChunkSize ));
-        return (pabyData != nullptr);
-    }
-
-    vsi_l_offset   iBlock = 0;
-
-    VSICacheChunk *poLRUPrev = nullptr;
-    VSICacheChunk *poLRUNext = nullptr;
-
-    size_t          nDataFilled = 0;
-    GByte          *pabyData = nullptr;
-};
 
 /************************************************************************/
 /* ==================================================================== */
@@ -100,27 +69,18 @@ class VSICachedFile final : public VSIVirtualHandle
                    size_t nCacheSize );
     ~VSICachedFile() override { VSICachedFile::Close(); }
 
-    void          FlushLRU();
-    int           LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
+    bool         LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
                               void *pBuffer, size_t nBufferSize );
-    void          Demote( VSICacheChunk * );
 
-    VSIVirtualHandle *poBase = nullptr;
+    std::unique_ptr<VSIVirtualHandle> m_poBase{};
 
-    vsi_l_offset  nOffset = 0;
-    vsi_l_offset  nFileSize = 0;
-
-    GUIntBig      nCacheUsed = 0;
-    GUIntBig      nCacheMax = 0;
+    vsi_l_offset  m_nOffset = 0;
+    vsi_l_offset  m_nFileSize = 0;
 
     size_t        m_nChunkSize = 0;
+    lru11::Cache<vsi_l_offset, cpl::NonCopyableVector<GByte>> m_oCache; // can only been initialized in constructor
 
-    VSICacheChunk *poLRUStart = nullptr;
-    VSICacheChunk *poLRUEnd = nullptr;
-
-    std::map<vsi_l_offset, std::unique_ptr<VSICacheChunk>> oMapOffsetToCache{};
-
-    bool           bEOF = false;
+    bool           m_bEOF = false;
 
     int Seek( vsi_l_offset nOffset, int nWhence ) override;
     vsi_l_offset Tell() override;
@@ -136,25 +96,47 @@ class VSICachedFile final : public VSIVirtualHandle
     int Flush() override;
     int Close() override;
     void *GetNativeFileDescriptor() override
-        { return poBase->GetNativeFileDescriptor(); }
+        { return m_poBase->GetNativeFileDescriptor(); }
+
+    bool HasPRead() const override { return m_poBase->HasPRead(); }
+    size_t PRead( void* pBuffer, size_t nSize, vsi_l_offset nOffset ) const override
+        { return m_poBase->PRead(pBuffer, nSize, nOffset); }
 };
 
+/************************************************************************/
+/*                           GetCacheMax()                              */
+/************************************************************************/
+
+static size_t GetCacheMax(size_t nCacheSize)
+{
+    return nCacheSize ? nCacheSize :
+        static_cast<size_t>(std::min(
+            static_cast<GUIntBig>(std::numeric_limits<size_t>::max() / 2),
+            CPLScanUIntBig(
+                 CPLGetConfigOption( "VSI_CACHE_SIZE", "25000000" ), 40 )));
+}
+
+/************************************************************************/
+/*                           DIV_ROUND_UP()                             */
+/************************************************************************/
+
+template<class T> inline T DIV_ROUND_UP(T a, T b)
+{
+    return a / b + (((a % b) == 0) ? 0 : 1);
+}
+
 /************************************************************************/
 /*                           VSICachedFile()                            */
 /************************************************************************/
 
 VSICachedFile::VSICachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
                               size_t nCacheSize ) :
-    poBase(poBaseHandle),
-    nCacheMax(nCacheSize),
-    m_nChunkSize(nChunkSize)
+    m_poBase(poBaseHandle),
+    m_nChunkSize(nChunkSize ? nChunkSize : 32768),
+    m_oCache{DIV_ROUND_UP(GetCacheMax(nCacheSize), m_nChunkSize), 0}
 {
-    if( nCacheSize == 0 )
-        nCacheMax = CPLScanUIntBig(
-             CPLGetConfigOption( "VSI_CACHE_SIZE", "25000000" ), 40 );
-
-    poBase->Seek( 0, SEEK_END );
-    nFileSize = poBase->Tell();
+    m_poBase->Seek( 0, SEEK_END );
+    m_nFileSize = m_poBase->Tell();
 }
 
 /************************************************************************/
@@ -164,18 +146,12 @@ VSICachedFile::VSICachedFile( VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
 int VSICachedFile::Close()
 
 {
-    oMapOffsetToCache.clear();
+    m_oCache.clear();
 
-    poLRUStart = nullptr;
-    poLRUEnd = nullptr;
-
-    nCacheUsed = 0;
-
-    if( poBase )
+    if( m_poBase )
     {
-        poBase->Close();
-        delete poBase;
-        poBase = nullptr;
+        m_poBase->Close();
+        m_poBase.reset();
     }
 
     return 0;
@@ -188,7 +164,7 @@ int VSICachedFile::Close()
 int VSICachedFile::Seek( vsi_l_offset nReqOffset, int nWhence )
 
 {
-    bEOF = false;
+    m_bEOF = false;
 
     if( nWhence == SEEK_SET )
     {
@@ -196,14 +172,14 @@ int VSICachedFile::Seek( vsi_l_offset nReqOffset, int nWhence )
     }
     else if( nWhence == SEEK_CUR )
     {
-        nReqOffset += nOffset;
+        nReqOffset += m_nOffset;
     }
     else if( nWhence == SEEK_END )
     {
-        nReqOffset += nFileSize;
+        nReqOffset += m_nFileSize;
     }
 
-    nOffset = nReqOffset;
+    m_nOffset = nReqOffset;
 
     return 0;
 }
@@ -215,68 +191,7 @@ int VSICachedFile::Seek( vsi_l_offset nReqOffset, int nWhence )
 vsi_l_offset VSICachedFile::Tell()
 
 {
-    return nOffset;
-}
-
-/************************************************************************/
-/*                              FlushLRU()                              */
-/************************************************************************/
-
-void VSICachedFile::FlushLRU()
-
-{
-    CPLAssert( poLRUStart != nullptr );
-
-    VSICacheChunk *poBlock = poLRUStart;
-
-    CPLAssert( nCacheUsed >= poBlock->nDataFilled );
-
-    nCacheUsed -= poBlock->nDataFilled;
-
-    poLRUStart = poBlock->poLRUNext;
-    if( poLRUEnd == poBlock )
-        poLRUEnd = nullptr;
-
-    if( poBlock->poLRUNext != nullptr )
-        poBlock->poLRUNext->poLRUPrev = nullptr;
-
-    oMapOffsetToCache.erase(oMapOffsetToCache.find(poBlock->iBlock));
-}
-
-/************************************************************************/
-/*                               Demote()                               */
-/*                                                                      */
-/*      Demote the indicated block to the end of the LRU list.          */
-/*      Potentially integrate the link into the list if it is not       */
-/*      already there.                                                  */
-/************************************************************************/
-
-void VSICachedFile::Demote( VSICacheChunk *poBlock )
-
-{
-    // Already at end?
-    if( poLRUEnd == poBlock )
-        return;
-
-    if( poLRUStart == poBlock )
-        poLRUStart = poBlock->poLRUNext;
-
-    if( poBlock->poLRUPrev != nullptr )
-        poBlock->poLRUPrev->poLRUNext = poBlock->poLRUNext;
-
-    if( poBlock->poLRUNext != nullptr )
-        poBlock->poLRUNext->poLRUPrev = poBlock->poLRUPrev;
-
-    poBlock->poLRUNext = nullptr;
-    poBlock->poLRUPrev = nullptr;
-
-    if( poLRUEnd != nullptr )
-        poLRUEnd->poLRUNext = poBlock;
-
-    poLRUEnd = poBlock;
-
-    if( poLRUStart == nullptr )
-        poLRUStart = poBlock;
+    return m_nOffset;
 }
 
 /************************************************************************/
@@ -284,16 +199,14 @@ void VSICachedFile::Demote( VSICacheChunk *poBlock )
 /*                                                                      */
 /*      Load the desired set of blocks.  Use pBuffer as a temporary     */
 /*      buffer if it would be helpful.                                  */
-/*                                                                      */
-/*  RETURNS: TRUE on success; FALSE on failure.                         */
 /************************************************************************/
 
-int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
-                               void *pBuffer, size_t nBufferSize )
+bool VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
+                                void *pBuffer, size_t nBufferSize )
 
 {
     if( nBlockCount == 0 )
-        return TRUE;
+        return true;
 
 /* -------------------------------------------------------------------- */
 /*      When we want to load only one block, we can directly load it    */
@@ -301,28 +214,31 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
 /* -------------------------------------------------------------------- */
     if( nBlockCount == 1 )
     {
-        poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
-                      SEEK_SET );
-
-        auto poBlock = std::unique_ptr<VSICacheChunk>(new VSICacheChunk());
-        if( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
+        if( m_poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
+                          SEEK_SET ) != 0 )
         {
-            return FALSE;
+            return false;
         }
 
-        poBlock->iBlock = nStartBlock;
-        poBlock->nDataFilled =
-            poBase->Read( poBlock->pabyData, 1, m_nChunkSize );
-        if( poBlock->nDataFilled == 0 )
-            return FALSE;
-        nCacheUsed += poBlock->nDataFilled;
+        try
+        {
+            cpl::NonCopyableVector<GByte> oData(m_nChunkSize);
+            const auto nDataRead =
+                m_poBase->Read( oData.data(), 1, m_nChunkSize );
+            if( nDataRead == 0 )
+                return false;
+            oData.resize(nDataRead);
 
-        // Merges into the LRU list.
-        Demote( poBlock.get() );
+            m_oCache.insert(nStartBlock, std::move(oData));
+        }
+        catch( const std::exception & )
+        {
+            CPLError(CE_Failure, CPLE_OutOfMemory,
+                     "Out of memory situation in VSICachedFile::LoadBlocks()");
+            return false;
+        }
 
-        oMapOffsetToCache[nStartBlock] = std::move(poBlock);
-
-        return TRUE;
+        return true;
     }
 
 /* -------------------------------------------------------------------- */
@@ -335,14 +251,14 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
         && nBufferSize < nBlockCount * m_nChunkSize )
     {
         if( !LoadBlocks( nStartBlock, 2, pBuffer, nBufferSize ) )
-            return FALSE;
+            return false;
 
         return LoadBlocks( nStartBlock+2, nBlockCount-2, pBuffer, nBufferSize );
     }
 
-    if( poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
+    if( m_poBase->Seek( static_cast<vsi_l_offset>(nStartBlock) * m_nChunkSize,
                       SEEK_SET ) != 0 )
-        return FALSE;
+        return false;
 
 /* -------------------------------------------------------------------- */
 /*      Do we need to allocate our own buffer?                          */
@@ -350,51 +266,53 @@ int VSICachedFile::LoadBlocks( vsi_l_offset nStartBlock, size_t nBlockCount,
     GByte *pabyWorkBuffer = static_cast<GByte *>(pBuffer);
 
     if( nBufferSize < m_nChunkSize * nBlockCount )
+    {
         pabyWorkBuffer =
-            static_cast<GByte *>( CPLMalloc(m_nChunkSize * nBlockCount) );
+            static_cast<GByte *>( VSI_MALLOC_VERBOSE(m_nChunkSize * nBlockCount) );
+        if( pabyWorkBuffer == nullptr )
+            return false;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Read the whole request into the working buffer.                 */
 /* -------------------------------------------------------------------- */
 
     const size_t nDataRead =
-        poBase->Read( pabyWorkBuffer, 1, nBlockCount*m_nChunkSize);
+        m_poBase->Read( pabyWorkBuffer, 1, nBlockCount*m_nChunkSize);
 
     if( nBlockCount * m_nChunkSize > nDataRead + m_nChunkSize - 1 )
         nBlockCount = (nDataRead + m_nChunkSize - 1) / m_nChunkSize;
 
+    bool ret = true;
     for( size_t i = 0; i < nBlockCount; i++ )
     {
         const vsi_l_offset iBlock = nStartBlock + i;
 
-        auto poBlock = std::unique_ptr<VSICacheChunk>(new VSICacheChunk());
-        if( !poBlock || !poBlock->Allocate( m_nChunkSize ) )
+        const auto nDataFilled = ( nDataRead >= (i+1) * m_nChunkSize ) ?
+            m_nChunkSize :  nDataRead - i*m_nChunkSize;
+        try
         {
-            return FALSE;
+            cpl::NonCopyableVector<GByte> oData(nDataFilled);
+
+            memcpy( oData.data(), pabyWorkBuffer + i*m_nChunkSize,
+                    nDataFilled );
+
+            m_oCache.insert(iBlock, std::move(oData));
+        }
+        catch( const std::exception & )
+        {
+            CPLError(CE_Failure, CPLE_OutOfMemory,
+                     "Out of memory situation in VSICachedFile::LoadBlocks()");
+            ret = false;
+            break;
         }
 
-        poBlock->iBlock = iBlock;
-
-        if( nDataRead >= (i+1) * m_nChunkSize )
-            poBlock->nDataFilled = m_nChunkSize;
-        else
-            poBlock->nDataFilled = nDataRead - i*m_nChunkSize;
-
-        memcpy( poBlock->pabyData, pabyWorkBuffer + i*m_nChunkSize,
-                poBlock->nDataFilled );
-
-        nCacheUsed += poBlock->nDataFilled;
-
-        // Merges into the LRU list.
-        Demote( poBlock.get() );
-
-        oMapOffsetToCache[iBlock] = std::move(poBlock);
     }
 
     if( pabyWorkBuffer != pBuffer )
         CPLFree( pabyWorkBuffer );
 
-    return TRUE;
+    return ret;
 }
 
 /************************************************************************/
@@ -410,27 +328,26 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
 
     // nFileSize might be set wrongly to 0 by underlying layers, such as
     // /vsicurl_streaming/https://query.data.world/s/jgsghstpphjhicstradhy5kpjwrnfy
-    if( nFileSize > 0 && nOffset >= nFileSize )
+    if( m_nFileSize > 0 && m_nOffset >= m_nFileSize )
     {
-        bEOF = true;
+        m_bEOF = true;
         return 0;
     }
 
 /* ==================================================================== */
 /*      Make sure the cache is loaded for the whole request region.     */
 /* ==================================================================== */
-    const vsi_l_offset nStartBlock = nOffset / m_nChunkSize;
+    const vsi_l_offset nStartBlock = m_nOffset / m_nChunkSize;
     const vsi_l_offset nEndBlock =
-        (nOffset + nRequestedBytes - 1) / m_nChunkSize;
+        (m_nOffset + nRequestedBytes - 1) / m_nChunkSize;
 
     for( vsi_l_offset iBlock = nStartBlock; iBlock <= nEndBlock; iBlock++ )
     {
-        auto oIter = oMapOffsetToCache.find(iBlock);
-        if( oIter == oMapOffsetToCache.end() )
+        if( !m_oCache.contains(iBlock) )
         {
             size_t nBlocksToLoad = 1;
             while( iBlock + nBlocksToLoad <= nEndBlock &&
-                   oMapOffsetToCache.find(iBlock + nBlocksToLoad) == oMapOffsetToCache.end() )
+                   !m_oCache.contains(iBlock + nBlocksToLoad) )
             {
                 nBlocksToLoad++;
             }
@@ -446,54 +363,47 @@ size_t VSICachedFile::Read( void * pBuffer, size_t nSize, size_t nCount )
 
     while( nAmountCopied < nRequestedBytes )
     {
-        const vsi_l_offset iBlock = (nOffset + nAmountCopied) / m_nChunkSize;
-        auto oIter = oMapOffsetToCache.find(iBlock);
-        if( oIter == oMapOffsetToCache.end() )
+        const vsi_l_offset iBlock = (m_nOffset + nAmountCopied) / m_nChunkSize;
+        const cpl::NonCopyableVector<GByte>* poData = m_oCache.getPtr(iBlock);
+        if( poData == nullptr )
         {
             // We can reach that point when the amount to read exceeds
             // the cache size.
             LoadBlocks(iBlock, 1,
                        static_cast<GByte *>(pBuffer) + nAmountCopied,
                        std::min(nRequestedBytes - nAmountCopied, m_nChunkSize));
-            oIter = oMapOffsetToCache.find(iBlock);
-            if( oIter == oMapOffsetToCache.end() )
+            poData = m_oCache.getPtr(iBlock);
+            if( poData == nullptr )
             {
                 break;
             }
         }
 
-        VSICacheChunk* poBlock = oIter->second.get();
         const vsi_l_offset nStartOffset =
             static_cast<vsi_l_offset>(iBlock) * m_nChunkSize;
-        if(nStartOffset + poBlock->nDataFilled < nAmountCopied + nOffset)
+        if(nStartOffset + poData->size() < nAmountCopied + m_nOffset)
             break;
         const size_t nThisCopy = std::min(
             nRequestedBytes - nAmountCopied,
             static_cast<size_t>(
-                ((nStartOffset + poBlock->nDataFilled)
-                 - nAmountCopied - nOffset)));
+                ((nStartOffset + poData->size())
+                 - nAmountCopied - m_nOffset)));
         if( nThisCopy == 0 )
             break;
 
         memcpy( static_cast<GByte *>(pBuffer) + nAmountCopied,
-                poBlock->pabyData
-                + (nOffset + nAmountCopied) - nStartOffset,
+                poData->data()
+                + (m_nOffset + nAmountCopied) - nStartOffset,
                 nThisCopy );
 
         nAmountCopied += nThisCopy;
     }
 
-    nOffset += nAmountCopied;
-
-/* -------------------------------------------------------------------- */
-/*      Ensure the cache is reduced to our limit.                       */
-/* -------------------------------------------------------------------- */
-    while( nCacheUsed > nCacheMax )
-        FlushLRU();
+    m_nOffset += nAmountCopied;
 
     const size_t nRet = nAmountCopied / nSize;
     if( nRet != nCount )
-        bEOF = true;
+        m_bEOF = true;
     return nRet;
 }
 
@@ -506,7 +416,7 @@ int VSICachedFile::ReadMultiRange( int const nRanges, void ** const ppData,
                                    const size_t* const panSizes )
 {
     // If the base is /vsicurl/
-    return poBase->ReadMultiRange( nRanges, ppData, panOffsets, panSizes );
+    return m_poBase->ReadMultiRange( nRanges, ppData, panOffsets, panSizes );
 }
 
 /************************************************************************/
@@ -527,7 +437,7 @@ size_t VSICachedFile::Write( const void * /* pBuffer */,
 int VSICachedFile::Eof()
 
 {
-    return bEOF;
+    return m_bEOF;
 }
 
 /************************************************************************/
