@@ -41,7 +41,6 @@
 
 #include "cpl_google_cloud.h"
 
-CPL_CVSID("$Id$")
 
 #ifndef HAVE_CURL
 
@@ -99,6 +98,9 @@ class VSIGSFSHandler final : public IVSIS3LikeFSHandler
     // Multipart upload
     bool SupportsParallelMultipartUpload() const override { return true; }
 
+    bool SupportsSequentialWrite( const char* /* pszPath */, bool /* bAllowLocalTempFile */ ) override { return true; }
+    bool SupportsRandomWrite( const char* /* pszPath */, bool /* bAllowLocalTempFile */ ) override;
+
     char** GetFileMetadata( const char * pszFilename, const char* pszDomain,
                             CSLConstList papszOptions ) override;
 
@@ -182,8 +184,7 @@ VSIVirtualHandle* VSIGSFSHandler::Open( const char *pszFilename,
 
     if( strchr(pszAccess, 'w') != nullptr || strchr(pszAccess, 'a') != nullptr )
     {
-        if( strchr(pszAccess, '+') != nullptr &&
-            !CPLTestBool(CPLGetConfigOption("CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE", "NO")) )
+        if( strchr(pszAccess, '+') != nullptr && !SupportsRandomWrite(pszFilename, true) )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                         "w+ not supported for /vsigs, unless "
@@ -215,6 +216,16 @@ VSIVirtualHandle* VSIGSFSHandler::Open( const char *pszFilename,
         VSICurlFilesystemHandlerBase::Open(pszFilename, pszAccess, bSetError, papszOptions);
 }
 
+/************************************************************************/
+/*                        SupportsRandomWrite()                         */
+/************************************************************************/
+
+bool VSIGSFSHandler::SupportsRandomWrite( const char* /* pszPath */, bool bAllowLocalTempFile )
+{
+    return bAllowLocalTempFile &&
+           CPLTestBool(CPLGetConfigOption("CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE", "NO"));
+}
+
 /************************************************************************/
 /*                           GetOptions()                               */
 /************************************************************************/
