@@ -42,7 +42,6 @@
 
 #include "cpl_swift.h"
 
-CPL_CVSID("$Id: cpl_vsil_swift.cpp b0fb9dd96e27fa263058223b86e8289f5264c3f3 2021-08-31 21:44:33 +0200 Even Rouault $")
 
 #ifndef HAVE_CURL
 
@@ -58,6 +57,8 @@ void VSIInstallSwiftFileHandler( void )
 
 #define ENABLE_DEBUG 0
 
+#define unchecked_curl_easy_setopt(handle,opt,param) CPL_IGNORE_RET_VAL(curl_easy_setopt(handle,opt,param))
+
 namespace cpl {
 
 /************************************************************************/
@@ -257,6 +258,9 @@ public:
         const char* GetOptions() override;
 
         std::string GetStreamingFilename(const std::string& osFilename) const override { return osFilename; }
+
+        bool SupportsSequentialWrite( const char* /* pszPath */, bool /* bAllowLocalTempFile */ ) override { return true; }
+        bool SupportsRandomWrite( const char* /* pszPath */, bool /* bAllowLocalTempFile */ ) override;
 };
 
 /************************************************************************/
@@ -273,7 +277,7 @@ class VSISwiftHandle final : public IVSIS3LikeHandle
     struct curl_slist* GetCurlHeaders(
         const CPLString& osVerb,
         const struct curl_slist* psExistingHeaders ) override;
-    virtual bool Authenticate() override;
+    virtual bool Authenticate(const char* pszFilename) override;
 
   public:
     VSISwiftHandle( VSISwiftFSHandler* poFS,
@@ -296,8 +300,7 @@ VSIVirtualHandle* VSISwiftFSHandler::Open( const char *pszFilename,
 
     if( strchr(pszAccess, 'w') != nullptr || strchr(pszAccess, 'a') != nullptr )
     {
-        if( strchr(pszAccess, '+') != nullptr &&
-            !CPLTestBool(CPLGetConfigOption("CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE", "NO")) )
+        if( strchr(pszAccess, '+') != nullptr && !SupportsRandomWrite(pszFilename, true) )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                         "w+ not supported for /vsiswift, unless "
@@ -330,6 +333,16 @@ VSIVirtualHandle* VSISwiftFSHandler::Open( const char *pszFilename,
         VSICurlFilesystemHandlerBase::Open(pszFilename, pszAccess, bSetError, papszOptions);
 }
 
+/************************************************************************/
+/*                        SupportsRandomWrite()                         */
+/************************************************************************/
+
+bool VSISwiftFSHandler::SupportsRandomWrite( const char* /* pszPath */, bool bAllowLocalTempFile )
+{
+    return bAllowLocalTempFile &&
+           CPLTestBool(CPLGetConfigOption("CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE", "NO"));
+}
+
 /************************************************************************/
 /*                       ~VSISwiftFSHandler()                           */
 /************************************************************************/
@@ -607,27 +620,27 @@ char** VSISwiftFSHandler::GetFileList( const char *pszDirname,
             struct curl_slist* headers =
                 VSICurlSetOptions(hCurlHandle, poS3HandleHelper->GetURL(), nullptr);
             // Disable automatic redirection
-            curl_easy_setopt(hCurlHandle, CURLOPT_FOLLOWLOCATION, 0 );
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_FOLLOWLOCATION, 0 );
 
-            curl_easy_setopt(hCurlHandle, CURLOPT_RANGE, nullptr);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_RANGE, nullptr);
 
             VSICURLInitWriteFuncStruct(&sWriteFuncData, nullptr, nullptr, nullptr);
-            curl_easy_setopt(hCurlHandle, CURLOPT_WRITEDATA, &sWriteFuncData);
-            curl_easy_setopt(hCurlHandle, CURLOPT_WRITEFUNCTION,
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_WRITEDATA, &sWriteFuncData);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_WRITEFUNCTION,
                             VSICurlHandleWriteFunc);
 
             WriteFuncStruct sWriteFuncHeaderData;
             VSICURLInitWriteFuncStruct(&sWriteFuncHeaderData, nullptr, nullptr, nullptr);
-            curl_easy_setopt(hCurlHandle, CURLOPT_HEADERDATA, &sWriteFuncHeaderData);
-            curl_easy_setopt(hCurlHandle, CURLOPT_HEADERFUNCTION,
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HEADERDATA, &sWriteFuncHeaderData);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HEADERFUNCTION,
                             VSICurlHandleWriteFunc);
 
             char szCurlErrBuf[CURL_ERROR_SIZE+1] = {};
-            curl_easy_setopt(hCurlHandle, CURLOPT_ERRORBUFFER, szCurlErrBuf );
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_ERRORBUFFER, szCurlErrBuf );
 
             headers = VSICurlMergeHeaders(headers,
                                 poS3HandleHelper->GetCurlHeaders("GET", headers));
-            curl_easy_setopt(hCurlHandle, CURLOPT_HTTPHEADER, headers);
+            unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_HTTPHEADER, headers);
 
             MultiPerform(hCurlMultiHandle, hCurlHandle);
 
@@ -745,9 +758,9 @@ struct curl_slist* VSISwiftHandle::GetCurlHeaders( const CPLString& osVerb,
 /*                           Authenticate()                             */
 /************************************************************************/
 
-bool VSISwiftHandle::Authenticate()
+bool VSISwiftHandle::Authenticate(const char* pszFilename)
 {
-    return m_poHandleHelper->Authenticate();
+    return m_poHandleHelper->Authenticate(pszFilename);
 }
 
 
@@ -762,13 +775,15 @@ bool VSISwiftHandle::Authenticate()
 /*                     VSIInstallSwiftFileHandler()                     */
 /************************************************************************/
 
-/**
- * \brief Install /vsiswift/ OpenStack Swif Object Storage (Swift) file
- * system handler (requires libcurl)
- *
- * @see <a href="gdal_virtual_file_systems.html#gdal_virtual_file_systems_vsiswift">/vsiswift/ documentation</a>
- *
- * @since GDAL 2.3
+/*!
+ \brief Install /vsiswift/ OpenStack Swif Object Storage (Swift) file
+ system handler (requires libcurl)
+
+ \verbatim embed:rst
+ See :ref:`/vsiswift/ documentation <vsiswift>`
+ \endverbatim
+
+ @since GDAL 2.3
  */
 void VSIInstallSwiftFileHandler( void )
 {
