@@ -45,13 +45,15 @@
 #include "cpl_string.h"
 #include "cpl_vsi_virtual.h"
 
-CPL_CVSID("$Id: cpl_vsil_tar.cpp 9cc19c14578d9365ede5e2806c2ae9f746f8c48e 2021-10-09 15:49:40 +0200 Even Rouault $")
 
 #if (defined(DEBUG) || defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)) && !defined(HAVE_FUZZER_FRIENDLY_ARCHIVE)
 /* This is a completely custom archive format that is rather inefficient */
 /* but supports random insertions or deletions, since it doesn't record */
 /* explicit file size or rely on files starting on a particular boundary */
 #define HAVE_FUZZER_FRIENDLY_ARCHIVE 1
+#endif
+
+#ifdef HAVE_FUZZER_FRIENDLY_ARCHIVE
 constexpr int HALF_BUFFER_SIZE = 1024;
 constexpr int BUFFER_SIZE = 2 * HALF_BUFFER_SIZE;
 #endif
@@ -619,37 +621,20 @@ VSIVirtualHandle* VSITarFilesystemHandler::Open( const char *pszFilename,
 /*                    VSIInstallTarFileHandler()                        */
 /************************************************************************/
 
-/**
- * \brief Install /vsitar/ file system handler.
- *
- * A special file handler is installed that allows reading on-the-fly in TAR
- * (regular .tar, or compressed .tar.gz/.tgz) archives.
- *
- * All portions of the file system underneath the base path "/vsitar/" will be
- * handled by this driver.
- *
- * The syntax to open a file inside a tar file is /vsitar/path/to/the/file.tar/path/inside/the/tar/file
- * were path/to/the/file.tar is relative or absolute and path/inside/the/tar/file
- * is the relative path to the file inside the archive.
- *
- * Starting with GDAL 2.2, an alternate syntax is available so as to enable
- * chaining and not being dependent on .tar extension :
- * /vsitar/{/path/to/the/archive}/path/inside/the/tar/file. Note that /path/to/the/archive
- * may also itself this alternate syntax.
- *
- * If the path is absolute, it should begin with a / on a Unix-like OS (or C:\ on Windows),
- * so the line looks like /vsitar//home/gdal/...
- * For example gdalinfo /vsitar/myarchive.tar/subdir1/file1.tif
- *
- * Syntactic sugar : if the tar archive contains only one file located at its
- * root, just mentioning "/vsitar/path/to/the/file.tar" will work
- *
- * VSIStatL() will return the uncompressed size in st_size member and file
- * nature- file or directory - in st_mode member.
- *
- * Directory listing is available through VSIReadDir().
- *
- * @since GDAL 1.8.0
+/*!
+ \brief Install /vsitar/ file system handler.
+
+ A special file handler is installed that allows reading on-the-fly in TAR
+ (regular .tar, or compressed .tar.gz/.tgz) archives.
+
+ All portions of the file system underneath the base path "/vsitar/" will be
+ handled by this driver.
+
+ \verbatim embed:rst
+ See :ref:`/vsitar/ documentation <vsitar>`
+ \endverbatim
+
+ @since GDAL 1.8.0
  */
 
 void VSIInstallTarFileHandler(void)
