@@ -37,7 +37,6 @@
 #include "cpl_vsi.h"
 
 
-CPL_CVSID("$Id: cpl_worker_thread_pool.cpp 9771b16ddd8d2e577e1875984b0737b220964b99 2020-05-21 11:31:01 +0200 Even Rouault $")
 
 struct CPLWorkerThreadJob
 {
@@ -45,6 +44,8 @@ struct CPLWorkerThreadJob
     void          *pData;
 };
 
+static thread_local CPLWorkerThreadPool* threadLocalCurrentThreadPool = nullptr;
+
 /************************************************************************/
 /*                         CPLWorkerThreadPool()                        */
 /************************************************************************/
@@ -96,6 +97,8 @@ void CPLWorkerThreadPool::WorkerThreadFunction(void* user_data)
     CPLWorkerThread* psWT = static_cast<CPLWorkerThread*>(user_data);
     CPLWorkerThreadPool* poTP = psWT->poTP;
 
+    threadLocalCurrentThreadPool = poTP;
+
     if( psWT->pfnInitFunc )
         psWT->pfnInitFunc( psWT->pInitData );
 
@@ -129,12 +132,40 @@ void CPLWorkerThreadPool::WorkerThreadFunction(void* user_data)
  */
 bool CPLWorkerThreadPool::SubmitJob( CPLThreadFunc pfnFunc, void* pData )
 {
-    CPLAssert( !aWT.empty() );
+    CPLAssert( m_nMaxThreads > 0 );
+
+    bool bMustIncrementWaitingWorkerThreadsAfterSubmission = false;
+    if( threadLocalCurrentThreadPool == this )
+    {
+        // If there are waiting threads or we have not started all allowed
+        // threads, we can submit this job asynchronously
+        {
+            std::unique_lock<std::mutex> oGuard(m_mutex);
+            if( nWaitingWorkerThreads > 0 || static_cast<int>(aWT.size()) < m_nMaxThreads )
+            {
+                bMustIncrementWaitingWorkerThreadsAfterSubmission = true;
+                nWaitingWorkerThreads --;
+            }
+        }
+        if( !bMustIncrementWaitingWorkerThreadsAfterSubmission )
+        {
+            // otherwise there is a risk of deadlock, so execute synchronously.
+            pfnFunc(pData);
+            return true;
+        }
+    }
 
     CPLWorkerThreadJob* psJob = static_cast<CPLWorkerThreadJob *>(
         VSI_MALLOC_VERBOSE(sizeof(CPLWorkerThreadJob)));
     if( psJob == nullptr )
+    {
+        if( bMustIncrementWaitingWorkerThreadsAfterSubmission )
+        {
+            std::unique_lock<std::mutex> oGuard(m_mutex);
+            nWaitingWorkerThreads ++;
+        }
         return false;
+    }
     psJob->pfnFunc = pfnFunc;
     psJob->pData = pData;
 
@@ -143,12 +174,42 @@ bool CPLWorkerThreadPool::SubmitJob( CPLThreadFunc pfnFunc, void* pData )
     if( psItem == nullptr )
     {
         VSIFree(psJob);
+        if( bMustIncrementWaitingWorkerThreadsAfterSubmission )
+        {
+            std::unique_lock<std::mutex> oGuard(m_mutex);
+            nWaitingWorkerThreads ++;
+        }
         return false;
     }
     psItem->pData = psJob;
 
     std::unique_lock<std::mutex> oGuard(m_mutex);
 
+    if( bMustIncrementWaitingWorkerThreadsAfterSubmission )
+        nWaitingWorkerThreads ++;
+
+    if( static_cast<int>(aWT.size()) < m_nMaxThreads )
+    {
+        // CPLDebug("CPL", "Starting new thread...");
+        std::unique_ptr<CPLWorkerThread> wt(new CPLWorkerThread);
+        wt->pfnInitFunc = nullptr;
+        wt->pInitData = nullptr;
+        wt->poTP = this;
+        wt->bMarkedAsWaiting = false;
+        wt->hThread =
+            CPLCreateJoinableThread(WorkerThreadFunction, wt.get());
+        if( wt->hThread == nullptr )
+        {
+            VSIFree(psJob);
+            VSIFree(psItem);
+            return false;
+        }
+        else
+        {
+            aWT.emplace_back(std::move(wt));
+        }
+    }
+
     psItem->psNext = psJobQueue;
     psJobQueue = psItem;
     nPendingJobs++;
@@ -198,7 +259,18 @@ bool CPLWorkerThreadPool::SubmitJob( CPLThreadFunc pfnFunc, void* pData )
 bool CPLWorkerThreadPool::SubmitJobs(CPLThreadFunc pfnFunc,
                                      const std::vector<void*>& apData)
 {
-    CPLAssert( !aWT.empty() );
+    CPLAssert( m_nMaxThreads > 0 );
+
+    if( threadLocalCurrentThreadPool == this )
+    {
+        // If SubmitJob() is called from a worker thread of this queue,
+        // then synchronously run the task to avoid deadlock.
+        for(size_t i=0;i<apData.size();i++)
+        {
+            pfnFunc(apData[i]);
+        }
+        return true;
+    }
 
     std::unique_lock<std::mutex> oGuard(m_mutex);
 
@@ -207,6 +279,26 @@ bool CPLWorkerThreadPool::SubmitJobs(CPLThreadFunc pfnFunc,
 
     for(size_t i=0;i<apData.size();i++)
     {
+        if( static_cast<int>(aWT.size()) < m_nMaxThreads )
+        {
+            std::unique_ptr<CPLWorkerThread> wt(new CPLWorkerThread);
+            wt->pfnInitFunc = nullptr;
+            wt->pInitData = nullptr;
+            wt->poTP = this;
+            wt->bMarkedAsWaiting = false;
+            wt->hThread =
+                CPLCreateJoinableThread(WorkerThreadFunction, wt.get());
+            if( wt->hThread == nullptr )
+            {
+                if( aWT.empty() )
+                    return false;
+            }
+            else
+            {
+                aWT.emplace_back(std::move(wt));
+            }
+        }
+
         CPLWorkerThreadJob* psJob = static_cast<CPLWorkerThreadJob*>(
             VSI_MALLOC_VERBOSE(sizeof(CPLWorkerThreadJob)));
         if( psJob == nullptr )
@@ -324,6 +416,7 @@ void CPLWorkerThreadPool::WaitEvent()
             break;
         }
         m_cv.wait(oGuard);
+        // cppcheck-suppress knownConditionTrueFalse
         if( nPendingJobs < nPendingJobsBefore )
         {
             break;
@@ -366,6 +459,16 @@ bool CPLWorkerThreadPool::Setup(int nThreads,
 {
     CPLAssert( nThreads > 0 );
 
+    if( nThreads > static_cast<int>(aWT.size()) &&
+        pfnInitFunc == nullptr && pasInitData == nullptr &&
+        !bWaitallStarted )
+    {
+        std::lock_guard<std::mutex> oGuard(m_mutex);
+        if( nThreads > m_nMaxThreads )
+            m_nMaxThreads = nThreads;
+        return true;
+    }
+
     bool bRet = true;
     for(int i=static_cast<int>(aWT.size());i<nThreads;i++)
     {
@@ -385,6 +488,12 @@ bool CPLWorkerThreadPool::Setup(int nThreads,
         aWT.emplace_back(std::move(wt));
     }
 
+    {
+        std::lock_guard<std::mutex> oGuard(m_mutex);
+        if( nThreads > m_nMaxThreads )
+            m_nMaxThreads = nThreads;
+    }
+
     if( bWaitallStarted )
     {
         // Wait all threads to be started
