@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_worker_thread_pool.h 1df63aa4d30bc389133c38822002d5ecfb91c314 2020-05-21 11:30:55 +0200 Even Rouault $
+ * $Id$
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  CPL worker thread pool
@@ -84,11 +84,13 @@ class CPL_DLL CPLWorkerThreadPool
         std::condition_variable m_cv{};
         volatile CPLWorkerThreadState eState = CPLWTS_OK;
         CPLList* psJobQueue = nullptr;
-        volatile int nPendingJobs = 0;
+        int nPendingJobs = 0;
 
         CPLList* psWaitingWorkerThreadsList = nullptr;
         int nWaitingWorkerThreads = 0;
 
+        int m_nMaxThreads = 0;
+
         static void WorkerThreadFunction(void* user_data);
 
         void DeclareJobFinished();
@@ -115,7 +117,7 @@ class CPL_DLL CPLWorkerThreadPool
         void WaitEvent();
 
         /** Return the number of threads setup */
-        int GetThreadCount() const { return static_cast<int>(aWT.size()); }
+        int GetThreadCount() const { return m_nMaxThreads; }
 };
 
 /** Job queue */
