@@ -17,12 +17,12 @@ else
     HAS_NUMPY=no
 fi
 
-INCFLAGS="-I${PYTHONHOME}/include -I../../port -I../../generated_headers -I../../gcore -I../../alg -I../../ogr/ -I../../gnm -I../../apps/"
-LINKFLAGS="-L../../.libs -lgdal -L${PYTHONHOME}/libs -l${PYTHONLIB}"
+INCFLAGS="-I${PYTHONHOME}/include -I../../port -I../../build/port -I../../gcore -I../../build/gcore -I../../alg -I../../ogr/ -I../../gnm -I../../apps/"
+LINKFLAGS="-L../../build -lgdal -L${PYTHONHOME}/libs -l${PYTHONLIB}"
 CFLAGS="-O2 -D__MSVCRT_VERSION__=0x0601"
 
 # Run native python
-wine "$PYTHONHOME/python" setup.py build
+wine "$PYTHONHOME/python" setup_fallback_mingw.py build
 
 # Determine OUTDIR
 if test -d build/lib.win32-3.7; then
